program estimate_model

use enum_mod
use model_params_mod
use interp_support_mod

use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use normal_mod
use stat_estimates_mod

use likelihood_mod
use bayesian_mod

use csv_file
use mpi_mod


   integer, parameter :: arg_no_params_file = 1
   integer, parameter :: arg_no_seed_file = 2
   integer, parameter :: arg_no_running_out_file = 3
   integer, parameter :: arg_no_bayes_export_file = 4

   character(len=30) :: running_output_file_here

   character(len=30) :: init_params_file
   character(len=30),parameter :: sipp_data_file= "sipp_fortran_big.csv"
   character(len=30)          :: interact_file = "interact_matrix.csv"
   character(len=30) :: seed_file


   type(bayes_obj_type) :: bayes_obj
   type(mcmc_output_type), allocatable :: mcmc_output !make allocatable because I only need this object for the root proc.
   type(value_grid_type)  :: value_grid
   type(like_type), dimension(:), allocatable :: like_object
   type(like_target_type), dimension(:), allocatable :: like_target
   type(data_i_type), dimension(:), allocatable ::  hh_data

   real*8, dimension(n_main_params) :: params_init
   integer :: unit_int, i

   call mpi_setup()

   call get_command_argument(arg_no_params_file,      init_params_file )
   call get_command_argument(arg_no_seed_file,        seed_file )
   call get_command_argument(arg_no_running_out_file, running_output_file_here )
   call get_command_argument(arg_no_bayes_export_file, bayes_export_file )


   running_output_file = running_output_file_here


   if( myrank .eq. 0) print *, "start of estimate"


   if (myrank .eq. 0 ) then
      print *, "N_obs_total", N_obs_total
      print *, "data file", sipp_data_file
      print *, "init params file", init_params_file
      print *, "max iter", n_mcmc_iter
      print *, "max time", max_time/3600.0, "hours"
      print *, "seed save file", seed_file
      print *, "running output file", running_output_file
      print *, "n_iter_bayes", n_iter_bayes

      print *, "  "
      print *, "est_dynamic", est_dynamic
      print *, "est_savings", est_savings
      print *, "est_hetero", est_hetero
      print *, "ind_draws_free",  ind_draws_free
      print *, "read_interp_matrices", read_interp_matrices
      print *, "error check", error_check

      print *, "number of processes", nprocs

   end if

 !  allocate(mcmc_output)
 !  print *, "allocated mcmc_output"
 !  read(*,*)




   !initialize random numbers
  ! call init_random_seed_mpi(same = .false. )
  ! call export_random_seed_mpi( seed_file )
    call read_random_seed_mpi( same = .false., filename_opt = seed_file)

   call print_random_seed_mpi()

   if (myrank .eq. 0) print *, "about to setup objects"

   call read_sipp_data( sipp_data_file, hh_data  )
   call bayes_obj % setup( init_params_file )



   if (est_dynamic .eqv. .false. ) then
      bayes_obj % params_current( param_weight_utility) = 2
   end if

   call interp_setup_all( interact_file )
   call value_grid % setup( bayes_obj % params_current )
 !  call bayes_obj % read_ols_init()

   !read in other interp matrices if I want
 !  if ( (read_interp_matrices .eqv. .true. )  .and. (est_dynamic .eqv. .true. ) ) then
   if  (read_interp_matrices .eqv. .true. ) then
      call read_interp_init( bayes_obj )
      if (myrank .eq. 0)  print *, "read in initial interp matrices"
   end if




!   print *, "free blocks", free_blocks
!   print *, "free params", free_param_list
!   print *, "rw joboffer", rw_chol_joboffer_block
!   print *, "rw wage", rw_chol_wage_block
!   print *, "rw leisure", rw_chol_leisure_block
!   print *, "rw asset_error", rw_chol_asset_error_block



   params_init(:n_main_params) = bayes_obj % params_current(:n_main_params)

   !set discount factor to 0
  ! bayes_obj % params_current(param_discount_factor)=0.0
  ! bayes_obj % params_current(param_sd_non_pec)=0.25
  ! bayes_obj % params_current(param_weight_utility)=1.0

   call like_setup_all( hh_data, &
                        bayes_obj % params_current, &
                        est_savings, &
                        bayes_obj % value_matrix_current, &
                        bayes_obj % asset_matrix_current, &
                        like_object, &
                        like_target )


   if (myrank .eq. 0)  print *, "setup everything"


 !  print *, bayes_obj % params_proposal

!   stop

   call mcmc_iter_main( bayes_obj, value_grid, hh_data, like_object, like_target)


   if ( myrank .eq. 0 ) then

      if (running_on_screen .eqv. .false.) open(newunit= print_unit,file=running_output_file,position="append")

         write(print_unit,*), "   "
         write(print_unit,*), " ****"
         write(print_unit,*), " done with mcmc_iter_main"
         write(print_unit,*), " "

       if (running_on_screen .eqv. .false.) close(print_unit)

   end if

   if ( myrank .eq. 0 ) then
      allocate(mcmc_output)
   end if

   if (myrank .eq. 0 ) then
      call mcmc_output_create( mcmc_output, bayes_obj )
   else
      call mcmc_output_create( bayes_obj_in = bayes_obj )
   end if

   call mpi_barrier(mpi_comm_world, err)

   if( myrank .eq. 0 ) call mcmc_output % write(params_init)
  ! call bayes_obj % random_walk_write()

  call mpi_finalize(err)


end program estimate_model









