program hetero_param_draw_test
!tests drawing unobserved heterogeneity parameters

use enum_mod
use model_params_mod
use interp_support_mod

use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use normal_mod
use stat_estimates_mod

use likelihood_mod
use bayesian_mod

use csv_file
use mpi_mod


   character(len=30),parameter :: init_params_file = "init_params_sept24.txt"
   character(len=30),parameter :: sipp_data_file= "sipp_fortran_small.csv"
   character(len=30), parameter :: interact_file_in  = "interact_matrix.csv"
   character(len=30), parameter :: seed_file = "seed_save5.txt"


   character(len=30),parameter :: value_init_file   = "value_matrix_out.csv"
   character(len=30),parameter :: asset_init_file   = "asset_matrix_out.csv"



   type(bayes_obj_type) :: bayes_obj
   type(like_type), dimension(:), allocatable :: like_object
   type(like_target_type), dimension(:), allocatable :: like_target
   type(data_i_type), dimension(:), allocatable ::  hh_data

   real*8, dimension(n_hetero, n_hetero) :: hetero_chol_old, hetero_chol_new

   integer :: i, t, b,j,block_in,n_params_block_in

   integer, parameter :: i_print = 1
   integer, parameter :: t_print = 5

   integer, parameter :: n_init_draws = 100 !number of draws to do initially

   real*8, dimension(iter_m_job_offer: iter_hours,N_t,N_obs_total) :: joboffer_random
   real*8, dimension(N_obs_total) ::  mh_random



   !first setup based on block of interest


   call mpi_setup

   if( myrank .eq. 0 ) then

      print *, "i_print ", i_print
      print *, "t_print ", t_print
      print *, " "

   end if

   !initialize random numbers
 !  call init_random_seed_mpi(same = .false. )
 !  call export_random_seed_mpi( seed_file )
   call read_random_seed_mpi( same = .false., filename_opt = seed_file)

   call gen_labels()
   call interp_setup_all(interact_file_in)

   call read_sipp_data(sipp_data_file, hh_data)
   call bayes_obj % setup(init_params_file)

   bayes_obj % iter = 1

   !read in other ols vecs if I want
   if (read_interp_matrices .eqv. .true. ) then
      call init_interp_matrices( value_init_file, asset_init_file, &
         bayes_obj % value_matrix_current, bayes_obj % asset_matrix_current )

   else
      bayes_obj % value_matrix_current = 0
      bayes_obj % asset_matrix_current = 0
      bayes_obj % asset_matrix_current(1) = 2.9957322735539909

   end if
   print *, "   "


   hetero_chol_old = bayes_obj % hetero_chol_current


   call like_setup_all( hh_data, &
                        bayes_obj % params_current, &
                        .true., &
                        bayes_obj % value_matrix_current, &
                        bayes_obj % asset_matrix_current, &
                        like_object, &
                        like_target )

   !switch individual variables
   !do one loop first

   bayes_obj % iter=1

   do i=1,n_init_draws
      call random_number(joboffer_random)
      call random_number(mh_random)

      call mh_step_ind( bayes_obj, block_joboffers_ind, hh_data, like_object, like_target, &
                     joboffer_random(:,:,min_obs:max_obs), mh_random(min_obs:max_obs) )
   end do
   print *, "got individual joboffers"



   call wishart_step_hetero( bayes_obj, like_object)

   hetero_chol_new = bayes_obj % hetero_chol_current

   if (myrank .eq. 0 ) then

      print *, "old heter chol"
      do i=1,n_hetero
         print *, (hetero_chol_old(i,j), j=1,n_hetero )
      end do


      print *, "new heter chol"
      do i=1,n_hetero
         print *, (hetero_chol_new(i,j), j=1,n_hetero )
      end do

   end if

   call mpi_finalize(err)

end program
