program interp_kernel_test

!tests the interp kernel weighting stuff

use enum_mod
use model_params_mod
use interp_support_mod

use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use normal_mod
use stat_estimates_mod

use likelihood_mod
use bayesian_mod

use csv_file
use mpi_mod

   type(bayes_obj_type) :: bayes_obj
   type(value_grid_type) :: value_grid

   integer, parameter :: n_param_draws_test = 100

   real*8, dimension(n_main_params) :: params_init
   real*8, dimension( n_grid_points )  :: value_matrix_main, value_matrix_est, &
      asset_matrix_scratch, value_matrix_post

   character(len=30),parameter :: init_params_file = "init_params_simple4.txt"
   character(len=30),parameter :: interact_file  = "interact_matrix.csv"
   character(len=30), parameter :: seed_file = "seed_save5.txt"


   real*8 :: real_ignore
   integer :: i,b
   logical :: kernel_error
 !  allocate(mcmc_output)
 !  print *, "allocated mcmc_output"
 !  read(*,*)

   call mpi_setup


   if (n_param_draws_test > max_iter_saved  ) then
      print *, " n_param_draws_test  needs to be smaller than max_iter_saved"
      stop
   end if



   !initialize random numbers
 !  call init_random_seed_mpi(same = .false. )
 !  call export_random_seed_mpi( seed_file )
   call read_random_seed_mpi( same = .false., filename_opt = seed_file)

   print *, "about to setup objects"

   call gen_labels()
   call bayes_obj % setup(init_params_file)

   print *, "finished setting up objects"

   bayes_obj % iter = 1
    bayes_obj % iter_saved = 0


   !need to change this
   call interp_setup_all( interact_file )
   call value_grid % setup( bayes_obj % params_current )



   params_init = bayes_obj % params_current(1:n_main_params)
   !iterate once at the initial variables, and then store this ols interpolation vector


   call iterate_once(   value_grid, &
                        bayes_obj % params_current, &
                        bayes_obj % asset_matrix_current, &
                        bayes_obj % value_matrix_current, &
                        value_matrix_main )

   !now gets ols vectors for random nearby parameters
   asset_matrix_scratch= 0

   do i= 1,  n_param_draws_test
      if (myrank .eq. 0 ) print *, "iter # ", i

      bayes_obj % params_current(1:n_main_params) = params_init(1:n_main_params)
      do b=1, n_blocks_params
         call draw_new_params(b, bayes_obj % params_current, bayes_obj % params_proposal)
      end do

    !  call write_params( bayes_obj % params_current, 6, .false.)
    !  read(*,*)

   !   bayes_obj % past_params_proposal(:,i) = bayes_obj % params_proposal

      call iterate_once(   value_grid, &
                           bayes_obj % params_current, &
                           bayes_obj % asset_matrix_current, &
                           bayes_obj % value_matrix_current, &
                           value_matrix_post )
   !   print *, "value matrix post", value_matrix_post(1,0,0)

      call save_bayesian_params_interp( bayes_obj, value_matrix_post, &
         asset_matrix_scratch)

   end do


   !update the bandwidth
!   bayes_obj % iter_saved = n_param_draws_test
   call bayes_obj % update_bandwidth()




   !now get the kernel estimate

   bayes_obj % time_weighting = .false.

   call interp_kernel(  bayes_obj, &
                     params_init, &
                     value_matrix_est, &
                     asset_matrix_scratch, &
                     kernel_error )

   if (myrank .eq. 0 ) then

      print *, "difference between original and kernel estimate"


      do i=1, n_grid_points
         print *, i,  value_matrix_main(i),  value_matrix_est(i), &
                      value_matrix_main(i) - value_matrix_est(i)

      end do


   end if

   call mpi_finalize(err)

contains

subroutine iterate_once( value_grid_inout, params_in, asset_matrix_in, value_matrix_in, value_matrix_out )
   !iterates the value function once without updating the asset policy funciton

   type(value_grid_type), intent(in out) :: value_grid_inout
   real*8, dimension(n_params) :: params_in
   real*8, dimension(n_grid_points), intent(in)  :: asset_matrix_in, value_matrix_in
   real*8, dimension(n_grid_points), intent(out) :: value_matrix_out


   !now iterate the value function
   call update_probs_all_node (  value_grid_inout, params_in )
   call update_asset_util_all_node ( value_grid_inout, params_in, asset_matrix_in )
   call iterate_value_all_node(  value_grid_inout, &
                                 value_matrix_in, &
                                 params_in, &
                                 value_matrix_out )


end subroutine iterate_once



end program









