program ols_kernel_test

!draws ols matrices for parameters centered around a mean

use enum_mod
use model_params_mod
use interp_support_mod

use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use normal_mod
use stat_estimates_mod

use likelihood_mod
use bayesian_mod

use csv_file
use mpi_mod

   integer, parameter :: arg_no_params_file = 1
   integer, parameter :: arg_no_seed_file = 2

   type(bayes_obj_type) :: bayes_obj
   type(value_grid_type) :: value_grid
   real*8, dimension(n_grid_points) :: value_matrix_temp
   real*8, dimension(n_grid_points ) :: asset_matrix_temp

   character(len=30) :: init_params_file
   character(len=30)          :: interact_file  = "interact_matrix.csv"
   character(len=30) :: seed_file


   integer :: i, unit_int, m_job, job, j
   real*8 :: real_ignore

   call mpi_setup()

   call get_command_argument(arg_no_params_file,      init_params_file )
   call get_command_argument(arg_no_seed_file,        seed_file )

   !initialize random numbers
 !  call init_random_seed_mpi(same = .false. )
 !  call export_random_seed_mpi( seed_file )
   call read_random_seed_mpi( same = .false., filename_opt = seed_file)

   if (est_dynamic .eqv. .false. ) then
      print *, "est_dynamic needs to be true, stopping"
      stop
   end if

   if( myrank .eq. 0 ) print *, "about to setup objects"

   call gen_labels()
   call bayes_obj % setup(init_params_file)
   call interp_setup_all( interact_file )
   call value_grid % setup( bayes_obj % params_current )




   call interp_matrix_bayesian_init( bayes_obj, value_grid )

   !now see if I can call the read routine without it crashing
   call read_interp_init( bayes_obj )

   !now printing to see if read correctly
   if( myrank .eq. 0 ) then

      print *, bayes_obj % params_current(1), bayes_obj % value_matrix_current(1)
      do i=1, n_draws_interp_init
         print *, i, bayes_obj % past_params_kernel(1,i), bayes_obj % past_value_matrix(1,i)
      end do

   end if


   if (myrank .eq. 0 ) then
      !reread base value matrix to see how kernel weighting is doing
      open(newunit = unit_int, file=init_interp_file, status = "old" )

      read(unit_int, *) real_ignore


      read(unit_int,*) ( value_matrix_temp(j), j=1,n_grid_points)
      read(unit_int,*) ( asset_matrix_temp(j), j=1,n_grid_points)


      close(unit_int )


      if( myrank .eq. 0 ) print *, "difference between kernel estimates and original"


      do i=1, n_grid_points
         print *, i,  bayes_obj % value_matrix_current(i),  value_matrix_temp(i), &
                      bayes_obj % value_matrix_current(i) - value_matrix_temp(i)

      end do




   end if


   call mpi_finalize(err)

end program









