program iterate_converge_test

      !testing to see if my value function will converge

use enum_mod
use model_params_mod
use interp_support_mod


use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use normal_mod
use stat_estimates_mod

use csv_file


implicit none

   logical, parameter :: max_switch = .false. !controls whether to maximize the value function wrt asset in order to interate value function
      !or whether to speed things up with an asset interpolation function.

!   character(len=30),parameter   :: init_params_file  = "output_params_feb19.txt"
    integer, parameter :: arg_no_params_file = 1
    character(len=30)   :: init_params_file


!   character(len=30),parameter :: init_params_file = "init_params_test.txt"
   character(len=30),parameter :: value_init_file = "value_matrix_test_init.csv"
   character(len=30),parameter :: asset_init_file = "asset_matrix_test_init.csv"
   character(len=30),parameter :: interact_file = "interact_matrix.csv"


!   character(len=30),parameter :: stata_file        = "stata_file.csv"
   character(len=30),parameter :: value_matrix_out_file   = "value_matrix_out.csv"
   character(len=30),parameter :: asset_matrix_out_file   = "asset_matrix_out.csv"
   character(len=30),parameter :: error_matrix_out_file   = "error_matrix_out.csv"
   character(len=30),parameter :: value_grid_out_file     = "value_grid_out.csv"


 !  character(len=30),parameter :: value_matrix_out_file      = "value_matrix_july22.csv"
 !  character(len=30),parameter :: asset_matrix_out_file   = "asset_matrix_july22.csv"
 !  character(len=30),parameter :: derivative_file   = "derivative_value_july22.csv"
 !  character(len=30),parameter :: value_grid_out_file     = "value_grid_july22.csv"

   logical, parameter :: read_ols_matrices = .false. !for whether to init the ols matrices with other files

   integer, parameter :: iter_init = -1
   integer, parameter :: max_iter = 500
   real*8, parameter :: tol = 5.0D-2
   integer, parameter :: n_howard = 0  !number of times to do the howard-improvement algorithm in which I only iterate and don't update the choice probability
   real*8, parameter :: tol_howard = 1.0D-3


   type(value_grid_type) :: value_grid
   real*8, dimension( n_grid_points ) :: value_matrix_init, value_matrix_out
   real*8, dimension( n_grid_points ) :: asset_matrix_init, asset_matrix_out
   real*8, dimension(n_params)  :: params

   integer:: m_job,job
   logical :: file_exists
   integer :: i,j, unit_int
   real*8 :: start,finish


   call mpi_setup

   call get_command_argument(arg_no_params_file, init_params_file )

  ! print_test_global = .true.

   if( myrank .eq. 0 ) print *, "init params file", init_params_file
   !now read in the parameters


   if (myrank .eq. 0 ) call init_params(init_params_file, params)
   call mpi_bcast( params, size(params), mpi_real8, 0, mpi_comm_world, err )


   call gen_labels()




!   do i=1,n_params
!      print *, param_labels(i), params(i)
!   end do

   call interp_setup_all( interact_file )
   call value_grid % setup( params )


   if (read_ols_matrices .eqv. .true. ) then
      call init_interp_matrices( value_init_file, asset_init_file, value_matrix_init, asset_matrix_init )

   else
      value_matrix_init = 0
      asset_matrix_init = 0
      asset_matrix_init(1) = 2.9957322735539909

   end if

   !reading in an ols matrix, if needed
!   print *, "reading in a the ols matrix"
!   open(unit=1, file=ols_out_file )
!   call ols_matrix_read(ols_matrix_out,1)
!   close(1)

  ! print *, "sd non pec", params( param_sd_non_pec )
  ! print *, "utility_weight", params( param_weight_utility )

   if (myrank .eq. 0 ) print *, "iterating the value function to convergence"
   call cpu_time(start)

   call iterate_to_converge_all( max_switch, &
                                 iter_init, &
                                 max_iter, &
                                 tol, &
                                 n_howard, &
                                 tol_howard, &
                                 params, &
                                 value_matrix_init, &
                                 asset_matrix_init, &
                                 value_grid, &
                                 value_matrix_out, &
                                 asset_matrix_out )

   call cpu_time(finish)
   if( myrank .eq. 0 ) print *, "done with iterating the value function to convergence, time:", &
      finish-start

   if (myrank .eq. 0 ) then
      print *, "writting the interp coefficients"
      open(newunit = unit_int, file=value_matrix_out_file, status='replace' )
      call interp_matrix_write(value_matrix_out,unit_int )
      close(unit_int)


      open(newunit = unit_int, file=asset_matrix_out_file, status='replace' )
      call interp_matrix_write(asset_matrix_out,unit_int )
      close(unit_int)

   end if

!   if (myrank .eq. 0 ) print *, "writting the grid values"
!   if (myrank .eq. 0 ) open(newunit = unit_int, file=value_grid_out_file, status='replace')
!   call grid_value_write("v",value_grid, unit_int)
!   if (myrank .eq. 0 ) close(unit_int)



!   if (myrank .eq. 0 ) then
!      print *, "getting derivatives"
!      open( newunit = unit_int,file= derivative_file, status='replace' )
!      call value_derivative_write( value_matrix_out, asset_matrix_init, unit_int )

!     ! call value_derivative_write( value_matrix_init, asset_matrix_init, 1 )

!      close(unit_int)

!   end if

   call mpi_finalize(err)

end program

