program iterate_converge_test

      !testing to see if my value function will converge

use enum_mod
use model_params_mod
use interp_support_mod


use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use normal_mod
use stat_estimates_mod

use csv_file


implicit none


   character(len=30),parameter :: init_params_file = "output_params.txt"
   character(len=30),parameter :: value_init_file = "value_matrix_init.csv"
   character(len=30),parameter :: asset_init_file = "asset_matrix_init.csv"
   character(len=30),parameter :: interact_file = "interact_matrix.csv"


!   character(len=30),parameter :: stata_file        = "stata_file.csv"
   character(len=30),parameter :: value_matrix_out_file   = "value_matrix_out.csv"
   character(len=30),parameter :: asset_matrix_out_file   = "asset_matrix_out.csv"
   character(len=30),parameter :: value_grid_out_file     = "value_grid_out.csv"

   logical, parameter :: read_interp_matrices = .false. !for whether to init the ols matrices with other files
   logical, parameter :: est_savings = .false.

   integer, parameter :: max_iter = 5000
   real*8, parameter :: tol = 1.0D-3


   type(value_grid_type) :: value_grid
   real*8, dimension( n_grid_points ) :: value_matrix_init, value_matrix_out
   real*8, dimension( n_grid_points ) :: asset_matrix_init
   real*8, dimension(n_params)  :: params

   integer:: m_job,job
   logical :: file_exists
   integer :: i,j, unit_int
   real*8 :: start,finish,assets_temp, value_temp1, value_temp2

   call mpi_setup


   if( myrank .eq. 0) print *, "init params file", init_params_file
   !now read in the parameters
   call init_params(init_params_file, params)

   call gen_labels()

!   do i=1,n_params
!      print *, param_labels(i), params(i)
!   end do

   call interp_setup_all( interact_file )
   call value_grid % setup( params )


   if (read_interp_matrices .eqv. .true. ) then
      call init_interp_matrices( value_init_file, asset_init_file, value_matrix_init, asset_matrix_init )

   else
      value_matrix_init = 0
      asset_matrix_init = 0
      asset_matrix_init(1) = 2.9957322735539909

   end if

   !reading in an ols matrix, if needed
!   print *, "reading in a the ols matrix"
!   open(unit=1, file=ols_out_file )
!   call ols_matrix_read(ols_matrix_out,1)
!   close(1)

!   print *, "sd non pec", params( param_sd_non_pec )
!   print *, "utility_weight", params( param_weight_utility )

   if (myrank .eq. 0 ) print *, "iterating the value function to convergence"
   call iterate_to_converge_value(  max_iter, &
                                    tol, &
                                    est_savings, &
                                    params, &
                                    value_matrix_init, &
                                    asset_matrix_init, &
                                    value_grid, &
                                    value_matrix_out )


   if(myrank .eq. 0 ) print *, "done with iterating the value function to convergence"


   if(myrank .eq. 0 ) then
      print *, "writting the interp coefficients"
      open(newunit = unit_int, file=value_matrix_out_file, status='replace' )
      call interp_matrix_write(value_matrix_out,unit_int )
      close(unit_int)
   end if

   if(myrank .eq. 0 )  print *, "writting the grid values"
   if(myrank .eq. 0 )  open(newunit = unit_int, file=value_grid_out_file, status='replace')
   call grid_value_write("v",value_grid, unit_int)
   if(myrank .eq. 0 )  close(unit_int)




   call mpi_finalize(err)





end program

