program iterate_once_test

   !testing to see if iterating the value function once (giving a guess of the policy function) works

use enum_mod
use model_params_mod
use interp_support_mod


use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use normal_mod
use stat_estimates_mod

use csv_file
use random_gen_mod


implicit none


   character(len=30),parameter :: init_params_file = "output_params.txt"
   character(len=30),parameter :: value_init_file = "value_matrix_init.csv"
   character(len=30),parameter :: asset_init_file = "asset_matrix_init.csv"
   character(len=30),parameter :: interact_file = "interact_matrix.csv"


!   character(len=30),parameter :: stata_file        = "stata_file.csv"
   character(len=30),parameter :: value_matrix_out_file   = "value_matrix_out.csv"
   character(len=30),parameter :: asset_matrix_out_file   = "asset_matrix_out.csv"
   character(len=30),parameter :: error_matrix_out_file   = "error_matrix_out.csv"
   character(len=30),parameter :: value_grid_out_file     = "value_grid_out.csv"
 !  character(len=30),parameter :: derivative_file   = "derivative_value_func.csv"
 !  character(len=30),parameter :: derivative_file2  = "derivative_value_func2.csv"

   logical, parameter :: read_ols_matrices = .false. !for whether to init the ols matrices with other files


   type(value_grid_type)  :: value_grid
   real*8, dimension( n_grid_points ) :: value_matrix_init, value_matrix_out
   real*8, dimension( n_grid_points ) :: asset_matrix_init
   real*8, dimension( n_grid_points ) :: error_matrix
   real*8, dimension(n_params)  :: params

   type(mygrid_type) :: mygrid_temp

   real*8 :: asset_high, m_ui_ben_high, origin, my_interact, my_interact_coef

   integer:: m_job,job
   logical :: file_exists
   integer :: i,j, unit_int, s, k
   real*8 :: start,finish
   real*8 :: value_real, value_predict

   call mpi_setup

   print *, "myrank", myrank




   if (myrank .eq. 0 ) print *, "init params file", init_params_file
   !now read in the parameters
   call init_params(init_params_file, params)

  ! print *, "sd non pec", params( param_sd_non_pec )
  ! print *, "utility_weight", params( param_weight_utility )

   call interp_setup_all( interact_file )

   if (read_ols_matrices .eqv. .true. ) then

      call init_interp_matrices( value_init_file, asset_init_file, value_matrix_init, asset_matrix_init )

   else
      value_matrix_init = 0
      asset_matrix_init = 0
      asset_matrix_init(1) = 2.9957322735539909   !this makes the fraction of A+I households set aside for the following period equal to
        ! r/(1+r)

   end if



   call mpi_barrier(mpi_comm_world,err)

   if (myrank .eq. 0 )  print *, "about to allocate and setup the value grid object"

   call value_grid_setup( value_grid, params )
   call mpi_barrier(mpi_comm_world,err)

   if (myrank .eq. 0 )  print *, "about to get the probs of interest"
   call update_probs_all_node ( value_grid, params )


   call mpi_barrier(mpi_comm_world,err)
   if (myrank .eq. 0 )  print *, "about to get the asset choice and utility function"
   call update_asset_util_all_node ( value_grid, params, asset_matrix_init )





   call mpi_barrier(mpi_comm_world,err)
   if (myrank .eq. 0 )  print *, "now just iterating the value function once"
   call iterate_value_all_node( value_grid, value_matrix_init, params,  value_matrix_out )


   if (myrank .eq. 0 ) print *, "done iterating the value function once"



   call mpi_barrier(mpi_comm_world,err)


   if(myrank .eq. 0 ) then

      print *, "writting the interp coefficients"
      open(newunit = unit_int, file=value_matrix_out_file, status='replace' )
      call interp_matrix_write(value_matrix_out,unit_int )
      close(unit_int)

   end if

      if(myrank .eq. 0 )   print *, "writting the grid values"
      if(myrank .eq. 0 )   open(newunit = unit_int, file=value_grid_out_file, status='replace')
                           call grid_value_write("v",value_grid, unit_int)
      if(myrank .eq. 0 )   close(unit_int)



   if ( myrank .eq. 0 ) then
      print *, "writting the asset value function coefficients (just so I can initialize this matrix)"
      open(newunit=unit_int, file=asset_matrix_out_file, status='replace')
      call interp_matrix_write(asset_matrix_init,unit_int)
      close(unit_int)
   end if


   print *, "check the value grid and iterate objects for errors"
   call value_grid % check()


 !  print *, "wrtting output to import into stata"
 !  call value_asset_reg_write( value_grid(0,0) , stata_file )






   !now get the descrepencies at the grid point between actual and predicted values(which should be 0)
   if (myrank .eq. 0 ) print *, "printing decrepencies"
   error_matrix = 0

   do i = value_grid % min_state_point, value_grid % max_state_point
      value_real = value_grid % stored_value(i)
      value_predict = value_function( &
                        value_grid % mygrid_start(i), &
                        value_grid % mygrid_start(i) % rv(var_assets), &
                        value_matrix_out )
      error_matrix(i) = abs( value_real - value_predict )
      print *, i, grid_labels(i), value_real, value_predict, error_matrix(i)
   end do


   open(newunit = unit_int, file=error_matrix_out_file, status='replace' )
   call interp_matrix_write(error_matrix,unit_int )
   close(unit_int)

   print *, "max error (which should be 0) ", maxval( error_matrix ), "myrank", myrank

 !  call value_grid(0,0) % iter_obj(1,35) % mygrid_new % print( .true., .true. )

 ! call value_grid(0,0) % iter_obj(1,36) % mygrid_new % print( .true., .true. )


   call mpi_finalize(err)

!   print *, "getting derivatives2"
!   open( unit=1,file= derivative_file2 )
!   call value_derivative_write2( value_grid, 1 )
!   close(1)


!contains
!
!   !following subroutines calculate the derivatives bases on the EV's I actually got rather than the ones bases on the interpolation function.  Used to see if I'm getting errors with the interpolation
!subroutine value_asset_derivative2(value_grid_in, first_der_value_out, second_der_value_out )
!   !gets the first and second derivative of the value function at a given value of the state variables

!   type(value_grid_type), intent(in) :: value_grid_in
!   real*8, dimension(n_cont_vars),intent(out) :: first_der_value_out
!   real*8, dimension(n_cont_vars,n_cont_vars), intent(out) :: second_der_value_out

!   real*8 :: value_origin, value_perturb1, value_perturb2, value_perturb3, value_perturb4

!   integer :: m_job, job, m_hours, hours, i, j, v,k

!   !for now, won't look at derivative for hours


!   call der_delta_setup()
!   call gen_var_labels()

!   first_der_value_out = 0
!   second_der_value_out = 0


!   m_job    = value_grid_in % m_job
!   job      = value_grid_in % job


!   !get origin value
!   v=1
!   value_origin = value_grid_in % stored_value(v)



!   !first and second deriviative
!   !formula from http://www.math.ohiou.edu/courses/math3600/lecture27.pdf
!   do i=1,n_cont_vars
!      if (var_present(i, job, m_job) .eq. 1) then
!         !forward derivative
!         v=v+1
!         value_perturb1 = value_grid_in % stored_value(v)


!         !backward derivative
!         v=v+1
!         value_perturb2 = value_grid_in % stored_value(v)
!         !now combine to get derivatives
!         first_der_value_out(i) = (value_perturb1 - value_perturb2)/ ( 2*der_delta(i) )
!         second_der_value_out(i,i) = &
!            ( value_perturb1 - 2*value_origin + value_perturb2 )/ (der_delta(i)**2)

!      end if
!   end do

!   !cross-partial derivatives

!   do k = 1, n_value_interact_list(job,m_job)
!      i = value_interact_list( 1, k, job, m_job )
!      j = value_interact_list( 2, k, job, m_job )

!      v=v+1
!      value_perturb1 = value_grid_in % stored_value(v)

!      v=v+1
!      value_perturb2 = value_grid_in % stored_value(v)

!      v=v+1
!      value_perturb3 = value_grid_in % stored_value(v)

!      v=v+1
!      value_perturb4 = value_grid_in % stored_value(v)

!      second_der_value_out(i,j) = &
!         ( value_perturb1 - value_perturb2 - value_perturb3 + value_perturb4 ) /&
!         ( 4* der_delta(i) * der_delta(j) )

!   end do

!end subroutine value_asset_derivative2


!subroutine value_derivative_write2( value_grid_in,  unit_in)
!   !writes the first and second derivative for each (job,m_job) combo
!   type(value_grid_type), dimension(0:1,0:1), intent(in) :: value_grid_in
!   integer, intent(in) :: unit_in

!   integer :: m_job, job, i, i1,i2, j,k,v
!   real*8, dimension(n_cont_vars) :: first_der_value
!   real*8, dimension(n_cont_vars,n_cont_vars) :: second_der_value


!   do m_job = 0,1
!   do job = 0,1

!      !get the origin mygrid type
!      call value_asset_derivative2( value_grid_in(job,m_job ), first_der_value, second_der_value )


!      call csv_write(unit_in, "   ", .true. )

!      call csv_write(unit_in, "m_job", .false.)
!      call csv_write(unit_in, m_job, .false.)
!      call csv_write(unit_in, "job", .false.)
!      call csv_write(unit_in, job, .true.)

!      !value function first
!      call csv_write(unit_in, "   ", .true. )
!      call csv_write(unit_in, "value function", .true.)
!      call csv_write(unit_in, "   ", .true. )

!      !first derivative
!      do i=1,n_cont_vars
!         if (var_present(i, job, m_job) .eq. 1) then

!            call csv_write(unit_in, var_labels(i), .false.)
!            call csv_write(unit_in, first_der_value(i), .true.)

!         end if
!      end do

!      call csv_write(unit_in, "   ", .true. )


!      !second derivatives
!      call csv_write(unit_in, "  ", .false.)

!      do i2=1, n_vars_list (job,m_job)
!         k = var_list(i2, job, m_job)
!         call csv_write(unit_in, var_labels(k),.false.)
!      end do

!      call csv_next_record(unit_in)

!      do i1=1, n_vars_list (job,m_job)
!         j = var_list(i1, job, m_job)
!         call csv_write(unit_in, var_labels(j), .false.)

!         do i2=1, n_vars_list (job,m_job)
!            k = var_list(i2, job, m_job)
!            call csv_write(unit_in, second_der_value(j,k),.false.)

!         end do
!         call csv_next_record(unit_in)

!      end do

!   end do
!   end do

!end subroutine value_derivative_write2



!   print *, "assset nodes", asset_nodes

!   asset_high = value_grid(0,0) % stored_value(36)
!   m_ui_ben_high = value_grid(0,0) % stored_value(40)
!   origin = value_grid(0,0) % stored_value(1)
!   my_interact = value_grid(0,0) % stored_value(65)

!   my_interact_coef = my_interact - asset_high - m_ui_ben_high + origin

!   print *, " "
!   print *, "asset_high", asset_high
!   print *, "m_ui_ben_high", m_ui_ben_high
!   print *, "origin", origin
!   print *, "my interact coef", my_interact_coef

!   print *, "stored interact", value_matrix_out(65,0,0), abs( my_interact - value_matrix_out(65,0,0) )



end program
