program iterate_one_point_test

   !test to see if iterating the value function for one point works

use enum_mod
use model_params_mod
use interp_support_mod


use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use normal_mod
use stat_estimates_mod

use csv_file

implicit none



   character(len=30),parameter :: init_params_file = "output_params.txt"
   character(len=30),parameter :: value_init_file = "value_matrix_init.csv"
   character(len=30),parameter :: asset_init_file = "asset_matrix_init.csv"
   character(len=30),parameter :: interact_file = "interact_matrix.csv"
   logical, parameter :: read_ols_matrices = .false. !for whether to init the ols matrices with other files
   logical, parameter :: iter_switch = .true.

   real*8, parameter ::  h= .5
   integer, parameter :: m_job = 1
   integer, parameter :: job = 0
   integer, parameter :: m_pt = 0
   integer, parameter :: pt = 0
   integer, parameter :: m_no_ui = 0
   integer, parameter :: no_ui = 0

   real*8, dimension(n_params)  :: params, params_peturb
   real*8, dimension( n_grid_points ) :: value_matrix_init
   real*8, dimension( n_grid_points ) :: asset_matrix_init
   type(mygrid_type) :: mygrid, mygrid_origin
   real*8, dimension(n_interp_vars) :: cube_point, cube_point_origin


   real*8 :: value_origin, value_perturb, param_delta
   real*8, dimension(n_interp_vars) :: value_der1


   integer :: n_expect_points
   integer :: seed

   integer :: i,j,k,s

   !initialize job and hours variables

   call mpi_setup

   if( nprocs .ne. 1 ) then
      print *, "there should only be one process for this program"
      stop
   end if

   print *, "init params file    ", init_params_file

   !now read in the parameters
   call init_params(init_params_file, params)
   print *, "utility_weight", params( param_weight_utility )

   !other setup
   call interp_setup_all( interact_file )
   call n_expect_nodes_setup()

   if (read_ols_matrices .eqv. .true. ) then
      call init_interp_matrices( value_init_file, asset_init_file, value_matrix_init, asset_matrix_init )

   else
      value_matrix_init = 0
      asset_matrix_init = 0
      asset_matrix_init(1) = 2.9957322735539909   !this makes the fraction of A+I households set aside for the following period equal to
        !1- r/(1+r)

   end if




   !get up mygrid
   cube_point_origin = 0
   cube_point_origin( interp_m_job ) = m_job
   cube_point_origin( interp_job ) = job
   cube_point_origin( interp_m_pt ) = m_pt
   cube_point_origin( interp_pt ) = pt
   cube_point_origin( interp_m_no_ui ) = m_no_ui
   cube_point_origin( interp_no_ui ) = no_ui

  ! cube_point_origin( interp_m_ui_left ) = -1
  ! cube_point_origin( interp_ui_left ) = -1

   call interp_to_mygrid( cube_point_origin, iter_switch, mygrid_origin )



   call mygrid_origin % print(.true., .true.)
   call mygrid_origin % mygrid_error_hunt(.true.,.true.,1,1,1)
  ! read(*,*)



   !set up iterate object
   n_expect_points = n_expect_points_matrix( m_job, job )



   value_origin = value_one_point( n_expect_points, mygrid_origin, params, asset_matrix_init, &
      value_matrix_init, .false.)

   print *, "value origin", value_origin




   !now get the derivative
   print *, " "
   print *, "first derivative of variables"
   do k= n_interp_vars_discrete+ 1, n_interp_vars

      if (interp_var_present(k,m_job,job) .eq. 1) then


   !      print *, "getting derivative for ", var_labels(k)
         cube_point = cube_point_origin

         cube_point(k) = cube_point(k) + h

         call interp_to_mygrid( cube_point, iter_switch, mygrid )


         value_perturb = value_one_point( n_expect_points, mygrid, params, asset_matrix_init, &
            value_matrix_init, .false.)


         value_der1(k) = (value_perturb - value_origin) / h

         print *, interp_var_labels(k), value_der1(k)

       !  print *, "got derivative for ", var_labels(k)
       !  read(*,*)

      end if

   end do


   stop

   print *, " "
   print *, "first derivative of parameters"

   mygrid = mygrid_origin

   do i=1,n_params
      params_peturb = params
      param_delta = max(abs(.1*params(i)),1.0D-10)
      params_peturb(i)= params(i) + param_delta



      value_perturb = value_one_point( n_expect_points, mygrid, params_peturb, asset_matrix_init, &
            value_matrix_init, .false.)

      print *, param_labels(i), (value_perturb - value_origin)/ param_delta


   end do

   call mpi_finalize(err)

!   print *, "printing difference between origin and peturb values for joboffer_mean"
!   do i=1,n_draws
!      print *, i, value_origin_all(i), value_perturb_all(i,2), value_perturb_all(i,2) - value_origin_all(i)
!   end do

contains

function value_one_point( n_expect_points_in, mygrid_in, params_in, asset_matrix_in, &
   value_matrix_in, print_check_in)

   real*8 :: value_one_point
   integer, intent(in) :: n_expect_points_in
   type(mygrid_type), intent(in) :: mygrid_in
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension( n_grid_points ), intent(in) :: value_matrix_in
   real*8, dimension( n_grid_points ), intent(in) :: asset_matrix_in
   logical, intent(in) :: print_check_in !for whether to print and check things related to the iterate object


   type(iterate_type), dimension(:), allocatable :: iterate_object
   integer :: s
   real*8 :: weight_sum

   !first set up the iterate object
   allocate( iterate_object( n_expect_points_in ) )
   call iterate_object_setup( n_expect_points_in, mygrid_in, params_in, iterate_object )

   value_one_point = 0
   weight_sum = 0
   do s=1,n_expect_points_in
      call update_probs_one_node(   iterate_object(s), &
                                    params_in )
      call update_asset_util_one_node( iterate_object(s), &
                                       params_in, &
                                       asset_matrix_in )
      call iterate_value_one_node(  iterate_object(s), &
                                    value_matrix_in )

      weight_sum = weight_sum + iterate_object(s) % weight
      value_one_point = value_one_point + iterate_object(s) % emax_weighted
   end do

   if (print_check_in .eqv. .true. ) then

      print *, "weight sum", weight_sum
      read(*,*)

      do s=1,n_expect_points
         print *, "******************************"
         print *, "state point", s
         print *, "  "
         call iterate_object(s) % print()
         read(*,*)
      end do

   end if


end function value_one_point


end program





































!an older way with importing the normals

!   !first get the random numbers
!   call random_number( random_input_matrix )
!   do i=1,n_draws
!      do j=1,5
!         random_normal_matrix(j,i) = rnorm()
!      end do
!   end do


!   do i=1, n_draws
!    !  call random_number( random_input )
!       !  random_input(:) = random_input_matrix(:,i)
!   !   random_input ( iter_m_jobloss ) = 0
!   !   random_input ( iter_m_ui ) = 0

!      call iterate_one_point_normal_in(mygrid, params, ols_matrix_init, random_input_matrix(:,i), &
!         random_normal_matrix(:,i), value_temp)

!      value_origin = value_origin + value_temp
!      value_origin_all(i) = value_temp
!   end do
!   value_origin = value_origin / n_draws


!   print *, "got origin value"
! !  read(*,*)


!   !now get the derivative
!   print *, "first derivative"
!   do k=1,n_cont_vars
!!   do k = var_uimax, var_uirate

!      if (var_present(k,m_job,job)) then
!       !  call read_random_seed()

!      !   print *, "getting derivative for", var_labels(k)

!         mygrid = mygrid_original
!         mygrid % rv(k) = mygrid % rv(k) + der_delta(k)
!         call mygrid % real_to_int_duplicate()

!         value_perturb = 0
!         do i=1, n_draws
!           ! call random_number( random_input )

!       !     random_input(:) = random_input_matrix(:,i)

!        !    random_input ( iter_m_jobloss ) = 0
!        !    random_input ( iter_m_ui ) = 0

!            call iterate_one_point_normal_in(mygrid, params, ols_matrix_init, random_input_matrix(:,i), &
!               random_normal_matrix(:,i), value_temp)

!            value_perturb = value_perturb + value_temp
!            value_perturb_all(i,k) = value_temp
!         end do

!         value_perturb = value_perturb / n_draws

!         value_der1(k) = (value_perturb - value_origin) / der_delta(k)

!         print *, var_labels(k), value_der1(k)

!       !  print *, "got derivative for ", var_labels(k)
!       !  read(*,*)

!      end if

!   end do


!!   print *, "printing difference between origin and peturb values for joboffer_mean"
!!   do i=1,n_draws
!!      print *, i, value_origin_all(i), value_perturb_all(i,2), value_perturb_all(i,2) - value_origin_all(i)
!!   end do




!end program


