!module for creating tables of my output in latex for my job market paper


module latex_tables_mod

use string_mod
use misc_funcs_mod
use latex_file
use enum_mod
use model_params_mod
use chi_square_mod

use, intrinsic :: iso_fortran_env

   integer, parameter :: coef_char_length = 25
   integer, parameter :: results_char_length = 50
   integer, parameter :: format_length  = 15
   integer, parameter :: max_rows_results = 50
   integer, parameter :: sig_fig = 3

   real*8, dimension(3), parameter :: star_cutoff = (/ 1.6449, 1.96, 2.5758 /)

   character(len=30), dimension(n_params) :: param_labels_table


contains



function get_power_func( x_in ) result(power)
   !gets the power of x when x is expressed in scientific notation (ES)
   real*8,  intent(in) :: x_in
   integer :: power

   character(len=9) :: xstring
   character(len=1) :: sign_char
   integer :: length

   write( xstring, "(ES9.1E3)" ) x_in
   length = len(trim(xstring) )

   read( xstring( length-2 : length), * ) power

   read( xstring( length-3 : length - 3 ), * ) sign_char

   if (sign_char .eq. "-" ) then
      power = - power
   end if


end function get_power_func



function get_format_decimal_func( x_in, sig_fig_in ) result( format_string)


   real*8, intent(in) :: x_in
   integer, intent(in) :: sig_fig_in
   character(len= format_length) :: format_string

   integer :: power, w, d

   power = get_power_func( x_in )

   if ( power < 0 )  then

      d = abs(power) + ( sig_fig_in - 1 )

      w = d + 2

   else if ( power < sig_fig_in - 1) then

      d = sig_fig_in - power - 1
      w = sig_fig_in + 1

   else

     ! d = 0
      w = 1 + power

   end if


   if ( x_in < 0.0D+0 ) then
      w = w + 1

   end if

   if ( power < sig_fig_in - 1 ) then
      format_string = "(F" // trim(int_to_string(w)) // "." // trim(int_to_string(d)) // ")"
   else
      format_string = "(I" // trim( int_to_string(w) ) // ")"
   end if


end function get_format_decimal_func


function get_format_scientific_func( x_in, sig_fig_in ) result( format_string)

   real*8, intent(in) :: x_in
   integer, intent(in) :: sig_fig_in
   character(len= format_length ) :: format_string

   integer :: power, w, d


   d = sig_fig_in - 1

   w = d + 8 !8 somewhat random, longth enough to not give ***, but not too long

   format_string = "(ES" // trim(int_to_string(w)) // "." // trim(int_to_string(d)) // ")"


end function get_format_scientific_func

function thousands_separator_func( x_in) result( x_string_out )
   !write x to x_string with commas for thousand separators
   integer, intent(in) :: x_in
   character(len = results_char_length )  :: x_string_out
   character(len = results_char_length )  :: x_string_temp

   integer :: i,j

   x_string_out = " "


   write(x_string_temp, * ) x_in
   x_string_temp = adjustr( x_string_temp )

  ! print *, x_string_temp

   i = 0
   j= 0

   do while ( (x_string_temp(results_char_length-i:results_char_length-i) .ne. " ")  )

   !   print *, i, x_string_out

      if ( ( mod(i,3) .eq. 0 ) .and. (i .ne. 0) ) then
         x_string_out(results_char_length-j:results_char_length-j) =","
         j = j + 1
      end if

      x_string_out(results_char_length-j:results_char_length-j) = &
         x_string_temp(results_char_length-i:results_char_length-i)

      i = i + 1
      j = j + 1

      if ( ( i .eq. results_char_length ) .or. ( j .eq. results_char_length ) ) then
         print *, "error in thousands_separator_func, stopping"
         stop
      end if

   end do

   x_string_out = adjustl(x_string_out)

   !haven't really checked this
   if (x_in < 0 ) then
      x_string_temp = x_string_out

      x_string_out = "-"
      do i = 1, results_char_length - 1
         x_string_out( i+1:i+1) = x_string_out(i:i)
      end do


   end if

   !print *, x_string_out


  ! stop


end function

!subroutine format_coef_sd( sig_fig_in, coef_in, sd_in, coef_out, sd_out )
!   !formats the estimated coefficients and standard deviations for printing to my latex table in my paper
!   integer, intent(in) :: sig_fig_in
!   real*8, intent(in) :: coef_in
!   real*8, intent(in) :: sd_in
!   character(len=coef_char_length), intent(out) :: coef_out
!   character(len=coef_char_length), intent(out) :: sd_out

!   logical :: science_logical !declares whether to write coeficients in scientific notation
!   logical :: nint_coef_logical !declares whether to round these coefficients to the nearest integer
!   logical :: nint_sd_logical !declares whether to round these coefficients to the nearest integer

!   character(len = format_length ) :: format_string_coef
!   character(len = format_length ) :: format_string_sd
!   real*8 :: z_score


!   if ( ( abs(coef_in) > 1.0D+5 ) .or. ( abs(coef_in) < 1.0D-3 ) ) then
!      science_logical = .true.
!   else
!      science_logical = .false.
!   end if

!   if ( ( get_power_func( coef_in ) >= sig_fig_in - 1 ) .and. ( science_logical .eqv. .false. ) &
!      ) then
!      nint_coef_logical = .true.
!   else
!      nint_coef_logical = .false.
!   end if

!   if ( ( get_power_func( sd_in ) >= sig_fig_in - 1 ) .and. ( science_logical .eqv. .false. ) &
!      ) then
!      nint_sd_logical = .true.
!   else
!      nint_sd_logical = .false.
!   end if

!   if ( science_logical ) then
!      format_string_coef = get_format_scientific_func( coef_in, sig_fig_in )
!      format_string_sd   = get_format_scientific_func( sd_in, sig_fig_in )

!   else
!      format_string_coef = get_format_decimal_func( coef_in, sig_fig_in )
!      format_string_sd   = get_format_decimal_func( sd_in, sig_fig_in )

!   end if

!   if( nint_coef_logical ) then
!      write( coef_out, format_string_coef ) nint(coef_in )

!   else
!      write( coef_out, format_string_coef ) coef_in

!   end if

!   if( nint_coef_logical ) then
!      write( coef_out, format_string_coef ) nint(coef_in )

!   else
!      write( coef_out, format_string_coef ) coef_in

!   end if

!   if( nint_sd_logical ) then
!      write( sd_out, format_string_sd ) nint(sd_in )

!   else
!      write( sd_out, format_string_sd ) sd_in

!   end if

!   !now add stars coef_out
!   z_score = abs( coef_in / sd_in )

!   if ( z_score > star_cutoff(3) ) then
!      coef_out =  trim( coef_out )  // "***"

!   else if ( z_score > star_cutoff(2) ) then
!      coef_out = trim( coef_out )  // "**"

!   else if ( z_score > star_cutoff(1) ) then
!      coef_out =  trim( coef_out )  // "**"

!   end if
!   coef_out = adjustl( trim( coef_out ) )


!   !now add paranthesis to sd
!   sd_out = "(" // trim(adjustl(sd_out ) )// ")"








!end subroutine format_coef_sd



subroutine format_coef_sd( dec_in, coef_in, sd_in, coef_out, sd_out )
   !formats the estimated coefficients and standard deviations for printing to my latex table in my paper
   integer, intent(in) :: dec_in !number of significant figures (for estimates greater than 1) or number of deciminal places (for numberes less than 1)
   real*8, intent(in) :: coef_in
   real*8, intent(in) :: sd_in
   character(len=coef_char_length), intent(out) :: coef_out
   character(len=coef_char_length), intent(out) :: sd_out


   character(len = format_length ) :: format_string_coef
   character(len = format_length ) :: format_string_sd
   real*8 :: z_score


   if (abs(coef_in) > 1.0D+0 ) then
      format_string_coef = get_format_decimal_func( coef_in, dec_in )
   else
      format_string_coef = "(F" // trim(int_to_string(dec_in +4)) // "." // &
         trim(int_to_string(dec_in)) // ")"
   end if

   if (abs(sd_in) > 1.0D+0 ) then
      format_string_sd = get_format_decimal_func( sd_in, dec_in )
   else
      format_string_sd = "(F" // trim(int_to_string(dec_in +4)) // "." // &
         trim(int_to_string(dec_in)) // ")"
   end if


   write( coef_out, format_string_coef ) coef_in
   write( sd_out, format_string_sd ) sd_in


   !now add stars coef_out
   z_score = abs( coef_in / sd_in )

   if ( z_score > star_cutoff(3) ) then
      coef_out =  trim( coef_out )  // "***"

   else if ( z_score > star_cutoff(2) ) then
      coef_out = trim( coef_out )  // "**"

   else if ( z_score > star_cutoff(1) ) then
      coef_out =  trim( coef_out )  // "**"

   end if
   coef_out = adjustl( trim( coef_out ) )


   !now add paranthesis to sd
   sd_out = "(" // trim(adjustl(sd_out ) )// ")"








end subroutine format_coef_sd


subroutine gen_param_labels_table
   !generates the param lables for this table

   param_labels_table( param_joboffer_cons) =   'Constant'
   param_labels_table( param_joboffer_ur) =  'Unemployment Rate'
   param_labels_table( param_joboffer_work) =  'Employed'
   !param_labels( param_joboffer_age) =  'joboffer_age'
   !param_labels( param_joboffer_age2) =  'joboffer_age2'
   !param_labels( param_joboffer_race) =  'joboffer_race'
   !param_labels( param_joboffer_hs) =  'joboffer_hs'
   !param_labels( param_joboffer_some_college) =  'joboffer_some_college'
   !param_labels( param_joboffer_college) =  'joboffer_college'
  ! param_labels( param_joboffer_job_length) =  'joboffer_job_length'
   param_labels_table( param_joboffer_unemp_length) =  'Length Non-Employed'
   param_labels_table( param_joboffer_gender) =  'Female'
   param_labels_table( param_joboffer_seam) =  'Seam Effect'

   param_labels_table( param_wage_cons) =  'Constant'
   param_labels_table( param_wage_pt) =  'Part-Time'
   param_labels_table( param_wage_ur) =  'Unemployment Rate'
   param_labels_table( param_wage_age) =  'Age in Decades'
   param_labels_table( param_wage_age2) =  'Age in Decades'
   param_labels_table( param_wage_race) =  'White'
   param_labels_table( param_wage_hs) =  'High School'
   param_labels_table( param_wage_some_college) =  'Some College'
   param_labels_table( param_wage_college) =  'College'
   param_labels_table( param_wage_gender) =  'Female'

  ! param_labels_table( param_m_leisure_cons) =  'm_leisure_cons'
   !param_labels_table( param_m_leisure_age) =  'm_leisure_age'
   !param_labels( param_m_leisure_age2) =  'm_leisure_age2'
   !param_labels( param_m_leisure_race) =  'm_leisure_race'
   !param_labels( param_m_leisure_hs) =  'm_leisure_hs'
   !param_labels( param_m_leisure_some_college) =  'm_leisure_some_college'
   !param_labels( param_m_leisure_college) =  'm_leisure_college'
   !param_labels( param_m_leisure_kids) =  'm_leisure_kids'
   !param_labels( param_m_leisure_youngest) =  'm_leisure_youngest'

   param_labels_table( param_leisure_cons) =  "Wife's Leisure Constant"
   !param_labels( param_leisure_age) =  'leisure_age'
   !param_labels( param_leisure_age2) =  'leisure_age2'
   !param_labels( param_leisure_race) =  'leisure_race'
   !param_labels( param_leisure_hs) =  'leisure_hs'
   !param_labels( param_leisure_some_college) =  'leisure_some_college'
   !param_labels( param_leisure_college) =  'leisure_college'
   !param_labels( param_leisure_any_kids) =  'leisure_any_kids'
 !  param_labels( param_leisure_kids) =  'leisure_kids'
 !  param_labels( param_leisure_youngest) =  'leisure_youngest'
   param_labels_table( param_gender_reverse) =  "Reversed Gender Roles Cost"

   param_labels_table( param_hetero_m_wage_var) =  "Husband's Variance "
   param_labels_table( param_hetero_wage_var) =  "Wife's Variance "
   param_labels_table( param_hetero_wage_covar) = "Covariance"

  ! param_labels( param_sd_non_pec) =  'sd_non_pec'
   param_labels_table( param_sd_wage) =  'Standard Deviation'
   param_labels_table( param_prob_pt) =  'Probability'


   param_labels_table( param_weight_utility) =  'Utility Weight'
   param_labels_table( param_rra) =  'Relative Risk Aversion'

   param_labels_table( param_asset_error_cons) =  'Constant'
   param_labels_table( param_asset_error_linear) =  'Abs. Value Unobs. Assets'


end subroutine gen_param_labels_table



subroutine gen_param_est_table_latex( coef_in, sd_in, filename_in )

   real*8, dimension(n_main_params) :: coef_in, sd_in
   character(len=30), intent(in) :: filename_in

   real*8, dimension(n_main_params) :: coef_trans, sd_trans
   character(len=results_char_length), dimension(max_rows_results, 2) :: column1, column2
   logical, dimension(max_rows_results ) :: title_row1, title_row2


   integer :: unit_int,k, var, i

   call gen_param_labels_table()

   !transform some variables
   coef_trans = coef_in
   sd_trans = sd_in

   coef_trans(param_wage_age ) = coef_trans(param_wage_age )*10
   sd_trans(param_wage_age )  = sd_trans(param_wage_age )*10

   coef_trans(param_wage_age2 ) = coef_trans(param_wage_age2 )*10
   sd_trans(param_wage_age2 ) = sd_trans(param_wage_age2 )*10

   coef_trans( param_asset_error_linear ) = coef_trans( param_asset_error_linear )*100000
   sd_trans( param_asset_error_linear ) = sd_trans( param_asset_error_linear )*100000

   open(newunit = unit_int, file = filename_in, status = "replace" )

   title_row1 = .false.
   title_row2 = .false.
   column1 = " "
   column2 = " "

   !now first column
   k = 1

   !job offer probability
   column1(k,1) = "Job Offer Probability"
   title_row1(k) = .true.
   k = k+1

   call column_write( param_joboffer_cons, 1, k)
   call column_write( param_joboffer_ur, 1, k)
   call column_write( param_joboffer_work, 1, k)
   call column_write( param_joboffer_unemp_length, 1, k)
   call column_write( param_joboffer_gender, 1, k)
   call column_write( param_joboffer_seam, 1, k)

   k=k+1
   !Part_time probability
   column1(k,1) = "Part-time Probability"
   title_row1(k) = .true.
   k = k+1

   call column_write( param_prob_pt, 1, k)

   k = k +1
   !Utility Function
   column1(k,1) = "Utility Function"
   title_row1(k) = .true.
   k = k+1

   call column_write( param_leisure_cons, 1, k)
   call column_write( param_gender_reverse, 1, k)
   call column_write( param_weight_utility, 1, k)
   call column_write( param_rra, 1, k)


   k = k + 1
   !Asset Measurement Error
   column1(k,1) = "Asset Measurement Error"
   title_row1(k) = .true.
   k = k+1

   call column_write( param_asset_error_cons, 1, k)
   call column_write( param_asset_error_linear, 1, k)


   !now second column
   k = 1

   !wage distribution
   column2(k,1) = "Log Wage Distribution"
   title_row2(k) = .true.
   k = k+1

   call column_write( param_wage_cons, 2, k)
   call column_write( param_wage_pt, 2, k)
   call column_write( param_wage_ur, 2, k)
   call column_write( param_wage_age, 2, k)
   call column_write( param_wage_age2, 2, k)
   call column_write( param_wage_race, 2, k)
   call column_write( param_wage_hs, 2, k)
   call column_write( param_wage_some_college, 2, k)
   call column_write( param_wage_college, 2, k)
   call column_write( param_wage_gender, 2, k)
   call column_write( param_sd_wage, 2, k)


   k = k + 1
   !heterogeneity
   column2(k,1) = "Wage Unobserved Heterogeneity"
   title_row2(k) = .true.
   k = k+1

   call column_write( param_hetero_m_wage_var, 2, k)
   call column_write( param_hetero_wage_covar, 2, k)
   call column_write( param_hetero_wage_var, 2, k)


   !now write to file
   do i = 1 , max_rows_results

     ! print *, column2(i, 1), column2(i, 2)

      !column 1
      if ( title_row1(i) .eqv. .true. ) then
         call latex_write( unit_int,  "\multicolumn{2}{c |}{\textbf{" // &
            trim(column1(i,1)) // "} } ", .false. )

      else

         call latex_write( unit_int, column1(i,1)  , .false. )
         call latex_write( unit_int, column1(i,2) , .false. )

      end if

      !column2
      if ( title_row2(i) .eqv. .true. ) then
         call latex_write( unit_int,  "\multicolumn{2}{c }{\textbf{" // &
            trim(column2(i,1)) // "} }", .true. )

      else
         call latex_write( unit_int, column2(i,1) , .false. )
         call latex_write( unit_int, column2(i,2) , .true. )

      end if




   end do




contains

subroutine column_write(var_in, col_no_in, k_inout)
   integer, intent(in) :: var_in
   integer, intent(in) :: col_no_in
   integer, intent(in out) :: k_inout

   character(len=results_char_length), dimension(2, 2) ::column_part_temp

   character(len=coef_char_length) :: coef_string, sd_string

   column_part_temp(1,1) = param_labels_table(var_in)

   call format_coef_sd( sig_fig, coef_trans( var_in), sd_trans( var_in), coef_string, sd_string )

   column_part_temp(1,2) = coef_string
   column_part_temp(2,1) = " "
   column_part_temp(2,2) = sd_string

   if (col_no_in .eq. 1 ) then
      column1(k_inout:k_inout+1,:) = column_part_temp

   else if (col_no_in .eq. 2 ) then
      column2(k_inout:k_inout+1,:) = column_part_temp

   end if


   k_inout = k_inout + 2



end subroutine column_write



end subroutine gen_param_est_table_latex


subroutine gen_uimax_appx_table_latex( n_samples_in, job_freq_matrix_in, filename_in )

   integer, intent(in) :: n_samples_in
   real*8, dimension( n_samples_in*4, 4 ), intent(in) :: job_freq_matrix_in
   character(len=30), intent(in) :: filename_in

   character(len = 100 ), dimension(n_samples_in) :: header
   integer :: s, i, j, unit_int, k
   integer, dimension(4) :: uimax_level
   character(len=10) :: prob_string
   logical :: adv

   header(1) = "Husband Lost Job, Wife Was Not Working, With Assets"
   header(2) = "Wife Lost Job, Husband Was Not Working, With Assets"
   header(3) = "Husband Lost Job, Wife Was Working, With Assets"
   header(4) = "Wife Lost Job, Husband Was Working, With Assets"
   header(5) = "Both Were Working and Lost Their Jobs, With Assets"
   header(6) = "Husband Lost Job, Wife Was Not Working, No Assets"
   header(7) = "Wife Lost Job, Husband Was Not Working, No Assets"
   header(8) = "Husband Lost Job, Wife Was Working, No Assets"
   header(9) = "Wife Lost Job, Husband Was Working, No Assets"
   header(10) = "Both Were Working and Lost Their Jobs, No Assets"

   uimax_level(1) = 100
   uimax_level(2) = 200
   uimax_level(3) = 300
   uimax_level(4) = 400

   open(newunit = unit_int, file = filename_in, status = "replace" )
   k = 0


   do s = 1, n_samples_in

      call latex_write(unit_int, " ", .false. )
      call latex_write(unit_int, "\multicolumn{4}{c }{ \textbf{" // &
         trim(adjustl(header(s) ) ) // "}}", .true. )

      do i = 1, 4

         call latex_write( unit_int, uimax_level(i), .false. )
         k = k + 1

         do j = 1, 4

            if( j .eq. 4) then
               adv = .true.
            else
               adv = .false.
            end if

            write(prob_string, "(F8.4)" ) job_freq_matrix_in(k, j )

            call latex_write( unit_int, prob_string, adv )

         end do

      end do

      call latex_next_record(unit_int)


   end do



end subroutine gen_uimax_appx_table_latex


subroutine gen_uimax_ddd_table_latex( n_samples_in, job_freq_matrix_in, filename1_in, filename2_in )

   integer, intent(in) :: n_samples_in
   real*8, dimension( n_samples_in*4, 4 ), intent(in) :: job_freq_matrix_in
   character(len=30), intent(in) :: filename1_in, filename2_in

   real*8, dimension(2,2) :: wife_nowork_matrix, wife_work_matrix, husband_nowork_matrix, &
      husband_work_matrix
   real*8, dimension(2) :: wife_nowork_dif, wife_work_dif, husband_nowork_dif, husband_work_dif, &
      dif_in_dif1, dif_in_dif2
   real*8 :: ddd1, ddd2

   character(len=10) :: prob_string

   integer :: i, s, k, unit_int

   !now get the respective probabiliites probabilities

   s = 1

   k = (s-1)*4 + 2
   wife_nowork_matrix(1,1) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,2)
   k = (s-1)*4 + 4
   wife_nowork_matrix(1,2) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,2)


   s = 6

   k = (s-1)*4 + 2
   wife_nowork_matrix(2,1) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,2)
   k = (s-1)*4 + 4
   wife_nowork_matrix(2,2) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,2)


   wife_nowork_dif = wife_nowork_matrix(:,2) - wife_nowork_matrix(:,1)

   dif_in_dif1(1) = wife_nowork_dif(2) - wife_nowork_dif(1)


   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   s = 4

   k = (s-1)*4 + 2
   wife_work_matrix(1,1) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,2)
   k = (s-1)*4 + 4
   wife_work_matrix(1,2) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,2)

   s = 9

   k = (s-1)*4 + 2
   wife_work_matrix(2,1) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,2)
   k = (s-1)*4 + 4
   wife_work_matrix(2,2) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,2)

   wife_work_dif = wife_work_matrix(:,2) - wife_work_matrix(:,1)

   dif_in_dif1(2) = wife_work_dif(2) - wife_work_dif(1)

   ddd1 = dif_in_dif1(2) - dif_in_dif1(1)


   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1

   s = 2

   k = (s-1)*4 + 2
   husband_nowork_matrix(1,1) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,3)
   k = (s-1)*4 + 4
   husband_nowork_matrix(1,2) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,3)

   s = 7

   k = (s-1)*4 + 2
   husband_nowork_matrix(2,1) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,3)
   k = (s-1)*4 + 4
   husband_nowork_matrix(2,2) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,3)

   husband_nowork_dif = husband_nowork_matrix(:,2) - husband_nowork_matrix(:,1)

   dif_in_dif2(1) = husband_nowork_dif(2) - husband_nowork_dif(1)



   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   s = 5

   k = (s-1)*4 + 2
   husband_work_matrix(1,1) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,3)
   k = (s-1)*4 + 4
   husband_work_matrix(1,2) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,3)

   s = 10

   k = (s-1)*4 + 2
   husband_work_matrix(2,1) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,3)
   k = (s-1)*4 + 4
   husband_work_matrix(2,2) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,3)

   husband_work_dif = husband_work_matrix(:,2) - husband_work_matrix(:,1)

   dif_in_dif2(2) = husband_work_dif(2) - husband_work_dif(1)

   ddd2 = dif_in_dif2(2) - dif_in_dif2(1)


   !!!!!!!!!!!!!!!!!!!!!!!!

   !now write the latex files
   !first block in the husband's table
   open(newunit = unit_int, file = filename1_in, status = "replace" )

   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, "UI Max = \$200 ", .false. )
   call latex_write( unit_int, "UI Max = \$400 ", .false. )
   call latex_write( unit_int, "Difference ", .true. )

   write(unit_int, *) "\hline"
   call latex_write( unit_int, "Group A: Wife Not Working", .true., "i" )

   call latex_write( unit_int,"Households With Assets", .false. )
   write(prob_string, "(F8.4)" ) wife_nowork_matrix(1,1)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) wife_nowork_matrix(1,2)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) wife_nowork_dif(1)
      call latex_write( unit_int, prob_string, .true. )


   call latex_write( unit_int,"Households Without Assets", .false. )
   write(prob_string, "(F8.4)" ) wife_nowork_matrix(2,1)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) wife_nowork_matrix(2,2)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) wife_nowork_dif(2)
      call latex_write( unit_int, prob_string, .true. )

   call latex_write( unit_int,"Difference-in-Difference", .false. )
   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, " ", .false. )
   write(prob_string, "(F8.4)" ) dif_in_dif1(1)
      call latex_write( unit_int, prob_string, .true., "b" )

   call latex_next_record(unit_int)

   !!!!!!!!!!!!!!!!!!!!

   !second block in the husband's table

   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, "UI Max = \$200 ", .false. )
   call latex_write( unit_int, "UI Max = \$400 ", .false. )
   call latex_write( unit_int, "Difference ", .true. )

   write(unit_int, *) "\hline"
   call latex_write( unit_int, "Group B: Wife Working", .true., "i" )

   call latex_write( unit_int,"Households With Assets", .false. )
   write(prob_string, "(F8.4)" ) wife_work_matrix(1,1)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) wife_work_matrix(1,2)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) wife_work_dif(1)
      call latex_write( unit_int, prob_string, .true. )


   call latex_write( unit_int,"Households Without Assets", .false. )
   write(prob_string, "(F8.4)" ) wife_work_matrix(2,1)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) wife_work_matrix(2,2)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) wife_work_dif(2)
      call latex_write( unit_int, prob_string, .true. )

   call latex_write( unit_int,"Difference-in-Difference", .false. )
   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, " ", .false. )
   write(prob_string, "(F8.4)" ) dif_in_dif1(2)
      call latex_write( unit_int, prob_string, .true., "b" )

   call latex_next_record(unit_int)

   call latex_write( unit_int,"Triple Difference", .false. )
   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, " ", .false. )
   write(prob_string, "(F8.4)" ) ddd1
      call latex_write( unit_int, prob_string, .true., "b" )

   close(unit_int)

   !!!!!!!!!!!!!!!!!!!!!!!!

   !now write the other latex files
   !first block in the husband's table
   open(newunit = unit_int, file = filename2_in, status = "replace" )

   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, "UI Max = \$200 ", .false. )
   call latex_write( unit_int, "UI Max = \$400 ", .false. )
   call latex_write( unit_int, "Difference ", .true. )

   write(unit_int, *) "\hline"
   call latex_write( unit_int, "Group A: Husband Not Working", .true., "i" )

   call latex_write( unit_int,"Households With Assets", .false. )
   write(prob_string, "(F8.4)" ) husband_nowork_matrix(1,1)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) husband_nowork_matrix(1,2)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) husband_nowork_dif(1)
      call latex_write( unit_int, prob_string, .true. )


   call latex_write( unit_int,"Households Without Assets", .false. )
   write(prob_string, "(F8.4)" ) husband_nowork_matrix(2,1)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) husband_nowork_matrix(2,2)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) husband_nowork_dif(2)
      call latex_write( unit_int, prob_string, .true. )

   call latex_write( unit_int,"Difference-in-Difference", .false. )
   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, " ", .false. )
   write(prob_string, "(F8.4)" ) dif_in_dif2(1)
      call latex_write( unit_int, prob_string, .true., "b" )

   call latex_next_record(unit_int)

   !!!!!!!!!!!!!!!!!!!!

   !second block in the husband's table

   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, "UI Max = \$200 ", .false. )
   call latex_write( unit_int, "UI Max = \$400 ", .false. )
   call latex_write( unit_int, "Difference ", .true. )

   write(unit_int, *) "\hline"
   call latex_write( unit_int, "Group B: Husband Working", .true., "i" )

   call latex_write( unit_int,"Households With Assets", .false. )
   write(prob_string, "(F8.4)" ) husband_work_matrix(1,1)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) husband_work_matrix(1,2)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) husband_work_dif(1)
      call latex_write( unit_int, prob_string, .true. )


   call latex_write( unit_int,"Households Without Assets", .false. )
   write(prob_string, "(F8.4)" ) husband_work_matrix(2,1)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) husband_work_matrix(2,2)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) husband_work_dif(2)
      call latex_write( unit_int, prob_string, .true. )

   call latex_write( unit_int,"Difference-in-Difference", .false. )
   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, " ", .false. )
   write(prob_string, "(F8.4)" ) dif_in_dif2(2)
      call latex_write( unit_int, prob_string, .true., "b" )

   call latex_next_record(unit_int)

   call latex_write( unit_int,"Triple Difference", .false. )
   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, " ", .false. )
   write(prob_string, "(F8.4)" ) ddd2
      call latex_write( unit_int, prob_string, .true., "b" )

   close(unit_int)






end subroutine gen_uimax_ddd_table_latex


subroutine gen_uilength_ddd_table_latex( n_samples_in, job_freq_matrix_in, filename1_in, filename2_in )

   integer, intent(in) :: n_samples_in
   real*8, dimension( n_samples_in*4, 4 ), intent(in) :: job_freq_matrix_in
   character(len=30), intent(in) :: filename1_in, filename2_in

   real*8, dimension(2,2) :: wife_nowork_matrix, wife_work_matrix, husband_nowork_matrix, &
      husband_work_matrix
   real*8, dimension(2) :: wife_nowork_dif, wife_work_dif, husband_nowork_dif, husband_work_dif, &
      dif_in_dif1, dif_in_dif2
   real*8 :: ddd1, ddd2

   character(len=10) :: prob_string

   integer :: i, s, k, unit_int

   !now get the respective probabiliites probabilities

   s = 1

   k = (s-1)*4 + 1
   wife_nowork_matrix(1,1) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,2)
   k = (s-1)*4 + 2
   wife_nowork_matrix(1,2) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,2)


   s = 6

   k = (s-1)*4 + 1
   wife_nowork_matrix(2,1) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,2)
   k = (s-1)*4 + 2
   wife_nowork_matrix(2,2) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,2)


   wife_nowork_dif = wife_nowork_matrix(:,2) - wife_nowork_matrix(:,1)

   dif_in_dif1(1) = wife_nowork_dif(2) - wife_nowork_dif(1)


   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   s = 4

   k = (s-1)*4 + 1
   wife_work_matrix(1,1) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,2)
   k = (s-1)*4 + 2
   wife_work_matrix(1,2) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,2)

   s = 9

   k = (s-1)*4 + 1
   wife_work_matrix(2,1) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,2)
   k = (s-1)*4 + 2
   wife_work_matrix(2,2) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,2)

   wife_work_dif = wife_work_matrix(:,2) - wife_work_matrix(:,1)

   dif_in_dif1(2) = wife_work_dif(2) - wife_work_dif(1)

   ddd1 = dif_in_dif1(2) - dif_in_dif1(1)


   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1

   s = 2

   k = (s-1)*4 + 1
   husband_nowork_matrix(1,1) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,3)
   k = (s-1)*4 + 2
   husband_nowork_matrix(1,2) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,3)

   s = 7

   k = (s-1)*4 + 1
   husband_nowork_matrix(2,1) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,3)
   k = (s-1)*4 + 2
   husband_nowork_matrix(2,2) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,3)

   husband_nowork_dif = husband_nowork_matrix(:,2) - husband_nowork_matrix(:,1)

   dif_in_dif2(1) = husband_nowork_dif(2) - husband_nowork_dif(1)



   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   s = 5

   k = (s-1)*4 + 1
   husband_work_matrix(1,1) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,3)
   k = (s-1)*4 + 2
   husband_work_matrix(1,2) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,3)

   s = 10

   k = (s-1)*4 + 1
   husband_work_matrix(2,1) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,3)
   k = (s-1)*4 + 2
   husband_work_matrix(2,2) = job_freq_matrix_in(k,1) + job_freq_matrix_in(k,3)

   husband_work_dif = husband_work_matrix(:,2) - husband_work_matrix(:,1)

   dif_in_dif2(2) = husband_work_dif(2) - husband_work_dif(1)

   ddd2 = dif_in_dif2(2) - dif_in_dif2(1)


   !!!!!!!!!!!!!!!!!!!!!!!!

   !now write the latex files
   !first block in the husband's table
   open(newunit = unit_int, file = filename1_in, status = "replace" )

   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, "UI Length = 7 Months ", .false. )
   call latex_write( unit_int, "UI Length = 8 Months ", .false. )
   call latex_write( unit_int, "Difference ", .true. )

   write(unit_int, *) "\hline"
   call latex_write( unit_int, "Group A: Wife Not Working", .true., "i" )

   call latex_write( unit_int,"Households With Assets", .false. )
   write(prob_string, "(F8.4)" ) wife_nowork_matrix(1,1)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) wife_nowork_matrix(1,2)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) wife_nowork_dif(1)
      call latex_write( unit_int, prob_string, .true. )


   call latex_write( unit_int,"Households Without Assets", .false. )
   write(prob_string, "(F8.4)" ) wife_nowork_matrix(2,1)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) wife_nowork_matrix(2,2)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) wife_nowork_dif(2)
      call latex_write( unit_int, prob_string, .true. )

   call latex_write( unit_int,"Difference-in-Difference", .false. )
   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, " ", .false. )
   write(prob_string, "(F8.4)" ) dif_in_dif1(1)
      call latex_write( unit_int, prob_string, .true., "b" )

   call latex_next_record(unit_int)

   !!!!!!!!!!!!!!!!!!!!

   !second block in the husband's table

   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, "UI Max = \$200 ", .false. )
   call latex_write( unit_int, "UI Max = \$400 ", .false. )
   call latex_write( unit_int, "Difference ", .true. )

   write(unit_int, *) "\hline"
   call latex_write( unit_int, "Group B: Wife Working", .true., "i" )

   call latex_write( unit_int,"Households With Assets", .false. )
   write(prob_string, "(F8.4)" ) wife_work_matrix(1,1)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) wife_work_matrix(1,2)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) wife_work_dif(1)
      call latex_write( unit_int, prob_string, .true. )


   call latex_write( unit_int,"Households Without Assets", .false. )
   write(prob_string, "(F8.4)" ) wife_work_matrix(2,1)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) wife_work_matrix(2,2)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) wife_work_dif(2)
      call latex_write( unit_int, prob_string, .true. )

   call latex_write( unit_int,"Difference-in-Difference", .false. )
   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, " ", .false. )
   write(prob_string, "(F8.4)" ) dif_in_dif1(2)
      call latex_write( unit_int, prob_string, .true., "b" )

   call latex_next_record(unit_int)

   call latex_write( unit_int,"Triple Difference", .false. )
   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, " ", .false. )
   write(prob_string, "(F8.4)" ) ddd1
      call latex_write( unit_int, prob_string, .true., "b" )

   close(unit_int)

   !!!!!!!!!!!!!!!!!!!!!!!!

   !now write the other latex files
   !first block in the husband's table
   open(newunit = unit_int, file = filename2_in, status = "replace" )

   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, "UI Max = \$200 ", .false. )
   call latex_write( unit_int, "UI Max = \$400 ", .false. )
   call latex_write( unit_int, "Difference ", .true. )

   write(unit_int, *) "\hline"
   call latex_write( unit_int, "Group A: Husband Not Working", .true., "i" )

   call latex_write( unit_int,"Households With Assets", .false. )
   write(prob_string, "(F8.4)" ) husband_nowork_matrix(1,1)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) husband_nowork_matrix(1,2)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) husband_nowork_dif(1)
      call latex_write( unit_int, prob_string, .true. )


   call latex_write( unit_int,"Households Without Assets", .false. )
   write(prob_string, "(F8.4)" ) husband_nowork_matrix(2,1)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) husband_nowork_matrix(2,2)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) husband_nowork_dif(2)
      call latex_write( unit_int, prob_string, .true. )

   call latex_write( unit_int,"Difference-in-Difference", .false. )
   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, " ", .false. )
   write(prob_string, "(F8.4)" ) dif_in_dif2(1)
      call latex_write( unit_int, prob_string, .true., "b" )

   call latex_next_record(unit_int)

   !!!!!!!!!!!!!!!!!!!!

   !second block in the husband's table

   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, "UI Max = \$200 ", .false. )
   call latex_write( unit_int, "UI Max = \$400 ", .false. )
   call latex_write( unit_int, "Difference ", .true. )

   write(unit_int, *) "\hline"
   call latex_write( unit_int, "Group B: Husband Working", .true., "i" )

   call latex_write( unit_int,"Households With Assets", .false. )
   write(prob_string, "(F8.4)" ) husband_work_matrix(1,1)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) husband_work_matrix(1,2)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) husband_work_dif(1)
      call latex_write( unit_int, prob_string, .true. )


   call latex_write( unit_int,"Households Without Assets", .false. )
   write(prob_string, "(F8.4)" ) husband_work_matrix(2,1)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) husband_work_matrix(2,2)
      call latex_write( unit_int, prob_string, .false. )
   write(prob_string, "(F8.4)" ) husband_work_dif(2)
      call latex_write( unit_int, prob_string, .true. )

   call latex_write( unit_int,"Difference-in-Difference", .false. )
   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, " ", .false. )
   write(prob_string, "(F8.4)" ) dif_in_dif2(2)
      call latex_write( unit_int, prob_string, .true., "b" )

   call latex_next_record(unit_int)

   call latex_write( unit_int,"Triple Difference", .false. )
   call latex_write( unit_int, " ", .false. )
   call latex_write( unit_int, " ", .false. )
   write(prob_string, "(F8.4)" ) ddd2
      call latex_write( unit_int, prob_string, .true., "b" )

   close(unit_int)






end subroutine gen_uilength_ddd_table_latex




subroutine gen_ui_income_vs_sub_table( n_samples_in, job_freq_matrix_in, filename_in )


   !subroutine for generating the liquidity constraint/income effect vs moral hazard/substitution effect
      !tables

   integer, intent(in) :: n_samples_in
   real*8, dimension( n_samples_in*4, 4 ), intent(in) :: job_freq_matrix_in
   character(len=30), intent(in) :: filename_in

   real*8, dimension( n_samples_in*4 ) :: husband_prob, wife_prob
   real*8 :: husband_dif_base, wife_dif_base
   real*8 :: husband_dif_assets, wife_dif_assets
   real*8 :: husband_dif_spouse, wife_dif_spouse

   !note, these indexes are speciific to how job_freq_matrix_in is created.  I currently do not have a
      !more robust algorithm

   integer, parameter :: husband_base_index = 6
   integer, parameter :: wife_base_index = 7
   integer, parameter :: husband_assets_index = 1
   integer, parameter :: wife_assets_index = 2
   integer, parameter :: husband_spouse_index = 9
   integer, parameter :: wife_spouse_index = 10

   integer :: unit_int

   !generate the probability of the husband and the wife being employed after 6 months from job_freq_matrix_in
   husband_prob = job_freq_matrix_in(:,1) +  job_freq_matrix_in(:,2)
   wife_prob    = job_freq_matrix_in(:,1) +  job_freq_matrix_in(:,3)



   husband_dif_base = husband_prob( (husband_base_index-1)*4 + 4 ) - &
                      husband_prob( (husband_base_index-1)*4 + 2 )

   husband_dif_assets = husband_prob( (husband_assets_index-1)*4 + 4 ) - &
                        husband_prob( (husband_assets_index-1)*4 + 2 )

   husband_dif_spouse = husband_prob( (husband_spouse_index-1)*4 + 4 ) - &
                        husband_prob( (husband_spouse_index-1)*4 + 2 )




   wife_dif_base =   wife_prob( (wife_base_index-1)*4 + 4 ) - &
                     wife_prob( (wife_base_index-1)*4 + 2 )

   wife_dif_assets = wife_prob( (wife_assets_index-1)*4 + 4 ) - &
                     wife_prob( (wife_assets_index-1)*4 + 2 )

   wife_dif_spouse = wife_prob( (wife_spouse_index-1)*4 + 4 ) - &
                     wife_prob( (wife_spouse_index-1)*4 + 2 )

   open(newunit = unit_int, file = filename_in, status = "replace" )

   write( unit_int, * ) "table about moral hazard vs liquidity contraints"

   write( unit_int, *) "comparision husbands with assets vs no assets (both no workign spouse)", &
      husband_dif_assets/husband_dif_base
   write( unit_int, *) "comparision husbands with working spouse vs no working spouse (both no assets)", &
      husband_dif_spouse/husband_dif_base

   write( unit_int, *) "comparision wives with assets vs no assets (both no workign spouse)", &
      wife_dif_assets/wife_dif_base
   write( unit_int, *) "comparision wives with working spouse vs no working spouse (both no assets)", &
      wife_dif_spouse/wife_dif_base

   close(unit_int)




end subroutine gen_ui_income_vs_sub_table



subroutine gen_stationary_table_latex( input_filename1_in, input_filename2_in, input_filename3_in, &
   output_filename_in )
   !generates a table for evaluating the fit of the stationary distribution in my model

   character(len=30), intent(in) :: input_filename1_in, input_filename2_in, input_filename3_in, &
      output_filename_in

   integer, parameter :: sig_fig_stationary = 4
   integer, parameter :: n_cols_in1 = 6
   integer, parameter :: n_rows_in1 = 5 !doesn't include the column header row
   integer, parameter :: n_cols_in2 = 2
   integer, parameter :: n_rows_in2 = 4
   integer, parameter :: n_cols_in3 = 2
   integer, parameter :: n_rows_in3 = 5

   real*8, dimension(n_rows_in1, n_cols_in1 ) :: input_matrix1
   real*8, dimension(n_rows_in2, n_cols_in2 ) :: input_matrix2
   real*8, dimension(n_rows_in3, n_cols_in3 ) :: input_matrix3

   character(len=30), dimension(n_rows_in1 ) :: row_header1
   character(len=30), dimension(n_rows_in2 ) :: row_header2
   character(len=30), dimension(n_rows_in3 ) :: row_header3
   character(len=100), dimension(n_cols_in1/2 ) :: col_header_pre1
   character(len=30), dimension(0:n_cols_in1 ) :: col_header1
   character(len=30), dimension(0:n_cols_in2 ) :: col_header2
   character(len=30), dimension(0:n_cols_in3 ) :: col_header3

   integer :: unit_int, i, j
   character(len=30) :: char_scrap

   character(len= format_length) :: format_string_temp
   character(len=20), dimension(n_rows_in1, n_cols_in1 ) :: input_matrix_string1
   character(len=20), dimension(n_rows_in2, n_cols_in2 ) :: input_matrix_string2
   character(len=20), dimension(n_rows_in3, n_cols_in3 ) :: input_matrix_string3

   !labels for the output table

   row_header1(1) = "\nth{10}"
   row_header1(2) = "\nth{25}"
   row_header1(3) = "\nth{50}"
   row_header1(4) = "\nth{75}"
   row_header1(5) = "\nth{90}"

   row_header2(1) = "Both Not Working"
   row_header2(2) = "Husband Working"
   row_header2(3) = "Wife Working"
   row_header2(4) = "Both Working"

   row_header3 = row_header1

   col_header_pre1(1) = "\multicolumn{2}{c | }{ \textit{Husband's Wage}  }"
   col_header_pre1(2) = "\multicolumn{2}{c | }{  \textit{Wife's Wage}   }"
   col_header_pre1(3) = "\multicolumn{2}{c  }{  \textit{Household Assets}   }"

   col_header1(0) = "Percentile"
   col_header1(1) = "Obs."
   col_header1(2) = "Predict."
   col_header1(3) = "Obs."
   col_header1(4) = "Predict."
   col_header1(5) = "Obs."
   col_header1(6) = "Predict."

   col_header2(0) = "Outcome"
   col_header2(1) = "Obs."
   col_header2(2) = "Predict."

   col_header3(0) = "Percentile"
   col_header3(1) = "Obs. Starting Wage"
   col_header3(2) = "Obs. New Wage"



   !inpute matrices
   open( newunit = unit_int, file = input_filename1_in, status = "old" )
   read( unit_int, * ) char_scrap

   do i = 1, n_rows_in1
      read( unit_int, * ) ( input_matrix1(i,j), j = 1, n_cols_in1 )
   end do

   close( unit_int )

   !!!

   open( newunit = unit_int, file = input_filename2_in, status = "old" )
   read( unit_int, * ) char_scrap

   do i = 1, n_rows_in2
      read( unit_int, * ) ( input_matrix2(i,j), j = 1, n_cols_in2 )
   end do

   close( unit_int )

   open( newunit = unit_int, file = input_filename3_in, status = "old" )
   read( unit_int, * ) char_scrap

   do i = 1, n_rows_in3
      read( unit_int, * ) ( input_matrix3(i,j), j = 1, n_cols_in3 )
   end do

   close( unit_int )

   !now write these input matrices to a character matrix in the right format
   do i = 1, n_rows_in1
      do j = 1, n_cols_in1 - 2
        ! format_string_temp = get_format_decimal_func( input_matrix1(i,j), sig_fig_stationary )
         format_string_temp = "(F14.2)"
         write( input_matrix_string1(i,j) , format_string_temp )  input_matrix1(i,j)


      end do

      do j = n_cols_in1 -1 , n_cols_in1
         input_matrix_string1(i,j) = &
            trim( thousands_separator_func( nint( input_matrix1(i,j) ) ) )

    !  print *, input_matrix_string1(i,j)

      end do

   end do

   do i = 1, n_rows_in2
      do j = 1, n_cols_in2
        ! format_string_temp = get_format_decimal_func( input_matrix2(i,j), sig_fig_stationary )
        format_string_temp = "(F14.4)"
         write( input_matrix_string2(i,j) , format_string_temp )  input_matrix2(i,j)
      end do
   end do

   do i = 1, n_rows_in3
      do j = 1, n_cols_in3
        ! format_string_temp = get_format_decimal_func( input_matrix1(i,j), sig_fig_stationary )
         format_string_temp = "(F14.2)"
         write( input_matrix_string3(i,j) , format_string_temp )  input_matrix3(i,j)


      end do

   end do



   !now write to the output file
   open( newunit = unit_int, file = output_filename_in, status = "replace")

   write(unit_int, * ) "\begin{tabular}{ l | l l | l  l | l l }"
   write(unit_int, * ) "\label{tab:model_fit2}"
   call latex_write( unit_int, "  ", .false. )
   call latex_write( unit_int, col_header_pre1, .true. )
   call latex_write( unit_int, col_header1, .true. )

   do i = 1, n_rows_in1
      call latex_write(unit_int, row_header1(i), .false. )
      call latex_write(unit_int, input_matrix_string1(i,:), .true. )
   end do

   write(unit_int, * ) "& & & & &  \\"
   write(unit_int, * ) "\hline"
   write(unit_int, * ) "\end{tabular}"
   write(unit_int, * ) "  "
   write(unit_int, * ) "\bigskip"
   write(unit_int, * ) "  "

   write(unit_int, * ) "\begin{tabular}{ l | c c }"
   write(unit_int, * ) "\label{tab:model_fit3}"
   call latex_write( unit_int, "  ", .false. )
   write(unit_int, * ) "  \multicolumn{2}{ c }{ \textit{Husband's Wage (2004 Panel)}  }  \\"
   call latex_write( unit_int, col_header3, .true. )

   do i = 1, n_rows_in1
      call latex_write(unit_int, row_header3(i), .false. )
      call latex_write(unit_int, input_matrix_string3(i,:), .true. )
   end do

   write(unit_int, * ) "& &   \\"
   write(unit_int, * ) "\hline"
   write(unit_int, * ) "\end{tabular}"
   write(unit_int, * ) "  "
   write(unit_int, * ) "\bigskip"
   write(unit_int, * ) "  "

   write(unit_int, * ) "\begin{tabular}{ l | l l }"
   write(unit_int, * ) "\multicolumn{3}{c }{ \textit{Employment Outcomes}  }  \\"
   call latex_write(unit_int, col_header2, .true. )

   do i = 1, n_rows_in2
      call latex_write(unit_int, row_header2(i), .false. )
      call latex_write(unit_int, input_matrix_string2(i,:), .true. )
   end do

   write(unit_int, * ) "& & \\ \hline"
   write(unit_int, * ) "\end{tabular}"
   write(unit_int, * ) "  "
   write(unit_int, * ) "\bigskip"
   write(unit_int, * ) "  "





end subroutine gen_stationary_table_latex


subroutine gen_model_fit_table_latex( input_filename_in, output_filename_in )
   !generates a table for evaluating the transition probabilities in my model

   character(len=30), intent(in) :: input_filename_in, output_filename_in


   integer, parameter :: n_cols = 4
   integer, parameter :: n_rows = 6 !doesn't include the column header row


   real*8, dimension(n_rows, n_cols ) :: input_matrix1, input_matrix2

   character(len=30), dimension(n_rows) :: row_header
   character(len=30), dimension(0:n_cols ) :: col_header

   integer :: unit_int, i, j
   character(len=30) :: char_scrap
   character(len=10) :: chi_print1, chi_print2

   character(len= format_length) :: format_string_temp
   character(len=20), dimension(n_rows, n_cols ) :: input_matrix_string1
   character(len=20), dimension(n_rows, n_cols) :: input_matrix_string2

   real*8 :: chi2_test_val, chi2_pval !for doing a chi-square test of a goodness of fit

   !labels for the output table

   row_header(1) = "1st"
   row_header(2) = "2nd"
   row_header(3) = "3rd"
   row_header(4) = "4th"
   row_header(5) = "5th"
   row_header(6) = "6th"


   col_header(0) = "Sextile"
   col_header(1) = "Both Reject"
   col_header(2) = "Husband Accepts"
   col_header(3) = "Wife Accepts"
   col_header(4) = "Both Accept"


   !inpute matrices
   open( newunit = unit_int, file = input_filename_in, status = "old" )
   read( unit_int, * ) char_scrap
   read( unit_int, * ) char_scrap

   do i = 1, n_rows
      read( unit_int, * ) char_scrap, ( input_matrix1(i,j), j = 1, n_cols )
   end do

   read( unit_int, * ) char_scrap
   read( unit_int, * ) char_scrap

   do i = 1, n_rows
      read( unit_int, * ) char_scrap, ( input_matrix2(i,j), j = 1, n_cols )
   end do

   close( unit_int )

   !gets the chi-square test statistics
   call pearson_test_matrix_input(n_rows, n_cols, input_matrix1, input_matrix2, &
      chi2_test_val, chi2_pval )

   !now write these input matrices to a character matrix in the right format
   do i = 1, n_rows
      do j = 1, n_cols
        ! format_string_temp = get_format_decimal_func( input_matrix1(i,j), sig_fig_stationary )
         format_string_temp = "(F14.4)"
         write( input_matrix_string1(i,j) , format_string_temp )  input_matrix1(i,j)
         write( input_matrix_string2(i,j) , format_string_temp )  input_matrix2(i,j)


      end do


   end do



   !now write to the output file
   open( newunit = unit_int, file = output_filename_in, status = "replace")

   write(unit_int, * ) "\begin{tabular}{ l l l l l }"
   write(unit_int, * ) "\multicolumn{5}{c }{ \textit{Predicted Probabilities} }  \\ \hline"
   call latex_write( unit_int, col_header, .true. )

   do i = 1, n_rows
      call latex_write(unit_int, row_header(i), .false. )
      call latex_write(unit_int, input_matrix_string1(i,:), .true. )
   end do

   write(unit_int, * ) "\\ "
   write(unit_int, * ) "\multicolumn{5}{c }{ \textit{Observed Frequencies} }  \\ \hline"
   call latex_write( unit_int, col_header, .true. )

   do i = 1, n_rows
      call latex_write(unit_int, row_header(i), .false. )
      call latex_write(unit_int, input_matrix_string2(i,:), .true. )
   end do

   write(unit_int, * ) "\\ "
   write(chi_print1, "(F7.4)")  chi2_test_val
   write(chi_print2, "(F7.4)")  chi2_pval

   write(unit_int, * ) "\multicolumn{5}{l }{Statistical Test: $\chi^2$(" // &
                        trim(int_to_string(n_rows*n_cols -1 ) ) // &
                        ")=  " // &
                         chi_print1// &
                         ", p= " // &
                         chi_print2 // &
                         "}"


   write(unit_int, * ) "\end{tabular}"



end subroutine gen_model_fit_table_latex

subroutine gen_uilength_table_latex( input_filename_in, output_filename_in )
   !generates a table for evaluating the effects of changing in the ui length

   character(len=30), intent(in) :: input_filename_in, output_filename_in


   integer, parameter :: n_cols = 3
   integer, parameter :: n_rows = 4 !doesn't include the column header row


   real*8, dimension(n_rows, n_cols ) :: input_matrix

   character(len=30), dimension(n_rows) :: row_header
   character(len=30), dimension(0:n_cols ) :: col_header

   integer :: unit_int, i, j
   character(len=30) :: char_scrap


   character(len= format_length) :: format_string_temp
   character(len=20), dimension(n_rows, n_cols ) :: input_matrix_string


   !labels for the output table

   row_header(1) = "Both Not Working"
   row_header(2) = "Husband Working"
   row_header(3) = "Wife Working"
   row_header(4) = "Both Working"



   col_header(0) = " "
   col_header(1) = "6 Months"
   col_header(2) = "14 Months"
   col_header(3) = "Difference"


   print *, "about to read"

   !input matrices
   open( newunit = unit_int, file = input_filename_in, status = "old" )
   read( unit_int, * ) char_scrap

   print *, "got past first line"

   do i = 1, n_rows
      print *, "reading row", i
      !do n_chols-1 because the last row wil have the differences in the employment probabilities
      read( unit_int, * ) char_scrap, ( input_matrix(i,j), j = 1, n_cols -1 )
   end do

   close( unit_int )

   print *, "read all of it"

   input_matrix(:,3) = input_matrix(:,2) - input_matrix(:,1)


   !gets the chi-square test statistics

   !now write these input matrices to a character matrix in the right format
   do i = 1, n_rows
      do j = 1, n_cols
        ! format_string_temp = get_format_decimal_func( input_matrix1(i,j), sig_fig_stationary )
         format_string_temp = "(F14.4)"
         write( input_matrix_string(i,j) , format_string_temp )  input_matrix(i,j)

      end do
   end do




   !now write to the output file
   open( newunit = unit_int, file = output_filename_in, status = "replace")

   write(unit_int, * ) "\begin{tabular}{ l l l }"
   write(unit_int, * ) "& \multicolumn{2}{c }{ \textit{Max. Duration of Unemployment} }  \\ \hline"
   call latex_write( unit_int, col_header, .true. )

   do i = 1, n_rows
      call latex_write(unit_int, row_header(i), .false. )
      call latex_write(unit_int, input_matrix_string(i,:), .true. )
   end do

   write(unit_int, * ) "\end{tabular}"



end subroutine gen_uilength_table_latex




end module latex_tables_mod
