program like_test

!tests the derivatives of the likelihood funciton

use enum_mod
use model_params_mod
use interp_support_mod

use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use likelihood_mod
use sipp_data_mod

use normal_mod
use stat_estimates_mod

use csv_file

!use statespace_trans_params_mod


implicit none

   character(len=30),parameter :: init_params_file       = "init_params_new.txt"
   character(len=30),parameter :: interact_file_in       = "interact_matrix.csv"
   character(len=30),parameter :: value_matrix_init_file = "value_matrix_out.csv"
   character(len=30),parameter :: asset_matrix_init_file = "asset_matrix_out.csv"

   character(len=30),parameter :: like_der_file = "like_der.csv"

   logical, parameter :: read_ols_matrices = .true. !for whether to init the ols matrices with other files

   real*8, dimension(n_params) :: params
   real*8, dimension( max_n_grid_points, 0:1,0:1 ) :: value_matrix_init
   real*8, dimension( max_n_grid_points, 0:1,0:1 ) :: asset_matrix_init
   real*8, dimension(0:1,0:1,n_choice_der_vars) :: first_derivative
   real*8, dimension(0:1,0:1,n_choice_der_vars, n_choice_der_vars) :: second_derivative

   type(like_type)   :: like_object
   type(mygrid_type) :: mygrid


   integer :: i,t,m_job, job, m_receive_new, receive_new
   integer :: m_hours_new, hours_new
   real*8 :: m_wage_new, wage_new, m_non_pec_new, non_pec_new
   integer :: unit_int

   call mpi_setup

   if( nprocs .ne. 0 ) then
      print *, "there should only be once process for this program"
      stop
   end if

   m_job = 0
   job =0
   m_receive_new = 1
   receive_new = 0


   !now read in the parameters
   call init_params(init_params_file, params)
   !change the discount factor for initial testing
 !  params(param_discount_factor) = .1

   call gen_var_labels()
   call der_delta_setup()
   call interp_list_setup_all(interact_file_in)

   !init ols matrix to 0
   if (read_ols_matrices .eqv. .true. ) then
      call init_interp_matrices( value_matrix_init_file, asset_matrix_init_file, value_matrix_init, asset_matrix_init )

   else
      value_matrix_init = 0
      asset_matrix_init = 0
      asset_matrix_init(1,:,:) = 2.9957322735539909

   end if

   call get_mygrid_origin(m_job, job, mygrid)

   mygrid % rv( var_m_ui_ben ) = 20
   mygrid % rv( var_ui_ben ) = 20
   mygrid % rv( var_assets ) = 3000

   m_wage_new  = grid_mean( var_m_wage )
   wage_new    = grid_mean( var_wage )

   m_non_pec_new = grid_mean( var_m_non_pec_norm )
   non_pec_new = grid_mean( var_non_pec_norm )

   m_hours_new = grid_mean( var_m_hours )
   hours_new = grid_mean( var_hours )

   print *, "setting up the derivative object"
   call choice_prob_der_setup( mygrid, m_wage_new, wage_new, m_non_pec_new, &
      non_pec_new, m_hours_new, hours_new, like_object )

   print *, "getting derivatives"
   call choice_prob_derivatives( m_receive_new, receive_new, like_object, &
      params, value_matrix_init, asset_matrix_init, first_derivative, second_derivative )


   print *, "writting the derivatives to a file"
   open(newunit = unit_int, file=like_der_file, status='replace' )
   call choice_prob_der_write( m_receive_new, receive_new, first_derivative, second_derivative, unit_int )
   close(unit_int)

   call mpi_finalize(err)


end program

