program like_test

!tests the likelihood evaluated for the whole sample

use enum_mod
use model_params_mod
use interp_support_mod


use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use likelihood_mod
use sipp_data_mod

use normal_mod
use stat_estimates_mod

use csv_file
use, intrinsic :: iso_fortran_env !gives standard in, out and error

!use statespace_trans_params_mod


implicit none


   integer, parameter :: i_test_index = 249 !household of interest
   integer, parameter :: t_test_index = 27 !time index of interest

   character(len=30),parameter :: init_params_file       = "init_params_simple2.txt"
   character(len=30),parameter :: sipp_data_file         = "sipp_fortran_small.csv"
   character(len=30),parameter :: interact_file          = "interact_matrix.csv"
   character(len=30),parameter :: value_matrix_init_file = "value_matrix_out.csv"
   character(len=30),parameter :: asset_matrix_init_file = "asset_matrix_out.csv"

   logical, parameter :: est_savings = .false.
   logical, parameter :: read_interp_matrices = .false. !for whether to init the ols matrices with other files


   type(data_i_type), dimension(:), allocatable                :: hh_data
   type(like_type), dimension(:), allocatable                  :: like_object
   type(like_target_type), dimension(:), allocatable, target   :: like_target
   real*8, dimension(n_params)                        :: params
   real*8, dimension( n_grid_points ) :: value_matrix_init
   real*8, dimension( n_grid_points ) :: asset_matrix_init
   integer :: i,t
   real*8 :: start, finish, start_all, finish_all, like_func_eval


   call mpi_setup()

   if (myrank .eq. 0 ) then

      print *, "N_obs", N_obs_total
      print *, "data file", sipp_data_file
      print *, "init params file", init_params_file

   end if

   call read_sipp_data(sipp_data_file, hh_data)
   if (myrank .eq. 0 ) print *, "read in data"

   !now read in the parameters
   call init_params(init_params_file, params)
   !change the discount factor for initial testing
  ! params(param_discount_factor) = .1
  ! params(param_weight_utility) = 7
  ! params(param_sd_non_pec) = 10

!   call write_params(params,6,.false.)
!   call gen_param_labels()
!   read(*,*)

   if (myrank .eq. 0) call write_params( params, output_unit, .false. )



   call gen_grid_mean_sd
   call interp_setup_all( interact_file )


   if (read_interp_matrices .eqv. .true. ) then
      call init_interp_matrices( value_matrix_init_file, asset_matrix_init_file, value_matrix_init, asset_matrix_init )

   else
      value_matrix_init = 0
      asset_matrix_init = 0
      asset_matrix_init(1) = 2.9957322735539909

   end if




   !read in an ols matrix if I want to


 !  print *, "read in parameters complete"

   !now for calling all the likelihood subroutines
   allocate(like_object(min_obs:max_obs) )
   allocate(like_target(min_obs:max_obs) )

   call cpu_time(start)
   do i=min_obs,max_obs
      call like_target(i) % allocate( hh_data(i) )
   end do
   call cpu_time(finish)
   if (myrank .eq. 0 ) print *, "time for like_target_allocate", finish-start

   call cpu_time(start)
   do i=min_obs,max_obs
      call like_object(i) % like_init_pointer( like_target(i), hh_data(i) )
   end do
   call cpu_time(finish)
   if (myrank .eq. 0 ) print *, "time for like_init_pointer", finish-start

   call cpu_time(start)
   do i=min_obs,max_obs
      call like_object(i) % like_main_setup( hh_data(i) )
   end do
   call cpu_time(finish)
   if (myrank .eq. 0 ) print *, "time for like_main_setup", finish-start

!   call cpu_time(start)
!   do i=1,N_obs
!      call like_object(i) % get_birth_ui_means( hh_data(i), params)
!   end do
!   call cpu_time(finish)
!   print *, "time for getting birth means", finish-start



   call cpu_time(start)
   do i=min_obs,max_obs
      call like_target(i) % copy_targets_init()
   end do
   call cpu_time(finish)
   if (myrank .eq. 0 ) print *, "time for like_target_copy_init", finish-start


   call mpi_barrier(mpi_comm_world, err)

   call cpu_time(start_all)

   call cpu_time(start)
   do i=min_obs,max_obs
      call like_object(i) % joboffer_like_update( hh_data(i), params )
   end do
   call cpu_time(finish)
   if (myrank .eq. 0 ) print *, "time for joboffer probs", finish-start

   call cpu_time(start)
   do i=min_obs,max_obs
      call like_object(i) % job_probs_like_update( hh_data(i), params )
   end do
   call cpu_time(finish)
   if (myrank .eq. 0 ) print *, "time for job_probs", finish-start

   call cpu_time(start)
   do i=min_obs,max_obs
      call like_object(i) % utility_params_like_update( hh_data(i), params )
   end do
   call cpu_time(finish)
   if (myrank .eq. 0 ) print *, "time for utility_params", finish-start

   if( est_savings .eqv. .true. ) then
      call cpu_time(start)
      do i=min_obs,max_obs
         call like_object(i) % asset_choice_like_update( asset_matrix_init, params )
      end do
      call cpu_time(finish)
      if (myrank .eq. 0 ) print *, "time for asset_choice", finish-start
   end if


   call cpu_time(start)
   do i=min_obs,max_obs
      call like_object(i) % utility_like_update( params )
   end do
   call cpu_time(finish)
   if (myrank .eq. 0 ) print *, "time for utility", finish-start

!   call cpu_time(start)
!   do i=min_obs,max_obs
!      call like_object(i) % jobloss_like_update( hh_data(i), params )
!   end do
!   call cpu_time(finish)
!   if (myrank .eq. 0 ) print *, "time for jobloss_like", finish-start

!   call cpu_time(start)
!   do i=1,N_obs
!      call like_object(i) % ols_partial_like_update( ols_vec )
!   end do
!   call cpu_time(finish)
!   print *, "time for choice probs", finish-start


   call cpu_time(start)
   do i=min_obs,max_obs
      call like_object(i) % choice_prob_like_update(value_matrix_init )
   end do
   call cpu_time(finish)
   if (myrank .eq. 0 ) print *, "time for choice probs", finish-start


   if (est_savings .eqv. .true. ) then
      call cpu_time(start)
      do i=min_obs,max_obs
         call like_object(i) % asset_error_prob_like_update( hh_data(i), params )
      end do
      call cpu_time(finish)
      if (myrank .eq. 0 ) print *, "time for asset_error", finish-start
   end if

   call mpi_barrier(mpi_comm_world, err)

   call cpu_time(finish_all)
   if (myrank .eq. 0 ) print *, " "
   if (myrank .eq. 0 ) print *, "time for main components", finish_all - start_all



   call cpu_time(start)
   do i=min_obs,max_obs
      call like_object(i) % like_error_hunt(i)
   end do
   call cpu_time(finish)
   if (myrank .eq. 0 ) print *, "time for hunting for error", finish-start



!   !now see if any households have a really low choice prob
!   do i=1,N_obs
!      do t=2, like_object(i) % max_time
!         if (like_object(i) % choice_prob(t) < choice_prob_min) then
!            print *, "low choice prob for hh", i, "at time", t
!            print *, like_object(i) % choice_prob(9)
!            call low_choice_prob_adjust(like_object(i), params, ols_matrix, t )
!            print *, like_object(i) % choice_prob(9)
!         end if
!      end do
!   end do

!   i=1
!   print *, "choice probs"
!  ! print *, like_object(i) % choice_prob(9)
!  ! call get_log_like_params(like_object_in, block_in, log_like_val_out)




   !now printing low choice probabilities
   do i=min_obs,max_obs
      do t=2, like_object(i) % max_time
         if ( like_object(i) % choice_prob(t) < choice_prob_lb + zero_tol ) then
            print *, "low choice prob for hh", i, "at time", t

            call like_object(i) % print(t, value_matrix_init)

            read(*,*)
         end if
      end do
   end do

   !printing key hh
!   if ( ( i_test_index .ge. min_obs ) .and. ( i_test_index .le. max_obs ) ) then
!      call like_object(i_test_index) % print(t_test_index,value_matrix_init)
!   end if


  ! read(*,*)


   !now get likelihood for whole sample
   like_func_eval = log_like_func_all( like_object, block_leisure_params1 )
   if( myrank .eq. 0 ) print *, "log_like_all", like_func_eval



   !now print some statistics
   call like_stats_print( like_object, hh_data, params, output_unit, est_savings )

   call mpi_finalize(err)






end program
