module model_fit_mod

!module for determing how well my model fits the data
!overall idea: For each household-month observation, I do simulation to compute the probability of 4 outcomes happening
   !  1: Both Reject
   !  2: Husband Accepts
   !  3: Wife Accepts
   !  4: Both Reject


!then once I have these probabilities, I then sort them and put them all into bins.  For each bin, I calculate the observed fequencies of the 4 outcomes, and I then see how that compares to the average probability in that bin to guage model fit.


use enum_mod
use model_params_mod
use interp_support_mod


use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use likelihood_mod
use sipp_data_mod

use normal_mod
use stat_estimates_mod

use bayesian_mod

use csv_file
use, intrinsic :: iso_fortran_env !gives standard in, out and error


   integer, parameter :: n_sim_model_fit =   5 !number of simulation to do to generate the predicted probabilities
   integer, parameter :: n_fit_cells = 6
   logical, parameter :: savings_switch_model_fit = .false.

contains

subroutine fit_outcome_prob_one_t( like_object_inout, params_in, value_matrix_in, t, outcome_prob_out)
   !gets the value function, choice probabilities, outcome probabiliites, and asset choices for the model fit exercises
   !need to get the probabilities for each potential outcomes, not just the outcome that occured


   class(like_type), intent(in out) :: like_object_inout
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
   integer, intent(in) :: t
   real*8, dimension(4), intent(out) :: outcome_prob_out

   real*8, dimension(4) :: value_vec

   real*8 :: m_non_pec_norm_new, m_wage_xb, m_wage_new, non_pec_norm_new, wage_xb, wage_new
   integer :: m_hours_new, hours_new

   type(mygrid_type) :: mygrid_husband_accept, mygrid_wife_accept, mygrid_both_accept

   real*8, dimension(2) :: choice_prob_husband_receive, choice_prob_wife_receive
   real*8, dimension(4) :: choice_prob_both_receive

   real*8 :: value_both_reject, value_husband_accept, value_wife_accept, value_both_accept
   real*8 :: m_joboffer_prob, joboffer_prob

   !want to draw new potential joboffers for all households, regardless of whether they accepted a job offer or not
      !redoing the wages for the households that accept helps me get average marginal effects that integrates over the distribution of
      !potential wages


   !husband first

   !non pec
   m_non_pec_norm_new = rnorm()

   !hours
   if ( r_uniform(dble(0),dble(1) ) < params_in(param_prob_pt) ) then
      m_hours_new = 20
   else
      m_hours_new = 40
   end if

   m_wage_xb = &
      m_wage_xb_func(   like_object_inout % mygrid_t_invar, &
                        like_object_inout % mygrid_t_var(t), &
                        params_in, &
                        m_hours_new)

   m_wage_new = exp( m_wage_xb + params_in(param_sd_wage) * rnorm() )

   !incorproate bounds
   m_non_pec_norm_new = bounds_func( m_non_pec_norm_new, grid_min( var_m_non_pec_norm ), &
                                                         grid_max( var_m_non_pec_norm ) )
   m_wage_new = bounds_func( m_wage_new, grid_min( var_m_wage ), grid_max( var_m_wage ) )




   !now wife
   non_pec_norm_new = rnorm()
!    like_object_inout % m_non_pec_prob(t) = ln_pdf_norm( m_non_pec_norm_new )

   !hours
   if ( r_uniform( dble(0), dble(1) ) < params_in(param_prob_pt) ) then
      hours_new = 20
   else
      hours_new = 40
   end if

   wage_xb = &
      wage_xb_func(  like_object_inout % mygrid_t_invar, &
                     like_object_inout % mygrid_t_var(t), &
                     params_in, &
                     hours_new)

   wage_new = exp( wage_xb + params_in(param_sd_wage) * rnorm() )

   !incorproate bounds
   non_pec_norm_new = bounds_func( non_pec_norm_new, grid_min( var_non_pec_norm ), &
                                                         grid_max( var_non_pec_norm ) )
   wage_new = bounds_func( wage_new, grid_min( var_wage ), grid_max( var_wage ) )



   !now store these on the mygrid objects
   mygrid_husband_accept = like_object_inout % mygrid_husband_accept(t)
   mygrid_wife_accept = like_object_inout % mygrid_wife_accept(t)
   mygrid_both_accept = like_object_inout % mygrid_both_accept(t)

   mygrid_husband_accept % rv( var_m_wage )              = m_wage_new
   mygrid_husband_accept % rv( var_m_non_pec_norm )      = m_non_pec_norm_new
   mygrid_husband_accept % iv( var_m_hours )             = m_hours_new

   mygrid_wife_accept % rv( var_wage )             = wage_new
   mygrid_wife_accept % rv( var_non_pec_norm )     = non_pec_norm_new
   mygrid_wife_accept % iv( var_hours )            = hours_new

   mygrid_both_accept % rv( var_m_wage )              = m_wage_new
   mygrid_both_accept % rv( var_m_non_pec_norm )      = m_non_pec_norm_new
   mygrid_both_accept % iv( var_m_hours )             = m_hours_new

   mygrid_both_accept % rv( var_wage )             = wage_new
   mygrid_both_accept % rv( var_non_pec_norm )     = non_pec_norm_new
   mygrid_both_accept % iv( var_hours )            = hours_new

   !now copy over some state variables from mygrid_t_invar to the husband and wife accept objects
   mygrid_husband_accept % rv( var_non_pec_norm ) = &
      like_object_inout % mygrid_t_var(t) % rv( var_non_pec_norm )
   mygrid_husband_accept % rv( var_wage ) = &
      like_object_inout % mygrid_t_var(t) % rv( var_wage )
   mygrid_husband_accept % rv( var_ui_ben ) = &
      like_object_inout % mygrid_t_var(t) % rv( var_ui_ben )

   mygrid_wife_accept % rv( var_m_non_pec_norm ) = &
      like_object_inout % mygrid_t_var(t) % rv( var_m_non_pec_norm )
   mygrid_wife_accept % rv( var_m_wage ) = &
      like_object_inout % mygrid_t_var(t) % rv( var_m_wage )
   mygrid_wife_accept % rv( var_m_ui_ben ) = &
      like_object_inout % mygrid_t_var(t) % rv( var_m_ui_ben )


   !both reject value
   call max_value_sub_split(  like_object_inout % mygrid_t_invar, &
                              like_object_inout % mygrid_t_var(t), &
                              like_object_inout % utility_params(:,t), &
                              params_in, &
                              value_matrix_in, &
                              like_object_inout % asset_choice_both_reject(t), &
                              value_both_reject )


   !husband accept value
   call max_value_sub_split(  like_object_inout % mygrid_t_invar, &
                              like_object_inout % mygrid_husband_accept(t), &
                              like_object_inout % utility_params(:,t), &
                              params_in, &
                              value_matrix_in, &
                              like_object_inout % asset_choice_husband_accept(t), &
                              value_husband_accept )

   !wife accept value
   call max_value_sub_split(  like_object_inout % mygrid_t_invar, &
                              like_object_inout % mygrid_wife_accept(t), &
                              like_object_inout % utility_params(:,t), &
                              params_in, &
                              value_matrix_in, &
                              like_object_inout % asset_choice_wife_accept(t), &
                              value_wife_accept )

   !both accept value
   call max_value_sub_split(  like_object_inout % mygrid_t_invar, &
                              like_object_inout % mygrid_both_accept(t), &
                              like_object_inout % utility_params(:,t), &
                              params_in, &
                              value_matrix_in, &
                              like_object_inout % asset_choice_both_accept(t), &
                              value_both_accept )


   !!!!!!!!!!!!!!!!!!!!!!!

   !now get the choice probabilities
   value_vec(1) = value_both_reject

   !choice probs for only husband receives a job offer
   value_vec(2) = value_husband_accept

   choice_prob_husband_receive(1) = mlogit_prob(2,1,value_vec(1:2) )
   choice_prob_husband_receive(2) = 1- choice_prob_husband_receive(1)


   !choice probs for only wife receiving a job offer
   value_vec(2) = value_wife_accept

   choice_prob_wife_receive(1) = mlogit_prob(2,1,value_vec(1:2) )
   choice_prob_wife_receive(2) = 1- choice_prob_wife_receive(1)


   !now for both receiving an job offer
   value_vec(1) = value_both_reject
   value_vec(2) = value_husband_accept
   value_vec(3) = value_wife_accept
   value_vec(4) = value_both_accept

   choice_prob_both_receive(1) = mlogit_prob(4,1,value_vec )
   choice_prob_both_receive(2) = mlogit_prob(4,2,value_vec )
   choice_prob_both_receive(3) = mlogit_prob(4,3,value_vec )
   choice_prob_both_receive(4) = 1 - choice_prob_both_receive(1) - choice_prob_both_receive(2) - &
      choice_prob_both_receive(3)

   !now get the probabilities for the observed outcomes, which are functions of the receive probabilities and the choice probabilities
   m_joboffer_prob = like_object_inout % m_joboffer_prob(t)
   joboffer_prob = like_object_inout % joboffer_prob(t)

   !both don't take a job offer
   outcome_prob_out(1) = (1- m_joboffer_prob ) * ( 1- joboffer_prob ) +   &
                     m_joboffer_prob  * ( 1- joboffer_prob ) * choice_prob_husband_receive(1) +  &
                     (1- m_joboffer_prob ) * joboffer_prob * choice_prob_wife_receive(1) +  &
                     m_joboffer_prob * joboffer_prob * choice_prob_both_receive(1)

   !only husband takes a job offer
   outcome_prob_out(2) = m_joboffer_prob  * ( 1- joboffer_prob ) * choice_prob_husband_receive(2) +  &
                     m_joboffer_prob * joboffer_prob * choice_prob_both_receive(2)

   !only wife takes a job offer
   outcome_prob_out(3) = (1- m_joboffer_prob ) * joboffer_prob * choice_prob_wife_receive(2) +  &
                     m_joboffer_prob * joboffer_prob * choice_prob_both_receive(3)

   !both take a job offer
   outcome_prob_out(4) = m_joboffer_prob * joboffer_prob * choice_prob_both_receive(4)


end subroutine fit_outcome_prob_one_t


subroutine fit_outcome_prob( like_object_inout, params_in, value_matrix_in, prob_matrix_out )
   class(like_type), intent(in out) :: like_object_inout
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
   real*8, dimension(4, 2:N_t), intent(out) :: prob_matrix_out
   integer :: t

   !note: this subroutine hasn't been debugged yet

   do t=2, like_object_inout % max_time
    !  print *, "asset choice updating, time = ", t
      !update asset state variable 1st.  for t=2, the value is based on the initial draw asset variable
      call asset_state_like_update_one_t( like_object_inout, params_in, t)
      call fit_outcome_prob_one_t( like_object_inout, params_in, value_matrix_in, t, prob_matrix_out(:,t) )

   end do !over t


end subroutine fit_outcome_prob


!subroutine I'm commenting out where I get these choice probabilties my doing interpolation to get the assets

!subroutine fit_outcome_prob_one_t(like_object_inout, value_matrix_in, t, choice_prob_out)

!   class(like_type), intent(in out) :: like_object_inout
!   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
!   integer, intent(in) :: t
!   real*8, dimension(4), intent(out) :: choice_prob_out

!   real*8,dimension(4) :: value_vec
!   integer :: choice_index

!   if (  (like_object_inout % m_receive_new(t) .eq. 0) .and.  &
!         (like_object_inout % receive_new(t) .eq. 0) ) then
!  !    print *, "both no offer"
!      choice_prob_out(1) = 1
!      choice_prob_out(2) = 0
!      choice_prob_out(3) = 0
!      choice_prob_out(4) = 0

!   else if (like_object_inout % m_receive_new(t) .eq. 0) then !only wife receives an offer
!   !   print *, "only wife receives offer"
!      value_vec(1) = like_object_inout % utility_both_reject(t) + &
!         value_function_split_in(   like_object_inout % mygrid_t_invar, &
!                                    like_object_inout % mygrid_t_var(t), &
!                                    like_object_inout % asset_choice_both_reject(t), &
!                                    value_matrix_in )


!      value_vec(2) = like_object_inout % utility_wife_accept(t) + &
!         value_function_split_in(   like_object_inout % mygrid_t_invar, &
!                                    like_object_inout % mygrid_wife_accept(t), &
!                                    like_object_inout % asset_choice_wife_accept(t), &
!                                    value_matrix_in )

!      choice_prob_out(1) = mlogit_prob(2,1,value_vec(1:2) )
!      choice_prob_out(2) = 1- choice_prob_out(1)
!      choice_prob_out(3) = 0
!      choice_prob_out(4) = 0

!   else if (like_object_inout % receive_new(t) .eq. 0) then !only husband
!    !  print *, "only husband receives offer"
!      value_vec(1) = like_object_inout % utility_both_reject(t) + &
!         value_function_split_in(   like_object_inout % mygrid_t_invar, &
!                                    like_object_inout % mygrid_t_var(t), &
!                                    like_object_inout % asset_choice_both_reject(t), &
!                                    value_matrix_in )


!      value_vec(2) = like_object_inout % utility_husband_accept(t) + &
!         value_function_split_in(   like_object_inout % mygrid_t_invar, &
!                                    like_object_inout % mygrid_husband_accept(t), &
!                                    like_object_inout % asset_choice_husband_accept(t), &
!                                    value_matrix_in )

!      choice_prob_out(1) = mlogit_prob(2,1,value_vec(1:2) )
!      choice_prob_out(2) = 0
!      choice_prob_out(3) = 1- choice_prob_out(1)
!      choice_prob_out(4) = 0

!   else !both receive an offer
!    !  print *, "both receive offer"

!      value_vec(1) = like_object_inout % utility_both_reject(t) + &
!         value_function_split_in(   like_object_inout % mygrid_t_invar, &
!                                    like_object_inout % mygrid_t_var(t), &
!                                    like_object_inout % asset_choice_both_reject(t), &
!                                    value_matrix_in )

!      value_vec(2) = like_object_inout % utility_husband_accept(t) + &
!         value_function_split_in(   like_object_inout % mygrid_t_invar, &
!                                    like_object_inout % mygrid_husband_accept(t), &
!                                    like_object_inout % asset_choice_husband_accept(t), &
!                                    value_matrix_in )

!      value_vec(3) = like_object_inout % utility_wife_accept(t) + &
!         value_function_split_in(   like_object_inout % mygrid_t_invar, &
!                                    like_object_inout % mygrid_wife_accept(t), &
!                                    like_object_inout % asset_choice_wife_accept(t), &
!                                    value_matrix_in )

!      value_vec(4) = like_object_inout % utility_both_accept(t) + &
!         value_function_split_in(   like_object_inout % mygrid_t_invar, &
!                                    like_object_inout % mygrid_both_accept(t), &
!                                    like_object_inout % asset_choice_both_accept(t), &
!                                    value_matrix_in )

!      choice_prob_out(1) = mlogit_prob(4,1,value_vec )
!      choice_prob_out(2) = mlogit_prob(4,2,value_vec )
!      choice_prob_out(3) = mlogit_prob(4,3,value_vec )
!      choice_prob_out(4) = 1 - choice_prob_out(1) - choice_prob_out(2) - choice_prob_out(3)

!   end if



!end subroutine fit_outcome_prob_one_t

!subroutine fit_outcome_prob(like_object_inout, value_matrix_in, choice_prob_out)
!   !gets the accept/reject choice probability, given that a particular joboffer has been received
!   class(like_type), intent(in out) :: like_object_inout
!   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
!   real*8, dimension(4, 2:N_t), intent(out) :: choice_prob_out


!   real*8,dimension(4) :: vals_prob_input
!   integer :: t, choice_index



!   !now go through time to get the choice probabilities
!   do t=2, like_object_inout % max_time
!    !  print *, t
!      call fit_outcome_prob_one_t(like_object_inout,value_matrix_in,t, choice_prob_out(:,t) )

!   end do !over t

!end subroutine fit_outcome_prob


subroutine fit_asset_init( like_object_inout, hh_data_in, params_in )
   !draws the initial assets based on the observed assets in the data

   type(like_type), intent(in out) :: like_object_inout
   type(data_i_type), intent(in) :: hh_data_in
   real*8, dimension(n_params) :: params_in

   real*8 :: asset_observed, sd_asset, asset_new

   !get initial assets, or set equal to median level if assets are not observed
   if ( hh_data_in % liquid_time1 .le. 0 ) then
      asset_observed = asset_median
   else
      asset_observed = hh_data_in % liquid_net1
   end if

   !get the standard deviation
   sd_asset = exp(   params_in( param_asset_error_cons ) + &
                  params_in( param_asset_error_linear )*abs(asset_observed) )

   !now draw the assets
   asset_new = rand_trunc_norm (   asset_observed, &
                                    sd_asset, &
                                    grid_min( var_assets ) + 1, & !+1 so that I don't start at the lb of assets (IMPORTANT, not sure if I should do this)
                                    grid_max( var_assets ) )

   like_object_inout % mygrid_t_var(1) % rv(var_assets) = asset_new


end subroutine fit_asset_init


subroutine fit_asset_weight( like_object_inout, hh_data_in, asset_weight_out )

   !gets the weight to multiply a choice probability by in my importance sampling procedure

   !IMPORTANT: for now, I'm setting these weights equal to 1, but that's wrong, I need to change that in the future


   type(like_type), intent(in out) :: like_object_inout
   type(data_i_type), intent(in) :: hh_data_in
   real*8, intent(out) :: asset_weight_out

   asset_weight_out = 1

end subroutine fit_asset_weight


subroutine gen_predicted_probs( hh_data_in, params_in, value_matrix_in,  &
   like_object_inout, like_target_inout, prob_matrix_out )
   !calculates the predected employement transition probabilities.  These transition probablities are integrals of logit probablities,
    !so I do simulation to calculate them
   type(data_i_type), intent(in) :: hh_data_in
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
   type(like_type), intent(in out) :: like_object_inout
   type(like_target_type), intent(in out) :: like_target_inout
   real*8, dimension(4,2:N_t) :: prob_matrix_out

   real*8, dimension(n_hetero,n_hetero) :: hetero_chol !choleksy decomp of the unobserved heterogeneity matrix
   real*8, dimension(iter_m_job_offer: iter_hours,N_t) :: joboffer_random_uniform
   real*8, dimension(4,2:N_t) :: outcome_prob_matrix
  ! real*8, dimension(2:N_t) :: receive_prob
   real*8 :: asset_weight

   integer :: t, s

   !initialize the prob matrix to 0 because I'm going to be adding stuff to it
   prob_matrix_out = 0


   !unpack the cholesky matrix into a 2 by 2 matrix
   hetero_chol = 0
   hetero_chol(1,1) = params_in( param_hetero_m_wage_var )
   hetero_chol(2,1) = params_in( param_hetero_wage_covar )
   hetero_chol(2,2) = params_in( param_hetero_wage_var )




   !Note: the likelihood objects and the joboffer probabilities need to have been calculated already
   do s= 1, n_sim_model_fit

      !draw new unobserved heterogeneity terms
      call get_new_heteo_ind( like_object_inout, &
                              params_in, &
                              hetero_chol)



      !draw new initial assets
      call fit_asset_init( like_object_inout, &
                           hh_data_in, &
                           params_in )


      !draw a new joboffer variables and update that part of the likelihood object
      call random_number( joboffer_random_uniform )

    !  print *, "calling get_new_joboffers_ind"


      call get_new_joboffers_ind(   like_object_inout, &
                                    hh_data_in, &
                                    params_in, &
                                    joboffer_random_uniform )

    !  print *, "called get_new_joboffers_ind"

      !now get the new choice probabilities and update the asset states
      call fit_outcome_prob( like_object_inout, params_in, value_matrix_in, outcome_prob_matrix )

      !now get the weight for this prob_matrix_temp based on the assets that were just drawn
      call fit_asset_weight( like_object_inout, hh_data_in, asset_weight )


      !now finally add all these parts to the prob_matrix_out

      do t=2, like_object_inout % max_time

         prob_matrix_out(:,t) = prob_matrix_out(:,t) + asset_weight  * &
            outcome_prob_matrix(:,t)

      end do



   end do !over s


   prob_matrix_out = prob_matrix_out / n_sim_model_fit


end subroutine gen_predicted_probs



subroutine gen_prob_fit( hh_data_in, params_in, value_matrix_in,  &
   like_object_inout, like_target_inout, predict_avg_out, obs_freq_out )

!   !subroutine to determine how good the predicted probabilties match up with observed probabiltiies with that cell.

!    1) Simulate to calculate the choice probability for all households
!    2) Combine all these probabilities into a vector for each process
!    3) Send these vectors to the root process in order to calculate the percentiles for the probabilties
!    4) Use these percentitles to create cells, and then calculate the observed frequency in each cell

   type(data_i_type), dimension(min_obs:max_obs), intent(in) :: hh_data_in
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
   type(like_type), dimension(min_obs:max_obs), intent(in out) :: like_object_inout
   type(like_target_type), dimension(min_obs:max_obs), intent(in out) :: like_target_inout

   real*8, dimension(n_fit_cells,4), intent(out) :: predict_avg_out
   real*8, dimension(n_fit_cells,4), intent(out) :: obs_freq_out

   real*8, dimension(n_fit_cells,4) :: cell_ub !upper bound for the probabiliity in this cell
   integer, dimension(n_fit_cells,4) :: cell_count_local, cell_count_global
   real*8, dimension(n_fit_cells,4) :: predict_sum_local, predict_sum_global
   integer, dimension(n_fit_cells,4) :: obs_sum_local, obs_sum_global


   real*8, dimension(4,2:N_t,min_obs:max_obs) :: prob_matrix
   real*8, dimension(:,:), allocatable :: prob_vec_local, prob_vec_global !prob_matrix but in vector form (or a vector for each choice, so it's still a matrix....)
   integer :: count_local, count_global !number of observed probabilities for each process and overall
   integer, dimension(0:nprocs-1) :: recvcounts, displs !some mpi variables


   real*8, dimension(n_fit_cells -1 ) :: percent_nodes2
   real*8, dimension(n_fit_cells - 1, 4) :: prob_percentiles

   integer :: i, k, j, t, lb, ub, s, max_time




   !first get the probabilities for each household in the sample
   do i=min_obs, max_obs

     ! print *, "obs no", i

      call gen_predicted_probs( hh_data_in(i), params_in, value_matrix_in,  &
         like_object_inout(i), like_target_inout(i), prob_matrix(:,:,i) )
   end do

 !  call mpi_barrier( mpi_comm_world, err)
 !  if (myrank .eq. 0 ) print *, "got all the predicted probabilities"

   !now get number of observed probabilities for each process
   count_local = 0
   do i=min_obs,max_obs
    !  print *, i
      count_local = count_local + like_object_inout(i) % max_time - 1
   end do



   !now collect local sums to the root node
   call mpi_gather( count_local, 1, mpi_int, recvcounts, 1, mpi_int, 0, mpi_comm_world, err )
   count_global = sum( recvcounts )
   call mpi_bcast( count_global, 1, mpi_int, 0, mpi_comm_world, err )







 !  call mpi_barrier( mpi_comm_world, err)
 !  if (myrank .eq. 0 ) print *, "got sums to root node"

   !set up displacement vectors, which will be used later to collect all the probabilities
   displs(0) = 0
   do i=1,nprocs-1
      displs(i) = recvcounts(i-1) + displs(i-1)
   end do

   !put the probabilies for each choice into a vector for each process
   allocate( prob_vec_local(4,count_local) )
   j = 0
   do i = min_obs,max_obs
      ub = like_object_inout(i) % max_time - 1

      do s = 1, 4
         prob_vec_local(s,j+1: j+ ub ) = prob_matrix( s, 2: ub + 1 , i )
      end do

      j = j+ ub
   end do

 !  call mpi_barrier( mpi_comm_world, err)
 !  if (myrank .eq. 0 ) print *, "just put all probabilities into a vector"

   !now collect these proability vectors to the root process to the root process
   allocate( prob_vec_global( 4, count_global ) )

!   call mpi_barrier( mpi_comm_world, err)
!   if (myrank .eq. 0 ) print *, "allocated prob_vec_global"



   call mpi_barrier( mpi_comm_world, err)


   do s = 1, 4
      call mpi_gatherv( prob_vec_local(s,:), &
                        count_local, &
                        mpi_real8, &
                        prob_vec_global(s,:), &
                        recvcounts, &
                        displs, &
                        mpi_real8, &
                        0, &
                        mpi_comm_world, &
                        err )
   end do

!   call mpi_barrier( mpi_comm_world, err)
!   if (myrank .eq. 0 ) print *, "about to get percentiles"

   !now get the percentiles
   do k = 1, n_fit_cells - 1
      percent_nodes2(k) = dble(k) / dble( n_fit_cells )
   end do

!   call mpi_barrier( mpi_comm_world, err)
!   if (myrank .eq. 0 ) print *, "got percentiles"

   do s = 1, 4
      prob_percentiles(:,s) = percent_func2( n_fit_cells-1, percent_nodes2, &
         count_global, prob_vec_global(s,: ) )
   end do


   do s = 1, 4
      do k = 1,n_fit_cells -1
         cell_ub( k, s ) = prob_percentiles( k,s )
      end do

      cell_ub( n_fit_cells, s ) = huge( cell_ub( n_fit_cells, s )  )  !useful for the do_while loop present a little bit later
   end do

   !now broadcast these upperbounds to all nodes
   call mpi_bcast( cell_ub, size(cell_ub), mpi_real8, 0, mpi_comm_world, err )

!   call mpi_barrier( mpi_comm_world, err)
!   if (myrank .eq. 0 ) print *, "about to get average predicted prob for each cell"

   !now get the average predicted probabilites and observed frequencies for each cell for each process
   cell_count_local = 0
   predict_sum_local = 0
   obs_sum_local = 0


   do i = min_obs, max_obs
      do t = 2, like_object_inout(i) % max_time
         do s = 1 ,4

            !figure out which bin to be in
            k = 1
            do while ( prob_matrix(s,t,i) > cell_ub(k,s) )
               k = k+ 1
            end do

            !increase the counter and add to the sum of predicted probabilities
            cell_count_local(k,s) = cell_count_local(k,s) + 1
            predict_sum_local(k,s) = predict_sum_local(k,s) + prob_matrix(s,t,i)

            !now figure out if the choice s was chosen by this person
            select case(s)
               case( index_both_reject )

                  if(   ( like_object_inout(i) % m_accept_new(t) .eq. 0 ) .and. &
                        ( like_object_inout(i) % accept_new(t) .eq. 0 ) ) then

                     obs_sum_local(k,s) = obs_sum_local(k,s) + 1
                  end if

               case( index_husband_accept )

                  if(   ( like_object_inout(i) % m_accept_new(t) .eq. 1 ) .and. &
                        ( like_object_inout(i) % accept_new(t) .eq. 0 ) ) then

                     obs_sum_local(k,s) = obs_sum_local(k,s) + 1
                  end if

               case( index_wife_accept )

                  if(   ( like_object_inout(i) % m_accept_new(t) .eq. 0 ) .and. &
                        ( like_object_inout(i) % accept_new(t) .eq. 1 ) ) then

                     obs_sum_local(k,s) = obs_sum_local(k,s) + 1
                  end if

               case( index_both_accept )

                  if(   ( like_object_inout(i) % m_accept_new(t) .eq. 1 ) .and. &
                        ( like_object_inout(i) % accept_new(t) .eq. 1 ) ) then

                     obs_sum_local(k,s) = obs_sum_local(k,s) + 1
                  end if

            end select

         end do
      end do
   end do

!   call mpi_barrier( mpi_comm_world, err)
!   if (myrank .eq. 0 ) print *, "about to gather across all processes"

   !now gather across all processes
   do s = 1 , 4
      do k = 1, n_fit_cells
         call mpi_reduce( cell_count_local(k,s), cell_count_global(k,s), 1, mpi_int, mpi_sum, 0, &
            mpi_comm_world, err )
         call mpi_reduce( predict_sum_local(k,s), predict_sum_global(k,s), 1, mpi_real8, mpi_sum, 0, &
            mpi_comm_world, err )
         call mpi_reduce( obs_sum_local(k,s), obs_sum_global(k,s), 1, mpi_int, mpi_sum, 0, &
            mpi_comm_world, err )

      end do
   end do



   !now finally take the average to get the output
   do s = 1 , 4
      do k = 1, n_fit_cells
         predict_avg_out(k,s) = predict_sum_global(k,s) /  cell_count_global(k,s)
         obs_freq_out(k,s) = dble(obs_sum_global(k,s) ) /  dble( cell_count_global(k,s) )

      end do
   end do

!   call mpi_barrier( mpi_comm_world, err)
!   if (myrank .eq. 0 ) print *, "done with gen_prob_fit"


contains

function percent_func2(n_percent2, percent_nodes2, n, x)
   !same a percent_fun in stats.f90, but with having the option of inputing custom percentiles
   integer, intent(in) :: n_percent2
   real*8, dimension(n_percent2), intent(in) :: percent_nodes2
   integer, intent(in) :: n
   real*8, dimension(n), intent(in) :: x
   real*8, dimension(n_percent2) :: percent_func2

   real*8, dimension(n) :: x_sorted
   integer, dimension(n_percent2) :: percent_index
   integer :: i

   x_sorted = x
   call QsortC(x_sorted)

   percent_index = nint(percent_nodes2*n)
   percent_index = max(percent_index,1) !just in case n is small and the nearest nint is 0

   do i=1,n_percent2
      percent_func2(i) = x_sorted(percent_index(i))
   end do

end function percent_func2


end subroutine gen_prob_fit


end module model_fit_mod
