
program model_fit_prog

!tests the likelihood evaluated for the whole sample

!code uses like_test_onepoint as a template

use enum_mod
use model_params_mod
use interp_support_mod

use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use likelihood_mod
use sipp_data_mod

use normal_mod
use stat_estimates_mod

use bayesian_mod
use model_fit_mod

use csv_file

!use statespace_trans_params_mod


implicit none


!  character(len=30),parameter   :: params_file  = "output_params_feb19.txt"
   integer, parameter :: arg_no_params_file = 1
   character(len=30)   :: params_file
   character(len=30),parameter :: sipp_data_file         = "sipp_fortran_big.csv"
   character(len=30),parameter :: interact_file      = "interact_matrix.csv"
   character(len=30), parameter  :: seed_file = "seed_save_sim1.txt"

   character(len=30),parameter :: value_matrix_init_file = "value_matrix_out.csv"
   character(len=30),parameter :: asset_matrix_init_file = "asset_matrix_out.csv"

   character(len=30),parameter :: prob_out_file = "model_fit_probs.csv"

   character(len=30), dimension(4) :: column_header
   character(len=30), dimension(n_fit_cells) :: row_header


   type(data_i_type), dimension(:), allocatable   :: hh_data
   type(like_type), dimension(:), allocatable   :: like_object
   type(like_target_type),dimension(:), allocatable, target :: like_target
   real*8, dimension(n_params) :: params
   real*8, dimension( n_grid_points ) :: value_matrix
   real*8, dimension( n_grid_points ) :: asset_matrix
   integer :: i,t, unit_int, k, s

   type(mygrid_type) :: mygrid_temp
   real*8, dimension(n_fit_cells,4) :: predict_avg
   real*8, dimension(n_fit_cells,4) :: obs_freq

   call mpi_setup()

   call get_command_argument(arg_no_params_file, params_file )

  ! call init_random_seed_mpi(same = .false. )
  ! call export_random_seed_mpi( seed_file )
   call read_random_seed_mpi( same = .false., filename_opt = seed_file)

   if (myrank .eq. 0 ) then

      print *, "N_obs", N_obs_total
      print *, "data file", sipp_data_file
      print *, "params file", params_file

   end if

   call read_sipp_data(sipp_data_file, hh_data)
   !print *, "read in data"



   !now read in the parameters
   call init_params(params_file, params)
   !change the discount factor for initial testing
 !  params(param_discount_factor) = .1

   call interp_setup_all( interact_file )


   call init_interp_matrices( value_matrix_init_file, asset_matrix_init_file, value_matrix, asset_matrix )

   call like_setup_all(hh_data, params, .true., value_matrix, asset_matrix, &
      like_object, like_target)

   !now get the simulated choice probabilities

 !  print *, "calling gen prob fit"

   call gen_prob_fit( hh_data, params, value_matrix, like_object, like_target, predict_avg, obs_freq )

   !now write these to a file (if root node)
   if ( myrank .eq. 0 ) then

      open( newunit = unit_int, file = prob_out_file, status = "replace" )

      !first create some headers
      column_header(1) = "both reject"
      column_header(2) = "husband accept"
      column_header(3) = "wife accept"
      column_header(4) = "both accept"

      do k = 1, n_fit_cells
         row_header(k) = "cell_" // trim(int_to_string(k))
      end do

      !now write predicted probabilities first
      call csv_write( unit_int, "predicted probs", .true. )
      call csv_write( unit_int, " ", .false. )
      do s = 1, 4
         call csv_write( unit_int, column_header(s), .false. )
      end do

      call csv_next_record( unit_int)

      do k = 1, n_fit_cells
         call csv_write( unit_int, row_header(k), .false. )

         do s = 1, 4
            call csv_write( unit_int, predict_avg( k, s ), .false. )
         end do
         call csv_next_record( unit_int)
      end do

      call csv_next_record( unit_int)

      !now the observed frequencies
      call csv_write( unit_int, "observed frequencies", .true. )
      call csv_write( unit_int, " ", .false. )
      do s = 1, 4
         call csv_write( unit_int, column_header(s), .false. )
      end do

      call csv_next_record( unit_int)

      do k = 1, n_fit_cells
         call csv_write( unit_int, row_header(k), .false. )

         do s = 1, 4
            call csv_write( unit_int, obs_freq( k, s ), .false. )
         end do
         call csv_next_record( unit_int)
      end do

      call csv_next_record( unit_int)

         print *, "done, wrote to file  ", prob_out_file

   end if !myrank .eq. 0

   call mpi_finalize(err)


end program

