

module iterate_object_mod
!module containing subroutines for an object that entials iterating the value function at one state point



use enum_mod
use model_params_mod
use interp_support_mod


use utility_function_mod
use model_probs_mod
use logit_mod

use normal_mod
use stat_estimates_mod

use csv_file
use mpi_mod

implicit none


!following type for storing a bunch of variables neede to iterate the value function


   real*8, parameter :: age_increment = 1.0/12.0
 !  character(40), parameter :: val_der_file="./val_derivative_file.txt"
 !  character(40), parameter :: choice_der_file="./choice_derivative_file.txt"
   real*8,parameter :: gh_node_n2 = .5* sqrt(2.0D+0)!gauss-hermite node for 2 points

   integer, dimension(0:1,0:1) :: n_expect_points_matrix !contains the number of expectation nodes for each point in my state space

   type :: iterate_type
      !object used for iterating the value function

      integer :: birth
      integer :: m_jobloss
      integer :: jobloss
      integer :: m_ui
      integer :: ui

      integer :: m_receive
      integer :: receive
      real*8 :: m_wage_z_score !z score for the new wage
      real*8 :: wage_z_score

      real*8, dimension(n_utility_params) :: utility_params

      type(mygrid_type) :: mygrid_new !this is also equal to the mygrid for both rejecting
      type(mygrid_type) :: mygrid_husband_accept
      type(mygrid_type) :: mygrid_wife_accept
      type(mygrid_type) :: mygrid_both_accept

      real*8 :: asset_choice_both_reject
      real*8 :: asset_choice_husband_accept
      real*8 :: asset_choice_wife_accept
      real*8 :: asset_choice_both_accept


      real*8 :: utility_both_reject
      real*8 :: utility_husband_accept
      real*8 :: utility_wife_accept
      real*8 :: utility_both_accept

      real*8 :: value_both_reject
      real*8 :: value_husband_accept
      real*8 :: value_wife_accept
      real*8 :: value_both_accept

      real*8 :: birth_prob
      real*8 :: m_jobloss_prob, jobloss_prob
      real*8 :: m_ui_prob, ui_prob
      real*8 :: ur_weight

      real*8 :: m_joboffer_prob, joboffer_prob
      real*8 :: m_hours_prob, hours_prob
      real*8 :: joboffer_weight

      real*8 :: weight
      real*8 :: emax_weighted !the emax for this expecation node multiplied by the respected weigths and probabilities

   contains
      procedure :: print => print_iterate_type
      procedure :: check => iterate_type_check

   end type





contains

subroutine print_iterate_type(iter_in)
   class(iterate_type), intent(in) :: iter_in

   print *,  'birth ', iter_in % birth
   print *,  'm_jobloss ', iter_in % m_jobloss
   print *,  'jobloss ', iter_in % jobloss
   print *,  'm_ui ', iter_in % m_ui
   print *,  'ui ', iter_in % ui
   print *, " "
   print *,  'm_receive ', iter_in % m_receive
   print *,  'receive ', iter_in % receive
   print *,  'm_wage_z_score  ', iter_in % m_wage_z_score
   print *,  'wage_z_score ', iter_in % wage_z_score
   print *, " "
   print *,  'mygrid_new '
               call iter_in % mygrid_new % print(.true.,.true.)
   print *,  'mygrid_husband_accept '
               if( iter_in % m_receive .eq. 1) call iter_in % mygrid_husband_accept % print(.true.,.true.)
   print *,  'mygrid_wife_accept '
               if( iter_in % receive .eq. 1) call iter_in % mygrid_wife_accept % print(.true.,.true.)
   print *,  'mygrid_both_accept '
               if(( iter_in % m_receive .eq. 1) .and. ( iter_in % receive .eq. 1) ) &
                  call iter_in % mygrid_both_accept % print(.true.,.true.)

   print *, " "
   print *,  'asset_choice_both_reject ', iter_in % asset_choice_both_reject
   print *, '    percentage', iter_in % asset_choice_both_reject/ &
         ( inc_func( iter_in % mygrid_new ) + iter_in % mygrid_new % rv( var_assets ) )

   if( iter_in % m_receive .eq. 1) then
      print *,  'asset_choice_husband_accept ', iter_in % asset_choice_husband_accept
      print *, '    percentage', iter_in % asset_choice_husband_accept/ &
         ( inc_func( iter_in % mygrid_husband_accept ) + iter_in % mygrid_husband_accept % rv( var_assets ) )
   end if


   if( iter_in % receive .eq. 1) then
      print *,  'asset_choice_wife_accept ', iter_in % asset_choice_wife_accept
      print *, '    percentage', iter_in % asset_choice_wife_accept/ &
         ( inc_func( iter_in % mygrid_wife_accept ) + iter_in % mygrid_wife_accept % rv( var_assets ) )
   end if

   if(( iter_in % m_receive .eq. 1) .and. ( iter_in % receive .eq. 1) ) then
      print *, 'asset_choice_both_accept ', iter_in % asset_choice_both_accept
      print *, '    percentage', iter_in % asset_choice_both_accept/ &
         ( inc_func( iter_in % mygrid_both_accept ) + iter_in % mygrid_both_accept % rv( var_assets ) )
   end if

   print *, " "
   print *,  'utility_both_reject ', iter_in % utility_both_reject
   if( iter_in % m_receive .eq. 1) print *, 'utility_husband_accept ', iter_in % utility_husband_accept
   if( iter_in % receive .eq. 1) print *, 'utility_wife_accept ', iter_in % utility_wife_accept
   if(( iter_in % m_receive .eq. 1) .and. ( iter_in % receive .eq. 1) )  &
      print *, 'utility_both_accept ', iter_in % utility_both_accept

   print *, " "
   print *,  'value_both_reject ', iter_in % value_both_reject
   if( iter_in % m_receive .eq. 1) print *,  'value_husband_accept ', iter_in % value_husband_accept
   if( iter_in % receive .eq. 1) print *, 'value_wife_accept ', iter_in % value_wife_accept
   if(( iter_in % m_receive .eq. 1) .and. ( iter_in % receive .eq. 1) )  &
      print *, 'value_both_accept ', iter_in % value_both_accept

   print *, " "
   print *,  'birth_prob ', iter_in % birth_prob
   print *,  'm_jobloss_prob ', iter_in % m_jobloss_prob
   print *,  'jobloss_prob ', iter_in % jobloss_prob
   print *,  'm_ui_prob ', iter_in % m_ui_prob
   print *,  'ui_prob ', iter_in % ui_prob
   print *,  'ur_weight ', iter_in % ur_weight

   print *, " "
   print *,  'm_joboffer_prob ', iter_in % m_joboffer_prob
   print *,  'joboffer_prob ', iter_in % joboffer_prob
   print *,  'm_hours_prob ', iter_in % m_hours_prob
   print *,  'hours_prob ', iter_in % hours_prob
   print *,  'joboffer_weight ', iter_in % joboffer_weight

   print *, " "
   print *,  'weight',  iter_in % weight
   print *,  'emax_weighted', iter_in % emax_weighted



end subroutine print_iterate_type


subroutine iterate_type_check(iter_in)
   !checks the iter type for errors
   class(iterate_type), intent(in) :: iter_in


   !first check some of the indicators

   if( .not. ((iter_in % birth .eq. 0) .or. (iter_in % birth .eq. 1)) ) then
      print *, "error with birth indicator", iter_in % birth
   end if

   if( .not. ((iter_in % m_jobloss .eq. 0) .or. (iter_in % m_jobloss .eq. 1)) ) then
      print *, "error with m_jobloss indicator", iter_in % m_jobloss
   end if

   if( .not. ((iter_in % jobloss .eq. 0) .or. (iter_in % jobloss .eq. 1)) ) then
      print *, "error with jobloss indicator", iter_in % jobloss
   end if

   if( .not. ((iter_in % m_ui .eq. 0) .or. (iter_in % m_ui .eq. 1)) ) then
      print *, "error with m_ui indicator", iter_in % m_ui
   end if

   if( .not. ((iter_in % ui .eq. 0) .or. (iter_in % ui .eq. 1)) ) then
      print *, "error with birth indicator", iter_in % ui
   end if

   if( .not. ((iter_in % m_receive .eq. 0) .or. (iter_in % m_receive .eq. 1)) ) then
      print *, "error with m_receive indicator", iter_in % m_receive
   end if

   if( .not. ((iter_in % receive .eq. 0) .or. (iter_in % receive .eq. 1)) ) then
      print *, "error with receive indicator", iter_in % receive
   end if

   if ( (iter_in % m_jobloss .eq. 0) .and. (iter_in % m_ui .eq. 1) ) then
      print *, "error with jobloss and ui indicators", iter_in % m_jobloss, iter_in % m_ui
   end if

   if ( (iter_in % jobloss .eq. 0) .and. (iter_in % ui .eq. 1) ) then
      print *, "error with jobloss and ui indicators", iter_in % jobloss, iter_in % ui
   end if


   !the z-score
   if ( nan_inf_check( iter_in % m_wage_z_score ) ) then
      print *, "error with m_wage_z_score", iter_in % m_wage_z_score
   end if

   if ( nan_inf_check( iter_in % wage_z_score ) ) then
      print *, "error with wage_z_score", iter_in % wage_z_score
   end if

   !check the mygrids
   call iter_in % mygrid_new % mygrid_error_hunt(.true., .true., 0,0,1)

   if ( iter_in % m_receive .eq. 1 ) then
      call iter_in % mygrid_husband_accept % mygrid_error_hunt(.true., .true., 0,0,2)
      call mygrid_same_check( "f",  iter_in % mygrid_new, &
                                    iter_in % mygrid_husband_accept, 0,0 )
   end if

   if ( iter_in % receive .eq. 1 ) then
      call iter_in % mygrid_wife_accept % mygrid_error_hunt(.true., .true., 0,0,3)
      call mygrid_same_check( "m",  iter_in % mygrid_new, &
                                    iter_in % mygrid_wife_accept, 0,0 )
   end if

   if ( ( iter_in % m_receive .eq. 1 ) .and. ( iter_in % receive .eq. 1 )  ) then
      call iter_in % mygrid_both_accept % mygrid_error_hunt(.true., .true., 0,0,4)
      call mygrid_same_check( "m",  iter_in % mygrid_both_accept, &
                                    iter_in % mygrid_husband_accept, 0,0 )
      call mygrid_same_check( "f",  iter_in % mygrid_both_accept, &
                                    iter_in % mygrid_wife_accept, 0,0 )

   end if



   !asset choice
   if( .not. ( ( iter_in % asset_choice_both_reject >  -zero_tol ) .and. &
               ( iter_in % asset_choice_both_reject .le. ( inc_func( iter_in % mygrid_new ) + &
                  iter_in % mygrid_new % rv( var_assets) + zero_tol )  )  ) ) then
      print *, "error with asset_choice_both_reject", iter_in % asset_choice_both_reject, &
         inc_func( iter_in % mygrid_new ) + iter_in % mygrid_new % rv( var_assets)
   end if

   if ( iter_in % m_receive .eq. 1 ) then
      if( .not. ( ( iter_in % asset_choice_husband_accept >  -zero_tol ) .and. &
                  ( iter_in % asset_choice_husband_accept < ( inc_func( iter_in % mygrid_husband_accept ) + &
                     iter_in % mygrid_husband_accept % rv( var_assets) + zero_tol )  )  ) ) then
         print *, "error with asset_choice_husband_accept", iter_in % asset_choice_husband_accept, &
            inc_func( iter_in % mygrid_husband_accept ) + iter_in % mygrid_husband_accept % rv( var_assets)
      end if
   end if

   if ( iter_in % receive .eq. 1 ) then
      if( .not. ( ( iter_in % asset_choice_wife_accept >  -zero_tol ) .and. &
                  ( iter_in % asset_choice_wife_accept < ( inc_func( iter_in % mygrid_wife_accept ) + &
                     iter_in % mygrid_wife_accept % rv( var_assets) + zero_tol )  )  ) ) then
         print *, "error with asset_choice_wife_accept", iter_in % asset_choice_wife_accept, &
            inc_func( iter_in % mygrid_wife_accept ) + iter_in % mygrid_wife_accept % rv( var_assets)
      end if
   end if

   if ( ( iter_in % m_receive .eq. 1 ) .and. ( iter_in % receive .eq. 1 )  ) then
      if( .not. ( ( iter_in % asset_choice_both_accept >  -zero_tol ) .and. &
                  ( iter_in % asset_choice_both_accept < ( inc_func( iter_in % mygrid_both_accept ) + &
                     iter_in % mygrid_both_accept % rv( var_assets) + zero_tol )  )  ) ) then
         print *, "error with asset_choice_both_accept", iter_in % asset_choice_both_accept, &
            inc_func( iter_in % mygrid_both_accept ) + iter_in % mygrid_both_accept % rv( var_assets)
      end if
   end if


   !utilities
   if ( nan_inf_check( iter_in % utility_both_reject ) ) then
      print *, "error with utility_both_reject", iter_in % utility_both_reject
   end if

   if ( iter_in % m_receive .eq. 1 ) then
      if ( nan_inf_check( iter_in % utility_husband_accept ) ) then
         print *, "error with utility_husband_accept", iter_in % utility_husband_accept
      end if
   end if

   if ( iter_in % receive .eq. 1 ) then
      if ( nan_inf_check( iter_in % utility_wife_accept ) ) then
         print *, "error with utility_wife_accept", iter_in % utility_wife_accept
      end if
   end if

   if ( ( iter_in % m_receive .eq. 1 ) .and. ( iter_in % receive .eq. 1 )  ) then
      if ( nan_inf_check( iter_in % utility_both_accept ) ) then
         print *, "error with utility_both_accept", iter_in % utility_both_accept
      end if
   end if

   !value function
   if ( nan_inf_check( iter_in % value_both_reject ) ) then
      print *, "error with value_both_reject", iter_in % value_both_reject
   end if

   if ( iter_in % m_receive .eq. 1 ) then
      if ( nan_inf_check( iter_in % value_husband_accept ) ) then
         print *, "error with value_husband_accept", iter_in % value_husband_accept
      end if
   end if

   if ( iter_in % receive .eq. 1 ) then
      if ( nan_inf_check( iter_in % value_wife_accept ) ) then
         print *, "error with value_wife_accept", iter_in % value_wife_accept
      end if
   end if

   if ( ( iter_in % m_receive .eq. 1 ) .and. ( iter_in % receive .eq. 1 )  ) then
      if ( nan_inf_check( iter_in % value_both_accept ) ) then
         print *, "error with value_both_accept", iter_in % value_both_accept
      end if
   end if


   !now for some of the probabilities
   if( .not. ( (iter_in % birth_prob >0) .and. ( iter_in % birth_prob <1+zero_tol)) ) then
         print *, "error with birth_prob", iter_in % birth_prob
   end if

   if( .not. ( (iter_in % m_jobloss_prob >0) .and. ( iter_in % m_jobloss_prob <1+zero_tol)) ) then
         print *, "error with m_jobloss_prob", iter_in % m_jobloss_prob
   end if

   if( .not. ( (iter_in % jobloss_prob >0) .and. ( iter_in % jobloss_prob <1+zero_tol)) ) then
         print *, "error with jobloss_prob", iter_in % jobloss_prob
   end if


   if( .not. ( (iter_in % birth_prob >0) .and. ( iter_in % birth_prob <1+zero_tol)) ) then
         print *, "error with birth_prob", iter_in % birth_prob
   end if

   if ( nan_inf_check( iter_in % ur_weight ) ) then
      print *, "error with ur_weight", iter_in % ur_weight
   end if

   !some more probabilities
   if( .not. ( (iter_in % m_joboffer_prob >0) .and. ( iter_in % m_joboffer_prob <1 )) ) then
      print *, "error with m_joboffer_prob", iter_in % m_joboffer_prob
   end if

   if( .not. ( (iter_in % joboffer_prob >0) .and. ( iter_in % joboffer_prob <1 )) ) then
      print *, "error with joboffer_prob", iter_in % joboffer_prob
   end if

   if( .not. ( (iter_in % m_hours_prob >0) .and. ( iter_in % m_hours_prob <1 + zero_tol )) ) then
      print *, "error with m_hours_prob", iter_in % m_hours_prob
   end if

   if( .not. ( (iter_in % hours_prob >0) .and. ( iter_in % hours_prob <1 + zero_tol )) ) then
      print *, "error with hours_prob", iter_in % hours_prob
   end if

   if ( nan_inf_check( iter_in % joboffer_weight ) ) then
      print *, "error with joboffer_weight", iter_in % joboffer_weight
   end if

   !finally, check the weighted emax
   if ( nan_inf_check( iter_in % emax_weighted ) ) then
      print *, "error with emax_weighted", iter_in % emax_weighted
   end if


end subroutine iterate_type_check



subroutine n_expect_nodes_setup()
   !sets up n_expect_points_matrix

   integer :: m_job, job

   do m_job = 0, 1
      do job = 0, 1

         n_expect_points_matrix( m_job, job ) = n_expect_points_get( m_job, job )

      end do
   end do

contains

function n_expect_points_get(m_job_in, job_in )
   !gets the number of expectatioin points I need for a given pair of job indicators
   integer, intent(in) :: m_job_in
   integer, intent(in) :: job_in
   integer :: n_expect_points_get

   integer :: birth_index,m_jobloss_ui_index, jobloss_ui_index, ur_index, &
      m_receive_index, receive_index, m_hours_index, hours_index, m_joboffer_index, joboffer_index


   n_expect_points_get = 0

   !this is the structure of the do loop from the iterate_object_setup subroutine, which I copy here to figure out how many state variables I need
   do birth_index=1,1
   do m_jobloss_ui_index=1,1+2*m_job_in
   do jobloss_ui_index=1,1+2*job_in
   do ur_index=-1,1,2
   do m_receive_index=0,1
   do receive_index=0,1

     if( (m_receive_index .eq. 1) .and. (receive_index .eq. 1) ) then

         do m_hours_index = 0,1
         do hours_index = 0,1
         do m_joboffer_index = -1,1,2
         do joboffer_index = -1,1,2

            n_expect_points_get = n_expect_points_get + 1

         end do
         end do
         end do
         end do


      else if (m_receive_index .eq. 1) then
         do m_hours_index = 0,1
         do m_joboffer_index = -1,1,2

            n_expect_points_get = n_expect_points_get + 1

         end do
         end do

      else if (receive_index .eq. 1)  then

         do hours_index = 0,1
         do joboffer_index = -1,1,2

            n_expect_points_get = n_expect_points_get + 1

         end do
         end do

      else

         n_expect_points_get = n_expect_points_get + 1

      end if

   end do
   end do
   end do
   end do
   end do
   end do




end function n_expect_points_get

end subroutine n_expect_nodes_setup



subroutine iterate_object_setup(n_expect_points_in, mygrid_in, params_in, iter_out)
   !sets of various iterate_objects for a given state point
   integer, intent(in) :: n_expect_points_in
   type(mygrid_type), intent(in) :: mygrid_in
   real*8, dimension(n_params), intent(in) :: params_in
   type(iterate_type), dimension(n_expect_points_in) , intent(out) :: iter_out

   integer :: s
   integer :: birth_index,m_jobloss_ui_index, jobloss_ui_index, ur_index, &
      m_receive_index, receive_index, m_hours_index, hours_index, m_joboffer_index, joboffer_index
   integer :: m_job_start, job_start
   real*8 :: m_hours_new, hours_new, m_non_pec_norm_new, non_pec_norm_new
   type(iterate_type) :: iter_object_save


   m_job_start = mygrid_in % iv(var_m_job)
   job_start = mygrid_in % iv(var_job)




   s=1

   do birth_index=1,1
   do m_jobloss_ui_index=1,1+2*m_job_start
   do jobloss_ui_index=1,1+2*job_start
   do ur_index=-1,1,2
   do m_receive_index=0,1
   do receive_index=0,1




      !set the both reject mygrid equal to the input mygrid
      iter_out(s) % mygrid_new = mygrid_in

      !just set hours prob equal to 1 for initialization
      iter_out(s) % m_hours_prob = 1
      iter_out(s) % hours_prob = 1

      !birth
      iter_out(s) % birth = 0
      iter_out(s) % mygrid_new % rv(var_youngest) = &
         min( iter_out(s) % mygrid_new % rv(var_youngest) +1.0/12.0, dble(18) )

      iter_out(s) % birth_prob = 1

      !husband jobloss and ui
      if ( mygrid_in % iv(var_m_job) .eq. 0 ) then !don't have job, so don't need to do anything
         iter_out(s) % m_jobloss = 0
         iter_out(s) % m_ui = 0

         iter_out(s) % m_jobloss_prob = 1
         iter_out(s) % m_ui_prob = 1

      else
         if (m_jobloss_ui_index .eq. 1) then
            iter_out(s) % m_jobloss = 0
            iter_out(s) % m_ui = 0

            iter_out(s) % m_jobloss_prob = &
               1-  m_jobloss_prob_iter_func( iter_out(s) % mygrid_new,  params_in )
            iter_out(s) % m_ui_prob = 1

         else
            iter_out(s) % m_jobloss = 1
            iter_out(s) % m_jobloss_prob = &
               m_jobloss_prob_iter_func( iter_out(s) % mygrid_new,  params_in )

            if (m_jobloss_ui_index .eq. 2) then
               iter_out(s) % m_ui = 1
               iter_out(s) % mygrid_new % rv( var_m_ui_ben ) = &
                  mygrid_in % iter( interp_m_ui_ben_pot )
               iter_out(s) % mygrid_new % iter( interp_m_ui_left )  = &
                  nint( mygrid_in % rv( var_uilength) )

               iter_out(s) % m_ui_prob = m_ui_prob_iter_func( iter_out(s) % mygrid_new , params_in )

            else
               iter_out(s) % m_ui = 0
               iter_out(s) % mygrid_new % rv( var_m_ui_ben ) = 0

               iter_out(s) % m_ui_prob = 1 - m_ui_prob_iter_func( iter_out(s) % mygrid_new , params_in )
            end if

            iter_out(s) % mygrid_new % iv( var_m_job )        = 0

            iter_out(s) % mygrid_new % rv( var_m_wage )           = 0
            iter_out(s) % mygrid_new % iv( var_m_hours )          = 0
            iter_out(s) % mygrid_new % rv( var_m_non_pec_norm )   = 0
            iter_out(s) % mygrid_new % rv( var_m_job_length )     = 0

            iter_out(s) % mygrid_new % iter( interp_m_ui_ben_pot )  = 0

            !make sure husband's ui doesn't exceed bounds
            iter_out(s) % mygrid_new % rv( var_m_ui_ben ) = bounds_func ( &
               iter_out(s) % mygrid_new % rv( var_m_ui_ben ), &
               grid_min( var_m_ui_ben), grid_max( var_m_ui_ben) )



         end if
      end if !of whether have a job

      !wife jobloss and ui
      if ( mygrid_in % iv(var_job) .eq. 0 ) then !don't have job, so don't need to do anything
         iter_out(s) % jobloss = 0
         iter_out(s) % ui = 0

         iter_out(s) % jobloss_prob = 1
         iter_out(s) % ui_prob = 1

      else
         if (jobloss_ui_index .eq. 1) then
            iter_out(s) % jobloss = 0
            iter_out(s) % ui = 0

            iter_out(s) % jobloss_prob = &
               1-  jobloss_prob_iter_func( iter_out(s) % mygrid_new,  params_in )
            iter_out(s) % ui_prob = 1

         else
            iter_out(s) % jobloss = 1
            iter_out(s) % jobloss_prob = &
               jobloss_prob_iter_func( iter_out(s) % mygrid_new,  params_in )

            if (jobloss_ui_index .eq. 2) then
               iter_out(s) % ui = 1
               iter_out(s) % mygrid_new % rv( var_ui_ben ) = &
                  mygrid_in % iter( interp_ui_ben_pot )
               iter_out(s) % mygrid_new % iter( interp_ui_left )  = &
                  nint( mygrid_in % rv( var_uilength) )


               iter_out(s) % ui_prob = ui_prob_iter_func( iter_out(s) % mygrid_new , params_in )
            else
               iter_out(s) % ui = 0
               iter_out(s) % mygrid_new % rv( var_ui_ben ) = 0

               iter_out(s) % ui_prob = 1 - ui_prob_iter_func( iter_out(s) % mygrid_new , params_in )
            end if

            iter_out(s) % mygrid_new % iv( var_job )        = 0

            iter_out(s) % mygrid_new % rv( var_wage )           = 0
            iter_out(s) % mygrid_new % iv( var_hours )          = 0
            iter_out(s) % mygrid_new % rv( var_non_pec_norm )   = 0
            iter_out(s) % mygrid_new % rv( var_job_length )     = 0

            iter_out(s) % mygrid_new % iter( interp_ui_ben_pot )  = 0


            !make sure wife's ui doesn't exceed bounds
            iter_out(s) % mygrid_new % rv( var_ui_ben ) = bounds_func ( &
               iter_out(s) % mygrid_new % rv( var_ui_ben ), &
               grid_min( var_ui_ben), grid_max( var_ui_ben) )


         end if
      end if !of whether have a job


      !unemployment rate
      iter_out(s) % mygrid_new % rv(var_ur_state) = params_in(param_state_ur_cons) + &
      iter_out(s) % mygrid_new % rv(var_ur_state) * params_in(param_state_ur_on_state_ur) + &
         params_in(param_state_ur_sd) * gh_node_n2 * ur_index

      !make sure ur doesn't go past bounds
      iter_out(s) % mygrid_new % rv(var_ur_state) = bounds_func( &
         iter_out(s) % mygrid_new % rv(var_ur_state) , &
         grid_min( var_ur_state), grid_max( var_ur_state) )


      iter_out(s) % ur_weight = .5


      !incorporate assets growing
      iter_out(s) % mygrid_new % rv(var_assets) = &
         iter_out(s) % mygrid_new % rv(var_assets) * (1 + params_in( param_interest_rate) )

      !make sure assets don't go past bounds
      iter_out(s) % mygrid_new % rv(var_assets) = bounds_func( &
         iter_out(s) % mygrid_new % rv(var_assets) , &
         grid_min( var_assets) , grid_max( var_assets) )



      !increment the appropriate length variables
      if (iter_out(s) % mygrid_new % iv( var_m_job ) .eq. 1 ) then
         iter_out(s) % mygrid_new % rv( var_m_job_length ) = &
         min(  iter_out(s) % mygrid_new % rv( var_m_job_length ) + 1, grid_max( var_m_job_length) )
      else
         iter_out(s) % mygrid_new % rv( var_m_unemp_length ) = &
         min(  iter_out(s) % mygrid_new % rv( var_m_unemp_length ) + 1, grid_max( var_m_unemp_length) )

         if ( iter_out(s) % mygrid_new % rv( var_m_ui_ben ) > zero_tol ) then
            iter_out(s) % mygrid_new % iter( interp_m_ui_left ) = &
               nint( iter_out(s) % mygrid_new % iter( interp_m_ui_left ) - 1 )
         end if


      end if


      if (iter_out(s) % mygrid_new % iv( var_job ) .eq. 1 ) then
         iter_out(s) % mygrid_new % rv( var_job_length ) = &
         min(  iter_out(s) % mygrid_new % rv( var_job_length ) + 1, grid_max( var_job_length) )
      else
         iter_out(s) % mygrid_new % rv( var_unemp_length ) = &
         min(  iter_out(s) % mygrid_new % rv( var_unemp_length ) + 1, grid_max( var_unemp_length)  )

         if ( iter_out(s) % mygrid_new % rv( var_ui_ben ) > zero_tol ) then
            iter_out(s) % mygrid_new % iter( interp_ui_left ) = &
               nint( iter_out(s) % mygrid_new % iter( interp_ui_left ) - 1 )
         end if

      end if

      !set ui to 0 if have exceded max duration
      if (  nint( iter_out(s) % mygrid_new % iter( interp_m_ui_left ) ) .eq. - 1 ) then

         iter_out(s) % mygrid_new % rv(var_m_ui_ben) = 0
         iter_out(s) % mygrid_new % iter(interp_m_ui_left) = 0
      end if

      if (  nint( iter_out(s) % mygrid_new % iter( interp_ui_left ) ) .eq. - 1 ) then

         iter_out(s) % mygrid_new % rv(var_ui_ben) = 0
         iter_out(s) % mygrid_new % iter(interp_ui_left) = 0
      end if


      !set ui to 0 if replacement rate set to 0
      if (  replacement_rate_param < zero_tol ) then
            iter_out(s) % mygrid_new % rv(var_m_ui_ben) = 0
            iter_out(s) % mygrid_new % rv(var_ui_ben) = 0
      end if


      !store the receive indexes
      iter_out(s) % m_receive = m_receive_index
      iter_out(s) % receive   = receive_index

      !save the iterate object that I have so far before I loop throught the joboffer variables
      iter_object_save = iter_out(s)

      !now for joboffer receipts
      if( (m_receive_index .eq. 1) .and. (receive_index .eq. 1) ) then

         do m_hours_index = 0,1
         do hours_index = 0,1
         do m_joboffer_index = -1,1,2
         do joboffer_index = -1,1,2

            !copy over the saved object
            iter_out(s) = iter_object_save

            iter_out(s) % joboffer_weight = .25

            if (m_hours_index .eq. 1) then
               m_hours_new = 40
            else
               m_hours_new = 20
            end if

            if (hours_index .eq. 1) then
               hours_new = 40
            else
               hours_new = 20
            end if

            m_non_pec_norm_new = m_joboffer_index * gh_node_n2
            non_pec_norm_new = joboffer_index * gh_node_n2
            iter_out(s) % m_wage_z_score = m_joboffer_index * gh_node_n2
            iter_out(s) % wage_z_score = joboffer_index * gh_node_n2

            !now put these new variables on the appropriate object
            !husband accept grid
            iter_out(s) % mygrid_husband_accept = iter_out(s) % mygrid_new

            iter_out(s) % mygrid_husband_accept % iv( var_m_job )             = 1
            iter_out(s) % mygrid_husband_accept % rv( var_m_non_pec_norm )    = m_non_pec_norm_new
            iter_out(s) % mygrid_husband_accept % iv( var_m_hours )           = m_hours_new
            iter_out(s) % mygrid_husband_accept % rv( var_m_ui_ben )          = 0

            iter_out(s) % mygrid_husband_accept % rv( var_m_job_length )      = 0
            iter_out(s) % mygrid_husband_accept % rv( var_m_unemp_length )    = 0

            iter_out(s) % mygrid_husband_accept % iter( interp_m_ui_left )    = 0

            !!!!!!

            !wife accept grid
            iter_out(s) % mygrid_wife_accept = iter_out(s) % mygrid_new

            iter_out(s) % mygrid_wife_accept % iv( var_job )            = 1
            iter_out(s) % mygrid_wife_accept % rv( var_non_pec_norm )   = non_pec_norm_new
            iter_out(s) % mygrid_wife_accept % iv( var_hours )          = hours_new
            iter_out(s) % mygrid_wife_accept % rv( var_ui_ben )         = 0

            iter_out(s) % mygrid_wife_accept % rv( var_job_length )     = 0
            iter_out(s) % mygrid_wife_accept % rv( var_unemp_length )   = 0

            iter_out(s) % mygrid_wife_accept % iter( interp_ui_left )   = 0



            !!!!!!!!!

            !both accept grid
            iter_out(s) % mygrid_both_accept = iter_out(s) % mygrid_new

            iter_out(s) % mygrid_both_accept % iv( var_m_job )             = 1
            iter_out(s) % mygrid_both_accept % rv( var_m_non_pec_norm )    = m_non_pec_norm_new
            iter_out(s) % mygrid_both_accept % iv( var_m_hours )           = m_hours_new
            iter_out(s) % mygrid_both_accept % rv( var_m_ui_ben )          = 0

            iter_out(s) % mygrid_both_accept % rv( var_m_job_length )      = 0
            iter_out(s) % mygrid_both_accept % rv( var_m_unemp_length )    = 0

            iter_out(s) % mygrid_both_accept % iv( var_job )               = 1
            iter_out(s) % mygrid_both_accept % rv( var_non_pec_norm )      = non_pec_norm_new
            iter_out(s) % mygrid_both_accept % iv( var_hours )             = hours_new
            iter_out(s) % mygrid_both_accept % rv( var_ui_ben )            = 0

            iter_out(s) % mygrid_both_accept % rv( var_job_length )        = 0
            iter_out(s) % mygrid_both_accept % rv( var_unemp_length )      = 0

            iter_out(s) % mygrid_both_accept % iter( interp_m_ui_left )    = 0
            iter_out(s) % mygrid_both_accept % iter( interp_ui_left )      = 0



            !finally, inrease the state variable index I'm dealing wiht
            s=s+1

         end do
         end do
         end do
         end do

      else if (m_receive_index .eq. 1) then
         do m_hours_index = 0,1
         do m_joboffer_index = -1,1,2

            !copy over the saved object
            iter_out(s) = iter_object_save

            iter_out(s) % joboffer_weight = .5

            if (m_hours_index .eq. 1) then
               m_hours_new = 40
            else
               m_hours_new = 20
            end if

            m_non_pec_norm_new = m_joboffer_index * gh_node_n2
            iter_out(s) % m_wage_z_score = m_joboffer_index * gh_node_n2




            !now put these new variables on the appropriate object
            !husband accept grid
            iter_out(s) % mygrid_husband_accept = iter_out(s) % mygrid_new

            iter_out(s) % mygrid_husband_accept % iv( var_m_job )             = 1
            iter_out(s) % mygrid_husband_accept % rv( var_m_non_pec_norm )    = m_non_pec_norm_new
            iter_out(s) % mygrid_husband_accept % iv( var_m_hours )           = m_hours_new
            iter_out(s) % mygrid_husband_accept % rv( var_m_ui_ben )          = 0

            iter_out(s) % mygrid_husband_accept % rv( var_m_job_length )      = 0
            iter_out(s) % mygrid_husband_accept % rv( var_m_unemp_length )    = 0

            iter_out(s) % mygrid_husband_accept % iter( interp_m_ui_left )    = 0



            !finally, inrease the state variable index I'm dealing wiht
            s=s+1

         end do
         end do

      else if (receive_index .eq. 1)  then

         do hours_index = 0,1
         do joboffer_index = -1,1,2

            !copy over the saved object
            iter_out(s) = iter_object_save


            iter_out(s) % joboffer_weight = .5

            if (hours_index .eq. 1) then
               hours_new = 40
            else
               hours_new = 20
            end if

            non_pec_norm_new = joboffer_index * gh_node_n2
            iter_out(s) % wage_z_score = joboffer_index * gh_node_n2


            !now put these new variables on the appropriate object

            !wife accept grid
            iter_out(s) % mygrid_wife_accept = iter_out(s) % mygrid_new

            iter_out(s) % mygrid_wife_accept % iv( var_job )            = 1
            iter_out(s) % mygrid_wife_accept % rv( var_non_pec_norm )   = non_pec_norm_new
            iter_out(s) % mygrid_wife_accept % iv( var_hours )          = hours_new
            iter_out(s) % mygrid_wife_accept % rv( var_ui_ben )         = 0

            iter_out(s) % mygrid_wife_accept % rv( var_job_length )     = 0
            iter_out(s) % mygrid_wife_accept % rv( var_unemp_length )   = 0

            iter_out(s) % mygrid_wife_accept % iter( interp_ui_left )   = 0


            !finally, inrease the state variable index I'm dealing wiht
            s=s+1

         end do
         end do
      else


         iter_out(s) % joboffer_weight = 1

         s=s+1
      end if



   end do !receive
   end do !m_receive
   end do !ur_index
   end do !jobloss
   end do !m_jobloss
   end do !birth

 !  print *, "s", s

end subroutine iterate_object_setup



end module iterate_object_mod


module value_grid_object_mod
   !module for an object that helps iterate the value function at a bunch of state points


use enum_mod
use model_params_mod
use interp_support_mod

use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod

use normal_mod
use stat_estimates_mod

use csv_file
use mpi_mod



   integer :: state_interval_length !how many points for each process (expect for possibly the last node in case n_grid_points/nprocs isnt' an integer)

   type :: value_grid_type
      !object for iterating the value function for a variety of states points with the same (m_job,job) grid
      logical :: grid_allocated = .false.


      integer :: n_state_points !number of state points, this is a local variable in terms of parallelization
      integer :: min_state_point
      integer :: max_state_point
      integer, dimension(:), allocatable :: n_expect_points !number of expectation points for a given state point, which varies depending on what the initial employment categories are


      real*8, dimension(:,:), allocatable :: utility_params_start !used for constructing the asset interpolation function
      type(mygrid_type), dimension(:), allocatable :: mygrid_start !state points of interests before random events (jobloss, unemp rate changes), as well as for the asset interpolation function
      type(iterate_type), dimension(:), allocatable :: iter_obj

      real*8, dimension(:), allocatable :: stored_value !stores the value function
      real*8, dimension(:), allocatable :: stored_asset_choice !stores the asset choice !transformed

   contains
      procedure :: check => value_grid_check
      procedure :: setup => value_grid_setup

   end type


contains





subroutine value_grid_check(value_grid_in)
   !checks the value grid for errors
   class(value_grid_type), intent(in) :: value_grid_in
   integer :: s,i,j,k

   real*8 :: weight_sum = 0



   print *, "checking the mygrid_start objects"

   do i= 1, value_grid_in % n_state_points
  !    print *, i
     call value_grid_in % mygrid_start(i) % mygrid_error_hunt(.true., .true., 0,0,0)
   end do




   k = 1

   do i=  value_grid_in % min_state_point, value_grid_in % max_state_point
      do s= 1, value_grid_in % n_expect_points(i)

          ! print *, "state point", i, "expect point", s


           ! call value_grid_in % iter_obj(k) % print()
            call value_grid_in % iter_obj(k) % check()

          !  read(*,*)


         if ( value_grid_in % iter_obj(k) % weight < 0 ) then
            print *, "problem with weight for this one being less than 0"
            call value_grid_in % iter_obj(k) % print()
            read(*,*)
         end if


!           print *, "state point", i, "expect point", s

!            do j=1,n_cont_vars
!               if (var_present(j,0,0) .eq. 1) then
!               print *, var_labels(j), ( value_grid_in % iter_obj(s,i) % mygrid_new % rv(j) - &
!                  grid_mean(j) ) * grid_sd_inv(j)
!               end if
!            end do

!            print *, "mygrid start"
!            call value_grid_in % mygrid_start(i) % print( .true., .true. )

!            stop
!         end if

      !   read(*,*)

         k = k + 1
      end do
   end do

   !check to see if probabilities add up to one
   k = 1
   do i= value_grid_in % min_state_point, value_grid_in % max_state_point
      weight_sum = 0

      do s = 1, value_grid_in % n_expect_points(i)
         weight_sum = weight_sum + &
            value_grid_in % iter_obj(k) % weight

         k = k + 1
      end do

      if( abs(weight_sum - 1.0) > zero_tol ) then
         print *, "error with expect weight for i=", i
         print *, "     ", weight_sum
      end if

   end do




end subroutine value_grid_check





subroutine value_grid_setup( value_grid_out, params_in )
   !sets up iter_obj in the value_grid_object
   class(value_grid_type), intent(out) :: value_grid_out
   real*8,dimension(n_params), intent(in) :: params_in

   real*8, dimension(n_interp_vars, n_grid_points) :: grid_points
   integer :: ratio_temp, min_point, max_point
   integer :: i, k, m_job, job, expect_sum, n_temp


   if ( interp_setup_logical .eqv. .false. ) then
      print *, "interp_setup_logical need to be true, stopping"
      stop
   end if

   call n_expect_nodes_setup


   value_grid_out % grid_allocated = .true.


   ratio_temp = ceiling(dble(n_grid_points)/ dble(nprocs) )
   state_interval_length = ratio_temp
   min_point = ratio_temp*myrank + 1
   max_point = min( ratio_temp*(myrank+1), n_grid_points )

   value_grid_out % n_state_points = max_point - min_point + 1
   value_grid_out % min_state_point = min_point
   value_grid_out % max_state_point = max_point






   !now do some allocation
   allocate( value_grid_out % n_expect_points ( min_point : max_point ) )
   allocate( value_grid_out % utility_params_start ( n_utility_params, min_point : max_point ) )
   allocate( value_grid_out % mygrid_start ( min_point : max_point ) )


   allocate( value_grid_out % stored_value( min_point : max_point ) )
   allocate( value_grid_out % stored_asset_choice( min_point : max_point ) )


   !get the points on the hypercube
   call gen_grid_points( grid_points ) !all these points are on the -1,1 hypercube



   !now convert these to the mygrid object and get the number of expectation points
   expect_sum = 0

   do i = min_point, max_point
      call interp_to_mygrid( grid_points(:,i) , .true. , value_grid_out % mygrid_start(i) )

      m_job = value_grid_out % mygrid_start(i) % iv (var_m_job)
      job = value_grid_out % mygrid_start(i) % iv (var_job)

      value_grid_out % n_expect_points(i) = n_expect_points_matrix( m_job, job )

      expect_sum = expect_sum + value_grid_out % n_expect_points(i)


   end do

   !now I can setup the iter_obj
   allocate( value_grid_out % iter_obj( expect_sum ) )
   k = 0

   do i = min_point, max_point
      n_temp = value_grid_out % n_expect_points(i)

      call iterate_object_setup( n_temp, &
                                 value_grid_out % mygrid_start(i), &
                                 params_in, &
                                 value_grid_out % iter_obj(k + 1: k+n_temp ) )

      k = k + n_temp

   end do




end subroutine value_grid_setup


end module value_grid_object_mod


module value_func_mod
!module for evaluating the value function and maximization it + utility function wrt assets
   !also contains functions related to the asset policy funciton

use enum_mod
use model_params_mod
use interp_support_mod

use utility_function_mod
use model_probs_mod


use iterate_object_mod
use value_grid_object_mod

use logit_mod
use normal_mod
use stat_estimates_mod

use brent_mod
use csv_file



   real*8, parameter :: eps_brent = 1.0D-6 !error tolerance for the brent minimization routine
   real*8, parameter :: t_brent = 1.0D-6 !error tolerance for the brent minimization routine
   integer, parameter :: max_iter_brent = 1000



contains


function value_function( mygrid_in, assets_in, value_matrix_in )
   !gets the value function for my model

   real*8 :: value_function
   type(mygrid_type), intent(in) :: mygrid_in
   real*8, intent(in) :: assets_in
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in

   type(mygrid_type) :: mygrid_temp
   real*8, dimension(n_interp_vars) :: cube_point



   mygrid_temp = mygrid_in
   mygrid_temp % rv( var_assets ) = assets_in

   call mygrid_to_interp( mygrid_temp, cube_point )


   !now get the value
 !  value_function = interp_func( mygrid_temp, value_matrix_in(:, m_job, job) )

   value_function = interp_func( cube_point, value_matrix_in )



end function value_function




function value_function_split_in( mygrid_t_invar_in, mygrid_t_var_in, assets_in, &
   value_matrix_in ) result(value_function)
   !gets the value function for evaluation the likelihood when the time variant and in-variant variables
      !are split into two objects

   real*8 :: value_function
   type(mygrid_type), intent(in) :: mygrid_t_invar_in
   type(mygrid_type), intent(in) :: mygrid_t_var_in
   real*8, intent(in) :: assets_in
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in

   type(mygrid_type) :: mygrid_temp
   real*8, dimension(n_interp_vars) :: cube_point
  ! integer :: m_job, job

   !make some of the key integers easier to deal with
  ! m_job = mygrid_t_var_in % iv( var_m_job )
  ! job   = mygrid_t_var_in % iv( var_job )


   !setup the mygrid for interpolation
   call mygrid_temp % allocate()
   mygrid_temp % rv(:n_t_invar)   = mygrid_t_invar_in % rv(:n_t_invar)
   mygrid_temp % rv(n_t_invar+1:) = mygrid_t_var_in   % rv(n_t_invar+1:)
   mygrid_temp % iv = mygrid_t_var_in % iv
   mygrid_temp % rv( var_assets ) = assets_in


   call mygrid_to_interp( mygrid_temp, cube_point )

   !now get the value
 !  value_function = interp_func( mygrid_temp, value_matrix_in( :,m_job, job) )
   value_function = interp_func( cube_point, value_matrix_in )



end function value_function_split_in






function asset_policy_func(mygrid_in, asset_matrix_in )
   !policy function for next period assets, given state variables

   real*8 :: asset_policy_func
   type(mygrid_type), intent(in) :: mygrid_in
   real*8, dimension(n_grid_points), intent(in) :: asset_matrix_in

   real*8, dimension(n_interp_vars) :: cube_point

   call mygrid_to_interp( mygrid_in, cube_point )


   asset_policy_func = interp_func( cube_point, asset_matrix_in )

   !now transform into the actual choice
   asset_policy_func = asset_interp_transform( mygrid_in, asset_policy_func )

   !aug 7, put inbetween bounds
   asset_policy_func = bounds_func( &
         asset_policy_func , &
         grid_min( var_assets) , grid_max( var_assets) )


end function asset_policy_func


function asset_policy_func_split_in( mygrid_t_invar_in, mygrid_t_var_in, &
   asset_matrix_in ) result(asset_policy_func )

   real*8 :: asset_policy_func
   type(mygrid_type), intent(in) :: mygrid_t_invar_in
   type(mygrid_type), intent(in) :: mygrid_t_var_in
   real*8, dimension(n_grid_points), intent(in) :: asset_matrix_in

   type(mygrid_type) :: mygrid_temp
   real*8, dimension(n_interp_vars) :: cube_point


   !setup the mygrid for interpolation
   call mygrid_temp % allocate()
   mygrid_temp % rv(:n_t_invar)   = mygrid_t_invar_in % rv(:n_t_invar)
   mygrid_temp % rv(n_t_invar+1:) = mygrid_t_var_in   % rv(n_t_invar+1:)
   mygrid_temp % iv = mygrid_t_var_in % iv


   !now get the interp_value
   call mygrid_to_interp( mygrid_temp, cube_point )

   asset_policy_func = interp_func( cube_point, asset_matrix_in )

  ! print *, "asset_func_pretransform", asset_policy_func

   !now get the actual asset choice
   asset_policy_func = asset_interp_transform( mygrid_temp, asset_policy_func )


   !aug 7, put inbetween bounds
   asset_policy_func = bounds_func( &
         asset_policy_func , &
         grid_min( var_assets) , grid_max( var_assets) )


  ! print *, "asset_func_posttransform", asset_policy_func


end function asset_policy_func_split_in

function asset_choice_transform( mygrid_in, asset_choice_in )
   !transforms the asset choice into a point in which I use as a bases for constructing an ols interpolation function
   real*8 :: asset_choice_transform
   type(mygrid_type), intent(in) :: mygrid_in
   real*8, intent(in) :: asset_choice_in


   asset_choice_transform = asset_choice_in / ( inc_func( mygrid_in ) +  mygrid_in % rv( var_assets) )

   asset_choice_transform = bounds_func( asset_choice_transform, 1.0D-8, 1.0D+0 - 1.0D-8)

   asset_choice_transform = inv_logit_prob( asset_choice_transform )

end function asset_choice_transform

function asset_interp_transform( mygrid_in, assets_interp_in )
   !transforms the value gotten from the asset choice interpolation into the actual asset choice
   real*8 :: asset_interp_transform
   type(mygrid_type), intent(in) :: mygrid_in
   real*8, intent(in) :: assets_interp_in

   asset_interp_transform = logit_prob( assets_interp_in )
   asset_interp_transform = asset_interp_transform * &
      ( inc_func( mygrid_in ) + mygrid_in % rv( var_assets) )

end function asset_interp_transform


subroutine max_value_sub( mygrid_in, utility_params_in, params_in, value_matrix_in, &
   asset_choice_out, max_value_out )
   !gets the value of assets that maximizes utility + value function
   !and return the max_value as well if it's given
   type( mygrid_type ), intent(in) :: mygrid_in
   real*8, dimension(n_utility_params), intent(in) :: utility_params_in
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
   real*8, intent(out) :: asset_choice_out
   real*8, intent(out),optional :: max_value_out

  ! real*8, dimension(n_asset_nodes + 1) :: coef_assets
   procedure(func_template_one_arg), pointer :: func_to_min_pointer
   real*8, dimension(n_coef_asset_max) :: coef_asset_max
   real*8, dimension(n_interp_vars) :: cube_point
   real*8 :: local_min_value
   real*8 :: up
   real*8 :: lb

   integer :: i

   !set up the interpolation coefficients used in the maximization function
   call mygrid_to_interp( mygrid_in, cube_point )
   call gen_coef_asset_max( cube_point, value_matrix_in, coef_asset_max )


   func_to_min_pointer => func_to_min

  ! up= inc_func( mygrid_in ) + mygrid_in % rv( var_assets)
  ! lb = params_in ( param_asset_lb  )

   up = min( inc_func( mygrid_in ) + mygrid_in % rv( var_assets), grid_max( var_assets ) )
   lb = grid_min( var_assets )


 !  print *, "value for very low cons", -func_to_min( up - 5 )
 !  print *, "value for very high cons", -func_to_min( zero_tol*100 )


   call brent_local_min (  lb, &
                           up, &
                           eps_brent, &
                           t_brent, &
                           max_iter_brent, &
                           func_to_min_pointer, &
                           asset_choice_out, &
                           local_min_value )

 !  print *, "optimal value", -local_min_value

   !put between bounds
   !aug 7, put inbetween bounds
  ! asset_choice_out = bounds_func( &
  !       asset_choice_out , &
  !       grid_min( var_assets) , grid_max( var_assets) )

   !to get value funciton, re-evaluate the func_to_min in case it changes with asset_choice
   if (present (max_value_out) ) then
   !   max_value_out = - func_to_min( asset_choice_out )

      !sept 24 changes the if slightly because I incorporate bounds earlier
      max_value_out = - local_min_value

   end if




contains

function func_to_min(assets_next_in)
!the function that needs to be minimized in order to find the optimal value of assets
!in order to make the function of interest conform with the interface for the brent min function, I have assets be the only input
   real*8 :: func_to_min
   real*8, intent(in) :: assets_next_in

   real*8 :: util_temp, val_temp, assets_cube

   !utility
   util_temp = utility_function( mygrid_in, assets_next_in, utility_params_in, params_in)



   !now the value function
 !  val_temp = value_function( mygrid_in, assets_next_in, value_matrix_in )

   !first convert to the hypercube
   if ( assets_next_in < grid_mean( var_assets ) ) then
      assets_cube = ( assets_next_in - grid_mean(var_assets) ) * grid_sd_inv( var_assets )
   else
      assets_cube = ( assets_next_in - grid_mean( var_assets) ) * grid_sd_asset_inv2
   end if

   !now all the coefficients

   !intercept
   val_temp = coef_asset_max(1)

   !the 1-d points
   val_temp = val_temp + interp_asset_func( assets_cube, coef_asset_max(2:1+n_asset_nodes) )

  ! now the interaction term
   val_temp = val_temp + coef_asset_max(n_coef_asset_max)*assets_cube


   func_to_min =  - util_temp - val_temp

end function func_to_min

end subroutine max_value_sub

subroutine max_value_sub_split( mygrid_t_invar_in, mygrid_t_var_in, &
   utility_params_in, params_in, value_matrix_in, &
   asset_choice_out, max_value_out )

   !NOTE: I haven't fully debugged this subroutine, but that shouldn't be too big of a deal

   type( mygrid_type ), intent(in) :: mygrid_t_invar_in
   type( mygrid_type ), intent(in) :: mygrid_t_var_in
   real*8, dimension(n_utility_params), intent(in) :: utility_params_in
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
   real*8, intent(out) :: asset_choice_out
   real*8, intent(out),optional :: max_value_out

   type(mygrid_type) :: mygrid_temp

   !setup the mygrid_temp which will be used as an input
   call mygrid_temp % allocate()
   mygrid_temp % rv (:n_t_invar) = mygrid_t_invar_in % rv(:n_t_invar)

   mygrid_temp % iv = mygrid_t_var_in % iv
   mygrid_temp % rv( n_t_invar + 1 : ) = mygrid_t_var_in % rv( n_t_invar + 1 : )

   !now call the main maximizing program
   call max_value_sub( mygrid_temp, utility_params_in, params_in, value_matrix_in, &
   asset_choice_out, max_value_out )


end subroutine max_value_sub_split


subroutine gen_value_matrix ( value_grid_in, value_matrix_out )
   !gens my interpolation matrix for value functions

   type( value_grid_type ), intent(in) :: value_grid_in
   real*8, dimension(n_grid_points), intent(out) :: value_matrix_out


   real*8, dimension(state_interval_length):: value_vector_small
   real*8, dimension(state_interval_length*nprocs) :: value_vector_large
      !note that state_interval_length*nprocs may be greater than n_grid_points
   integer :: lb, ub, i

   !first put the stored values from node into a vector

   value_vector_small = 0

   lb = value_grid_in % min_state_point
   ub = value_grid_in % max_state_point

   value_vector_small( 1 : ub - lb + 1 ) = value_grid_in % stored_value(lb:ub)


   !now combine these into a larger vector

   call mpi_gather( value_vector_small, size(value_vector_small), mpi_real8, value_vector_large, &
      size(value_vector_small), mpi_real8, 0, mpi_comm_world, err )


   !now generate the interpolation coefficients from this larger vector
   if (myrank .eq. 0 ) then

      call gen_interp_coef( value_vector_large(:n_grid_points), value_matrix_out )

   end if

   !now broadcast this matrix to other nodes
   call mpi_bcast( value_matrix_out, size(value_matrix_out), mpi_real8, 0, mpi_comm_world, err )

end subroutine gen_value_matrix


subroutine gen_asset_matrix ( value_grid_in, asset_matrix_out )
   !gens my interpolation matrix for asset functions

   type( value_grid_type ), intent(in) :: value_grid_in
   real*8, dimension(n_grid_points), intent(out) :: asset_matrix_out


   real*8, dimension(state_interval_length):: asset_vector_small
   real*8, dimension(state_interval_length*nprocs) :: asset_vector_large
   integer :: lb, ub

   !first put the stored assets from node into a vector

   asset_vector_small = 0

   lb = value_grid_in % min_state_point
   ub = value_grid_in % max_state_point

   asset_vector_small( 1 : ub - lb + 1 ) = value_grid_in % stored_asset_choice(lb:ub)


   !now combine these into a larger vector

   call mpi_gather( asset_vector_small, size(asset_vector_small), mpi_real8, asset_vector_large, &
      size(asset_vector_small), mpi_real8, 0, mpi_comm_world, err )


   !now generate the interpolation coefficients from this larger vector
   if (myrank .eq. 0 ) then

      call gen_interp_coef(asset_vector_large(:n_grid_points), asset_matrix_out )

   end if

   !now broadcast this matrix to other nodes
   call mpi_bcast( asset_matrix_out, size(asset_matrix_out), mpi_real8, 0, mpi_comm_world, err )

end subroutine gen_asset_matrix



subroutine interp_matrix_write(interp_matrix_in,unit_in)
   !writes an interpolation matrix or a grid of values to a file
   real*8, dimension(n_grid_points), intent(in) :: interp_matrix_in
   integer, intent(in) :: unit_in

   integer :: i, j,k,v,s
   character(len=1) :: char_int


   do i = 1, n_grid_points
      call csv_write( unit_in, trim( grid_labels(i) ), .false. )
      call csv_write( unit_in, interp_matrix_in(i), .true. )

   end do




end subroutine interp_matrix_write



subroutine interp_matrix_read( interp_matrix_out,unit_in)
   !reads a interp matrix from a file
   real*8, dimension(n_grid_points), intent(out) :: interp_matrix_out
   integer, intent(in) :: unit_in

   character(len=30) :: char_ignore
   integer ::  i

!   print *, "n_grid_points", n_grid_points

   do i=1 , n_grid_points
      read(unit_in, *) char_ignore, interp_matrix_out(i)
   end do


end subroutine interp_matrix_read


subroutine grid_value_write( char_in, value_grid_in, unit_in )
   !writes the grid values to a file
   !this can be for either the value function or assets
   character(len=1), intent(in) :: char_in !declares whether to write values or assets
   type(value_grid_type), intent(in) :: value_grid_in
   integer, intent(in) :: unit_in

   real*8, dimension(state_interval_length) :: grid_vec_small
   real*8, dimension(state_interval_length*nprocs) :: grid_vec_large
   integer ::  lb, ub


   print *, "the subrouitne has some bugs that I haven't fixed yet, stopping"
   stop

   grid_vec_small = 0


   lb = value_grid_in % min_state_point
   ub = value_grid_in % max_state_point

   if (char_in .eq. "a") then
      grid_vec_small( 1 : ub-lb+1 ) = value_grid_in % stored_asset_choice(lb : ub)
   else if (char_in .eq. "v") then
      grid_vec_small( 1 : ub-lb+1 ) = value_grid_in % stored_value(lb : ub)
   else
      print *, "error with grid_value_write"
      stop
   end if


   !now combine into larger vector
   !2/24/14, I don't think this is the right way of collectign all these vectors
   call mpi_gather( grid_vec_small, size(grid_vec_small), mpi_real8, grid_vec_large, &
      size(grid_vec_small), mpi_real8, 0, mpi_comm_world, err )

   !now generate the interpolation coefficients from this larger vector
   if (myrank .eq. 0 ) then

      call interp_matrix_write( grid_vec_large(:n_grid_points), unit_in)

   end if



end subroutine grid_value_write


subroutine init_interp_matrices( value_file_in, asset_file_in, value_matrix_out, asset_matrix_out )

   character(len=30), intent(in) :: value_file_in
   character(len=30), intent(in) :: asset_file_in
   real*8, dimension( n_grid_points ) :: value_matrix_out
   real*8, dimension( n_grid_points ) :: asset_matrix_out

   integer :: unit_int
   logical :: file_exists



   inquire(file = value_file_in, exist=file_exists)
   if (.not. file_exists) then
      print *, "file  ", value_file_in, "not present, stopping"
      stop
   end if

   inquire(file = asset_file_in, exist=file_exists)
   if (.not. file_exists) then
      print *, "file  ", asset_file_in, "not present, stopping"
      stop
   end if

   open(newunit = unit_int , file = value_file_in, status="old" )
   call interp_matrix_read( value_matrix_out, unit_int )
   close( unit_int )

   open(newunit = unit_int , file = asset_file_in, status ="old" )
   call interp_matrix_read( asset_matrix_out, unit_int )
   close( unit_int )



end subroutine init_interp_matrices



end module value_func_mod






module iterate_mod


use enum_mod
use model_params_mod
use interp_support_mod

use utility_function_mod
use model_probs_mod


use iterate_object_mod
use value_grid_object_mod
use value_func_mod

use logit_mod
use normal_mod
use stat_estimates_mod

use csv_file

use, intrinsic :: iso_fortran_env !gives standard in, out and error

   logical, parameter :: print_iter = .true. !controls whether to print stuff during the iteration routines
 !  logical :: print_test_global = .false. !2/17/16, this prints out a bunch of stuff in iterate_to_converge_all


contains





subroutine update_probs_one_node( iter_inout, params_in )
   !updates probabilities and new wages for one expect node

   type(iterate_type), intent(in out) :: iter_inout
   real*8, dimension(n_params), intent(in) :: params_in

   real*8 :: m_wage_new, wage_new, m_wage_xb, wage_xb
   integer :: m_hours_new, hours_new
   real*8 :: m_ui_ben_pot_new, ui_ben_pot_new


   !joboffer probs
   iter_inout % m_joboffer_prob = m_joboffer_prob_iter_func( iter_inout % mygrid_new, params_in )
   iter_inout % joboffer_prob   = joboffer_prob_iter_func(   iter_inout % mygrid_new, params_in )



   !setup utility function
   call utility_function_setup(  iter_inout % mygrid_new % rv(var_kids), &
                                 iter_inout % mygrid_new % rv(var_youngest), &
                                 params_in, &
                                 iter_inout % utility_params )



   !if husband recieved joboffer, get new wage, the hours prob, and the income from it
   if ( iter_inout % m_receive .eq. 1)  then
      m_hours_new = iter_inout % mygrid_husband_accept % iv(var_m_hours)

      !wage
      m_wage_xb = m_wage_xb_func(   iter_inout % mygrid_new, &
                                    iter_inout % mygrid_new, &
                                    params_in, &
                                    m_hours_new )

      m_wage_new = exp( m_wage_xb + params_in(param_sd_wage) * iter_inout % m_wage_z_score )

      !make sure wage is within the bounds
      m_wage_new = bounds_func( m_wage_new, grid_min( var_m_wage), grid_max( var_m_wage) )

      iter_inout % mygrid_husband_accept % rv( var_m_wage ) = m_wage_new





      !hours
      if (m_hours_new .eq. 20 ) then
         iter_inout % m_hours_prob = params_in(param_prob_pt)
      else
         iter_inout % m_hours_prob = 1- params_in(param_prob_pt)
      end if

      !update potential ui
      m_ui_ben_pot_new = m_ui_ben_func ( iter_inout % mygrid_husband_accept, 1 )

      m_ui_ben_pot_new = bounds_func ( m_ui_ben_pot_new, &
         grid_min( var_m_ui_ben), grid_max( var_m_ui_ben) )

      iter_inout % mygrid_husband_accept % iter( interp_m_ui_ben_pot ) = &
         m_ui_ben_pot_new




   else !didn't get offer, so joboffer prob is really 1-P for this hh
     iter_inout % m_joboffer_prob = 1 - iter_inout % m_joboffer_prob
   end if

   !same for wife
   if ( iter_inout % receive .eq. 1)  then
      hours_new = iter_inout % mygrid_wife_accept % iv(var_hours)

      !wage
      wage_xb = wage_xb_func(    iter_inout % mygrid_new, &
                                 iter_inout % mygrid_new, &
                                 params_in, &
                                 hours_new )

      wage_new = exp( wage_xb + params_in(param_sd_wage) * iter_inout % wage_z_score )

      !make sure wage is within the bounds
      wage_new = bounds_func( wage_new, grid_min( var_m_wage), grid_max( var_m_wage) )

      iter_inout % mygrid_wife_accept % rv( var_wage ) = wage_new

      !hours
      if (hours_new .eq. 20 ) then
         iter_inout % hours_prob = params_in(param_prob_pt)
      else
         iter_inout % hours_prob = 1- params_in(param_prob_pt)
      end if

      !update potential ui
      ui_ben_pot_new = ui_ben_func ( iter_inout % mygrid_wife_accept, 1 )

      ui_ben_pot_new = bounds_func ( ui_ben_pot_new, &
         grid_min( var_ui_ben), grid_max( var_ui_ben) )

      iter_inout % mygrid_wife_accept % iter( interp_ui_ben_pot ) = &
         ui_ben_pot_new


   else
      iter_inout % joboffer_prob   = 1 - iter_inout % joboffer_prob
   end if

   !both accept
   if (( iter_inout % m_receive .eq. 1) .and. ( iter_inout % receive .eq. 1)) then

      iter_inout % mygrid_both_accept % rv( var_m_wage ) = m_wage_new
      iter_inout % mygrid_both_accept % rv( var_wage ) = wage_new

      iter_inout % mygrid_both_accept % iter( interp_m_ui_ben_pot ) = m_ui_ben_pot_new

      iter_inout % mygrid_both_accept % iter( interp_ui_ben_pot ) = ui_ben_pot_new


   end if

end subroutine update_probs_one_node


subroutine update_probs_all_node ( value_grid_inout, params_in )

   type(value_grid_type), intent(in out) :: value_grid_inout
   real*8, dimension(n_params), intent(in) :: params_in

   integer :: i, s, k

   k = 1

   do i= value_grid_inout % min_state_point, value_grid_inout % max_state_point
      do s=1, value_grid_inout % n_expect_points(i)

         call update_probs_one_node ( value_grid_inout % iter_obj(k), params_in )
         k=k+1

      end do
   end do



end subroutine update_probs_all_node



subroutine update_asset_util_one_node( iter_inout, params_in, asset_matrix_in )
   type(iterate_type), intent(in out) :: iter_inout
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(n_grid_points), intent(in) :: asset_matrix_in

   !update utility function if either (1) parameters different from previous iteration, or (2) updated the asset policy function

   !both reject
   iter_inout % asset_choice_both_reject = &
      asset_policy_func(   iter_inout % mygrid_new, &
                           asset_matrix_in  )

   iter_inout % utility_both_reject = &
         utility_function( iter_inout % mygrid_new, &
                           iter_inout % asset_choice_both_reject, &
                           iter_inout % utility_params, &
                           params_in )


   if ( iter_inout % m_receive .eq. 1)  then
      iter_inout % asset_choice_husband_accept = &
         asset_policy_func(   iter_inout % mygrid_husband_accept, &
                              asset_matrix_in  )

      iter_inout % utility_husband_accept = &
            utility_function( iter_inout % mygrid_husband_accept, &
                              iter_inout % asset_choice_husband_accept, &
                              iter_inout % utility_params, &
                              params_in )

   end if

   if ( iter_inout % receive .eq. 1)  then
      iter_inout % asset_choice_wife_accept = &
         asset_policy_func(   iter_inout % mygrid_wife_accept, &
                              asset_matrix_in  )


      iter_inout % utility_wife_accept = &
            utility_function( iter_inout % mygrid_wife_accept, &
                              iter_inout % asset_choice_wife_accept, &
                              iter_inout % utility_params, &
                              params_in )

   end if

   if ( ( iter_inout % m_receive .eq. 1) .and. ( iter_inout % receive .eq. 1) )  then
      iter_inout % asset_choice_both_accept = &
         asset_policy_func(   iter_inout % mygrid_both_accept, &
                              asset_matrix_in  )

      iter_inout % utility_both_accept = &
            utility_function( iter_inout % mygrid_both_accept, &
                              iter_inout % asset_choice_both_accept, &
                              iter_inout % utility_params, &
                              params_in )

   end if


end subroutine update_asset_util_one_node



subroutine update_asset_util_all_node ( value_grid_inout, params_in, asset_matrix_in )

   type(value_grid_type), intent(in out) :: value_grid_inout
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(n_grid_points), intent(in) :: asset_matrix_in

   integer :: i, s, k

   k = 1

   do i= value_grid_inout % min_state_point, value_grid_inout % max_state_point
      do s=1, value_grid_inout % n_expect_points(i)

         call update_asset_util_one_node ( value_grid_inout % iter_obj(k), &
            params_in, asset_matrix_in )

         k = k + 1

      end do
   end do



end subroutine update_asset_util_all_node

subroutine update_asset_util_no_savings ( value_grid_inout, params_in )
   !routine that ignores savings for hwo

   type(value_grid_type), intent(in out) :: value_grid_inout
   real*8, dimension(n_params), intent(in) :: params_in

   integer :: i, s, k


   k = 1

   do i= value_grid_inout % min_state_point, value_grid_inout % max_state_point
      do s=1, value_grid_inout % n_expect_points(i)

         call update_asset_util_one_node2 ( value_grid_inout % iter_obj(k), &
               params_in )

         k = k + 1

      end do
   end do



contains


subroutine update_asset_util_one_node2( iter_inout, params_in )
   type(iterate_type), intent(in out) :: iter_inout
   real*8, dimension(n_params), intent(in) :: params_in

   !temporary routine that igores assets and bases consumption on just current income

   !both reject
   iter_inout % asset_choice_both_reject = iter_inout % mygrid_new % rv( var_assets )

   iter_inout % utility_both_reject = &
         utility_function( iter_inout % mygrid_new, &
                           iter_inout % asset_choice_both_reject, &
                           iter_inout % utility_params, &
                           params_in )


   if ( iter_inout % m_receive .eq. 1)  then
      iter_inout % asset_choice_husband_accept = iter_inout % mygrid_husband_accept % rv( var_assets )

      iter_inout % utility_husband_accept = &
            utility_function( iter_inout % mygrid_husband_accept, &
                              iter_inout % asset_choice_husband_accept, &
                              iter_inout % utility_params, &
                              params_in )

   end if

   if ( iter_inout % receive .eq. 1)  then
      iter_inout % asset_choice_wife_accept = iter_inout % mygrid_wife_accept % rv( var_assets )


      iter_inout % utility_wife_accept = &
            utility_function( iter_inout % mygrid_wife_accept, &
                              iter_inout % asset_choice_wife_accept, &
                              iter_inout % utility_params, &
                              params_in )

   end if

   if ( ( iter_inout % m_receive .eq. 1) .and. ( iter_inout % receive .eq. 1) )  then
      iter_inout % asset_choice_both_accept = iter_inout % mygrid_both_accept % rv( var_assets )

      iter_inout % utility_both_accept = &
            utility_function( iter_inout % mygrid_both_accept, &
                              iter_inout % asset_choice_both_accept, &
                              iter_inout % utility_params, &
                              params_in )

   end if


end subroutine update_asset_util_one_node2

end subroutine update_asset_util_no_savings






subroutine update_util_one_node( iter_inout, params_in  )
   type(iterate_type), intent(in out) :: iter_inout
   real*8, dimension(n_params), intent(in) :: params_in

   !updates utility function, given asset choices already being stores

   !both reject

   iter_inout % utility_both_reject = &
         utility_function( iter_inout % mygrid_new, &
                           iter_inout % asset_choice_both_reject, &
                           iter_inout % utility_params, &
                           params_in )


   if ( iter_inout % m_receive .eq. 1)  then

      iter_inout % utility_husband_accept = &
            utility_function( iter_inout % mygrid_husband_accept, &
                              iter_inout % asset_choice_husband_accept, &
                              iter_inout % utility_params, &
                              params_in )

   end if

   if ( iter_inout % receive .eq. 1)  then

      iter_inout % utility_wife_accept = &
            utility_function( iter_inout % mygrid_wife_accept, &
                              iter_inout % asset_choice_wife_accept, &
                              iter_inout % utility_params, &
                              params_in )

   end if

   if ( ( iter_inout % m_receive .eq. 1) .and. ( iter_inout % receive .eq. 1) )  then

      iter_inout % utility_both_accept = &
            utility_function( iter_inout % mygrid_both_accept, &
                              iter_inout % asset_choice_both_accept, &
                              iter_inout % utility_params, &
                              params_in )

   end if


end subroutine update_util_one_node



subroutine update_util_all_node ( value_grid_inout, params_in )

   type(value_grid_type), intent(in out) :: value_grid_inout
   real*8, dimension(n_params), intent(in) :: params_in

   integer :: i, s, k

   k = 1

   do i= value_grid_inout % min_state_point, value_grid_inout % max_state_point
      do s=1, value_grid_inout % n_expect_points(i)

         call update_util_one_node ( value_grid_inout % iter_obj(k), &
            params_in )

         k = k + 1

      end do
   end do



end subroutine update_util_all_node







subroutine iterate_value_one_node( iter_inout, value_matrix_in )
   type(iterate_type), intent(in out) :: iter_inout
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in

   real*8,dimension(4) :: vals_emax_input
   real*8 :: emax


   !now calculate things I would need to calculate every iteration

   !now get the value function and the emax
   !always need to calculate value both reject
   iter_inout % value_both_reject = &
      value_function(   iter_inout % mygrid_new, &
                        iter_inout % asset_choice_both_reject, &
                        value_matrix_in )


   vals_emax_input(1) = iter_inout % utility_both_reject + &
                        iter_inout % value_both_reject


   !now for others
   if (( iter_inout % m_receive .eq. 1) .and. ( iter_inout % receive .eq. 1)) then

      iter_inout % value_husband_accept = &
         value_function(   iter_inout % mygrid_husband_accept , &
                           iter_inout % asset_choice_husband_accept, &
                           value_matrix_in )

      iter_inout % value_wife_accept = &
         value_function(   iter_inout % mygrid_wife_accept , &
                           iter_inout % asset_choice_wife_accept, &
                           value_matrix_in )

      iter_inout % value_both_accept = &
         value_function(   iter_inout % mygrid_both_accept , &
                           iter_inout % asset_choice_both_accept, &
                           value_matrix_in )

      vals_emax_input(2) = iter_inout % utility_husband_accept + &
                        iter_inout % value_husband_accept
      vals_emax_input(3) = iter_inout % utility_wife_accept + &
                        iter_inout % value_wife_accept
      vals_emax_input(4) = iter_inout % utility_both_accept + &
                        iter_inout % value_both_accept

      emax = emax_ev(4, vals_emax_input(1:4) )

   else  if ( iter_inout % m_receive .eq. 1)  then

      iter_inout % value_husband_accept = &
         value_function(   iter_inout % mygrid_husband_accept , &
                           iter_inout % asset_choice_husband_accept, &
                           value_matrix_in )

      vals_emax_input(2) = iter_inout % utility_husband_accept + &
                        iter_inout % value_husband_accept

      emax = emax_ev(2, vals_emax_input(1:2) )


   else if ( iter_inout % receive .eq. 1)  then

      iter_inout % value_wife_accept = &
         value_function(   iter_inout % mygrid_wife_accept , &
                           iter_inout % asset_choice_wife_accept, &
                           value_matrix_in )

      vals_emax_input(2) = iter_inout % utility_wife_accept + &
                        iter_inout % value_wife_accept

      emax = emax_ev(2, vals_emax_input(1:2) )

   else

      emax =  vals_emax_input(1)

   end if

   !now finally multiply the emax by the expectation weights
   iter_inout % weight =   iter_inout % birth_prob * &
                           iter_inout % m_jobloss_prob * &
                           iter_inout % jobloss_prob * &
                           iter_inout % m_ui_prob * &
                           iter_inout % ui_prob * &
                           iter_inout % ur_weight * &
                           iter_inout % m_joboffer_prob * &
                           iter_inout % joboffer_prob * &
                           iter_inout % m_hours_prob * &
                           iter_inout % hours_prob * &
                           iter_inout % joboffer_weight



   iter_inout % emax_weighted =  iter_inout % weight * &
                                 emax




end subroutine iterate_value_one_node



subroutine iterate_value_all_node( value_grid_inout, value_matrix_in, params_in, value_matrix_out )
   !iterates the value function once for all points in my state space and then does ols interpolation
   type(value_grid_type),  intent(in out) :: value_grid_inout
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(n_grid_points), intent(out) :: value_matrix_out

   real*8 :: value_temp
   integer :: i, s, k


   k = 1

   do i= value_grid_inout % min_state_point, value_grid_inout % max_state_point
      value_temp = 0

      do s=1, value_grid_inout % n_expect_points(i)

         call iterate_value_one_node(  value_grid_inout % iter_obj(k), value_matrix_in )

         value_temp = value_temp + value_grid_inout % iter_obj(k) % emax_weighted

         k = k + 1

      end do

      value_grid_inout % stored_value(i) = value_temp

   end do

   !finally, incorporate discount factor
   value_grid_inout % stored_value = &
      value_grid_inout % stored_value * params_in( param_discount_factor )


!   do i =1 , n_grid_points
!      if (  ( i >= value_grid_inout % min_state_point ) .and.  &
!            ( i <= value_grid_inout % max_state_point ) ) then

!         print *, i, value_grid_inout % stored_value(i)

!      end if


!   end do

!   call mpi_barrier( mpi_comm_world, err )
!   stop



   !now given these values, generate the interpolation matrix
   call gen_value_matrix ( value_grid_inout, value_matrix_out )


end subroutine iterate_value_all_node


subroutine iterate_value_max_one_node( iter_inout, params_in, value_matrix_in )
   !gets the value function viva maximizing wrt assets rather than using the policy interpolation funciton
   type(iterate_type), intent(in out) :: iter_inout
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in

   real*8,dimension(4) :: vals_emax_input
   real*8 :: emax


   !now get the value function and the emax
   !always need to calculate value both reject
   call max_value_sub(  iter_inout % mygrid_new, &
                        iter_inout % utility_params, &
                        params_in, &
                        value_matrix_in, &
                        iter_inout % asset_choice_both_reject, &
                        iter_inout % value_both_reject )


   vals_emax_input(1) = iter_inout % value_both_reject


   !now for others
   if (( iter_inout % m_receive .eq. 1) .and. ( iter_inout % receive .eq. 1)) then

      call max_value_sub(  iter_inout % mygrid_husband_accept, &
                           iter_inout % utility_params, &
                           params_in, &
                           value_matrix_in, &
                           iter_inout % asset_choice_husband_accept, &
                           iter_inout % value_husband_accept )

      call max_value_sub(  iter_inout % mygrid_wife_accept, &
                           iter_inout % utility_params, &
                           params_in, &
                           value_matrix_in, &
                           iter_inout % asset_choice_wife_accept, &
                           iter_inout % value_wife_accept )

      call max_value_sub(  iter_inout % mygrid_both_accept, &
                           iter_inout % utility_params, &
                           params_in, &
                           value_matrix_in, &
                           iter_inout % asset_choice_both_accept, &
                           iter_inout % value_both_accept )

      vals_emax_input(2) = iter_inout % value_husband_accept
      vals_emax_input(3) = iter_inout % value_wife_accept
      vals_emax_input(4) = iter_inout % value_both_accept

      emax = emax_ev(4, vals_emax_input(1:4) )

   else  if ( iter_inout % m_receive .eq. 1)  then

      call max_value_sub(  iter_inout % mygrid_husband_accept, &
                           iter_inout % utility_params, &
                           params_in, &
                           value_matrix_in, &
                           iter_inout % asset_choice_husband_accept, &
                           iter_inout % value_husband_accept )

      vals_emax_input(2) = iter_inout % value_husband_accept

      emax = emax_ev(2, vals_emax_input(1:2) )


   else if ( iter_inout % receive .eq. 1)  then

      call max_value_sub(  iter_inout % mygrid_wife_accept, &
                           iter_inout % utility_params, &
                           params_in, &
                           value_matrix_in, &
                           iter_inout % asset_choice_wife_accept, &
                           iter_inout % value_wife_accept )

      vals_emax_input(2) = iter_inout % value_wife_accept

      emax = emax_ev(2, vals_emax_input(1:2) )

   else

      emax =  vals_emax_input(1)

   end if

   !now finally multiply the emax by the expectation weights
   iter_inout % weight =   iter_inout % birth_prob * &
                           iter_inout % m_jobloss_prob * &
                           iter_inout % jobloss_prob * &
                           iter_inout % m_ui_prob * &
                           iter_inout % ui_prob * &
                           iter_inout % ur_weight * &
                           iter_inout % m_joboffer_prob * &
                           iter_inout % joboffer_prob * &
                           iter_inout % m_hours_prob * &
                           iter_inout % hours_prob * &
                           iter_inout % joboffer_weight



   iter_inout % emax_weighted =  iter_inout % weight * &
                                 emax




end subroutine iterate_value_max_one_node




subroutine iterate_value_max_all_node( value_grid_inout, value_matrix_in, params_in, value_matrix_out )
   !iterates the value function once for all points in my state space and then does ols interpolation
   type(value_grid_type),  intent(in out) :: value_grid_inout
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(n_grid_points), intent(out) :: value_matrix_out

   real*8 :: value_temp
   integer :: i, s, k


   k = 1

   do i= value_grid_inout % min_state_point, value_grid_inout % max_state_point
      value_temp = 0

      do s=1, value_grid_inout % n_expect_points(i)

         call iterate_value_max_one_node(  value_grid_inout % iter_obj(k), params_in, value_matrix_in )

         value_temp = value_temp + value_grid_inout % iter_obj(k) % emax_weighted

         k = k + 1

      end do

      value_grid_inout % stored_value(i) = value_temp

   end do

   !finally, incorporate discount factor
   value_grid_inout % stored_value = &
      value_grid_inout % stored_value * params_in( param_discount_factor )


!   do i =1 , n_grid_points
!      if (  ( i >= value_grid_inout % min_state_point ) .and.  &
!            ( i <= value_grid_inout % max_state_point ) ) then

!         print *, i, value_grid_inout % stored_value(i)

!      end if


!   end do

!   call mpi_barrier( mpi_comm_world, err )
!   stop



   !now given these values, generate the interpolation matrix
   call gen_value_matrix ( value_grid_inout, value_matrix_out )


end subroutine iterate_value_max_all_node



subroutine update_asset_utility_params( value_grid_inout, params_in )
   !updates the utility params used for constructing the asset interpolation function
   type(value_grid_type), intent(in out) :: value_grid_inout
   real*8, dimension(n_params), intent(in) :: params_in

   integer :: i


   do i= value_grid_inout % min_state_point, value_grid_inout % max_state_point

      call utility_function_setup(  &
         value_grid_inout % mygrid_start(i) % rv(var_kids), &
         value_grid_inout % mygrid_start(i) % rv(var_youngest), &
         params_in, &
         value_grid_inout % utility_params_start(:,i) )
   end do




end subroutine update_asset_utility_params



subroutine update_asset_policy_function( value_grid_inout, value_matrix_in, params_in, &
   asset_matrix_out )
   !iterates the value function once for all points in my state space and then does ols interpolation
   type(value_grid_type), intent(in out) :: value_grid_inout
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(n_grid_points), intent(out) :: asset_matrix_out

   integer :: i

   do i= value_grid_inout % min_state_point, value_grid_inout % max_state_point

      !now get the optimal value of assets
      call  max_value_sub(    value_grid_inout % mygrid_start(i), &
                              value_grid_inout % utility_params_start(:,i) , &
                              params_in, &
                              value_matrix_in, &
                              value_grid_inout % stored_asset_choice(i) )

      !now transform these values for the dependent variable used in a regression
      value_grid_inout % stored_asset_choice(i) = asset_choice_transform( &
         value_grid_inout % mygrid_start(i), &
         value_grid_inout % stored_asset_choice(i) )

   end do

   !finally, get the asset matrix
   call gen_asset_matrix ( value_grid_inout, asset_matrix_out )


end subroutine update_asset_policy_function





subroutine iterate_to_converge_value( max_iter_in, tol_in, savings_switch_in, params_in, &
   value_matrix_in, asset_matrix_in, value_grid_inout, value_matrix_out )
   !iterates the value function to covergence, holding the asset policy function fixed
   !iterate setup need to have been called already

   integer, parameter :: print_multiple = 25

   integer, intent(in) :: max_iter_in
   real*8, intent(in) :: tol_in
   logical, intent(in) :: savings_switch_in !says whether to do savings

   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension( n_grid_points ), intent(in) :: value_matrix_in
   real*8, dimension( n_grid_points ), intent(in) :: asset_matrix_in
   type(value_grid_type), intent(in out) :: value_grid_inout
   real*8, dimension( n_grid_points ), intent(out) :: value_matrix_out


   real*8, dimension( n_grid_points ) :: value_matrix_old,  value_matrix_new


   integer :: i, iter, m_job,job
   real*8 :: dif, start,finish

   !!!!!!!
   if (value_grid_inout % grid_allocated .eqv. .false. ) then
      print *, "value grid needed to be allocated and setup "
      stop
   end if

   if ( any( nan_inf_check(value_matrix_in) ) ) then
      print *, "problem with value matrix in "
      stop
   end if

   if ( any( nan_inf_check(asset_matrix_in) ) ) then
      print *, "problem with asset matrix in "
      stop
   end if


   !get the joboffer probs, wage values, and utility params for the expectation ndoes, which only need to be calculated once
   call update_probs_all_node ( value_grid_inout, params_in )

   !get the chosen assets and utility, which only need to be done once since I'm holding the asset policy function fixed.
   if (savings_switch_in .eqv. .true. ) then
      call update_asset_util_all_node ( value_grid_inout, params_in, asset_matrix_in )
   else
      call update_asset_util_no_savings ( value_grid_inout, params_in )
   end if

   value_matrix_new = value_matrix_in



   call cpu_time(start)
   iter = 0
   dif = tol_in + 1

   do while ( (iter .le. max_iter_in) .and. (dif .ge. tol_in) )
      iter=iter+1

      value_matrix_old = value_matrix_new

      call iterate_value_all_node( value_grid_inout, value_matrix_old, params_in, value_matrix_new )

      dif= maxval( abs(value_matrix_new-value_matrix_old) )



  !    print *, "check the value grid and iterate objects for errors"
      !   print *, "m_job, job", m_job, job
!      call value_grid_inout % check()

      if ( mod(iter,print_multiple) .eq. 0 )   then
         call cpu_time(finish)
      !   print *, "iter", iter, "dif", dif, "constant", ols_matrix_new(1,1,1), "time", finish-start
         if (myrank .eq. 0 ) print *, "iter", iter, "dif", dif, "origin", value_grid_inout % stored_value(1)
!         print *, value_grid_inout(1,1) % stored_value(2)
!         print *, value_grid_inout(1,1) % stored_value(3), &
!            value_grid_inout(1,1) % stored_value(3)- value_grid_inout(1,1) % stored_value(2)
!         print *, value_grid_inout(1,1) % stored_value(1), &
!            value_grid_inout(1,1) % stored_value(1)- value_grid_inout(1,1) % stored_value(3)
!         print *, value_grid_inout(1,1) % stored_value(4), &
!            value_grid_inout(1,1) % stored_value(4)- value_grid_inout(1,1) % stored_value(1)
!         print *, value_grid_inout(1,1) % stored_value(5), &
!            value_grid_inout(1,1) % stored_value(5)- value_grid_inout(1,1) % stored_value(4)
!         print *, "key dif", value_grid_inout(1,1) % iter_obj(1,4) % emax_weighted - &
!         value_grid_inout(1,1) % iter_obj(1,1) % emax_weighted



      end if

   end do

   value_matrix_out = value_matrix_new

   if ( myrank .eq. 0 ) then
      print *, "iteration", iter
      print *, "final dif", dif
   end if


end subroutine iterate_to_converge_value





subroutine iterate_to_converge_all( max_switch_in, &
   iter_init, max_iter_in, tol_in, n_howard_in, tol_howard_in, &
   params_in, value_matrix_in, asset_matrix_in, value_grid_inout, &
   value_matrix_out, asset_matrix_out, dif_out_opt )
   !iterates the value function to covergence
   !makes use of howard's improvement algorithm
   !iterate setup need to have been called already

   integer, parameter :: print_multiple = 10
   integer, parameter :: print_multiple_howard = 25

   logical, intent(in) :: max_switch_in  !controls whether to maximize the value function wrt assets for
      !each expectation node or whether to get assets via an asset policy interpolation function

   integer, intent(in) :: iter_init
   integer, intent(in) :: max_iter_in
   real*8, intent(in) :: tol_in
   integer, intent(in) :: n_howard_in !number of times to do the howard-improvement algorithm in which I only iterate and don't update the choice probability
   real*8, intent(in) :: tol_howard_in

   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension( n_grid_points ), intent(in) :: value_matrix_in
   real*8, dimension( n_grid_points ), intent(in) :: asset_matrix_in
   type(value_grid_type), intent(in out) :: value_grid_inout
   real*8, dimension( n_grid_points ), intent(out) :: value_matrix_out
   real*8, dimension( n_grid_points ), intent(out) :: asset_matrix_out
   real*8, intent(out), optional :: dif_out_opt

   real*8, dimension( n_grid_points ) :: value_matrix_old, value_matrix_very_old, value_matrix_new
   real*8, dimension( n_grid_points ) :: asset_matrix_new

 !  type(mygrid_type) :: mygrid_origin
   integer, dimension(3) :: maxloc_vec

   integer :: i, iter, iter_howard, m_job,job
   real*8 :: dif,dif_howard, start,finish
   real*8 :: inv_save_index

!   if (print_test_global .eqv. .true. ) then
!      print *, "************************"
!      print *, "        "
!      print *, "printing a bunch of stuff"
!      print *, "    "
!      print *, "max_switch_in", max_switch_in
!      print *, "iter_init", iter_init
!      print *, "max_iter_in", max_iter_in
!      print *, "n_howard_in", n_howard_in
!      print *, "tol_howard_in", tol_howard_in
!      print *, "params_in", params_in
!      print *, "value matrix"
!      call interp_matrix_write(value_matrix_in, output_unit)
!      print *, "asset matrix"
!      call interp_matrix_write(asset_matrix_in, output_unit)

!   end if

   !!!!!!!
   if (value_grid_inout % grid_allocated .eqv. .false. ) then
      print *, "value grid needed to be allocated and setup "
      stop
   end if

   if ( any( nan_inf_check(value_matrix_in) ) ) then
      print *, "problem with value matrix in "
      stop
   end if

   if ( any( nan_inf_check(asset_matrix_in) ) ) then
      print *, "problem with asset matrix in "
      stop
   end if

   if (  ( iter_init < -1 ) .or. &
         ( max_iter_in < 0 ) .or. &
         ( tol_in < zero_tol ) .or. &
         ( n_howard_in < 0 ) .or. & !number of times to do the howard-improvement algorithm in which I only iterate and don't update the choice probability
         ( tol_howard_in < zero_tol ) ) then

      print *, "error with inputs for iterate_to_converge_all "



   end if


   call cpu_time(start)


   !get the joboffer probs, wage values, and utility params for the expectation ndoes, which only need to be calculated once
   call update_probs_all_node ( value_grid_inout, params_in )

   !get the utilitiy params for the asset policy function creation, which only needs to be done once
   call update_asset_utility_params( value_grid_inout, params_in )



   value_matrix_new = value_matrix_in

   !do one howard loop to make sure I take a good initial update of the asset policy function
   call update_asset_util_all_node ( value_grid_inout, params_in, asset_matrix_in )

   iter_howard = 0
   dif_howard = tol_howard_in + 1

  ! if( myrank .eq. 0 ) print *, "iterating value function initially before going into main loop"
   do while ( (iter_howard .le. iter_init ) .and. (dif_howard .ge. tol_howard_in) )
      iter_howard = iter_howard + 1

      value_matrix_old = value_matrix_new

      call iterate_value_all_node( value_grid_inout, value_matrix_old, params_in, value_matrix_new )

      dif_howard= maxval( abs(value_matrix_new-value_matrix_old) )

      if (mod(iter_howard,print_multiple_howard) .eq. 0) then
         if ( ( print_iter .eqv. .true. ) .and. ( myrank .eq. 0 ) ) then
            print *, "iter_howard", iter_howard, "dif_howard", dif_howard, &
            "origin", value_grid_inout % stored_value(1)
         end if
      end if

   end do




   iter = 0
   dif = tol_in + 1

   if ( ( print_iter .eqv. .true. ) .and. ( myrank .eq. 0 ) )  print *, "start of main loop"

   do while ( (iter .le. max_iter_in) .and. (dif .ge. tol_in) )
      iter=iter+1


      value_matrix_very_old = value_matrix_new
 !     asset_matrix_old = asset_matrix_new

      if (max_switch_in .eqv. .true. ) then
         !get an update value and the value of assets that maximizes the value function

         call iterate_value_max_all_node( value_grid_inout, value_matrix_very_old, params_in, &
            value_matrix_new) !gets the optimal assets

         call update_util_all_node( value_grid_inout, params_in )

      else

         !construct the asset interpolation function

         call update_asset_policy_function( value_grid_inout, value_matrix_very_old, params_in, &
            asset_matrix_new )

         !update utilities params and the choosen assets for the expectation nodes
         call update_asset_util_all_node ( value_grid_inout, params_in, asset_matrix_new )

      end if

      !now the howard steps
 !     if( myrank .eq. 0 ) print *, "doing a howard step"

      iter_howard = 0
      dif_howard = tol_howard_in + 1
      do while ( (iter_howard .le. n_howard_in ) .and. (dif_howard .ge. tol_howard_in) )
         iter_howard = iter_howard + 1

         value_matrix_old = value_matrix_new

         call iterate_value_all_node( value_grid_inout, value_matrix_old, params_in, value_matrix_new )

         dif_howard= maxval( abs(value_matrix_new-value_matrix_old) )

         if ( ( print_iter .eqv. .true. ) .and. (mod(iter_howard, print_multiple_howard) .eq. 0) ) then
            if( myrank .eq. 0 ) print *, "    iter_howard", iter_howard, "dif_howard", dif_howard, &
               "origin", value_grid_inout % stored_value(1)
         end if

      end do


      !now convergence for main loop
      dif= maxval( abs(value_matrix_new-value_matrix_very_old) )


!      print *, "check the value grid and iterate objects for errors"

!      call value_grid % check()



      if ( ( print_iter .eqv. .true.) .and. (mod(iter, print_multiple) .eq. 0) )  then
         call cpu_time(finish)
      !   print *, "iter", iter, "dif", dif, "constant", ols_matrix_new(1,1,1), "time", finish-start
         if( myrank .eq. 0 ) print *, "iter", iter, "dif", dif, "origin", value_grid_inout % stored_value(1)


      end if

   end do

   call cpu_time(finish)

   value_matrix_out = value_matrix_new

   if ( max_switch_in .eqv. .true. ) then
      !get the asset interpolation funciton since I haven't been calculating it up until now
      call update_asset_policy_function( value_grid_inout, value_matrix_out, params_in, &
         asset_matrix_out )

   else

      asset_matrix_out = asset_matrix_new

   end if

!  do i=1,length_ols_vec_high
!      print *, ols_vec_old(i), ols_vec_new(i), dif_vector(i)
!  end do

   if ( ( print_iter .eqv. .true. ) .and. ( myrank .eq. 0 )  ) then
     print *, "iteration", iter
     print *, "final dif", dif
     print *, "time" , finish-start
   end if

   if (present( dif_out_opt ) ) then
      dif_out_opt = dif
   end if

end subroutine iterate_to_converge_all

















subroutine iterate_bayes( max_switch_in, &
   max_iter_in, tol_in, &
   params_in, value_matrix_in, value_grid_inout, &
   value_matrix_out, asset_matrix_out )
   !iterates the value function for the Bayesian McMC algorithm


   logical, intent(in) :: max_switch_in  !controls whether to maximize the value function wrt assets for
      !each expectation node or whether to get assets via an asset policy interpolation function

   integer, intent(in) :: max_iter_in
   real*8, intent(in) :: tol_in


   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension( n_grid_points ), intent(in) :: value_matrix_in
   type(value_grid_type), intent(in out) :: value_grid_inout
   real*8, dimension( n_grid_points ), intent(out) :: value_matrix_out
   real*8, dimension( n_grid_points ), intent(out) :: asset_matrix_out

   real*8, dimension( n_grid_points ) :: value_matrix_old, value_matrix_new
   real*8, dimension( n_grid_points ) :: asset_matrix_new


   integer :: i, iter
   real*8 :: dif, start,finish
   real*8 :: inv_save_index

   !!!!!!!
   if (value_grid_inout % grid_allocated .eqv. .false. ) then
      print *, "value grid needed to be allocated and setup "
      stop
   end if

   if ( any( nan_inf_check(value_matrix_in) ) ) then
      print *, "problem with value matrix in "
      stop
   end if


   if (  ( max_iter_in < 0 ) .or. &
         ( tol_in < zero_tol ) ) then


      print *, "error with inputs for iterate_to_converge_all "



   end if


   call cpu_time(start)


   !get the joboffer probs, wage values, and utility params for the expectation ndoes, which only need to be calculated once
   call update_probs_all_node ( value_grid_inout, params_in )

   !get the utilitiy params for the asset policy function creation, which only needs to be done once
   call update_asset_utility_params( value_grid_inout, params_in )



   value_matrix_new = value_matrix_in


   iter = 0
   dif = tol_in + 1

   do while ( (iter .le. max_iter_in) .and. (dif .ge. tol_in) )
      iter=iter+1


      value_matrix_old = value_matrix_new
 !     asset_matrix_old = asset_matrix_new

      if (max_switch_in .eqv. .true. ) then
         !get an update value and the value of assets that maximizes the value function

         call iterate_value_max_all_node( value_grid_inout, value_matrix_old, params_in, &
            value_matrix_new) !gets the optimal assets

         call update_util_all_node( value_grid_inout, params_in )

      else

         !construct the asset interpolation function

         call update_asset_policy_function( value_grid_inout, value_matrix_old, params_in, &
            asset_matrix_new )

         !update utilities params and the choosen assets for the expectation nodes
         call update_asset_util_all_node ( value_grid_inout, params_in, asset_matrix_new )

      end if

      !now the howard steps
 !     if( myrank .eq. 0 ) print *, "doing a howard step"

      call iterate_value_all_node( value_grid_inout, value_matrix_old, params_in, value_matrix_new )

      dif= maxval( abs(value_matrix_new-value_matrix_old) )



   end do

   call cpu_time(finish)

   value_matrix_out = value_matrix_new

   if ( max_switch_in .eqv. .true. ) then
      !get the asset interpolation funciton since I haven't been calculating it up until now
      call update_asset_policy_function( value_grid_inout, value_matrix_out, params_in, &
         asset_matrix_out )

   else

      asset_matrix_out = asset_matrix_new

   end if




end subroutine iterate_bayes





!Important, might need to relook at these derivative subroutines.


!subroutine value_asset_derivative(mygrid_in, value_matrix_in, asset_matrix_in,  &
!   first_der_value_out, second_der_value_out, first_der_asset_out, second_der_asset_out )
!   !gets the first and second derivative of the value function at a given value of the state variables

!   type(mygrid_type), intent(in) :: mygrid_in
!   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
!   real*8, dimension(n_grid_points), intent(in) :: asset_matrix_in
!   real*8, dimension(n_cont_vars),intent(out) :: first_der_value_out
!   real*8, dimension(n_cont_vars,n_cont_vars), intent(out) :: second_der_value_out
!   real*8, dimension(n_cont_vars),intent(out) :: first_der_asset_out
!   real*8, dimension(n_cont_vars,n_cont_vars), intent(out) :: second_der_asset_out

!   type(mygrid_type) :: perturb_point_temp
!   real*8 :: value_origin, value_perturb1, value_perturb2, value_perturb3, value_perturb4
!   real*8 :: asset_origin, asset_perturb1, asset_perturb2, asset_perturb3, asset_perturb4

!   integer :: m_job, job, m_hours, hours, i, j

!   !for now, won't look at derivative for hours


!   call der_delta_setup()
!   call gen_var_labels()

!   first_der_value_out = 0
!   second_der_value_out = 0
!   first_der_asset_out = 0
!   second_der_asset_out = 0

!   m_job    = mygrid_in % iv(var_m_job)
!   job      = mygrid_in % iv(var_job)


!   !get origin value
!   value_origin = value_function( mygrid_in, mygrid_in % rv( var_assets), value_matrix_in )
!   asset_origin = asset_policy_func(mygrid_in, asset_matrix_in )


!   !first and second deriviative
!   !formula from http://www.math.ohiou.edu/courses/math3600/lecture27.pdf
!   do i=1,n_cont_vars
!      if (var_present(i, m_job, job) .eq. 1) then
!         !forward derivative
!         perturb_point_temp    = mygrid_in
!         perturb_point_temp % rv(i) = mygrid_in % rv(i) + der_delta(i)

!         value_perturb1 = value_function( perturb_point_temp, &
!            perturb_point_temp % rv( var_assets), value_matrix_in )
!         asset_perturb1 = asset_policy_func( perturb_point_temp, asset_matrix_in )


!         !backward derivative
!         perturb_point_temp    = mygrid_in
!         perturb_point_temp % rv(i) = mygrid_in % rv(i) - der_delta(i)

!         value_perturb2 = value_function( perturb_point_temp, &
!            perturb_point_temp % rv( var_assets), value_matrix_in )
!         asset_perturb2 = asset_policy_func( perturb_point_temp, asset_matrix_in )

!         !now combine to get derivatives
!         first_der_value_out(i) = (value_perturb1 - value_perturb2)/ ( 2*der_delta(i) )
!         second_der_value_out(i,i) = &
!            ( value_perturb1 - 2*value_origin + value_perturb2 )/ (der_delta(i)**2)

!         first_der_asset_out(i) = (asset_perturb1 - asset_perturb2)/ ( 2*der_delta(i) )
!         second_der_asset_out(i,i) = &
!            ( asset_perturb1 - 2*asset_origin + asset_perturb2 )/ (der_delta(i)**2)

!!         if ( ( job .eq. 1) .and. (m_job .eq. 1) .and. (i .eq. 2 ) ) then
!!            print *, "value origin   ", value_origin
!!            print *, "value perturb 1", value_perturb1
!!            print *, "value perturb 2", value_perturb2
!!            print *, "first derivative", first_der_value_out(i)

!!            stop

!!         end if


!      end if
!   end do

!   !cross-partial derivatives

!   do i=1,n_cont_vars
!      do j=1,i-1
!         if ( (var_present(i, m_job, job) .eq. 1) .and. (var_present(j, m_job, job) .eq. 1) ) then

!            perturb_point_temp    = mygrid_in
!            perturb_point_temp % rv(i) = mygrid_in % rv(i) + der_delta(i)
!            perturb_point_temp % rv(j) = mygrid_in % rv(j) + der_delta(j)

!            value_perturb1 = value_function( perturb_point_temp, &
!               perturb_point_temp % rv( var_assets), value_matrix_in )
!            asset_perturb1 = asset_policy_func( perturb_point_temp, asset_matrix_in )

!            perturb_point_temp    = mygrid_in
!            perturb_point_temp % rv(i) = mygrid_in % rv(i) + der_delta(i)
!            perturb_point_temp % rv(j) = mygrid_in % rv(j) - der_delta(j)

!            value_perturb2 = value_function( perturb_point_temp, &
!               perturb_point_temp % rv( var_assets), value_matrix_in )
!            asset_perturb2 = asset_policy_func( perturb_point_temp, asset_matrix_in )

!            perturb_point_temp    = mygrid_in
!            perturb_point_temp % rv(i) = mygrid_in % rv(i) - der_delta(i)
!            perturb_point_temp % rv(j) = mygrid_in % rv(j) + der_delta(j)

!            value_perturb3 = value_function( perturb_point_temp, &
!               perturb_point_temp % rv( var_assets), value_matrix_in )
!            asset_perturb3 = asset_policy_func( perturb_point_temp, asset_matrix_in )

!            perturb_point_temp    = mygrid_in
!            perturb_point_temp % rv(i) = mygrid_in % rv(i) - der_delta(i)
!            perturb_point_temp % rv(j) = mygrid_in % rv(j) - der_delta(j)

!            value_perturb4 = value_function( perturb_point_temp, &
!               perturb_point_temp % rv( var_assets), value_matrix_in )
!            asset_perturb4 = asset_policy_func( perturb_point_temp, asset_matrix_in )

!            second_der_value_out(i,j) = &
!               ( value_perturb1 - value_perturb2 - value_perturb3 + value_perturb4 ) /&
!               ( 4* der_delta(i) * der_delta(j) )
!            second_der_asset_out(i,j) = &
!               ( asset_perturb1 - asset_perturb2 - asset_perturb3 + asset_perturb4 ) /&
!               ( 4* der_delta(i) * der_delta(j) )
!         end if
!      end do
!   end do



!end subroutine value_asset_derivative

!subroutine value_derivative_write(value_matrix_in, asset_matrix_in,  unit_in)
!   !writes the first and second derivative for each (m_job,job) combo
!   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
!   real*8, dimension(n_grid_points), intent(in) :: asset_matrix_in
!   integer, intent(in) :: unit_in

!   type(mygrid_type) :: mygrid_origin
!   integer :: m_job, job, i, i1,i2, j,k,v
!   real*8, dimension(n_cont_vars) :: first_der_value, first_der_asset
!   real*8, dimension(n_cont_vars,n_cont_vars) :: second_der_value, second_der_asset


!   do job = 0,1
!   do m_job = 0,1

!      !get the origin mygrid type
!      call get_mygrid_origin( m_job, job, mygrid_origin)

!      call value_asset_derivative( mygrid_origin, value_matrix_in, asset_matrix_in,  &
!         first_der_value, second_der_value, first_der_asset, second_der_asset )

!      call csv_write(unit_in, "   ", .true. )

!      call csv_write(unit_in, "m_job", .false.)
!      call csv_write(unit_in, m_job, .false.)
!      call csv_write(unit_in, "job", .false.)
!      call csv_write(unit_in, job, .true.)

!      !value function first
!      call csv_write(unit_in, "   ", .true. )
!      call csv_write(unit_in, "value function", .true.)
!      call csv_write(unit_in, "   ", .true. )

!      !first derivative
!      do i=1,n_cont_vars
!         if (var_present(i, m_job, job) .eq. 1) then

!            call csv_write(unit_in, var_labels(i), .false.)
!            call csv_write(unit_in, first_der_value(i), .true.)

!         end if
!      end do

!      call csv_write(unit_in, "   ", .true. )


!      !second derivatives
!      call csv_write(unit_in, "  ", .false.)

!      do i2=1, n_vars_list (m_job,job)
!         k = var_list(i2, m_job, job)
!         call csv_write(unit_in, var_labels(k),.false.)
!      end do

!      call csv_next_record(unit_in)

!      do i1=1, n_vars_list (m_job,job)
!         j = var_list(i1, m_job, job)
!         call csv_write(unit_in, var_labels(j), .false.)

!         do i2=1, n_vars_list (m_job,job)
!            k = var_list(i2, m_job, job)
!            call csv_write(unit_in, second_der_value(j,k),.false.)

!         end do
!         call csv_next_record(unit_in)

!      end do

!      !now assets
!      call csv_write(unit_in, "   ", .true. )
!      call csv_write(unit_in, "asset function", .true.)
!      call csv_write(unit_in, "   ", .true. )

!      !first derivative
!      do i=1,n_cont_vars
!         if (var_present(i, m_job, job) .eq. 1) then

!            call csv_write(unit_in, var_labels(i), .false.)
!            call csv_write(unit_in, first_der_asset(i), .true.)

!         end if
!      end do

!      call csv_write(unit_in, "   ", .true. )


!      !second derivatives
!      call csv_write(unit_in, "  ", .false.)

!      do i2=1, n_vars_list (m_job,job)
!         k = var_list(i2, m_job, job)
!         call csv_write(unit_in, var_labels(k),.false.)
!      end do

!      call csv_next_record(unit_in)

!      do i1=1, n_vars_list (m_job,job)
!         j = var_list(i1, m_job, job)
!         call csv_write(unit_in, var_labels(j), .false.)

!         do i2=1, n_vars_list (m_job,job)
!            k = var_list(i2, m_job, job)
!            call csv_write(unit_in, second_der_asset(j,k),.false.)

!         end do
!         call csv_next_record(unit_in)

!      end do



!   end do
!   end do

!end subroutine value_derivative_write


end module iterate_mod



















!subroutine max_value_sub( mygrid_in, utility_params_in, params_in, value_matrix_in, &
!   asset_choice_out, max_value_out )
!   !gets the value of assets that maximizes utility + value function
!   !and return the max_value as well if it's given
!   type( mygrid_type ), intent(in) :: mygrid_in
!   real*8, dimension(n_utility_params), intent(in) :: utility_params_in
!   real*8, dimension(n_params), intent(in) :: params_in
!   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
!   real*8, intent(out) :: asset_choice_out
!   real*8, intent(out),optional :: max_value_out

!   real*8, dimension(n_asset_nodes + 1) :: coef_assets
!   procedure(func_template_one_arg), pointer :: func_to_min_pointer
!   real*8 :: local_min_value
!   real*8 :: up
!   real*8 :: lb


!   call value_function_asset_setup( mygrid_in, value_matrix_in, coef_assets )


!   func_to_min_pointer => func_to_min
!   up= inc_func( mygrid_in ) + mygrid_in % rv( var_assets)
!   lb = params_in ( param_asset_lb  )


! !  print *, "value for very low cons", -func_to_min( up - 5 )
! !  print *, "value for very high cons", -func_to_min( zero_tol*100 )


!   call brent_local_min (  lb, &
!                           up, &
!                           eps_brent, &
!                           t_brent, &
!                           max_iter_brent, &
!                           func_to_min_pointer, &
!                           asset_choice_out, &
!                           local_min_value )

! !  print *, "optimal value", -local_min_value

!   !put between bounds
!   !aug 7, put inbetween bounds
!   asset_choice_out = bounds_func( &
!         asset_choice_out , &
!         grid_min( var_assets) , grid_max( var_assets) )

!   !to get value funciton, re-evaluate the func_to_min in case it changes with asset_choice
!   if (present (max_value_out) ) then
!      max_value_out = - func_to_min( asset_choice_out )

!   end if




!contains

!function func_to_min(assets_next_in)
!!the function that needs to be minimized in order to find the optimal value of assets
!!in order to make the function of interest conform with the interface for the brent min function, I have assets be the only input
!   real*8 :: func_to_min
!   real*8, intent(in) :: assets_next_in

!   real*8 :: util_temp, val_temp, assets_cube

!   !utility
!   util_temp = utility_function( mygrid_in, assets_next_in, utility_params_in, params_in)

!   if ( assets_next_in < grid_mean( var_assets ) ) then
!      assets_cube = ( assets_next_in - grid_mean(var_assets) ) * grid_sd_inv( var_assets )
!   else
!      assets_cube = ( assets_next_in - grid_mean( var_assets) ) * grid_sd_asset_inv2
!   end if

!   !now get the value function (not including the intercept, which is irrelevant for maximization)
!   val_temp = interp_asset_func( assets_cube, coef_assets(:n_asset_nodes) )

!   !now the interaction term
!   val_temp = val_temp + coef_assets(n_asset_nodes + 1)*assets_cube


!   func_to_min =  - util_temp - val_temp

!end function func_to_min

!end subroutine max_value_sub


