program iterate_converge_test

use enum_mod
use model_params_mod
use ols_interp_mymodel_mod

use random_gen_mod

use utility_function_mod
use emax_mod
use value_iterate_mod
use csv_file
use normal_mod

implicit none

   !testing to see if the probability functions (and the ui benefit function) are making sense

   character(len=30)          :: init_params_file = "init_params_new.txt"
      
   real*8, dimension(n_params)  :: params
   type(mygrid_type) :: mygrid, mygrid_original
   
   real*8 :: m_joboffer_xb_origin, joboffer_xb_origin, m_joboffer_prob_origin, joboffer_prob_origin, &
      m_wage_xb_origin, wage_xb_origin, &
      m_jobloss_xb_origin, jobloss_xb_origin, m_jobloss_prob_origin, jobloss_prob_origin, &
      birth_prob_origin, m_ui_prob_origin, ui_prob_origin, &
      m_ui_ben_origin, ui_ben_origin 

   real*8, dimension(n_vars) :: &
      m_joboffer_xb_der1, joboffer_xb_der1, m_joboffer_prob_der1, joboffer_prob_der1, &
      m_wage_xb_der1, wage_xb_der1, &
      m_jobloss_xb_der1, jobloss_xb_der1, m_jobloss_prob_der1, jobloss_prob_der1, &
      birth_prob_der1, m_ui_prob_der1, ui_prob_der1, &
      m_ui_ben_der1, ui_ben_der1 


   
   integer:: m_job,job, seam
   integer:: m_hours,hours
   
   integer :: i,j,k

   
   
  
   print *, "init params file    ", init_params_file
   

   call init_random_seed()
   call export_random_seed()
!  call read_random_seed()
  
   
  
  

   !now read in the parameters
   call init_params(init_params_file, params)
   
   call ols_setup_all
   call gen_param_labels()
   call gen_var_labels()
   call der_delta_setup
   
   
    
 !  params = 0 
  ! open(unit=1, file= init_params_file)
  ! call write_params(params,1,.true.)


   !get up mygrid
   
   seam = 1
   
   m_job = 1
   job = 1
   m_hours = 40 
   hours = 40
   
   call mygrid % allocate()
   
   
 !  call get_random_point(m_job, job, mygrid % rv(:) )
 !  call mygrid % real_to_int_duplicate()
 
 
   mygrid % rv(:) = grid_mean(:)
 
   
   mygrid % iv(var_m_job_int) = m_job
   mygrid % iv(var_job_int) = job
   mygrid % iv(var_m_hours_int) = m_hours
   mygrid % iv(var_hours_int) = hours
   
   mygrid % rv(var_m_ui_ben)  = 0
   mygrid % rv(var_ui_ben)    = 0
   mygrid % rv(var_m_unemp_length)  = 0
   mygrid % rv(var_unemp_length)    = 0
   
   mygrid % rv(var_uimax)  = 600
   
   mygrid % iv(var_kids_int) = 2
   
 !  mygrid % rv(var_uimax) = 525
   
   call mygrid % int_to_real_duplicate()

   call mygrid % print(.true., .true.)
 !  call mygrid % mygrid_error_hunt(.true.,.true.,1,1,1)
 !  read(*,*)

   mygrid_original = mygrid

   
   !first get the origin
   
   print *, "getting origin values"
     
   m_joboffer_xb_origin    = m_joboffer_xb_like_func  (mygrid,mygrid,params,seam)
   joboffer_xb_origin      = joboffer_xb_like_func    (mygrid,mygrid,params,seam) 
   m_joboffer_prob_origin  = m_joboffer_prob_iter_func(mygrid,params)
   joboffer_prob_origin    = joboffer_prob_iter_func  (mygrid,params)
   m_wage_xb_origin        = m_wage_xb_func           (mygrid,mygrid,params,m_hours)
   wage_xb_origin          = wage_xb_func             (mygrid,mygrid,params,hours)
   m_jobloss_xb_origin     = m_jobloss_xb_like_func   (mygrid,mygrid,params,seam)
   jobloss_xb_origin       = jobloss_xb_like_func     (mygrid,mygrid,params,seam)
   m_jobloss_prob_origin   = m_jobloss_prob_iter_func (mygrid,params)
   jobloss_prob_origin     = jobloss_prob_iter_func   (mygrid,params)
   birth_prob_origin       = birth_prob_iter_func     (mygrid,params)
   m_ui_prob_origin        = m_ui_prob_iter_func      (mygrid,params)
   ui_prob_origin          = ui_prob_iter_func        (mygrid,params)
   m_ui_ben_origin         = m_ui_ben_func            (mygrid,1)
   ui_ben_origin           = ui_ben_func              (mygrid,1)

   
 !  read(*,*)
   
   
   !now get the derivative
   print *, "now getting derivatives"
   do k=1,n_cont_vars
!   do k = var_uimax, var_uirate
  
      if (var_present(k,m_job,job) .eq. 1) then
      
         mygrid = mygrid_original
         mygrid % rv(k) = mygrid % rv(k) + der_delta(k)
         call mygrid % real_to_int_duplicate()
         
         m_joboffer_xb_der1(k)    = ( m_joboffer_xb_like_func  (mygrid,mygrid,params,seam) - &
         m_joboffer_xb_origin) / der_delta(k)
         
         joboffer_xb_der1(k)      = ( joboffer_xb_like_func    (mygrid,mygrid,params,seam) - &
         joboffer_xb_origin) / der_delta(k)
         
         m_joboffer_prob_der1(k)  = ( m_joboffer_prob_iter_func(mygrid,params) - &
         m_joboffer_prob_origin) / der_delta(k)
            
         joboffer_prob_der1(k)    = ( joboffer_prob_iter_func  (mygrid,params) - &
         joboffer_prob_origin) / der_delta(k)
            
         m_wage_xb_der1(k)        = ( m_wage_xb_func           (mygrid,mygrid,params,m_hours) - &
         m_wage_xb_origin) / der_delta(k)
            
         wage_xb_der1(k)          = ( wage_xb_func             (mygrid,mygrid,params,hours) - &
         wage_xb_origin) / der_delta(k)
            
         m_jobloss_xb_der1(k)     = ( m_jobloss_xb_like_func   (mygrid,mygrid,params,seam) - &
         m_jobloss_xb_origin) / der_delta(k)
            
         jobloss_xb_der1(k)       = ( jobloss_xb_like_func     (mygrid,mygrid,params,seam) - &
         jobloss_xb_origin) / der_delta(k)
            
         m_jobloss_prob_der1(k)   = ( m_jobloss_prob_iter_func (mygrid,params) - &
         m_jobloss_prob_origin) / der_delta(k)
            
         jobloss_prob_der1(k)     = ( jobloss_prob_iter_func   (mygrid,params) - &
         jobloss_prob_origin) / der_delta(k)
            
         birth_prob_der1(k)       = ( birth_prob_iter_func     (mygrid,params) - &
         birth_prob_origin) / der_delta(k)
            
         m_ui_prob_der1(k)        = ( m_ui_prob_iter_func      (mygrid,params) - &
         m_ui_prob_origin) / der_delta(k)
            
         ui_prob_der1(k)          = ( ui_prob_iter_func        (mygrid,params) - &
         ui_prob_origin) / der_delta(k)
            
         m_ui_ben_der1(k)         = ( m_ui_ben_func            (mygrid,1) - &
         m_ui_ben_origin) / der_delta(k)
            
         ui_ben_der1(k)           = ( ui_ben_func              (mygrid,1) - &
         ui_ben_origin) / der_delta(k)
         
         
      end if
   
   end do
   
   print *, "now printing derivatives and origin values"
   
   print *, " "
   print *, "*********************"
   print *, " "
   print *, "m_joboffer_xb"
   print *, "origin", m_joboffer_xb_origin
   print *, "deriviatve"
   do k=1,n_cont_vars
      if (var_present(k,m_job,job) .eq. 1) then
         print *, var_labels(k), m_joboffer_xb_der1(k)
      end if
   end do
   
   
      print *, " "
   print *, "*********************"
   print *, " "
   print *, "joboffer_xb"
   print *, "origin", joboffer_xb_origin
   print *, "deriviatve"
   do k=1,n_cont_vars
      if (var_present(k,m_job,job) .eq. 1) then
         print *, var_labels(k), joboffer_xb_der1(k)
      end if
   end do
   
   
      print *, " "
   print *, "*********************"
   print *, " "
   print *, "m_joboffer_prob"
   print *, "origin", m_joboffer_prob_origin
   print *, "deriviatve"
   do k=1,n_cont_vars
      if (var_present(k,m_job,job) .eq. 1) then
         print *, var_labels(k), m_joboffer_prob_der1(k)
      end if
   end do
   
   
      print *, " "
   print *, "*********************"
   print *, " "
   print *, "joboffer_prob"
   print *, "origin", joboffer_prob_origin
   print *, "deriviatve"
   do k=1,n_cont_vars
      if (var_present(k,m_job,job) .eq. 1) then
         print *, var_labels(k), joboffer_prob_der1(k)
      end if
   end do
   
   
      print *, " "
   print *, "*********************"
   print *, " "
   print *, "m_wage_xb"
   print *, "origin", m_wage_xb_origin
   print *, "deriviatve"
   do k=1,n_cont_vars
      if (var_present(k,m_job,job) .eq. 1) then
         print *, var_labels(k), m_wage_xb_der1(k)
      end if
   end do
   
   
      print *, " "
   print *, "*********************"
   print *, " "
   print *, "wage_xb"
   print *, "origin", wage_xb_origin
   print *, "deriviatve"
   do k=1,n_cont_vars
      if (var_present(k,m_job,job) .eq. 1) then
         print *, var_labels(k), wage_xb_der1(k)
      end if
   end do
   
   
      print *, " "
   print *, "*********************"
   print *, " "
   print *, "m_jobloss_xb"
   print *, "origin", m_jobloss_xb_origin
   print *, "deriviatve"
   do k=1,n_cont_vars
      if (var_present(k,m_job,job) .eq. 1) then
         print *, var_labels(k), m_jobloss_xb_der1(k)
      end if
   end do
   
   
      
      print *, " "
   print *, "*********************"
   print *, " "
   print *, "jobloss_xb"
   print *, "origin", jobloss_xb_origin
   print *, "deriviatve"
   do k=1,n_cont_vars
      if (var_present(k,m_job,job) .eq. 1) then
         print *, var_labels(k), jobloss_xb_der1(k)
      end if
   end do
   
         print *, " "
   print *, "*********************"
   print *, " "
   print *, "m_jobloss_prob"
   print *, "origin", m_jobloss_prob_origin
   print *, "deriviatve"
   do k=1,n_cont_vars
      if (var_present(k,m_job,job) .eq. 1) then
         print *, var_labels(k), m_jobloss_prob_der1(k)
      end if
   end do
   
   
      
      print *, " "
   print *, "*********************"
   print *, " "
   print *, "jobloss_prob"
   print *, "origin", jobloss_prob_origin
   print *, "deriviatve"
   do k=1,n_cont_vars
      if (var_present(k,m_job,job) .eq. 1) then
         print *, var_labels(k), jobloss_prob_der1(k)
      end if
   end do
   

   
   
      print *, " "
   print *, "*********************"
   print *, " "
   print *, "birth_prob"
   print *, "origin", birth_prob_origin
   print *, "deriviatve"
   do k=1,n_cont_vars
      if (var_present(k,m_job,job) .eq. 1) then
         print *, var_labels(k), birth_prob_der1(k)
      end if
   end do
   
   
      print *, " "
   print *, "*********************"
   print *, " "
   print *, "m_ui_prob"
   print *, "origin", m_ui_prob_origin
   print *, "deriviatve"
   do k=1,n_cont_vars
      if (var_present(k,m_job,job) .eq. 1) then
         print *, var_labels(k), m_ui_prob_der1(k)
      end if
   end do
   
   
      print *, " "
   print *, "*********************"
   print *, " "
   print *, "ui_prob"
   print *, "origin", ui_prob_origin
   print *, "deriviatve"
   do k=1,n_cont_vars
      if (var_present(k,m_job,job) .eq. 1) then
         print *, var_labels(k), ui_prob_der1(k)
      end if
   end do
   
   
      print *, " "
   print *, "*********************"
   print *, " "
   print *, "m_ui_ben"
   print *, "origin", m_ui_ben_origin
   print *, "deriviatve"
   do k=1,n_cont_vars
      if (var_present(k,m_job,job) .eq. 1) then
         print *, var_labels(k), m_ui_ben_der1(k)
      end if
   end do
   
   
      print *, " "
   print *, "*********************"
   print *, " "
   print *, "ui_ben"
   print *, "origin", ui_ben_origin
   print *, "deriviatve"
   do k=1,n_cont_vars
      if (var_present(k,m_job,job) .eq. 1) then
         print *, var_labels(k), ui_ben_der1(k)
      end if
   end do
   
   
   
   
!   print *, "printing difference between origin and peturb values for joboffer_mean"
!   do i=1,n_draws
!      print *, i, value_origin_all(i), value_perturb_all(i,2), value_perturb_all(i,2) - value_origin_all(i)
!   end do
   
   


end program





































!an older way with importing the normals

!   !first get the random numbers
!   call random_number( random_input_matrix )
!   do i=1,n_draws
!      do j=1,5
!         random_normal_matrix(j,i) = rnorm()
!      end do
!   end do

     
!   do i=1, n_draws
!    !  call random_number( random_input ) 
!       !  random_input(:) = random_input_matrix(:,i)
!   !   random_input ( iter_m_jobloss ) = 0
!   !   random_input ( iter_m_ui ) = 0
      
!      call iterate_one_point_normal_in(mygrid, params, ols_vec_init, random_input_matrix(:,i), &
!         random_normal_matrix(:,i), value_temp)

!      value_origin = value_origin + value_temp
!      value_origin_all(i) = value_temp
!   end do
!   value_origin = value_origin / n_draws
   
   
!   print *, "got origin value"
! !  read(*,*)
   
   
!   !now get the derivative
!   print *, "first derivative"
!   do k=1,n_cont_vars
!!   do k = var_uimax, var_uirate
   
!      if (var_present(k,m_job,job)) then
!       !  call read_random_seed()
         
!      !   print *, "getting derivative for", var_labels(k)
      
!         mygrid = mygrid_original
!         mygrid % rv(k) = mygrid % rv(k) + der_delta(k)
!         call mygrid % real_to_int_duplicate()
      
!         value_perturb = 0
!         do i=1, n_draws
!           ! call random_number( random_input ) 
            
!       !     random_input(:) = random_input_matrix(:,i)
       
!        !    random_input ( iter_m_jobloss ) = 0
!        !    random_input ( iter_m_ui ) = 0
            
!            call iterate_one_point_normal_in(mygrid, params, ols_vec_init, random_input_matrix(:,i), &
!               random_normal_matrix(:,i), value_temp)

!            value_perturb = value_perturb + value_temp
!            value_perturb_all(i,k) = value_temp
!         end do
      
!         value_perturb = value_perturb / n_draws
      
!         value_der1(k) = (value_perturb - value_origin) / der_delta(k)
      
!         print *, var_labels(k), value_der1(k)
         
!       !  print *, "got derivative for ", var_labels(k)
!       !  read(*,*)
         
!      end if
   
!   end do
   
   
!!   print *, "printing difference between origin and peturb values for joboffer_mean"
!!   do i=1,n_draws
!!      print *, i, value_origin_all(i), value_perturb_all(i,2), value_perturb_all(i,2) - value_origin_all(i)
!!   end do
   
   


!end program


