
module mpi_mod

use mpi

!module for doing parallel stuff

   integer :: myrank, nprocs
   integer :: err
   logical :: mpi_setup_called = .false.

contains

!combines the setup routines into one
subroutine mpi_setup()

   call mpi_init(err)
   call mpi_comm_rank(mpi_comm_world, myrank, err)
   call mpi_comm_size(mpi_comm_world, nprocs, err)

   mpi_setup_called = .true.

end subroutine mpi_setup



!got online from the GNU Fortran site, for intializing random seed
!modifying it so that the processes can either have same seed or different seed
SUBROUTINE init_random_seed_mpi(same)
   logical, intent(in) :: same
   INTEGER :: i, n, clock
   INTEGER, DIMENSION(:), ALLOCATABLE :: seed

   CALL RANDOM_SEED(size = n)
   ALLOCATE(seed(n))

   if (myrank .eq. 0 ) then
      CALL SYSTEM_CLOCK(COUNT=clock)
      seed = clock + 37 * (/ (i - 1, i = 1, n) /)
   end if

   call mpi_bcast( seed, size(seed), mpi_int, 0, mpi_comm_world, err )

   if( same .eqv. .false. ) seed = seed + myrank


   CALL RANDOM_SEED(PUT = seed)

  ! print *,"random seed"
  ! do i=1,n
  !    print *, seed(i)
  ! end do

   DEALLOCATE(seed)
END SUBROUTINE


!writes the random seed to a file if the root noe
subroutine export_random_seed_mpi(filename_opt)
   character(len=30), intent(in), optional :: filename_opt
   integer :: i,n,unit_loc
   integer, dimension(:), allocatable :: seed

   if( myrank .eq. 0 ) then

      call random_seed(size =n )
      allocate(seed(n))

      call random_seed(get=seed)

      if ( present(filename_opt) ) then
         open (newunit=unit_loc,file=filename_opt, status = "replace")
      else
         open (newunit=unit_loc,file="seed_save.txt", status = "replace")
      end if

      do i=1,n
         write(unit_loc,*), seed(i)
      end do
      close(unit_loc)

   end if

end subroutine


subroutine print_random_seed_mpi()
   integer :: i,n
   integer, dimension(:), allocatable :: seed

   if( myrank .eq. 0 ) then

      call random_seed(size =n )
      allocate(seed(n))

      call random_seed(get=seed)

      print *, "random seed"

      do i=1,n
         print *, seed(i)
      end do

      print *, " "

   end if

end subroutine

!reads random seed from a file
subroutine read_random_seed_mpi(same, filename_opt)
   logical, intent(in) :: same
   character(len=30), intent(in), optional :: filename_opt

   integer :: i,n,unit_loc
   integer, dimension(:), allocatable :: seed
 !  logical ::file_exists

   call random_seed(size =n )
   allocate(seed(n))


!   inquire(file="seed_save.txt",exist=file_exists)

!   if (file_exists .eqv. .false.) then
!      print *, "seed file not found"
!      stop
!   end if

   if( present(filename_opt) ) then
      open (newunit=unit_loc,file=filename_opt, status = "old" )
   else
      open (newunit=unit_loc,file="seed_save.txt", status = "old" )
   end if

   do i=1,n
      read(unit_loc,*), seed(i)
   end do
   close(unit_loc)

   if( same .eqv. .false. ) seed = seed + myrank

   call random_seed(put = seed)


end subroutine







end module mpi_mod
