program iterate_one_point_test

   !test to see if iterating the value function for one point works

use enum_mod
use model_params_mod
use interp_support_mod


use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use normal_mod
use stat_estimates_mod

use csv_file

implicit none


   character(len=30), parameter :: seed_file = "seed_save6.txt"
   character(len=30), parameter :: interact_file = "interact_matrix.csv"

   logical :: iter_switch = .false.

   type(mygrid_type) :: mygrid_original, mygrid_duplicate
   real*8, dimension(n_interp_vars) :: interp_vec
   real*8, dimension(n_interp_vars_discrete) :: rand_uniform_discrete
   real*8, dimension(n_cont_vars) :: rand_uniform_cont
   real*8, dimension( interp_m_ui_ben_pot: interp_ui_left ) :: rand_iter

   integer :: i



   call mpi_setup()

   call init_random_seed_mpi(same = .false. )
   call export_random_seed_mpi( seed_file )
 !  call read_random_seed_mpi( same = .false., filename_opt = seed_file)

   call interp_setup_all( interact_file )


   call random_number(rand_uniform_discrete)
   call random_number(rand_uniform_cont)
   call random_number( rand_iter )

   rand_uniform_discrete( interp_no_ui )  = 1

   call gen_mygrid_random_test( mygrid_original, iter_switch , &
      rand_uniform_discrete, rand_uniform_cont, rand_iter )

!   mygrid_original % rv( var_assets ) = 5000

   print *, "printing mygrid original and hunting for errors"
   call mygrid_original % mygrid_error_hunt( .true. , .true., 1, 1, 1 )
   call mygrid_original % print( .true., .true. )
   print *, " "


   call mygrid_to_interp( mygrid_original, interp_vec)

   print *, "printing the interpolation vector"
   call print_interp_vec( interp_vec )
   print *, " "

   call interp_to_mygrid( interp_vec, iter_switch , mygrid_duplicate )

   print *, "printing mygrid transform and hunting for errors"
   call mygrid_duplicate % mygrid_error_hunt( .true. , .true., 1, 1, 1 )
   call mygrid_duplicate % print( .true., .true. )
   print *, " "


   !now compare the mygrid original and duplciates
   print *, "printing difference between the two mygrid objects"
   do i =1 , n_int_vars
      print *, i, mygrid_original % iv(i) - mygrid_duplicate % iv(i)
   end do

   print *," "

   do i =1 , n_cont_vars
      print *, var_labels(i), mygrid_original % rv(i) - mygrid_duplicate % rv(i)
   end do

   if ( iter_switch .eqv. .true. ) then
      do i = interp_m_ui_ben_pot, interp_ui_left
         print *, interp_var_labels(i), mygrid_original % iter(i) - mygrid_duplicate % iter(i)
      end do
   end if


   print *, " "
   print *, "husband's potential ui ben", m_ui_ben_func( mygrid_original, 1 )
   print *, "wife's potential ui ben", ui_ben_func( mygrid_original, 1 )
   print *, "any receiving ui?", (mygrid_original % rv( var_m_ui_ben ) > zero_tol) .or. &
                                 (mygrid_original % rv( var_ui_ben ) > zero_tol)





contains

subroutine gen_mygrid_random_test( mygrid_out, iter_switch_in, rand1_in_opt, rand2_in_opt, rand3_in_opt )

   type(mygrid_type), intent(out) :: mygrid_out
   logical, intent(in) :: iter_switch_in
   real*8, dimension(n_interp_vars_discrete), intent(in), optional :: rand1_in_opt
   real*8, dimension(n_cont_vars), intent(in), optional :: rand2_in_opt
   real*8, dimension( interp_m_ui_ben_pot: interp_ui_left ), intent(in) , optional :: &
      rand3_in_opt


   real*8, parameter :: job_prob = .5
   real*8, parameter :: pt_prob = .5
   real*8, parameter :: no_ui_prob = .5

   real*8, dimension(n_interp_vars_discrete) :: rand_uniform_discrete
   real*8, dimension(n_cont_vars) :: rand_uniform_cont
   real*8, dimension( interp_m_ui_ben_pot: interp_ui_left ) :: rand_iter

   integer :: i, m_job, job, m_no_ui, no_ui, m_pt, pt

   if ( iter_switch .eqv. .true. ) then
      call mygrid_out % iter_allocate()
   else
      call mygrid_out % allocate()
   end if

   if ( present(rand1_in_opt) )then
      rand_uniform_discrete = rand1_in_opt
   else
      call random_number( rand_uniform_discrete )
   end if

   if ( present(rand2_in_opt) )then
      rand_uniform_cont = rand2_in_opt
   else
      call random_number( rand_uniform_cont )
   end if

   if ( present(rand3_in_opt) )then
      rand_iter = rand3_in_opt
   else
      call random_number( rand_iter )
   end if

   rand_uniform_cont = rand_uniform_cont * 2 -1
   rand_iter = rand_iter * 2 -1



   if ( rand_uniform_discrete( interp_m_job ) < job_prob ) then
      m_job = 1
   else
      m_job = 0
   end if

   if ( rand_uniform_discrete( interp_job ) < job_prob ) then
      job = 1
   else
      job = 0
   end if

   mygrid_out % iv( var_m_job )  = m_job
   mygrid_out % iv( var_job )    = job



   if ( m_job .eq. 1 ) then

      if ( rand_uniform_discrete( interp_m_pt ) < pt_prob ) then
         mygrid_out % iv( var_m_hours )  = 20
      else
         mygrid_out % iv( var_m_hours )  = 40
      end if

      m_no_ui = 0

   else

      mygrid_out % iv( var_m_hours )  = 0

      if ( rand_uniform_discrete( interp_m_no_ui ) < no_ui_prob ) then
         m_no_ui = 1
      else
         m_no_ui = 0
      end if

   end if



   if ( job .eq. 1 ) then

      if ( rand_uniform_discrete( interp_pt ) < pt_prob ) then
         mygrid_out % iv( var_hours )  = 20
      else
         mygrid_out % iv( var_hours )  = 40
      end if

      no_ui = 0

   else

      mygrid_out % iv( var_hours )  = 0

      if ( rand_uniform_discrete( interp_no_ui ) < no_ui_prob ) then
         no_ui = 1
      else
         no_ui = 0
      end if

   end if

   print *, m_no_ui, no_ui
   print *, iter_switch_in




   !now get the continuous variables
   do i=1,n_cont_vars
      if ( var_present(i, m_job, job ) .eq. 1) then
         mygrid_out % rv(i) = rand_uniform_cont(i)*grid_sd(i) + grid_mean(i)
      else
         mygrid_out % rv(i) = 0
      end if
   end do


   if( iter_switch_in .eqv. .true. ) then

      if ( m_job .eq. 1 ) then
         mygrid_out % iter( interp_m_ui_ben_pot ) = &
            rand_iter( interp_m_ui_ben_pot ) * grid_sd( var_m_ui_ben ) + grid_mean( var_m_ui_ben )
      else if ( m_no_ui .eq. 0 ) then
         mygrid_out % iter( interp_m_ui_left ) = &
            rand_iter( interp_m_ui_left ) * grid_sd( var_m_unemp_length) + grid_mean( var_m_unemp_length)
      end if

      if ( job .eq. 1 ) then
         mygrid_out % iter( interp_ui_ben_pot ) = &
            rand_iter( interp_ui_ben_pot ) * grid_sd( var_ui_ben ) + grid_mean( var_ui_ben )
      else if ( no_ui .eq. 0 ) then
         mygrid_out % iter( interp_ui_left ) = &
            rand_iter( interp_ui_left ) * grid_sd( var_unemp_length) + grid_mean( var_unemp_length)
      end if

   end if



   !now adjust some of the ui variables
   if ( m_no_ui .eq. 1 ) then
      mygrid_out % rv( var_m_ui_ben ) = 0

   end if

   if ( no_ui .eq. 1 ) then
      mygrid_out % rv( var_ui_ben ) = 0

   end if

   if ( mygrid_out % rv( var_m_unemp_length ) > mygrid_out % rv( var_uilength ) ) then
      mygrid_out % rv( var_m_ui_ben ) = 0

      if ( iter_switch_in .eqv. .true. ) mygrid_out % iter( interp_m_ui_left ) = 0

   end if

   if ( mygrid_out % rv( var_unemp_length ) > mygrid_out % rv( var_uilength ) ) then
      mygrid_out % rv( var_ui_ben ) = 0


      if ( iter_switch_in .eqv. .true. ) mygrid_out % iter( interp_ui_left ) = 0


   end if



end subroutine

subroutine print_interp_vec(interp_vec_in )
   real*8, dimension(n_interp_vars) ::interp_vec_in
   integer :: i

   do i = 1, n_interp_vars_discrete
      print *, interp_var_labels(i), interp_vec_in(i)

   end do

   print *, " "
   do i = interp_m_wage_mean, interp_assets
      print *, interp_var_labels(i), interp_vec_in(i)

   end do

   print *, " "
   do i = interp_m_wage, interp_ui_ben
      print *, interp_var_labels(i), interp_vec_in(i)

   end do

   print *, " "
   do i = interp_m_job_length, interp_unemp_length
      print *, interp_var_labels(i), interp_vec_in(i)

   end do

   print *, " "
   do i = interp_m_ui_ben_pot, interp_ui_left
      print *, interp_var_labels(i), interp_vec_in(i)

   end do



end subroutine print_interp_vec






end program





































!an older way with importing the normals

!   !first get the random numbers
!   call random_number( random_input_matrix )
!   do i=1,n_draws
!      do j=1,5
!         random_normal_matrix(j,i) = rnorm()
!      end do
!   end do


!   do i=1, n_draws
!    !  call random_number( random_input )
!       !  random_input(:) = random_input_matrix(:,i)
!   !   random_input ( iter_m_jobloss ) = 0
!   !   random_input ( iter_m_ui ) = 0

!      call iterate_one_point_normal_in(mygrid, params, ols_matrix_init, random_input_matrix(:,i), &
!         random_normal_matrix(:,i), value_temp)

!      value_origin = value_origin + value_temp
!      value_origin_all(i) = value_temp
!   end do
!   value_origin = value_origin / n_draws


!   print *, "got origin value"
! !  read(*,*)


!   !now get the derivative
!   print *, "first derivative"
!   do k=1,n_cont_vars
!!   do k = var_uimax, var_uirate

!      if (var_present(k,m_job,job)) then
!       !  call read_random_seed()

!      !   print *, "getting derivative for", var_labels(k)

!         mygrid = mygrid_original
!         mygrid % rv(k) = mygrid % rv(k) + der_delta(k)
!         call mygrid % real_to_int_duplicate()

!         value_perturb = 0
!         do i=1, n_draws
!           ! call random_number( random_input )

!       !     random_input(:) = random_input_matrix(:,i)

!        !    random_input ( iter_m_jobloss ) = 0
!        !    random_input ( iter_m_ui ) = 0

!            call iterate_one_point_normal_in(mygrid, params, ols_matrix_init, random_input_matrix(:,i), &
!               random_normal_matrix(:,i), value_temp)

!            value_perturb = value_perturb + value_temp
!            value_perturb_all(i,k) = value_temp
!         end do

!         value_perturb = value_perturb / n_draws

!         value_der1(k) = (value_perturb - value_origin) / der_delta(k)

!         print *, var_labels(k), value_der1(k)

!       !  print *, "got derivative for ", var_labels(k)
!       !  read(*,*)

!      end if

!   end do


!!   print *, "printing difference between origin and peturb values for joboffer_mean"
!!   do i=1,n_draws
!!      print *, i, value_origin_all(i), value_perturb_all(i,2), value_perturb_all(i,2) - value_origin_all(i)
!!   end do




!end program


