program output_graphs

!tests whether the difference in choice probabilities seems reasonable for changes in wage mean, wage, ui, and assets
!derivatives represent marginal effects at the mean

use enum_mod
use model_params_mod
use interp_support_mod

use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use likelihood_mod
use sipp_data_mod

use normal_mod
use stat_estimates_mod

use csv_file
use output_mod

!use statespace_trans_params_mod


implicit none

   character(len=30), parameter :: init_params_file       = "output_params.txt"
   character(len=30), parameter :: interact_file_in       = "interact_matrix.csv"
   character(len=30), parameter :: value_matrix_init_file = "value_matrix_output.csv"
   character(len=30), parameter :: asset_matrix_init_file = "asset_matrix_output.csv"

   character(len=output_len), parameter :: husband_title = "P( Husband Accepts a Job Offer  )"
   !   "P( Husband Accepts a Joboffer | Only Husband Received an Offer this Month )"

   character(len=output_len), parameter :: wife_title = "P( Wife Accepts a Job Offer )"
   !   "P( Wife Accepts a Joboffer | Only Wife Received an Offer this Month )"

   character(len=output_len), parameter :: husband_save_file = "husband_cp"
   character(len=output_len), parameter :: wife_save_file =    "wife_cp"

   integer, parameter :: n_x_nodes = 10
   integer, parameter :: n_z_nodes = 4

   real*8, dimension(n_params) :: params
   real*8, dimension( n_grid_points ) :: value_matrix
   real*8, dimension( n_grid_points ) :: asset_matrix

   real*8 :: m_wage_new, wage_new, m_non_pec_new, non_pec_new
   integer :: m_hours_new, hours_new

   type(mygrid_type) :: mygrid_object
   type(graph_csv_obj_type) :: graph_obj

   !first some setup
   call init_params(init_params_file, graph_obj % params)

   call gen_var_labels()
   call gen_var_labels_long()
   call der_delta_setup()
   call interp_list_setup_all(interact_file_in)

   call init_interp_matrices( value_matrix_init_file, asset_matrix_init_file, &
      graph_obj % value_matrix, graph_obj % asset_matrix)


   !get the "orgin" value of all my variables

   m_wage_new  = grid_mean( var_m_wage )
   wage_new    = grid_mean( var_wage )

   m_non_pec_new  = grid_mean( var_m_non_pec_norm )
   non_pec_new    = grid_mean( var_non_pec_norm )

   m_hours_new = grid_mean( var_m_hours )
   hours_new   = grid_mean( var_hours )

   call get_mygrid_origin(0, 0, mygrid_object)

   !have wife be a stay at home mom
   mygrid_object % rv( var_ui_ben ) = 0
 !  mygrid_object % rv( var_unemp_length ) = grid_min( var_unemp_length )

  ! mygrid_object % rv( var_m_unemp_length ) = grid_min( var_unemp_length )


   call choice_prob_der_setup( mygrid_object, m_wage_new, wage_new, m_non_pec_new, &
      non_pec_new, m_hours_new, hours_new, graph_obj % like_original )

   !done with setting up the "origin" value



   !inputs that I could change for other programs
   graph_obj % title = husband_title
   graph_obj % dv =  dv_choice_prob_husband
   graph_obj % two_vars = .true.
   graph_obj % n_x_nodes = n_x_nodes
   graph_obj % n_z_nodes = n_z_nodes
   allocate( graph_obj % x_nodes( graph_obj % n_x_nodes ) )
   allocate( graph_obj % z_nodes( graph_obj % n_z_nodes ) )


   !now for ui_ben and assets
   graph_obj % x_in = var_m_ui_ben
   graph_obj % z_in = var_assets
   graph_obj % save_file = save_file_func( husband_save_file,  graph_obj % x_in, graph_obj % z_in )

   graph_obj % x_nodes = nint( evenly_spaced_func( graph_obj % n_x_nodes, 0.0D+0, 350.0D+0 ) )
   graph_obj % z_nodes = nint( evenly_spaced_func( graph_obj % n_z_nodes, 1.0D+0, 1500.0D+0 ) )

   call graph_csv_gen( graph_obj )

   print *, "created file", graph_obj % save_file


   !now for husbands's new wage and ui_mean
   graph_obj % x_in = var_m_wage_new
   graph_obj % z_in = var_m_wage_mean
   graph_obj % save_file = save_file_func( husband_save_file,  graph_obj % x_in, graph_obj % z_in )



   graph_obj % x_nodes = nint( evenly_spaced_func( graph_obj % n_x_nodes, &
      grid_min(var_m_wage), grid_max(var_m_wage) ) )
   graph_obj % z_nodes = nint( exp( evenly_spaced_func( graph_obj % n_z_nodes, &
      grid_min(var_m_wage_mean) +1, grid_max(var_m_wage_mean) ) ) )

   call graph_csv_gen( graph_obj )

   print *, "created file", graph_obj % save_file






   !now for wife's stuff
   graph_obj % title = wife_title
   graph_obj % dv =  dv_choice_prob_wife

   !now for ui_ben and assets
   graph_obj % x_in = var_m_ui_ben
   graph_obj % z_in = var_assets
   graph_obj % save_file = save_file_func( wife_save_file,  graph_obj % x_in, graph_obj % z_in )

   graph_obj % x_nodes = nint( evenly_spaced_func( graph_obj % n_x_nodes, 0.0D+0, 350.0D+0 ) )
   graph_obj % z_nodes = nint( evenly_spaced_func( graph_obj % n_z_nodes, 1.0D+0, 1500.0D+0 ) )

   call graph_csv_gen( graph_obj )

   print *, "created file", graph_obj % save_file


   !now for wife's new wage and husban'd wage mean
   graph_obj % x_in = var_wage_new
   graph_obj % z_in = var_m_wage_mean
   graph_obj % save_file = save_file_func( wife_save_file,  graph_obj % x_in, graph_obj % z_in )

   graph_obj % x_nodes = nint( evenly_spaced_func( graph_obj % n_x_nodes, &
      grid_min(var_wage), grid_max(var_wage) ) )
   graph_obj % z_nodes =  nint( exp( evenly_spaced_func( graph_obj % n_z_nodes, &
      grid_min(var_m_wage_mean) +1 , grid_max(var_m_wage_mean) ) ) )

   call graph_csv_gen( graph_obj )

   print *, "created file", graph_obj % save_file




end program






