program output_graphs

!tests whether the difference in choice probabilities seems reasonable for changes in wage mean, wage, ui, and assets
!derivatives represent marginal effects at the mean

use enum_mod
use model_params_mod
use interp_support_mod

use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use likelihood_mod
use sipp_data_mod

use normal_mod
use stat_estimates_mod

use csv_file
use output_mod

!use statespace_trans_params_mod


implicit none

   character(len=30), parameter :: init_params_file       = "output_params.txt"
   character(len=30), parameter :: interact_file_in       = "interact_matrix.csv"
   character(len=30), parameter :: value_matrix_init_file = "value_matrix_output.csv"
   character(len=30), parameter :: asset_matrix_init_file = "asset_matrix_output.csv"

   character(len=output_len), parameter :: base_vals_file = "./graphs/graph2.0.base_values.csv"
   character(len=output_len), parameter :: save_base = "graph2.0"
   character(len=output_len), parameter :: subtitle = "Base Case: No UI with Assets"
   character(len=output_len), parameter :: label1 = "Both Not Working Both No UI"
   character(len=output_len), parameter :: label2 = "Only Wife Working Husband No UI"
   character(len=output_len), parameter :: label3 = "Only Husband Working Wife No UI"

   integer, parameter :: n_nodes = 10
   integer, parameter :: n_lines = 3
   logical, parameter :: max_switch = .false.


   real*8 :: m_wage_new, wage_new, m_non_pec_new, non_pec_new
   integer :: m_hours_new, hours_new, m_job, job

   type(mygrid_type) :: mygrid_object
   type(graph_obj_type2) :: graph_obj

   real*8, dimension(n_like_vars) :: grid_mean_new
   real*8, dimension(n_lines) :: m_prob_base, prob_base

   !first some setup

   call gen_var_labels()
   call gen_var_labels_long()
   call der_delta_setup()
   call gen_param_labels
   call interp_setup_all(interact_file_in)

   call graph_obj % setup( n_lines, n_nodes, max_switch )

   call init_params(init_params_file, graph_obj % params)
   call init_interp_matrices( value_matrix_init_file, asset_matrix_init_file, &
      graph_obj % value_matrix, graph_obj % asset_matrix)

   graph_obj % subtitle = subtitle
   graph_obj % save_file_base = save_base
   graph_obj % labels(1) = label1
   graph_obj % labels(2) = label2
   graph_obj % labels(3) = label3


   !now set up the base case values
   grid_mean_new(:n_cont_vars) = grid_mean(:n_cont_vars)

   grid_mean_new( var_m_wage_new )           = grid_mean( var_m_wage )
   grid_mean_new( var_m_non_pec_norm_new )   = grid_mean( var_m_non_pec_norm )
   grid_mean_new( var_wage_new )             = grid_mean( var_wage )
   grid_mean_new( var_non_pec_norm_new )     = grid_mean( var_non_pec_norm )

   !now change some of these mean values
  ! grid_mean_new( var_m_ui_ben ) = 0
  ! grid_mean_new( var_ui_ben ) = 0

   grid_mean_new( var_m_unemp_length ) = 4
   grid_mean_new( var_unemp_length ) = 4

  ! grid_mean_new( var_assets ) =  2000 !!! max(grid_mean( var_assets) - 20000 - 9000 , 1.0D+0)  !.5 * grid_sd( var_assets )  !10000
   grid_mean_new( var_ur_state ) = 7



   !set new variable values equal to origin values

   m_wage_new  = grid_mean_new( var_m_wage_new )
   wage_new    = grid_mean_new( var_wage_new )

   m_non_pec_new  = grid_mean_new( var_m_non_pec_norm_new )
   non_pec_new    = grid_mean_new( var_non_pec_norm_new )

   m_hours_new = 40
   hours_new   = 40


   !!!!!!!!!!!!!!!!!!!!!!!
   !line 1

   !first like object of both not working
   m_job = 0
   job = 0

   call get_mygrid_origin_output(m_job, job, grid_mean_new, mygrid_object )

   call mini_setup(1)




   !!!!!!!!!!!!!!!!!!!!!!!!!!
   !line 2

   !wife working
   m_job = 0
   job = 1

   call get_mygrid_origin_output(m_job, job, grid_mean_new, mygrid_object )

   call mini_setup(2)




   !!!!!!!!!!!!!!!!!!!!!!!!!!
   !line 3

   !husband working
   m_job = 1
   job = 0

   call get_mygrid_origin_output(m_job, job, grid_mean_new, mygrid_object )

   call mini_setup(3)




   !now generate graphs
   call graph_obj % gen( "m" )
   print *, "generated husband's graphs"

   call graph_obj % gen( "f" )
   print *, "generated wife's graphs"

   call write_base_val( base_vals_file, grid_mean_new, n_lines, graph_obj % labels, m_prob_base, prob_base )
   print *, "wrote base values"


contains

subroutine mini_setup(i_in)
   !little setupt routines to save me repeating so much code
   integer, intent(in) :: i_in

   call choice_prob_der_setup( mygrid_object, m_wage_new, wage_new, m_non_pec_new, &
      non_pec_new, m_hours_new, hours_new, graph_obj % like_obj(i_in) )

   m_prob_base(i_in) = choice_prob_output_func( dv_choice_prob_husband, &
                                                graph_obj % max_switch, &
                                                graph_obj % like_obj(i_in), &
                                                graph_obj % params, &
                                                graph_obj % value_matrix, &
                                                graph_obj % asset_matrix )

   prob_base(i_in) = choice_prob_output_func(   dv_choice_prob_wife, &
                                                graph_obj % max_switch, &
                                                graph_obj % like_obj(i_in), &
                                                graph_obj % params, &
                                                graph_obj % value_matrix, &
                                                graph_obj % asset_matrix )

end subroutine mini_setup




end program






