
module output_mod

   !module for creating graphs and other interpretable output from estimates of my model
use enum_mod
use model_params_mod
use interp_support_mod

use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use normal_mod
use stat_estimates_mod

use likelihood_mod
use bayesian_mod

use csv_file
use random_gen_mod
use mpi_mod

   integer, parameter :: n_labels = 4
   integer, parameter :: n_labels2_base = 3


   type :: graph_csv_obj_type !contains variables used for graphing output

      character(len=output_len) :: title
      type(like_type) :: like_original !the likelihood type that contains the original values
      character(len=output_len) :: save_file

      logical :: max_switch !controls whether to find assets via maximization

      integer :: dv !the dependent variable, such as choice probabilities
      integer :: x_in !the independent variable that will appear on the x axix
      integer :: z_in !the independent variable that will shift the lines
      logical  :: two_vars !says whether there should be one or two variables

      integer :: n_x_nodes
      integer :: n_z_nodes
      real*8, dimension(:), allocatable :: x_nodes
      real*8, dimension(:), allocatable :: z_nodes

      real*8, dimension(n_params) :: params
      real*8, dimension(n_grid_points) :: value_matrix
      real*8, dimension(n_grid_points) :: asset_matrix

   contains
      procedure :: graph_csv_gen => graph_csv_gen

   end type


   type :: graph_obj_type2 !contains variables used for graphing output

      character(len=output_len) :: subtitle
      character(len=output_len) :: save_file_base

      logical :: max_switch !controls whether to find assets via maximization

      integer :: n_lines !number of lines to draws per graph, which represent different types of households
      type(like_type), dimension(:), allocatable :: like_obj !the likelihood objects for the different lines
      character(len=output_len), dimension(:), allocatable :: labels

      integer :: n_nodes !number of nodes per horizontal axis
      real*8, dimension(:,:), allocatable :: x_grid !grid for explanatory variables

      real*8, dimension(n_params) :: params
      real*8, dimension(n_grid_points) :: value_matrix
      real*8, dimension(n_grid_points) :: asset_matrix

   contains
      procedure :: setup => graph_obj2_setup
      procedure :: gen => graph_csv_gen2
      procedure :: gen_value => graph_csv_value

   end type


contains


subroutine get_mygrid_origin_output(m_job_in, job_in, grid_mean_new_in, mygrid_origin_out)
   !same as  get_mygrid_origin but with being able to import a modified grid mean
   integer, intent(in) :: job_in
   integer, intent(in) :: m_job_in
   real*8, dimension( n_like_vars ), intent(in) :: grid_mean_new_in
   type(mygrid_type), intent(out) :: mygrid_origin_out

   integer :: i

   call mygrid_origin_out % allocate()

   do i=1,n_cont_vars
      if ( var_present(i, m_job_in, job_in ) .eq. 1) then
         mygrid_origin_out % rv(i) = grid_mean_new_in(i)
      else
         mygrid_origin_out % rv(i) = 0
      end if
   end do

   mygrid_origin_out % iv(var_m_job) = m_job_in
   mygrid_origin_out % iv(var_job) = job_in

   if ( m_job_in .eq. 1) then
      mygrid_origin_out % iv(var_m_hours) = 40
   else
      mygrid_origin_out % iv(var_m_hours) = 0
   end if

   if ( job_in .eq. 1) then
      mygrid_origin_out % iv(var_hours) = 40
   else
      mygrid_origin_out % iv(var_hours) = 0
   end if


end subroutine get_mygrid_origin_output

subroutine write_base_val( filename_in, grid_mean_new_in, n_in, labels_in, m_prob_in, prob_in )
   !writes the base values (grid values and their probabilities ) to a file
   character(len = output_len), intent(in) :: filename_in
   real*8, dimension( n_like_vars ), intent(in) :: grid_mean_new_in
   integer, intent(in) :: n_in
   character(len = output_len ), dimension(n_in) :: labels_in
   real*8, dimension(n_in), intent(in) :: m_prob_in, prob_in

   integer :: unit_int
   integer :: i


   open(newunit = unit_int, file = filename_in, status = "replace" )

   call csv_write( unit_int, "Base Case Values", .true. )
   call csv_next_record( unit_int )

   do i=1,n_cont_vars
      call csv_write( unit_int, var_labels_long(i), .false. )
      call csv_write( unit_int, grid_mean_new_in(i), .true. )
   end do

   call csv_next_record(unit_int)
   call csv_write( unit_int, "Husband base probability", .true. )
   do i=1,n_in
      call csv_write( unit_int, labels_in(i), .false. )
      call csv_write( unit_int, m_prob_in(i), .true. )
   end do

   call csv_next_record(unit_int)
   call csv_write( unit_int, "Wife base probability", .true. )
   do i=1,n_in
      call csv_write( unit_int, labels_in(i), .false. )
      call csv_write( unit_int, prob_in(i), .true. )
   end do

   close(unit_int)

end subroutine write_base_val


subroutine like_single_setup( mygrid_in, m_wage_new_in, wage_new_in, m_non_pec_new_in, &
   non_pec_new_in, m_hours_new_in, hours_new_in, like_object_out )

   !sets up a likelihood object for only one point in time

   type(mygrid_type), intent(in) :: mygrid_in
   real*8, intent(in) :: m_wage_new_in, wage_new_in, m_non_pec_new_in, non_pec_new_in
   integer, intent(in) :: m_hours_new_in, hours_new_in
   type(like_type), intent(out) :: like_object_out


   !setup the like_object_out of interest
   allocate( like_object_out % m_receive_new(1) )
   allocate( like_object_out % receive_new(1) )
   allocate( like_object_out % m_accept_new(1) )
   allocate( like_object_out % accept_new(1) )

   allocate( like_object_out % mygrid_t_invar )
   allocate( like_object_out % mygrid_t_var(1) )
   allocate( like_object_out % mygrid_husband_accept(1) )
   allocate( like_object_out % mygrid_wife_accept(1) )
   allocate( like_object_out % mygrid_both_accept(1) )

   allocate( like_object_out % utility_params_created(1) )
   allocate( like_object_out % utility_params(n_utility_params, 1) )
   allocate( like_object_out % utility_both_reject(1) )
   allocate( like_object_out % utility_husband_accept(1) )
   allocate( like_object_out % utility_wife_accept(1) )
   allocate( like_object_out % utility_both_accept(1) )

   allocate( like_object_out % asset_choice_both_reject(1) )
   allocate( like_object_out % asset_choice_husband_accept(1) )
   allocate( like_object_out % asset_choice_wife_accept(1) )
   allocate( like_object_out % asset_choice_both_accept(1) )

   allocate( like_object_out % choice_prob(1) )

   like_object_out % utility_params_created = 1

   !now setup the mygrids
   !first mygrid_t_invar and mygrid_t_var

   call like_object_out % mygrid_t_invar % allocate()
   call like_object_out % mygrid_t_var(1) % t_var_allocate()

   like_object_out % mygrid_t_invar % rv( : n_t_invar ) = mygrid_in % rv( : n_t_invar )

   like_object_out % mygrid_t_var(1) % rv(  n_t_invar + 1 :  ) = mygrid_in % rv( n_t_invar + 1 : )

   like_object_out % mygrid_t_var(1) % iv =  mygrid_in % iv


   !mygrid husband accept

   like_object_out % mygrid_husband_accept(1) = like_object_out % mygrid_t_var(1)

   like_object_out % mygrid_husband_accept(1) % rv( var_m_wage )          = m_wage_new_in
   like_object_out % mygrid_husband_accept(1) % rv( var_m_non_pec_norm )  = m_non_pec_new_in
   like_object_out % mygrid_husband_accept(1) % rv( var_m_job_length )    = 0

   like_object_out % mygrid_husband_accept(1) % rv( var_m_ui_ben )         = 0
   like_object_out % mygrid_husband_accept(1) % rv( var_m_unemp_length )   = 0

   like_object_out % mygrid_husband_accept(1) % iv( var_m_hours )         = m_hours_new_in
   like_object_out % mygrid_husband_accept(1) % iv( var_m_job )           = 1


   !mygrid wife accept
   like_object_out % mygrid_wife_accept(1) = like_object_out % mygrid_t_var(1)

   like_object_out % mygrid_wife_accept(1) % rv( var_wage )          = wage_new_in
   like_object_out % mygrid_wife_accept(1) % rv( var_non_pec_norm )  = non_pec_new_in
   like_object_out % mygrid_wife_accept(1) % rv( var_job_length )    = 0

   like_object_out % mygrid_wife_accept(1) % rv( var_ui_ben )         = 0
   like_object_out % mygrid_wife_accept(1) % rv( var_unemp_length )   = 0

   like_object_out % mygrid_wife_accept(1) % iv( var_hours )         = hours_new_in
   like_object_out % mygrid_wife_accept(1) % iv( var_job )           = 1

   !mygrid both accept
   like_object_out % mygrid_both_accept(1) = like_object_out % mygrid_t_var(1)

   like_object_out % mygrid_both_accept(1) % rv( var_m_wage )          = m_wage_new_in
   like_object_out % mygrid_both_accept(1) % rv( var_m_non_pec_norm )  = m_non_pec_new_in
   like_object_out % mygrid_both_accept(1) % rv( var_m_job_length )    = 0

   like_object_out % mygrid_both_accept(1) % rv( var_m_ui_ben )         = 0
   like_object_out % mygrid_both_accept(1) % rv( var_m_unemp_length )   = 0

   like_object_out % mygrid_both_accept(1) % iv( var_m_hours )         = m_hours_new_in
   like_object_out % mygrid_both_accept(1) % iv( var_m_job )           = 1

   like_object_out % mygrid_both_accept(1) % rv( var_wage )          = wage_new_in
   like_object_out % mygrid_both_accept(1) % rv( var_non_pec_norm )  = non_pec_new_in
   like_object_out % mygrid_both_accept(1)   % rv( var_job_length )    = 0

   like_object_out % mygrid_both_accept(1) % rv( var_ui_ben )         = 0
   like_object_out % mygrid_both_accept(1) % rv( var_unemp_length )   = 0

   like_object_out % mygrid_both_accept(1) % iv( var_hours )         = hours_new_in
   like_object_out % mygrid_both_accept(1) % iv( var_job )           = 1


end subroutine like_single_setup

subroutine like_single_duplicate(like_object_in, like_object_out)
   !copies over values for my like single object, with the components of these two objects having the same values but
      !different addresses at the end
   !this is needed because doing like_duplicate = like_original just copies the memory address locations for the pointer
      !(even if I allocated these pointer), which is not what I want

   type(like_type), intent(in) :: like_object_in
   type(like_type), intent(out) :: like_object_out


   allocate( like_object_out % m_receive_new(1) )
   allocate( like_object_out % receive_new(1) )
   allocate( like_object_out % m_accept_new(1) )
   allocate( like_object_out % accept_new(1) )

   allocate( like_object_out % mygrid_t_invar )
   allocate( like_object_out % mygrid_t_var(1) )
   allocate( like_object_out % mygrid_husband_accept(1) )
   allocate( like_object_out % mygrid_wife_accept(1) )
   allocate( like_object_out % mygrid_both_accept(1) )

   allocate( like_object_out % utility_params_created(1) )
   allocate( like_object_out % utility_params(n_utility_params, 1) )
   allocate( like_object_out % utility_both_reject(1) )
   allocate( like_object_out % utility_husband_accept(1) )
   allocate( like_object_out % utility_wife_accept(1) )
   allocate( like_object_out % utility_both_accept(1) )

   allocate( like_object_out % asset_choice_both_reject(1) )
   allocate( like_object_out % asset_choice_husband_accept(1) )
   allocate( like_object_out % asset_choice_wife_accept(1) )
   allocate( like_object_out % asset_choice_both_accept(1) )

   allocate( like_object_out % choice_prob(1) )

   like_object_out % m_receive_new(1) = like_object_in % m_receive_new(1)
   like_object_out % receive_new(1) = like_object_in % receive_new(1)
   like_object_out % m_accept_new(1) = like_object_in % m_accept_new(1)
   like_object_out % accept_new(1) = like_object_in % accept_new(1)

   like_object_out % mygrid_t_invar = like_object_in % mygrid_t_invar
   like_object_out % mygrid_t_var(1) = like_object_in % mygrid_t_var(1)
   like_object_out % mygrid_husband_accept(1) = like_object_in % mygrid_husband_accept(1)
   like_object_out % mygrid_wife_accept(1) = like_object_in % mygrid_wife_accept(1)
   like_object_out % mygrid_both_accept(1) = like_object_in % mygrid_both_accept(1)

   like_object_out % utility_params_created(1) = like_object_in % utility_params_created(1)
   like_object_out % utility_params(n_utility_params, 1) = like_object_in % utility_params(n_utility_params, 1)
   like_object_out % utility_both_reject(1) = like_object_in % utility_both_reject(1)
   like_object_out % utility_husband_accept(1) = like_object_in % utility_husband_accept(1)
   like_object_out % utility_wife_accept(1) = like_object_in % utility_wife_accept(1)
   like_object_out % utility_both_accept(1) =  like_object_in % utility_both_accept(1)

   like_object_out % asset_choice_both_reject(1) = like_object_in % asset_choice_both_reject(1)
   like_object_out % asset_choice_husband_accept(1) = like_object_in % asset_choice_husband_accept(1)
   like_object_out % asset_choice_wife_accept(1) = like_object_in % asset_choice_wife_accept(1)
   like_object_out % asset_choice_both_accept(1) = like_object_in % asset_choice_both_accept(1)

   like_object_out % choice_prob(1) = like_object_in % choice_prob(1)


end subroutine like_single_duplicate


function choice_prob_output_func( dv_in, max_switch_in, like_object_inout, params_in, value_matrix_in, asset_matrix_in ) &
   result(choice_prob)
   !this gets the choice probability for my output by evaluating the utility function, asset choice, and then
      !finally the choice probability

   real*8 :: choice_prob
   integer, intent(in) :: dv_in
   logical, intent(in) :: max_switch_in  !controls whether to get assets through maximization or through interpolation
   type(like_type), intent(in out) :: like_object_inout
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension( n_grid_points ), intent(in) :: value_matrix_in
   real*8, dimension( n_grid_points ), intent(in) :: asset_matrix_in

   if ( dv_in .eq. dv_choice_prob_husband ) then

      like_object_inout % m_receive_new(1) = 1
      like_object_inout % receive_new(1) = 0
      like_object_inout % m_accept_new(1) = 1
      like_object_inout % accept_new(1) = 0

   else if ( dv_in .eq. dv_choice_prob_wife ) then

      like_object_inout % m_receive_new(1) = 0
      like_object_inout % receive_new(1) = 1
      like_object_inout % m_accept_new(1) = 0
      like_object_inout % accept_new(1) = 1

   else
      print *, "error in choice_prob_output_func "
      stop
   end if


   call utility_function_setup(  like_object_inout % mygrid_t_var(1) % rv(var_kids), &
                                 like_object_inout % mygrid_t_var(1) % rv(var_youngest), &
                                 params_in, &
                                 like_object_inout % utility_params(:,1) )

   if ( max_switch_in .eqv. .true. ) then
      call asset_choice_via_max( like_object_inout, params_in, value_matrix_in, 1 )
   else
      call asset_choice_like_update_one_t(like_object_inout, asset_matrix_in, 1)
   end if

   call utility_like_update_one_t(like_object_inout, params_in,1)


   call choice_prob_like_update_one_t(like_object_inout,value_matrix_in,1)



contains

subroutine asset_choice_via_max( like_object_inout, params_in, value_matrix_in, t )
   !gets the asset choice via maximization
   type( like_type), intent(in out) :: like_object_inout
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
   integer, intent(in) :: t

   real*8 :: asset_temp, value_temp
   type( mygrid_type) :: mygrid_temp

   call mygrid_temp % allocate()
   mygrid_temp % rv( : n_t_invar) = like_object_inout % mygrid_t_invar % rv( : n_t_invar )




   ! print *, "both reject"
   mygrid_temp % rv( n_t_invar + 1: ) = like_object_inout % mygrid_t_var(t) % rv( n_t_invar + 1: )
   mygrid_temp % iv = like_object_inout % mygrid_t_var(t) % iv

   call max_value_sub(  mygrid_temp, &
                        like_object_inout % utility_params(:,t), &
                        params_in, &
                        value_matrix_in, &
                        asset_temp, &
                        value_temp )

   like_object_inout % asset_choice_both_reject(t) = asset_temp





   !  print *, "husband accept"
   if(like_object_inout % m_receive_new(t) .eq. 1)  then

      mygrid_temp % rv( n_t_invar + 1: ) = &
         like_object_inout % mygrid_husband_accept(t) % rv( n_t_invar + 1: )
      mygrid_temp % iv = like_object_inout % mygrid_husband_accept(t) % iv

      call max_value_sub(  mygrid_temp, &
                           like_object_inout % utility_params(:,t), &
                           params_in, &
                           value_matrix_in, &
                           asset_temp, &
                           value_temp )

      like_object_inout % asset_choice_husband_accept(t) = asset_temp

   end if

   ! print *, "wife accept"
   if(like_object_inout % receive_new(t) .eq. 1)  then

      mygrid_temp % rv( n_t_invar + 1: ) = &
         like_object_inout % mygrid_wife_accept(t) % rv( n_t_invar + 1: )
      mygrid_temp % iv = like_object_inout % mygrid_wife_accept(t) % iv

      call max_value_sub(  mygrid_temp, &
                           like_object_inout % utility_params(:,t), &
                           params_in, &
                           value_matrix_in, &
                           asset_temp, &
                           value_temp )

      like_object_inout % asset_choice_wife_accept(t) = asset_temp

   end if

   !  print *, "both accept"
   if (   (like_object_inout % m_receive_new(t) .eq. 1) .and. &
         (like_object_inout % receive_new(t) .eq. 1) ) then


      mygrid_temp % rv( n_t_invar + 1: ) = &
         like_object_inout % mygrid_both_accept(t) % rv( n_t_invar + 1: )
      mygrid_temp % iv = like_object_inout % mygrid_both_accept(t) % iv

      call max_value_sub(  mygrid_temp, &
                           like_object_inout % utility_params(:,t), &
                           params_in, &
                           value_matrix_in, &
                           asset_temp, &
                           value_temp )

      like_object_inout % asset_choice_both_accept(t) = asset_temp

   end if



end subroutine asset_choice_via_max

end function choice_prob_output_func


function value_arg_output_func( dv_in, asset_switch_in, like_object_inout, params_in, &
   value_matrix_in, asset_matrix_in ) result(value_vec)
   !prints out the value functions that go into the logit probabilites

   real*8, dimension(:), allocatable :: value_vec
   integer, intent(in) :: dv_in
   logical, intent(in) :: asset_switch_in !whether to do assets or not
   type(like_type), intent(in out) :: like_object_inout
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension( n_grid_points ), intent(in) :: value_matrix_in
   real*8, dimension( n_grid_points ), intent(in) :: asset_matrix_in

   if ( dv_in .eq. dv_choice_prob_husband ) then

      like_object_inout % m_receive_new(1) = 1
      like_object_inout % receive_new(1) = 0
      like_object_inout % m_accept_new(1) = 1
      like_object_inout % accept_new(1) = 0

   else if ( dv_in .eq. dv_choice_prob_wife ) then

      like_object_inout % m_receive_new(1) = 0
      like_object_inout % receive_new(1) = 1
      like_object_inout % m_accept_new(1) = 0
      like_object_inout % accept_new(1) = 1

   else
      print *, "error in choice_prob_output_func "
      stop
   end if


   if (asset_switch_in .eqv. .true. ) then

      call utility_function_setup(  like_object_inout % mygrid_t_var(1) % rv(var_kids), &
                                    like_object_inout % mygrid_t_var(1) % rv(var_youngest), &
                                    params_in, &
                                    like_object_inout % utility_params(:,1) )

      call asset_choice_like_update_one_t(like_object_inout, asset_matrix_in, 1)

      call utility_like_update_one_t(like_object_inout, params_in,1)

   else
      !make asset values unchanged

     like_object_inout % asset_choice_both_reject(1) = &
         like_object_inout % mygrid_t_var(1) % rv(var_assets )

      !  print *, "husband accept"
      like_object_inout % asset_choice_husband_accept(1) = &
         like_object_inout % mygrid_husband_accept(1) % rv(var_assets)

      ! print *, "wife accept"
      like_object_inout % asset_choice_wife_accept(1) = &
         like_object_inout % mygrid_wife_accept(1) % rv(var_assets)


      !  print *, "both accept"
      like_object_inout % asset_choice_both_accept(1) = &
         like_object_inout % mygrid_both_accept(1) % rv(var_assets )


   end if

   call value_args_one_t(like_object_inout, value_matrix_in, 1, value_vec)


end function value_arg_output_func


subroutine perturb_mygrid( k, new_val, like_obj_inout )
   !for peterbing the points

   integer, intent(in) :: k !which variable to perturb
   real*8, intent(in) :: new_val !the new value for this variable
   type(like_type), intent(in out) :: like_obj_inout

   integer :: k2



   select case(k)

      case ( : var_wage_mean )

         like_obj_inout % mygrid_t_invar % rv(k) =  log( new_val ) !important, I import the wage distribution mean(not log), and then
            ! I take log of it here.

      case (var_ur_state : var_assets)

         like_obj_inout % mygrid_t_var(1) % rv(k)           =  new_val
         like_obj_inout % mygrid_husband_accept(1) % rv(k)  =  new_val
         like_obj_inout % mygrid_wife_accept(1) % rv(k)     =  new_val
         like_obj_inout % mygrid_both_accept(1) % rv(k)     =  new_val

      case ( var_m_wage,  var_m_non_pec_norm, var_m_job_length )

         if( like_obj_inout % mygrid_t_var(1) % iv( var_m_job ) .eq. 1) then

            like_obj_inout % mygrid_t_var(1) % rv(k)        =  new_val
            like_obj_inout % mygrid_wife_accept(1) % rv(k)  =  new_val

         end if

      case ( var_m_ui_ben, var_m_unemp_length  )

         if( like_obj_inout % mygrid_t_var(1) % iv( var_m_job ) .eq. 0) then

            like_obj_inout % mygrid_t_var(1) % rv(k)        =  new_val
            like_obj_inout % mygrid_wife_accept(1) % rv(k)  = new_val

         end if

      case ( var_wage,  var_non_pec_norm, var_job_length )

         if( like_obj_inout % mygrid_t_var(1) % iv( var_job ) .eq. 1) then

            like_obj_inout % mygrid_t_var(1) % rv(k)           =  new_val
            like_obj_inout % mygrid_husband_accept(1) % rv(k) = new_val

         end if

      case ( var_ui_ben, var_unemp_length )

         if( like_obj_inout % mygrid_t_var(1) % iv( var_job ) .eq. 0) then

            like_obj_inout % mygrid_t_var(1) % rv(k)           = new_val
            like_obj_inout % mygrid_husband_accept(1) % rv(k)  = new_val

         end if

      case ( var_m_wage_new,  var_m_non_pec_norm_new )

         k2= k - der_vars_convert !gets the appropriate variable index from k2

         like_obj_inout % mygrid_husband_accept(1) % rv(k2) = new_val
         like_obj_inout % mygrid_both_accept(1) % rv(k2)    = new_val

      case ( var_wage_new,  var_non_pec_norm_new )

         k2= k - der_vars_convert !gets the appropriate variable index from k2

         like_obj_inout % mygrid_wife_accept(1) % rv(k2) = new_val
         like_obj_inout % mygrid_both_accept(1) % rv(k2) = new_val

      case default
         print *, "error in select case for getting the choice probabilitiy derivatives"
         stop

   end select

end subroutine perturb_mygrid

function like_var_present(index_in,m_job_in, job_in, m_receive_in, receive_in)
   !says whether a variable is present
   integer :: like_var_present
   integer, intent(in) :: index_in, m_job_in, job_in, m_receive_in, receive_in

   select case(index_in)
      case( :n_cont_vars)
         like_var_present = var_present(index_in,m_job_in,job_in )

      case(var_m_wage_new, var_m_non_pec_norm_new)
         like_var_present = m_receive_in

      case(var_wage_new, var_non_pec_norm_new)
         like_var_present = receive_in

      case default
         print *, "error in select case"
         stop

   end select


end function like_var_present

function like_var_present2(index_in,m_job_in, job_in, m_receive_in, receive_in) result(like_var_present)
   !same as above function but removing most variables from a job to new job transition
   integer :: like_var_present
   integer, intent(in) :: index_in, m_job_in, job_in, m_receive_in, receive_in

   select case(index_in)
      case( :n_cont_vars)
         like_var_present = var_present(index_in,m_job_in,job_in )

      case(var_m_wage_new, var_m_non_pec_norm_new)
         like_var_present = m_receive_in

      case(var_wage_new, var_non_pec_norm_new)
         like_var_present = receive_in

      case default
         print *, "error in select case"
         stop

   end select

   if ( ( m_job_in .eq. 1) .and. ( m_receive_in .eq. 1 ) ) then
      if (  ( index_in .eq. var_m_wage) .or. ( index_in .eq. var_m_non_pec_norm) .or. &
            ( index_in .eq. var_m_wage_new)  .or. ( index_in .eq. var_m_non_pec_norm_new) ) then

         like_var_present = 1
      else
         like_var_present = 0
      end if
   end if

   if ( ( job_in .eq. 1) .and. ( receive_in .eq. 1 ) ) then
      if (  ( index_in .eq. var_wage) .or. ( index_in .eq. var_non_pec_norm) .or. &
            ( index_in .eq. var_wage_new)  .or. ( index_in .eq. var_non_pec_norm_new) ) then

         like_var_present = 1
      else
         like_var_present = 0
      end if
   end if

   !turn of kids variables and non-pec benefit
   select case(index_in)
      case(var_youngest, var_kids, var_m_non_pec_norm, var_non_pec_norm, &
         var_m_non_pec_norm_new, var_non_pec_norm_new )

         like_var_present = 0

   end select




end function like_var_present2



subroutine graph_csv_gen(  obj_in )
   !generates a csv file for graphing output from my model
   class(graph_csv_obj_type), intent(in) :: obj_in

   integer :: i, j
   real*8, dimension( obj_in % n_x_nodes, obj_in % n_z_nodes ) :: y !contains the dependent variable of interest
   type(like_type) :: like_temp
   character(len=output_len), dimension(n_labels) :: label_info
   integer :: unit_int


 !  call gen_var_labels_long() !just to make sure this is called
   call like_single_duplicate( obj_in % like_original, like_temp)



   if (  ( obj_in % dv .eq. dv_choice_prob_husband ) .or. &
         ( obj_in % dv .eq. dv_choice_prob_wife ) ) then

      label_info(1) = obj_in % title
      label_info(2) = "Choice Probability"
      label_info(3) = var_labels_long( obj_in % x_in )
      if( obj_in % two_vars .eqv. .true. )  label_info(4) = var_labels_long( obj_in % z_in )

      if( obj_in % two_vars .eqv. .true. ) then

         do j = 1,  obj_in % n_z_nodes
            call perturb_mygrid( obj_in % z_in, &
                                 obj_in % z_nodes(j), &
                                 like_temp )

            do i = 1, obj_in % n_x_nodes
               call perturb_mygrid( obj_in % x_in, &
                                    obj_in % x_nodes(i), &
                                    like_temp )

               y(i,j) = choice_prob_output_func(   obj_in % dv, &
                                                   obj_in % max_switch, &
                                                   like_temp, &
                                                   obj_in % params, &
                                                   obj_in % value_matrix, &
                                                   obj_in % asset_matrix )
            end do !i
         end do !j

      else !only have one independent variable

         do i = 1, obj_in % n_x_nodes
            call perturb_mygrid( obj_in % x_in, &
                                 obj_in % x_nodes(i), &
                                 like_temp )

            y(i,1) = choice_prob_output_func(   obj_in % dv, &
                                                obj_in % max_switch, &
                                                like_temp, &
                                                obj_in % params, &
                                                obj_in % value_matrix, &
                                                obj_in % asset_matrix )
         end do !i

      end if !for one or two variables

      !now write output to a file
      open(newunit = unit_int, file = obj_in % save_file, status = "replace" )

      !first do headers
      call csv_write( unit_int, "labels", .false. )
      call csv_write( unit_int, "x_nodes", .false. )
      call csv_write( unit_int, "z_nodes", .false. )

      do j = 1, obj_in % n_z_nodes
         call csv_write( unit_int, "y" // trim( string_conv(j) ), .false. )
      end do

      call csv_next_record( unit_int )


      !now go through row by row
      do i=1, obj_in % n_x_nodes

         !first label info
         if ( i .le. n_labels ) then
            call csv_write( unit_int, label_info(i) , .false. )

         else
            call csv_write( unit_int, " " , .false. )

         end if

         !x_node
         call csv_write( unit_int, obj_in % x_nodes(i) , .false. )

         !z_node
         if ( i .le. obj_in % n_z_nodes ) then
            call csv_write( unit_int, obj_in % z_nodes(i) , .false. )

         else
            call csv_write( unit_int, " " , .false. )

         end if

         !now dependents variable
         do j = 1, obj_in % n_z_nodes
            call csv_write( unit_int, y(i,j), .false. )
         end do

         call csv_next_record( unit_int )


      end do !over rows


   else !for dependend variable
      print *, "other type of dependent variables not supported yet"
      stop

   end if


end subroutine graph_csv_gen

function save_file_func( base, var1, var2 ) result( save_file)
   !generate a string for my saving output
   character( len = output_len ) :: save_file
   character( len = output_len ) :: base
   integer, intent(in) :: var1
   integer, intent(in), optional :: var2

   if (present(var2) ) then
      save_file = "./graphs/" // trim( base ) // "_" // trim( like_var_labels( var1 ) ) // &
         "_" // trim( like_var_labels( var2 ) ) // ".csv"
   else
      save_file = "./graphs/" // trim( base ) // "_" // trim( like_var_labels( var1 ) ) // &
         "_" // trim( like_var_labels( var2 ) ) // ".csv"
   end if

end function save_file_func


function save_file_func2( base, gender, var ) result( save_file)
   !generate a string for my saving output
   character( len = output_len ) :: save_file
   character( len = output_len ), intent(in) :: base
   character( len=1 ), intent(in) :: gender
   integer, intent(in) :: var

   save_file = "./graphs/" // trim( base ) // "." // gender // "." // &
      trim( string_conv(var) ) // "." // trim( like_var_labels( var ) ) //  ".csv"

end function save_file_func2


subroutine graph_obj2_setup(obj_inout, n_lines_in, n_nodes_in, max_switch_in)
   class(graph_obj_type2), intent(in out) :: obj_inout
   integer, intent(in) :: n_lines_in, n_nodes_in
   logical, intent(in) :: max_switch_in

   integer :: i, k
   real*8, dimension(n_cont_vars) :: grid_min_temp !the largest possible value for a state variable
   real*8, dimension(n_cont_vars) :: grid_max_temp
   real*8, dimension(:,:), allocatable :: x_grid_round_temp

   obj_inout % n_lines = n_lines_in
   obj_inout % n_nodes = n_nodes_in
   obj_inout % max_switch = max_switch_in


   allocate( obj_inout % like_obj( n_lines_in ) )
   allocate( obj_inout % labels( n_lines_in ) )
   allocate( obj_inout % x_grid(n_nodes_in, n_like_vars ) )
   allocate( x_grid_round_temp(n_nodes_in, n_like_vars ) )

   grid_min_temp = grid_min
   grid_max_temp = grid_max

   grid_min_temp( var_m_wage_mean ) = exp( grid_min_temp( var_m_wage_mean ) + 1 )
   grid_max_temp( var_m_wage_mean ) = exp( grid_max_temp( var_m_wage_mean ) )
   grid_min_temp( var_wage_mean ) = exp( grid_min_temp( var_wage_mean ) + 1 )
   grid_max_temp( var_wage_mean ) = exp( grid_max_temp( var_wage_mean ) )
   grid_max_temp( var_assets ) = 10000

   !do something different for ui as well
   grid_min_temp( var_m_ui_ben ) = grid_mean( var_m_ui_ben ) - grid_sd( var_m_ui_ben )
   grid_min_temp( var_ui_ben )   = grid_mean( var_ui_ben ) - grid_sd( var_ui_ben )


   do i=1,n_cont_vars
      obj_inout % x_grid(:,i) = evenly_spaced_func( n_nodes_in, grid_min_temp(i), grid_max_temp(i) )
   end do

   do i=n_cont_vars +1, n_like_vars
      k = i - der_vars_convert
      obj_inout % x_grid(:,i) = evenly_spaced_func( n_nodes_in, grid_min_temp(k), grid_max_temp(k) )
   end do


   !now round grid values
   x_grid_round_temp = nint( obj_inout % x_grid )

   !some variables I don't want rounded to nearest integer
   x_grid_round_temp(:,var_ur_state) = nint( 10* obj_inout % x_grid(:,var_ur_state) ) / 10
   x_grid_round_temp(:,var_m_non_pec_norm) = &
      nint( 10* obj_inout % x_grid(:,var_m_non_pec_norm) ) / 10
   x_grid_round_temp(:,var_non_pec_norm) = &
      nint( 10* obj_inout % x_grid(:,var_non_pec_norm) ) / 10

   obj_inout % x_grid = x_grid_round_temp




end subroutine graph_obj2_setup




subroutine graph_csv_gen2(  obj_in, gender_in )
   !generates a csv file for graphing output from my model
   class(graph_obj_type2), intent(in) :: obj_in
   character(len=1), intent(in) :: gender_in

   integer :: i, j, k, m
   integer :: dv
   real*8, dimension(:,:), allocatable :: y !contains the dependent variable of interest
   type(like_type), dimension(:), allocatable :: like_temp
   character(len=output_len), dimension(:), allocatable :: label_info
   integer :: unit_int

   integer, dimension( obj_in % n_lines) :: line_present
   character(len=output_len) :: title
   character(len=output_len) :: save_file_temp

   integer :: n_lines_temp, m_receive, receive, n_labels_temp
   integer, dimension( obj_in % n_lines) :: index_vec


   if( gender_in .eq. "m" ) then
      m_receive = 1
      receive = 0
      title = "P( Husband Accepts a Job Offer  )"
      dv = dv_choice_prob_husband

   else if ( gender_in .eq. "f" ) then
      m_receive = 0
      receive = 1
      title = "P( Wife Accepts a Job Offer )"
      dv = dv_choice_prob_wife
   else
      print *, "error with gender", gender_in
      stop
   end if

 !  do i = var_m_wage_new , var_m_wage_new
   do i =1, n_like_vars

      print *, "variable number", i

      !count how many lines are for this variable
      do j = 1, obj_in % n_lines
         line_present(j) = &
            like_var_present2( i, &
                              obj_in % like_obj(j) % mygrid_t_var(1) % iv( var_m_job ), &
                              obj_in % like_obj(j) % mygrid_t_var(1) % iv( var_job ), &
                              m_receive, &
                              receive )

      end do

     ! print *, "line present", line_present

      n_lines_temp = sum( line_present)
      n_labels_temp = n_labels2_base + n_lines_temp

      !setup vector that converts indeces
      k = 1
      do j = 1, obj_in % n_lines

         if ( line_present(j) .eq. 1 ) then

            index_vec(k) = j
            k=k+1

         end if
      end do

     ! print *, "index vec", index_vec
     ! stop

      !now only create output if one or more lines contain variable i
      if ( n_lines_temp > 0 ) then

         !allocate some objects
         if ( allocated( y ) ) deallocate(y)
         if ( allocated( like_temp ) ) deallocate(like_temp)
         if ( allocated( label_info ) ) deallocate(label_info)

         allocate( y( obj_in % n_nodes, n_lines_temp ) )
         allocate( like_temp( n_lines_temp ) )
         allocate( label_info( n_labels_temp ) )

         !setup the like_temp object
         do k = 1, n_lines_temp
            call like_single_duplicate( obj_in % like_obj( index_vec(k) ), like_temp(k) )
         end do

         !generate labels
         label_info(1) = title
         label_info(2) = obj_in % subtitle
         label_info(3) = var_labels_long(i)

         do k = 1, n_lines_temp
            label_info(n_labels2_base + k) = obj_in % labels( index_vec(k) )
         end do

         !now perturb the explanatory variables and generate the perturbed choice prob
         do k = 1, n_lines_temp
            do m = 1, obj_in % n_nodes

               call perturb_mygrid( i, &
                                    obj_in % x_grid(m,i), &
                                    like_temp(k) )

               y(m,k) = choice_prob_output_func(   dv, &
                                                   obj_in % max_switch, &
                                                   like_temp(k), &
                                                   obj_in % params, &
                                                   obj_in % value_matrix, &
                                                   obj_in % asset_matrix )
            end do
         end do



         !now write output to a file
         save_file_temp = save_file_func2( obj_in % save_file_base, gender_in, i )
         open(newunit = unit_int, file = save_file_temp, status = "replace" )

         !first do headers
         call csv_write( unit_int, "labels", .false. )
         call csv_write( unit_int, "x_nodes", .false. )

         do k = 1, n_lines_temp
            call csv_write( unit_int, "y" // trim( string_conv(k) ), .false. )
         end do

         call csv_next_record( unit_int )

         !now go through row by row
         do m=1, obj_in % n_nodes

            !first label info
            if ( m .le. n_labels_temp ) then
               call csv_write( unit_int, label_info(m) , .false. )

            else
               call csv_write( unit_int, " " , .false. )

            end if

            !x_node
            call csv_write( unit_int, obj_in % x_grid(m,i) , .false. )


            !now dependents variable
            do k = 1, n_lines_temp
               call csv_write( unit_int, y(m,k), .false. )
            end do

            call csv_next_record( unit_int )


         end do !over rows

      else !n_lines_temp = 0
         print *, "skipping variable   ", like_var_labels(i), "   for gender   ", gender_in

      end if !n_lines_temp > 0

   end do !over variables



end subroutine graph_csv_gen2


subroutine graph_csv_value(  obj_in, gender_in, base_combo_in, asset_switch_in )
   !generates the value functions that enter the probabilities
   integer, parameter :: n_values = 2

   class(graph_obj_type2), intent(in) :: obj_in
   character(len=1), intent(in) :: gender_in
   integer, intent(in) :: base_combo_in !indicator for which job combination
   logical, intent(in) :: asset_switch_in !whether to do assets for this subroutine

   integer :: i, m, v
   integer :: dv
   real*8, dimension(:,:), allocatable :: y !contains the dependent variable of interest
   real*8, dimension(:), allocatable :: value_vec_temp
   type(like_type), dimension(:), allocatable :: like_temp
   character(len=output_len), dimension(:), allocatable :: label_info
   integer :: unit_int

   integer, dimension( obj_in % n_lines) :: line_present
   character(len=output_len) :: title
   character(len=output_len) :: save_file_temp

   integer :: m_receive, receive, n_labels_temp
   integer, dimension( obj_in % n_lines) :: index_vec

   if ( obj_in % n_lines .ne. 1 ) then
      print *, "number of lines needs to equal one for this subroutine"

   end if




   if( gender_in .eq. "m" ) then
      m_receive = 1
      receive = 0
      title = "Values for Husband Accepts a Job Offer"
      dv = dv_choice_prob_husband

   else if ( gender_in .eq. "f" ) then
      m_receive = 0
      receive = 1
      title = "Values for Wife Accepts a Job Offer"
      dv = dv_choice_prob_wife
   else
      print *, "error with gender", gender_in
      stop
   end if

 !  do i = var_m_wage_new , var_m_wage_new
   do i =1, n_like_vars

      print *, "variable number", i

      !count how many lines are for this variable
      line_present(1) = &
         like_var_present2( i, &
                           obj_in % like_obj(1) % mygrid_t_var(1) % iv( var_m_job ), &
                           obj_in % like_obj(1) % mygrid_t_var(1) % iv( var_job ), &
                           m_receive, &
                           receive )


     ! print *, "line present", line_present

      n_labels_temp = n_labels2_base + n_values !this is different from teh other routine



     ! print *, "index vec", index_vec
     ! stop

      !now only create output if one or more lines contain variable i
      if ( line_present(1) .eq. 1 ) then

         !allocate some objects
         if ( allocated( y ) ) deallocate(y)
         if ( allocated( like_temp ) ) deallocate(like_temp)
         if ( allocated( label_info ) ) deallocate(label_info)

         allocate( y( obj_in % n_nodes, n_values ) )
         allocate( like_temp( 1 ) )
         allocate( label_info( n_labels_temp ) )

         !setup the like_temp object
         call like_single_duplicate( obj_in % like_obj( 1 ), like_temp(1) )

         !generate labels
         label_info(1) = title
         label_info(2) = obj_in % subtitle
         label_info(3) = var_labels_long(i)

         !now labels depend on gender in and the base case
         select case( base_combo_in )

            !joboffer probabilities
            case ( both_job )
               label_info(4) = "Both with job"
               label_info(5) = "Both with job"

            !wage, hours, and non_pec probs
            case ( husband_job )
               label_info(4) = "Only husband with job"

               if( gender_in .eq. "m" ) then
                  label_info(5) = "Only husband with job"
               else
                  label_info(5) = "Both with job"
               end if

            case ( wife_job )
               label_info(4) = "Only wife with job"

               if( gender_in .eq. "m" ) then
                  label_info(5) = "Both with job"
               else
                  label_info(5) = "Only wife with job"
               end if

            case ( both_nojob )
               label_info(4) = "Both without a job"

               if( gender_in .eq. "m" ) then
                  label_info(5) = "Only husband with job"
               else
                  label_info(5) = "Only wife with job"
               end if

            case default
               print *, "error with select case for graph_csv_value"
               stop

         end select

         !now perturb the explanatory variables and generate the perturbed choice prob
         do m = 1, obj_in % n_nodes

            call perturb_mygrid( i, &
                                 obj_in % x_grid(m,i), &
                                 like_temp(1) )

            value_vec_temp = value_arg_output_func(   dv, &
                                                      asset_switch_in, &
                                                      like_temp(1), &
                                                      obj_in % params, &
                                                      obj_in % value_matrix, &
                                                      obj_in % asset_matrix )

            y(m,1:2) = value_vec_temp(1:2)
         end do



         !now write output to a file
         save_file_temp = save_file_func2( obj_in % save_file_base, gender_in, i )
         open(newunit = unit_int, file = save_file_temp, status = "replace" )

         !first do headers
         call csv_write( unit_int, "labels", .false. )
         call csv_write( unit_int, "x_nodes", .false. )

         do v = 1, n_values
            call csv_write( unit_int, "y" // trim( string_conv(v) ), .false. )
         end do

         call csv_next_record( unit_int )

         !now go through row by row
         do m=1, obj_in % n_nodes

            !first label info
            if ( m .le. n_labels_temp ) then
               call csv_write( unit_int, label_info(m) , .false. )

            else
               call csv_write( unit_int, " " , .false. )

            end if

            !x_node
            call csv_write( unit_int, obj_in % x_grid(m,i) , .false. )


            !now dependents variable
            do v = 1, n_values
               call csv_write( unit_int, y(m,v), .false. )
            end do

            call csv_next_record( unit_int )


         end do !over rows

      else !n_lines_temp = 0
         print *, "skipping variable   ", like_var_labels(i), "   for gender   ", gender_in

      end if !n_lines_temp > 0

      print *, "stopping early"
      stop

   end do !over variables



end subroutine graph_csv_value






end module output_mod
