
program sim_path_test


use enum_mod
use model_params_mod
use interp_support_mod

use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use normal_mod
use stat_estimates_mod

use likelihood_mod
use bayesian_mod

use csv_file
use random_gen_mod
use mpi_mod

use sim_path_mod
use, intrinsic :: iso_fortran_env !gives standard in, out and error


   integer, parameter :: n_obs_sim = 200  !this is the number of observations per process if mpi is used
   integer, parameter :: n_t_sim = 1
   integer, parameter :: n_burn_in_param = 0
   integer, parameter :: max_t_param = n_t_sim + n_burn_in_param
   logical, parameter :: max_switch = .true. !controls whether to do asset maximization
   logical, parameter :: read_value_ui = .true. !controls whether to create value matrices for the ui case
   logical, parameter :: read_value_no_ui = .true. !controls whether to create value matrices for the no ui case

   integer :: unit_int

   !sample numbers
   integer, parameter :: n_samples = 2
   integer, parameter :: sample_ui = 1
   integer, parameter :: sample_no_ui = 2


   !parameters that control iteration of the value function
   integer, parameter :: iter_init = 50
   integer, parameter :: max_iter = 100
   real*8, parameter :: tol = 1.0D-3
   integer, parameter :: n_howard = 20  !number of times to do the howard-improvement algorithm in which I only iterate and don't update the choice probability
   real*8, parameter :: tol_howard = 1.0D-3
   integer :: i


   character(len=30),parameter :: params_file = "output_params.txt"
   character(len=30),parameter :: sipp_data_file= "sipp_fortran_small.csv"
   character(len=30),parameter  :: interact_file = "interact_matrix.csv"
   character(len=30), parameter :: seed_file = "seed_save_sim1.txt"

   character(len=30), parameter :: value_file_ui = "value_matrix_output.csv"
   character(len=30), parameter :: asset_file_ui = "asset_matrix_output.csv"
   character(len=30), parameter :: value_file_no_ui = "value_matrix_no_ui.csv"
   character(len=30), parameter :: asset_file_no_ui = "asset_matrix_no_ui.csv"
   character(len=30), parameter :: sim_file = "sim_output_no_ui.csv"

   real*8, dimension(n_params) :: params

   real*8, dimension( max_n_grid_points, 0:1,0:1 ) :: value_matrix_init
   real*8, dimension( max_n_grid_points, 0:1,0:1 ) :: asset_matrix_init
   real*8, dimension( max_n_grid_points, 0:1,0:1 ) :: value_matrix_ui
   real*8, dimension( max_n_grid_points, 0:1,0:1 ) :: asset_matrix_ui
   real*8, dimension( max_n_grid_points, 0:1,0:1 ) :: value_matrix_no_ui
   real*8, dimension( max_n_grid_points, 0:1,0:1 ) :: asset_matrix_no_ui

   type(value_grid_type), dimension(0:1,0:1) :: value_grid


   !following variables used in the simulaiton programs
   type(mygrid_type), dimension( n_obs_sim )             :: mygrid_init
   type(mygrid_type), dimension(n_t_sim, n_obs_sim, n_samples)      :: mygrid_sim
   real*8, dimension(n_sim_uniform, max_t_param, n_obs_sim)  :: rand_uniform
   real*8, dimension(n_sim_normal, max_t_param, n_obs_sim)   :: rand_normal
   real*8, dimension(n_sim_ev, max_t_param, n_obs_sim)       :: rand_ev


   !setup mpi, parameters, value function
   call mpi_setup()

   !initialize random numbers
  ! call init_random_seed_mpi(same = .false. )
  ! call export_random_seed_mpi( seed_file )
   call read_random_seed_mpi( same = .false., filename_opt = seed_file)


   !do some initial setup
   call init_params(params_file, params)

   call gen_var_labels()
   call gen_param_labels()
   call iterate_setup_all( interact_file, params, value_grid )



   !now get the value functions
   !first the base case with ui
   if (read_value_ui .eqv. .true. ) then

      call init_interp_matrices( value_file_ui, asset_file_ui, value_matrix_ui, asset_matrix_ui )

   else

      !generate this file
      replacement_rate_param = .5

      value_matrix_init = 0
      asset_matrix_init = 0
      asset_matrix_init(1,:,:) = 2.9957322735539909

      call iterate_to_converge_all( iter_init, &
                                    max_iter, &
                                    tol, &
                                    n_howard, &
                                    tol_howard, &
                                    params, &
                                    value_matrix_init, &
                                    asset_matrix_init, &
                                    value_grid, &
                                    value_matrix_ui, &
                                    asset_matrix_ui )

      if (myrank .eq. 0 ) then
         print *, "writting the interp coefficients"
         open(newunit = unit_int, file=value_file_ui, status='replace' )
         call interp_matrix_write(value_matrix_ui, unit_int )
         close(unit_int)


         open(newunit = unit_int, file=asset_file_ui, status='replace' )
         call interp_matrix_write(asset_matrix_ui, unit_int )
         close(unit_int)

      end if

   end if

   !now the case without ui
   if (read_value_no_ui .eqv. .true. ) then

      call init_interp_matrices( value_file_no_ui, asset_file_no_ui, value_matrix_no_ui, &
         asset_matrix_no_ui )

   else

      !generate this file
      replacement_rate_param = 0

      !setup the value grid to incorporate the change in the replacement rate
      call iterate_setup_all( interact_file, params, value_grid )

      call iterate_to_converge_all( iter_init, &
                                    max_iter, &
                                    tol, &
                                    n_howard, &
                                    tol_howard, &
                                    params, &
                                    value_matrix_init, &
                                    asset_matrix_init, &
                                    value_grid, &
                                    value_matrix_no_ui, &
                                    asset_matrix_no_ui )

      if (myrank .eq. 0 ) then
         print *, "writting the interp coefficients"
         open(newunit = unit_int, file=value_file_no_ui, status='replace' )
         call interp_matrix_write(value_matrix_no_ui, unit_int )
         close(unit_int)


         open(newunit = unit_int, file=asset_file_no_ui, status='replace' )
         call interp_matrix_write(asset_matrix_no_ui, unit_int )
         close(unit_int)

      end if

   end if





   !now setup for the simulation part
 !  call gen_mygrid_init_sim( n_obs_sim, sipp_data_file, params, mygrid_init )
   do i=1,n_obs_sim
      call get_mygrid_origin(0,0, mygrid_init(i) )
      mygrid_init(i) % rv ( var_assets ) = 0
   end do



   call mpi_barrier(mpi_comm_world, err)
   if (myrank .eq. 0 ) print *, "got initial myrid"

   call gen_rand_vars_sim( n_obs_sim, max_t_param, rand_uniform, rand_normal, rand_ev )

   call mpi_barrier(mpi_comm_world, err)
   if (myrank .eq. 0 ) print *, "setup random variables"



   !now for the actual construction of the simulated path

   !first the case with ui

   replacement_rate_param = .5
   call sim_path (n_obs_sim, max_t_param, n_burn_in_param, mygrid_init, params, rand_uniform, &
      rand_normal, rand_ev, value_matrix_ui, asset_matrix_ui, max_switch, &
      mygrid_sim(:,:,sample_ui) )

   !now case without ui

   replacement_rate_param = 0

   do i=1, n_obs_sim
      mygrid_init(i) % rv( var_m_ui_ben ) = 0
      mygrid_init(i) % rv( var_ui_ben ) = 0
   end do


   call sim_path ( n_obs_sim, max_t_param, n_burn_in_param, mygrid_init, params, rand_uniform, &
      rand_normal, rand_ev, value_matrix_no_ui, asset_matrix_no_ui, max_switch, &
      mygrid_sim(:,:,sample_no_ui) )



   call mpi_barrier(mpi_comm_world, err)
   if (myrank .eq. 0 ) print *, "created simulated path"

   !now write these paths to a file
   call sim_path_write_init( sim_file )
   call sim_path_write( sim_file, sample_ui, n_obs_sim, n_t_sim, mygrid_sim( :,:, sample_ui) )
   call sim_path_write( sim_file, sample_no_ui, n_obs_sim, n_t_sim, mygrid_sim( :,:, sample_no_ui) )

   if (myrank .eq. 0 ) print *, "wrote paths to file"

  call mpi_finalize(err)



end program sim_path_test
