
program stationary_dist_gen

!policy experiment to see how changes in ui lenghts affect the stationary distribution of employment frequencies


use enum_mod
use model_params_mod
use interp_support_mod

use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use normal_mod
use stat_estimates_mod

use likelihood_mod
use bayesian_mod

use csv_file
use random_gen_mod
use mpi_mod

use sim_path_mod
use, intrinsic :: iso_fortran_env !gives standard in, out and error


   integer, parameter :: ui_length_low = 6 !this is 26 weeeks, the base before the start of the great recession
   integer, parameter :: ui_length_high = 14 !this is about 26+20+14 weeks, a state during the great recession on EUC Tiers I and II

   integer, parameter :: n_obs_sim =  1000  !this is the number of observations per process if mpi is used
   integer, parameter :: n_t_sim = 30
   integer, parameter :: n_burn_in_param = 1000 !so get unemployment data after 6 months of events
   integer, parameter :: max_t_param = n_t_sim + n_burn_in_param

   logical, parameter :: max_switch = .true. !controls whether to do asset maximization
   logical, parameter :: init_rand_switch = .true. !controls whether to generated simulated households from random draws for the dataset
      !I want this to be true for actual results, but being false can be useful for debugging purposes

   integer :: unit_int


!  character(len=30),parameter   :: params_file  = "output_params_feb19.txt"
   integer, parameter :: arg_no_params_file = 1
   character(len=30)   :: params_file
   character(len=30),parameter   :: sipp_data_file= "sipp_fortran_big.csv"
   character(len=30),parameter   :: interact_file = "interact_matrix.csv"
   character(len=30), parameter  :: seed_file = "seed_save_sim1.txt"

   character(len=30), parameter :: value_file = "value_matrix_out.csv"
   character(len=30), parameter :: asset_file = "asset_matrix_out.csv"
   character(len=30), parameter :: output_file = "policy_exp_uilength.csv"


   real*8, dimension(n_params) :: params

   real*8, dimension( n_grid_points ) :: value_matrix
   real*8, dimension( n_grid_points ) :: asset_matrix


   !following variables used in the simulaiton programs
   type(mygrid_type), dimension( n_obs_sim )          :: mygrid_init
   type(mygrid_type), dimension(n_t_sim, n_obs_sim )  :: mygrid_sim
   real*8, dimension(n_sim_uniform, max_t_param)      :: rand_uniform
   real*8, dimension(n_sim_normal, max_t_param)       :: rand_normal
   real*8, dimension(n_sim_ev, max_t_param)           :: rand_ev

   integer :: i, j, k, hh
   integer, dimension(4) :: job_sum_low_ui, job_sum_high_ui

   if ( sim_print .eqv. .true. ) then
      print *, "I want sim_print to be false"
      stop
   end if


   !setup mpi, parameters, value function
   call mpi_setup()

   call get_command_argument(arg_no_params_file, params_file )

   !initialize random numbers
  ! call init_random_seed_mpi(same = .false. )
  ! call export_random_seed_mpi( seed_file )
   call read_random_seed_mpi( same = .false., filename_opt = seed_file)


   !do some initial setup
   call init_params(params_file, params)

   call gen_var_labels()
   call gen_param_labels()
   call interp_setup_all( interact_file )

   call init_interp_matrices( value_file, asset_file, value_matrix, asset_matrix )




   !now setup for the simulation part
   if ( init_rand_switch .eqv. .true. ) then
      call gen_mygrid_init_sim( n_obs_sim, sipp_data_file, params, mygrid_init )
   else
      do i = 1, n_obs_sim
         call get_mygrid_origin( 0, 0, mygrid_init(i) )
      end do

   end if

!   call mygrid_init(1) % print(.true. , .true. )
!   read(*,*)


   call mpi_barrier(mpi_comm_world, err)
   if (myrank .eq. 0 ) print *, "got initial myrid"






   call mpi_barrier(mpi_comm_world, err)
   if (myrank .eq. 0 ) print *, "setup random variables"


   job_sum_low_ui = 0
   job_sum_high_ui = 0

   !now create the simulated samples
   do i =1 , n_obs_sim

      !create the random variables
      call gen_rand_vars_sim_1hh( max_t_param, rand_uniform, rand_normal, rand_ev )

    !  print *, "printing random variables"
      !seting these to certain values useful for debugging


   !   rand_uniform( iter_birth, 1, 1)        = 1
   !   rand_uniform( iter_m_jobloss, :, :)    = 0
   !   rand_uniform( iter_m_ui, :, :)         = 0
   !   rand_uniform( iter_jobloss, :, :)      = 0
   !   rand_uniform( iter_ui, :, :)           = 0

   !   rand_uniform( iter_m_job_offer, :, :)  = 0
   !   rand_uniform( iter_job_offer, :, :)    = 1
   !   rand_uniform( iter_m_hours, :, :)      = 1
   !   rand_uniform( iter_hours, :, :)        = 1

   !   rand_normal( iter_norm_ur, :, :)           = 0
   !   rand_normal( iter_norm_m_wage, :, :)       = 0
   !   rand_normal( iter_norm_m_non_pec, :, :)    = 0
   !   rand_normal( iter_norm_wage, :, :)         = 0
   !   rand_normal( iter_norm_non_pec, :, :)      = 0

   !   rand_ev( index_both_reject, 1, 1)           = 0
   !   rand_ev( index_husband_accept, 1, 1)        = 0
   !   rand_ev( index_wife_accept, 1, 1)           = 0
   !   rand_ev( index_both_accept, 1, 1)         = 0

      !change the ui_length

      mygrid_init(i) % rv( var_uilength) = ui_length_low

      !simulate the path
      call sim_path_1hh ( max_t_param, n_burn_in_param, mygrid_init(i), params, rand_uniform, &
         rand_normal, rand_ev, value_matrix, asset_matrix, max_switch, mygrid_sim(:,i) )

      !now record the frequncy of job outcomes
      call job_sum_inc( mygrid_sim(:,i), job_sum_low_ui )

      !now for high ui

      !change the ui_length

      mygrid_init(i) % rv( var_uilength) = ui_length_high

      !simulate the path
      call sim_path_1hh ( max_t_param, n_burn_in_param, mygrid_init(i), params, rand_uniform, &
         rand_normal, rand_ev, value_matrix, asset_matrix, max_switch, mygrid_sim(:,i) )

      !now record the frequncy of job outcomes
      call job_sum_inc( mygrid_sim(:,i), job_sum_high_ui )

   end do !over i



   call mpi_barrier(mpi_comm_world, err)
   if (myrank .eq. 0 ) print *, "created simulated path"

   !now write the data to a file
   open( newunit = unit_int, file = output_file, status = "replace" )
   call csv_write( unit_int, "Employment frequencies for uilength = " // &
                              trim( int_to_string( ui_length_low) ) // " and "  // &
                              trim( int_to_string( ui_length_high) ) // " months", .true. )

   call csv_write( unit_int, "'Both not working'", .false. )
   call csv_write( unit_int, job_sum_low_ui(1)/dble(n_t_sim*n_obs_sim), .false. )
   call csv_write( unit_int, job_sum_high_ui(1)/dble(n_t_sim*n_obs_sim), .false. )
   call csv_write( unit_int, (job_sum_high_ui(1)-job_sum_low_ui(1))/ &
      dble(n_t_sim*n_obs_sim), .true. )

   call csv_write( unit_int, "'Husband working'", .false. )
   call csv_write( unit_int, job_sum_low_ui(2)/dble(n_t_sim*n_obs_sim), .false. )
   call csv_write( unit_int, job_sum_high_ui(2)/dble(n_t_sim*n_obs_sim), .false. )
   call csv_write( unit_int, (job_sum_high_ui(2)-job_sum_low_ui(2))/ &
      dble(n_t_sim*n_obs_sim), .true. )

   call csv_write( unit_int, "'Wife working'", .false. )
   call csv_write( unit_int, job_sum_low_ui(3)/dble(n_t_sim*n_obs_sim), .false. )
   call csv_write( unit_int, job_sum_high_ui(3)/dble(n_t_sim*n_obs_sim), .false. )
   call csv_write( unit_int, (job_sum_high_ui(3)-job_sum_low_ui(3))/ &
      dble(n_t_sim*n_obs_sim), .true. )

   call csv_write( unit_int, "'Both working'", .false. )
   call csv_write( unit_int, job_sum_low_ui(4)/dble(n_t_sim*n_obs_sim), .false. )
   call csv_write( unit_int, job_sum_high_ui(4)/dble(n_t_sim*n_obs_sim), .false. )
   call csv_write( unit_int, (job_sum_high_ui(4)-job_sum_low_ui(4))/ &
      dble(n_t_sim*n_obs_sim), .true. )

   print *, "simulated frequencies", dble(job_sum_low_ui)/(n_t_sim*n_obs_sim)


   call mpi_finalize(err)

contains

subroutine job_sum_inc(mygrid_in, job_sum_inout)
   type(mygrid_type), dimension(n_t_sim), intent(in) :: mygrid_in
   integer, dimension(4), intent(in out) :: job_sum_inout
   integer  :: t


   do t = 1, n_t_sim

      if (  ( mygrid_in(t) % iv( var_m_job ) .eq. 0 ) .and. &
            ( mygrid_in(t) % iv( var_job ) .eq. 0 ) ) then

         job_sum_inout(1) = job_sum_inout(1) + 1

      else if ( ( mygrid_in(t) % iv( var_m_job ) .eq. 1 ) .and. &
                ( mygrid_in(t) % iv( var_job ) .eq. 0  ) ) then

         job_sum_inout(2) = job_sum_inout(2) + 1

      else if ( ( mygrid_in(t) % iv( var_m_job ) .eq. 0 ) .and. &
                ( mygrid_in(t) % iv( var_job ).eq. 1  ) ) then

         job_sum_inout(3) = job_sum_inout(3) + 1

      else if ( ( mygrid_in(t) % iv( var_m_job ) .eq. 1 ) .and. &
                ( mygrid_in(t) % iv( var_job) .eq. 1 ) ) then

         job_sum_inout(4) = job_sum_inout(4) + 1

      else

         print *, "error, stoping"
         stop

      end if
   end do


end subroutine job_sum_inc

end program stationary_dist_gen
