
program sim_path_test


use enum_mod
use model_params_mod
use interp_support_mod

use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use normal_mod
use stat_estimates_mod

use likelihood_mod
use bayesian_mod

use csv_file
use mpi_mod

use sim_path_mod
use, intrinsic :: iso_fortran_env !gives standard in, out and error


   integer, parameter :: n_obs_sim =  40000  !this is the number of observations per process if mpi is used
   integer, parameter :: n_t_sim = 1
   integer, parameter :: n_burn_in_param = 5 !so get unemployment data after 6 months of events
   integer, parameter :: max_t_param = n_t_sim + n_burn_in_param
   integer, parameter :: n_uimax_nodes = 4
   integer, parameter :: n_init_conditions = 10 !number of initial cases I deal with (eg. only husband working before loss, both working before but had no assets, etc)
   integer, parameter :: n_samples = n_uimax_nodes * n_init_conditions

   logical, parameter :: max_switch = .true. !controls whether to do asset maximization
   logical, parameter :: init_rand_switch = .true. !controls whether to generated simulated households from random draws for the dataset

   integer :: unit_int


!  character(len=30),parameter   :: params_file  = "output_params_feb19.txt"
   integer, parameter :: arg_no_params_file = 1
   character(len=30)   :: params_file
   character(len=30),parameter   :: sipp_data_file= "sipp_fortran_big.csv"
   character(len=30),parameter   :: interact_file = "interact_matrix.csv"
   character(len=30), parameter  :: seed_file = "seed_save_sim1.txt"

   character(len=30), parameter :: value_file = "value_matrix_out.csv"
   character(len=30), parameter :: asset_file = "asset_matrix_out.csv"
   character(len=30), parameter :: sim_file = "freq_output_uimax.csv"

   character(len=sample_label_length), dimension(n_samples) :: sample_labels
   character(len=sample_label_length) :: label_part1, label_part2

   real*8, dimension(n_params) :: params

   real*8, dimension( n_grid_points ) :: value_matrix
   real*8, dimension( n_grid_points ) :: asset_matrix



   !following variables used in the simulaiton programs
   type(mygrid_type), dimension( n_obs_sim )                 :: mygrid_init, mygrid_init_adj
   type(mygrid_type), dimension(n_t_sim, n_obs_sim )      :: mygrid_sim
   real*8, dimension(n_sim_uniform, max_t_param, n_obs_sim)  :: rand_uniform
   real*8, dimension(n_sim_normal, max_t_param, n_obs_sim)   :: rand_normal
   real*8, dimension(n_sim_ev, max_t_param, n_obs_sim)       :: rand_ev

   logical, dimension(n_cont_vars ) :: rv_adj_logic_base, rv_adj_logic
   real*8, dimension(n_cont_vars ) :: rv_adj_val_base, rv_adj_val

   real*8, dimension(n_uimax_nodes ) :: uimax_nodes
   real*8, dimension(4, n_samples) :: job_freq_matrix

   integer :: sample_no, i, j, k, hh
   integer :: m_job_post, job_post, m_ui_post, ui_post

   if ( sim_print .eqv. .true. ) then
      print *, "I want sim_print to be false"
      stop
   end if


   !setup mpi, parameters, value function
   call mpi_setup()

   call get_command_argument(arg_no_params_file, params_file )

   !initialize random numbers
  ! call init_random_seed_mpi(same = .false. )
  ! call export_random_seed_mpi( seed_file )
   call read_random_seed_mpi( same = .false., filename_opt = seed_file)


   !do some initial setup
   if (myrank .eq. 0 ) call init_params(params_file, params)
   call mpi_bcast( params, size(params), mpi_real8, 0, mpi_comm_world, err )

   call gen_var_labels()
   call gen_param_labels()
   call interp_setup_all( interact_file )

   call init_interp_matrices( value_file, asset_file, value_matrix, asset_matrix )


   !setup the adjustment vectors
   rv_adj_logic_base = .false.

   rv_adj_logic_base( var_uimax ) = .true.
   rv_adj_logic_base( var_m_unemp_length ) = .true.
   rv_adj_logic_base( var_unemp_length ) = .true.

   rv_adj_val_base = 0

   rv_adj_val_base( var_m_unemp_length ) = 0
   rv_adj_val_base( var_unemp_length ) = 0


   !gen uimax_nodes
   uimax_nodes(1:n_uimax_nodes) = evenly_spaced_func( n_uimax_nodes , &
                                                      dble(100), &
                                                      dble(400) )

!   uimax_nodes(1) = 0
!   uimax_nodes(2:n_uimax_nodes) = evenly_spaced_func( n_uimax_nodes - 1, &
!                                                      grid_min( var_uimax), &
!                                                      grid_max( var_uimax) )

  ! print *, "uimax_nodes", uimax_nodes
  ! stop

   !generate sample labels
   k = 0

   do i = 1, n_init_conditions

      select case( mod( i, 5 ) )
         case(1)
            label_part1 = "Only_husband_worked"

         case(2)
            label_part1 = "Only_wife_worked"

         case(3)
            label_part1 = "Both_worked_both_lost_job"

         case(4)
            label_part1 = "Both_worked_husband_lost_job"

         case(0)
            label_part1 = "Both_worked_wife_lost_job"
         case default
            print *, "error in select case"
            stop

         end select

      if ( i > 5 ) then
         label_part1 = trim( label_part1 ) // "_low_assets"

      end if


      do j = 1, n_uimax_nodes
         k = k + 1
         label_part2 = "_uimax_" // trim( int_to_string( nint(uimax_nodes(j) ) ) )

         sample_labels(k) = trim( label_part1) // trim( label_part2 )

      end do
   end do

!   do i=1, n_samples
!      print *, i, sample_labels(i)
!   end do

!   stop



   !now setup for the simulation part
   if ( init_rand_switch .eqv. .true. ) then
      call gen_mygrid_init_sim( n_obs_sim, sipp_data_file, params, mygrid_init )

   else
      do i = 1, n_obs_sim
         call get_mygrid_origin( 0, 0, mygrid_init(i) )
      end do

   end if

!   call mygrid_init(1) % print(.true. , .true. )
!   read(*,*)


   call mpi_barrier(mpi_comm_world, err)
   if (myrank .eq. 0 ) print *, "got initial myrid"

   call gen_rand_vars_sim( n_obs_sim, max_t_param, rand_uniform, rand_normal, rand_ev )




 !  print *, "printing random variables"


!   rand_uniform( iter_birth, 1, 1)        = 1
!   rand_uniform( iter_m_jobloss, :, :)    = 0
!   rand_uniform( iter_m_ui, :, :)         = 0
!   rand_uniform( iter_jobloss, :, :)      = 0
!   rand_uniform( iter_ui, :, :)           = 0

!   rand_uniform( iter_m_job_offer, :, :)  = 0
!   rand_uniform( iter_job_offer, :, :)    = 1
!   rand_uniform( iter_m_hours, :, :)      = 1
!   rand_uniform( iter_hours, :, :)        = 1

!   rand_normal( iter_norm_ur, :, :)           = 0
!   rand_normal( iter_norm_m_wage, :, :)       = 0
!   rand_normal( iter_norm_m_non_pec, :, :)    = 0
!   rand_normal( iter_norm_wage, :, :)         = 0
!   rand_normal( iter_norm_non_pec, :, :)      = 0

!   rand_ev( index_both_reject, 1, 1)           = 0
!   rand_ev( index_husband_accept, 1, 1)        = 0
!   rand_ev( index_wife_accept, 1, 1)           = 0
!   rand_ev( index_both_accept, 1, 1)         = 0





   call mpi_barrier(mpi_comm_world, err)
   if (myrank .eq. 0 ) print *, "setup random variables"


   !now create the simulated samples

   k = 0

   do i = 1, n_init_conditions

      rv_adj_logic = rv_adj_logic_base
      rv_adj_val = rv_adj_val_base

      !setup the ui indicators
      select case( mod( i, 5 ) )
         case(1)
            m_job_post = 0
            job_post = 0
            m_ui_post = 1
            ui_post = 0

            rv_adj_val( var_m_unemp_length ) = 0
            rv_adj_val( var_unemp_length ) = grid_mean( var_unemp_length )

         case(2) !"Only_wife_worked"
            m_job_post = 0
            job_post = 0
            m_ui_post = 0
            ui_post = 1

            rv_adj_val( var_m_unemp_length ) = grid_mean( var_m_unemp_length )
            rv_adj_val( var_unemp_length ) = 0

         case(3) !"Both_worked_both_lost_job"
            m_job_post = 0
            job_post = 0
            m_ui_post = 1
            ui_post = 1

            rv_adj_val( var_m_unemp_length ) = 0
            rv_adj_val( var_unemp_length ) = 0

         case(4) !"Both_worked_husband_lost_job"
            m_job_post = 0
            job_post = 1
            m_ui_post = 1
            ui_post = 0

            rv_adj_val( var_m_unemp_length ) = 0


         case(0) !"Both_worked_wife_lost_job"
            m_job_post = 1
            job_post = 0
            m_ui_post = 0
            ui_post = 1

            rv_adj_val( var_unemp_length ) = 0

         case default
            print *, "error in select case"
            stop

      end select

      !setup the asset indicators
      if ( i > 5 ) then !low assets
         rv_adj_logic( var_assets ) = .true.
         rv_adj_val( var_assets ) = grid_min( var_assets )

      end if

      !now go through the ui nodes and do other stuff
      do j = 1, n_uimax_nodes
         k = k + 1

         if ( myrank .eq. 0 ) then
            print *, "**************************"
            print *, "case", k
            print *, " "
         end if

      !   if ( k .eq. 13 ) then

         mygrid_init_adj = mygrid_init
         rv_adj_val( var_uimax ) = uimax_nodes(j)


         !now adjust the initial mygrid
         call adjust_init_sim( n_obs_sim, mygrid_init_adj, rv_adj_logic, rv_adj_val, &
            m_job_post, job_post, m_ui_post, ui_post )


         !change assets for households with low assets for i<=5
         if ( i <= 5 )  then
            do hh = 1, n_obs_sim
               if( mygrid_init_adj(hh) % rv( var_assets) < 1000 ) then
                  mygrid_init_adj(hh) % rv( var_assets) = grid_mean( var_assets )

               end if
            end do
         end if


!         call mygrid_init_adj(1) % print( .true. , .true. )
!         read(*,*)


         !simulate the path
         call sim_path (n_obs_sim, max_t_param, n_burn_in_param, mygrid_init_adj, params, rand_uniform, &
            rand_normal, rand_ev, value_matrix, asset_matrix, max_switch, &
            mygrid_sim)

         !generate the frequencies
         call gen_job_freq( n_obs_sim, n_t_sim, mygrid_sim, job_freq_matrix(:,k) )

           ! stop
        ! end if

      end do
   end do



   !now write the frequencies to a file



   call mpi_barrier(mpi_comm_world, err)
   if (myrank .eq. 0 ) print *, "created simulated path"

   !now write the frequencies to a file
   call job_freq_write( n_samples, job_freq_matrix, sample_labels, sim_file )

   if (myrank .eq. 0 ) print *, "wrote frequencies to file"

   call mpi_finalize(err)



end program sim_path_test
