
program sim_path_test


use enum_mod
use model_params_mod
use interp_support_mod

use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use normal_mod
use stat_estimates_mod

use likelihood_mod
use bayesian_mod

use csv_file
use random_gen_mod
use mpi_mod

use sim_path_mod
use, intrinsic :: iso_fortran_env !gives standard in, out and error


   integer, parameter :: n_obs_sim = 100  !this is the number of observations per process if mpi is used
   integer, parameter :: n_t_sim = 20
   integer, parameter :: n_burn_in_param = 20
   integer, parameter :: max_t_param = n_t_sim + n_burn_in_param
   logical, parameter :: max_switch = .true. !controls whether to do asset maximization

   character(len=30),parameter :: params_file = "output_params.txt"
   character(len=30),parameter :: sipp_data_file= "sipp_fortran_small.csv"
   character(len=30),parameter  :: interact_file = "interact_matrix.csv"
   character(len=30), parameter :: seed_file = "seed_save_sim1.txt"

   character(len=30), parameter :: value_file = "value_matrix_output.csv"
   character(len=30), parameter :: asset_file = "asset_matrix_output.csv"
   character(len=30), parameter :: sim_file = "sim_output_test1.csv"

   real*8, dimension(n_params) :: params
   real*8, dimension( max_n_grid_points, 0:1,0:1 ) :: value_matrix
   real*8, dimension( max_n_grid_points, 0:1,0:1 ) :: asset_matrix

   !following variables used in the simulaiton programs
   type(mygrid_type), dimension( n_obs_sim )             :: mygrid_init
   type(mygrid_type), dimension(n_t_sim, n_obs_sim)      :: mygrid_sim
   real*8, dimension(n_sim_uniform, max_t_param, n_obs_sim)  :: rand_uniform
   real*8, dimension(n_sim_normal, max_t_param, n_obs_sim)   :: rand_normal
   real*8, dimension(n_sim_ev, max_t_param, n_obs_sim)       :: rand_ev


   !setup mpi, parameters, value function
   call mpi_setup()

   print *, "myrank", myrank, "nprocs", nprocs

   call init_params(params_file, params)

   call gen_grid_mean_sd()
   call gen_var_labels()
   call gen_param_labels()
   call interp_list_setup_all(interact_file)

   call init_interp_matrices( value_file, asset_file, value_matrix, asset_matrix )


   !initialize random numbers
  ! call init_random_seed_mpi(same = .false. )
  ! call export_random_seed_mpi( seed_file )
   call read_random_seed_mpi( same = .false., filename_opt = seed_file)




   !now setup for the simulation part
   call gen_mygrid_init_sim( n_obs_sim, sipp_data_file, params, mygrid_init )

      call mpi_barrier(mpi_comm_world, err)

   if (myrank .eq. 0 ) print *, "got initial myrid"



!   print *, "huting for errors with initial grid"
!   call mygrid_init(1) % mygrid_error_hunt( .true., .true., 1,2,1 )

!   call mygrid_init(1) % print( .true., .true. )

!   stop

 !  call get_mygrid_origin(  1,1, mygrid_init(1) )



   call gen_rand_vars_sim( n_obs_sim, max_t_param, rand_uniform, rand_normal, rand_ev )

   call mpi_barrier(mpi_comm_world, err)


   if (myrank .eq. 0 ) print *, "setup random variables"

   !adjusting the random variables.  For the uniforms, a descrete event happens if u<p, so if I
      !want an event to happen, then I should set u=0, and if I don't want it to happen, then I should set it equal to 1 )

  ! rand_uniform( iter_birth, 1, 1)        = 0
   rand_uniform( iter_m_jobloss, 1, 1)    = 0
   rand_uniform( iter_m_ui, 1, 1)         = 0
   rand_uniform( iter_jobloss, 1, 1)      = 0
   rand_uniform( iter_ui, 1, 1)           = 0

   rand_uniform( iter_m_job_offer, 1, 1)  = 0
   rand_uniform( iter_job_offer, 1, 1)    = 1
   rand_uniform( iter_m_hours, 1, 1)      = 1
   rand_uniform( iter_hours, 1, 1)        = 1

   rand_normal( iter_norm_ur, 1, 1)           = 0
   rand_normal( iter_norm_m_wage, 1, 1)       = 0
   rand_normal( iter_norm_m_non_pec, 1, 1)    = 0
   rand_normal( iter_norm_wage, 1, 1)         = 0
   rand_normal( iter_norm_non_pec, 1, 1)      = 0

   rand_ev( index_both_reject, 1, 1)           = 0
   rand_ev( index_husband_accept, 1, 1)        = 0
   rand_ev( index_wife_accept, 1, 1)           = 0
   rand_ev( index_both_accept, 1, 1)         = 0







   !now for the actual construction of the simulated path
   call sim_path (n_obs_sim, max_t_param, n_burn_in_param, mygrid_init, params, rand_uniform, &
      rand_normal, rand_ev, value_matrix, asset_matrix, max_switch, mygrid_sim )


   call mpi_barrier(mpi_comm_world, err)
   if (myrank .eq. 0 ) print *, "created simulated path"

   !now write these paths to a file
   call sim_path_write_init( sim_file )
   call sim_path_write( sim_file, 1, n_obs_sim, n_t_sim, mygrid_sim )

   if (myrank .eq. 0 ) print *, "wrote paths to file"

  call mpi_finalize(err)



end program sim_path_test
