!module for importing sipp data for Eggleston 2013

!NOTE, not sure at this stage if I should make the spouse-specific function means a state variable.  I can calculate it every time based on parameter, observed characteristics, and unobserved hetero.  Pre-calculating might save time, but would increase the memory needed for my model.  At this time, I'm not including these things


module sipp_data_mod
use mpi_mod

implicit none


   integer,parameter :: capital_T=56
   integer,parameter :: N_t = capital_t
   integer,parameter :: N_obs_total= 9622 !for the larger file
  ! integer, parameter :: N_obs_total = 800 !total number of obs for smaller file

  !  integer, parameter :: N_obs_total =  20 !total number of obs for smaller file

   integer :: min_obs, max_obs !local variables for each node

 !integer,parameter :: N_obs= 1 !for the smaller file

  !  integer,parameter :: N_obs= 100 !for the smaller file

 !  character(30),parameter :: sipp_data_file="./sipp_fortran.txt"
 !  character(30),parameter :: sipp_data_file="./sipp_fortran_big.txt"
!    character(30),parameter :: sipp_data_file="test_obs.txt"
   integer,parameter :: file_unit=1 !which unit to assign the file to

   !type to contains variables that change over time
   type :: data_t_type
   !   integer:: index_t
      integer:: seam

      integer:: job
      integer:: job_no
      integer:: hours
      real*8 :: wage
      integer:: jobloss
      integer:: ui_ever
      integer:: new_job
      integer:: had_job

      integer:: m_job
      integer:: m_job_no
      integer:: m_hours
      real*8 :: m_wage
      integer:: m_jobloss
      integer:: m_ui_ever
      integer:: m_new_job
      integer:: m_had_job

      real*8 :: youngest
      integer:: kids
      real*8 :: ur_state
      real*8 :: ur_nat
      real*8 :: uirate
      integer:: uimax
      integer:: eb_opt_trig  !whether the state has the optional eb trigger
      integer:: euc_bill_1
      integer:: euc_bill_2
      integer:: euc_bill_3


   contains
      procedure :: print => print_data_t
   end type data_t_type

   !type to contains variables that change over time
   type :: data_i_type
      type(data_t_type),dimension(:),allocatable :: data_t
      !NOTE, even though some households aren't in the sample for capital_T periods, I'm concered if I make this allocatable,that could slow down reading into memory and it doesn't seem to add that much to the memory needed by my program
    !  integer :: index_i
      integer :: max_time

      integer :: age
      integer :: white
      integer :: hs
      integer :: some_college
      integer :: college

      integer :: job_length_start
      integer :: ui_ben_init
      integer :: min_unemp_length
      integer :: max_unemp_length

      integer :: m_age
      integer :: m_white
      integer :: m_hs
      integer :: m_some_college
      integer :: m_college

      integer :: m_job_length_start
      integer :: m_ui_ben_init
      integer :: m_min_unemp_length
      integer :: m_max_unemp_length

      integer :: age_dif
      integer :: ed_dif
      integer :: liquid_time1
      integer :: liquid_time2
      integer :: liquid_time3
      real*8  :: liquid_net1
      real*8  :: liquid_net2
      real*8  :: liquid_net3
      real*8  :: weight


   contains
      procedure :: print => print_data_i
   end type data_i_type


contains

subroutine read_sipp_data(filename_in, sipp_data_out )!reads data from the SIPP already cleaned

   implicit none
   character(len=30), intent(in) :: filename_in
   type(data_i_type), dimension(:), allocatable, intent(out) :: sipp_data_out
   integer :: i,t,int_ignore,max_obs_read, i_read, t_read, max_read
   logical :: file_exists
   integer :: max_time_temp, ratio_temp


   inquire(file=filename_in, exist=file_exists)
   if (.not. file_exists) then
      print *, "file", filename_in, "not present, stopping"
      stop
   end if


  ! if( myrank .eq. 0 ) print *, "about to open file"
  ! read(*,*)

   open(unit=file_unit,file=filename_in)

   if (mpi_setup_called .eqv. .false. ) then
      print *, "mpi_setup needs to be called first, stopping"
      stop
   end if

   !now get the min and max number of observation for this node
   ratio_temp = ceiling(dble(N_obs_total)/dble(nprocs))
   min_obs = ratio_temp*myrank + 1
   max_obs = min( ratio_temp*(myrank+1), N_obs_total )
  ! N_obs = max_obs - min_obs + 1

   allocate( sipp_data_out( min_obs: max_obs ) )


   read(1,*) !for ignoring first line of variable names

   do i=1,max_obs

      t=1

      !first get the max time
      read(1,*), int_ignore, &
         int_ignore, &
         max_time_temp
      backspace(file_unit)

      !if this household doesn't belong to this node, skip over them
      if ( i < min_obs ) then

         do t=1, max_time_temp
            read(1,*)
         end do

      else
         !hh belongs, read their data
         allocate(sipp_data_out(i) % data_t(max_time_temp) )

         !get time-invariant data and data for t=1
         read(1,*), i_read, &
            t_read, &
            sipp_data_out(i) % max_time, &
            sipp_data_out(i) % data_t(t) % seam  , &
            sipp_data_out(i) % data_t(t) % job   , &
            sipp_data_out(i) % data_t(t) % job_no   , &
            sipp_data_out(i) % data_t(t) % hours , &
            sipp_data_out(i) % data_t(t) % wage  , &
            sipp_data_out(i) % data_t(t) % jobloss  , &
            sipp_data_out(i) % data_t(t) % ui_ever  , &
            sipp_data_out(i) % data_t(t) % new_job  , &
            sipp_data_out(i) % data_t(t) % had_job  , &
            sipp_data_out(i) % data_t(t) % m_job , &
            sipp_data_out(i) % data_t(t) % m_job_no , &
            sipp_data_out(i) % data_t(t) % m_hours  , &
            sipp_data_out(i) % data_t(t) % m_wage   , &
            sipp_data_out(i) % data_t(t) % m_jobloss   , &
            sipp_data_out(i) % data_t(t) % m_ui_ever   , &
            sipp_data_out(i) % data_t(t) % m_new_job   , &
            sipp_data_out(i) % data_t(t) % m_had_job   , &
            sipp_data_out(i) % data_t(t) % youngest , &
            sipp_data_out(i) % data_t(t) % kids  , &
            sipp_data_out(i) % data_t(t) % ur_state , &
            sipp_data_out(i) % data_t(t) % ur_nat   , &
            sipp_data_out(i) % data_t(t) % uirate   , &
            sipp_data_out(i) % data_t(t) % uimax , &
            sipp_data_out(i) % data_t(t) % eb_opt_trig , &
            sipp_data_out(i) % data_t(t) % euc_bill_1  , &
            sipp_data_out(i) % data_t(t) % euc_bill_2  , &
            sipp_data_out(i) % data_t(t) % euc_bill_3  , &
            sipp_data_out(i) %    age   , &
            sipp_data_out(i) %    white , &
            sipp_data_out(i) %    hs , &
            sipp_data_out(i) %    some_college   , &
            sipp_data_out(i) %    college  , &
            sipp_data_out(i) %    job_length_start  , &
            sipp_data_out(i) %    ui_ben_init , &
            sipp_data_out(i) %    min_unemp_length  , &
            sipp_data_out(i) %    max_unemp_length  , &
            sipp_data_out(i) %    m_age , &
            sipp_data_out(i) %    m_white  , &
            sipp_data_out(i) %    m_hs  , &
            sipp_data_out(i) %    m_some_college , &
            sipp_data_out(i) %    m_college   , &
            sipp_data_out(i) %    m_job_length_start   , &
            sipp_data_out(i) %    m_ui_ben_init  , &
            sipp_data_out(i) %    m_min_unemp_length   , &
            sipp_data_out(i) %    m_max_unemp_length   , &
            sipp_data_out(i) %    age_dif  , &
            sipp_data_out(i) %    ed_dif   , &
            sipp_data_out(i) %    liquid_time1   , &
            sipp_data_out(i) %    liquid_time2   , &
            sipp_data_out(i) %    liquid_time3   , &
            sipp_data_out(i) %    liquid_net1 , &
            sipp_data_out(i) %    liquid_net2 , &
            sipp_data_out(i) %    liquid_net3 , &
            sipp_data_out(i) %    weight

   !      if (i_read .ne. i) then
   !         print *, "error with current i and i_read", i, i_read
   !         stop
   !      end if

       !  print *, "i read", i_read

     !    print *, " "
     !    print *, i, t, sipp_data_out(i) % weight

         !for now, set liquid_net1 to 0 if have negative asset holding
         if( sipp_data_out(i) % liquid_net1 < 0 )  sipp_data_out(i) % liquid_net1 = 0
         if( sipp_data_out(i) % liquid_net2 < 0 )  sipp_data_out(i) % liquid_net2 = 0
         if( sipp_data_out(i) % liquid_net3 < 0 )  sipp_data_out(i) % liquid_net3 = 0



         if (sipp_data_out(i) % max_time>1) then
            do t=2, sipp_data_out(i) % max_time
        !       print *, "      t", t
               read(1,*), i_read, &
                  t_read, &
                  max_read, &
                  sipp_data_out(i) % data_t(t) % seam  , &
                  sipp_data_out(i) % data_t(t) % job   , &
                  sipp_data_out(i) % data_t(t) % job_no   , &
                  sipp_data_out(i) % data_t(t) % hours , &
                  sipp_data_out(i) % data_t(t) % wage  , &
                  sipp_data_out(i) % data_t(t) % jobloss  , &
                  sipp_data_out(i) % data_t(t) % ui_ever  , &
                  sipp_data_out(i) % data_t(t) % new_job  , &
                  sipp_data_out(i) % data_t(t) % had_job  , &
                  sipp_data_out(i) % data_t(t) % m_job , &
                  sipp_data_out(i) % data_t(t) % m_job_no , &
                  sipp_data_out(i) % data_t(t) % m_hours  , &
                  sipp_data_out(i) % data_t(t) % m_wage   , &
                  sipp_data_out(i) % data_t(t) % m_jobloss   , &
                  sipp_data_out(i) % data_t(t) % m_ui_ever   , &
                  sipp_data_out(i) % data_t(t) % m_new_job   , &
                  sipp_data_out(i) % data_t(t) % m_had_job   , &
                  sipp_data_out(i) % data_t(t) % youngest , &
                  sipp_data_out(i) % data_t(t) % kids  , &
                  sipp_data_out(i) % data_t(t) % ur_state , &
                  sipp_data_out(i) % data_t(t) % ur_nat   , &
                  sipp_data_out(i) % data_t(t) % uirate   , &
                  sipp_data_out(i) % data_t(t) % uimax , &
                  sipp_data_out(i) % data_t(t) % eb_opt_trig , &
                  sipp_data_out(i) % data_t(t) % euc_bill_1  , &
                  sipp_data_out(i) % data_t(t) % euc_bill_2  , &
                  sipp_data_out(i) % data_t(t) % euc_bill_3

            !      if (sipp_data_out(i) % index_i .ne. i) print *, "error, i does not match up"
            !      if (sipp_data_out(i) % data_t(t) % index_t .ne. t) print *, "error, t does not match up"

             !     print *, i, t, sipp_data_out(i) % data_t(t) % ur_state

   !            if (t_read .ne. t) then
   !               print *, "error with current t and t_read", t, t_read
   !               print *, "current i", i
   !               stop
   !            end if

             !  print *, "t read", t_read

            end do !over t
         end if !max_time > 1

      end if !for i < min_obs
   end do !over i

   close(file_unit)
 !  if( myrank .eq. 0 ) print *, "file read"
  ! print *, "file read, pause"
  ! read(*,*)

end subroutine read_sipp_data

subroutine print_data_t(data_t_in)
   class(data_t_type), intent(in) :: data_t_in

   print *, " "
   print *, "seam", data_t_in %  seam

   print *, " "
   print *, "job", data_t_in %  job
   print *, "job_no", data_t_in %  job_no
   print *, "hours", data_t_in %  hours
   print *, "wage", data_t_in %  wage
   print *, "jobloss", data_t_in %  jobloss
   print *, "ui_ever", data_t_in % ui_ever
   print *, "new_job", data_t_in %  new_job
   print *, "had_job", data_t_in %  had_job

   print *, " "
   print *, "m_job", data_t_in %  m_job
   print *, "m_job_no", data_t_in %  m_job_no
   print *, "m_hours", data_t_in %  m_hours
   print *, "m_wage", data_t_in %  m_wage
   print *, "m_jobloss", data_t_in %  m_jobloss
   print *, "m_ui_ever", data_t_in % m_ui_ever
   print *, "m_new_job", data_t_in %  m_new_job
   print *, "m_had_job", data_t_in %  m_had_job

   print *, " "
   print *, "youngest", data_t_in %  youngest
   print *, "kids", data_t_in %  kids
   print *, "ur_state", data_t_in %  ur_state
   print *, "ur_nat", data_t_in %  ur_nat
   print *, "uimax", data_t_in %  uimax
   print *, "uirate", data_t_in %  uirate
   print *, "eb_opt_trig  ", data_t_in %  eb_opt_trig
   print *, "euc_bill_1", data_t_in %  euc_bill_1
   print *, "euc_bill_2", data_t_in %  euc_bill_2
   print *, "euc_bill_3", data_t_in %  euc_bill_3

end subroutine print_data_t

subroutine print_data_i(data_i_in)
   class(data_i_type), intent(in) :: data_i_in
   integer :: t

   print *, "max_time", data_i_in % max_time

   print *, "age", data_i_in % age
   print *, "white", data_i_in % white
   print *, "hs", data_i_in % hs
   print *, "some_college", data_i_in % some_college
   print *, "college", data_i_in % college

   print *, "job_length_start", data_i_in % job_length_start
   print *, "ui_ben_init", data_i_in % ui_ben_init
   print *, "min_unemp_length", data_i_in % min_unemp_length
   print *, "max_unemp_length", data_i_in % max_unemp_length

   print *, "m_age", data_i_in % m_age
   print *, "m_white", data_i_in % m_white
   print *, "m_hs", data_i_in % m_hs
   print *, "m_some_college", data_i_in % m_some_college
   print *, "m_college", data_i_in % m_college

   print *, "m_job_length_start", data_i_in % m_job_length_start
   print *, "m_ui_ben_init", data_i_in % m_ui_ben_init
   print *, "m_min_unemp_length", data_i_in % m_min_unemp_length
   print *, "m_max_unemp_length", data_i_in % m_max_unemp_length

   print *, "age_dif", data_i_in % age_dif
   print *, "ed_dif", data_i_in % ed_dif
   print *, "liquid_time1", data_i_in % liquid_time1
   print *, "liquid_time2", data_i_in % liquid_time2
   print *, "liquid_time3", data_i_in % liquid_time3
   print *, "liquid_net1", data_i_in % liquid_net1
   print *, "liquid_net2", data_i_in % liquid_net2
   print *, "liquid_net3", data_i_in % liquid_net3
   print *, "weight", data_i_in % weight

!  print *, " "
!  print *, "now go through the time periods for vars that change over time"
!  do t=1, data_i_in % max_time
!      print *, " "
!      print *, "*************************************"
!      print *, "t=", t
!      call data_i_in % data_t(t) % print()
!  end do


end subroutine print_data_i


subroutine sipp_data_deallocate( sipp_data_inout )

   type(data_i_type), dimension(:), allocatable, intent(in out) :: sipp_data_inout
   integer :: i

   !deallocate the lower level compenents.  Not sure
   do i = min_obs, max_obs
      deallocate( sipp_data_inout(i) % data_t )
   end do

   deallocate (sipp_data_inout)

end subroutine sipp_data_deallocate


end module sipp_data_mod





