program like_test

!tests whether the difference in choice probabilities seems reasonable for changes in wage mean, wage, ui, and assets
!derivatives represent marginal effects at the mean

use enum_mod
use model_params_mod
use interp_support_mod

use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use likelihood_mod
use sipp_data_mod

use normal_mod
use stat_estimates_mod

use csv_file

!use statespace_trans_params_mod


implicit none

   character(len=30),parameter :: init_params_file       = "output_params.txt"
   character(len=30),parameter :: interact_file_in       = "interact_matrix.csv"
   character(len=30),parameter :: value_matrix_init_file = "value_matrix_output.csv"
   character(len=30),parameter :: asset_matrix_init_file = "asset_matrix_output.csv"

   character(len=30),parameter :: table_file = "table_prelim.csv"


   real*8, dimension(n_params) :: params
   real*8, dimension( max_n_grid_points, 0:1,0:1 ) :: value_matrix
   real*8, dimension( max_n_grid_points, 0:1,0:1 ) :: asset_matrix


   type(like_type)   :: like_object
   type(mygrid_type) :: mygrid_object


   integer :: i,t,m_job, job
   real*8 :: ui_low, ui_high, asset_low, asset_high, new_wage_low, new_wage_high, wage_mean_low, wage_mean_high
   real*8, dimension(2,2) :: choice_prob_matrix

   real*8 :: m_wage_new, wage_new, m_non_pec_new, non_pec_new
   integer :: m_hours_new, hours_new

   integer :: unit_int


   !now read in the parameters
   call init_params(init_params_file, params)

   call gen_var_labels()
   call der_delta_setup()
   call interp_list_setup_all(interact_file_in)

   call init_interp_matrices( value_matrix_init_file, asset_matrix_init_file, value_matrix, asset_matrix)


   !given the creation of the grid mean, I can now initialize the variable of interest
   ui_low = 0
   ui_high = grid_mean( var_m_ui_ben )

   asset_low = 1
   asset_high = 500

   new_wage_low = grid_min( var_m_wage )
   new_wage_high = grid_mean( var_m_wage )

   wage_mean_low = grid_mean( var_m_wage_mean )
   wage_mean_high = grid_max( var_m_wage_mean )

   m_wage_new  = grid_mean( var_m_wage )
   wage_new    = grid_mean( var_wage )

   m_non_pec_new = grid_mean( var_m_non_pec_norm )
   non_pec_new = grid_mean( var_non_pec_norm )

   m_hours_new = grid_mean( var_m_hours )
   hours_new = grid_mean( var_hours )


   call get_mygrid_origin(0, 0, mygrid_object)

   !have wife be a stay at home mom
   mygrid_object % rv( var_ui_ben ) = 0
   mygrid_object % rv( var_unemp_length ) = grid_max( var_unemp_length )

   call choice_prob_der_setup( mygrid_object, m_wage_new, wage_new, m_non_pec_new, &
      non_pec_new, m_hours_new, hours_new, like_object )


   open( newunit = unit_int, file= table_file, status = "replace" )

   call csv_write( unit_int, "ui_low", .false. )
   call csv_write( unit_int,  ui_low, .true. )

   call csv_write( unit_int, "ui_high", .false. )
   call csv_write( unit_int,  ui_high, .true. )

   call csv_write( unit_int, "asset_low", .false. )
   call csv_write( unit_int,  asset_low, .true. )

   call csv_write( unit_int, "asset_high", .false. )
   call csv_write( unit_int,  asset_high, .true. )

   call csv_write( unit_int, "new_wage_low", .false. )
   call csv_write( unit_int,  new_wage_low, .true. )

   call csv_write( unit_int, "new_wage_high", .false. )
   call csv_write( unit_int,  new_wage_high, .true. )

   call csv_write( unit_int, "wage_mean_low", .false. )
   call csv_write( unit_int,  wage_mean_low, .true. )

   call csv_write( unit_int, "wage_mean_high", .false. )
   call csv_write( unit_int,  wage_mean_high, .true. )



   !do prob husband accept first
   like_object % m_receive_new(1) = 1
   like_object % receive_new(1) = 0
   like_object % m_accept_new(1) = 1
   like_object % accept_new(1) = 0

!!!!!!!!!!!!!!!!!!!!!!!!!

   !ui_ben and assets
   !low low
   like_object % mygrid_t_var(1) % rv( var_m_ui_ben ) = ui_low
  ! like_object % mygrid_husband_accept(1) % rv( var_m_ui_ben ) = ui_low

   like_object % mygrid_t_var(1) % rv( var_assets ) = asset_low
   like_object % mygrid_husband_accept(1) % rv( var_assets ) = asset_low

   choice_prob_matrix(1,1) = choice_prob_table_func( like_object, params, value_matrix, asset_matrix )

   !low high
   like_object % mygrid_t_var(1) % rv( var_m_ui_ben ) = ui_low
  ! like_object % mygrid_husband_accept(1) % rv( var_m_ui_ben ) = ui_low

   like_object % mygrid_t_var(1) % rv( var_assets ) = asset_high
   like_object % mygrid_husband_accept(1) % rv( var_assets ) = asset_high

   choice_prob_matrix(1,2) = choice_prob_table_func( like_object, params, value_matrix, asset_matrix )

   !high low
   like_object % mygrid_t_var(1) % rv( var_m_ui_ben ) = ui_high
  ! like_object % mygrid_husband_accept(1) % rv( var_m_ui_ben ) = ui_high

   like_object % mygrid_t_var(1) % rv( var_assets ) = asset_low
   like_object % mygrid_husband_accept(1) % rv( var_assets ) = asset_low

   choice_prob_matrix(2,1) = choice_prob_table_func( like_object, params, value_matrix, asset_matrix )

   !high high
   like_object % mygrid_t_var(1) % rv( var_m_ui_ben ) = ui_high
  ! like_object % mygrid_husband_accept(1) % rv( var_m_ui_ben ) = ui_high

   like_object % mygrid_t_var(1) % rv( var_assets ) = asset_high
   like_object % mygrid_husband_accept(1) % rv( var_assets ) = asset_high

   choice_prob_matrix(2,2) = choice_prob_table_func( like_object, params, value_matrix, asset_matrix )

   call gen_dd_table( unit_int, choice_prob_matrix, "husband accept dd table", "ui", "assets" )




!!!!!!!!!!!!!!!!!!!!!!!!!

   !new wage and wage mean
   !low low
   like_object % mygrid_husband_accept(1) % rv( var_m_wage ) = new_wage_low
   like_object % mygrid_t_invar % rv( var_m_wage_mean ) = wage_mean_low

   choice_prob_matrix(1,1) = choice_prob_table_func( like_object, params, value_matrix, asset_matrix )

   !low high
   like_object % mygrid_husband_accept(1) % rv( var_m_wage ) = new_wage_low
   like_object % mygrid_t_invar % rv( var_m_wage_mean ) = wage_mean_high

   choice_prob_matrix(1,2) = choice_prob_table_func( like_object, params, value_matrix, asset_matrix )

   !high low
   like_object % mygrid_husband_accept(1) % rv( var_m_wage ) = new_wage_high
   like_object % mygrid_t_invar % rv( var_m_wage_mean ) = wage_mean_low

   choice_prob_matrix(2,1) = choice_prob_table_func( like_object, params, value_matrix, asset_matrix )

   !high high
   like_object % mygrid_husband_accept(1) % rv( var_m_wage ) = new_wage_high
   like_object % mygrid_t_invar % rv( var_m_wage_mean ) = wage_mean_high

   choice_prob_matrix(2,2) = choice_prob_table_func( like_object, params, value_matrix, asset_matrix )

   call gen_dd_table( unit_int, choice_prob_matrix, "husband accept dd table", "new wage", "wage mean" )



!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   !now for wife accepting a job offer


   like_object % m_receive_new(1) = 0
   like_object % receive_new(1) = 1
   like_object % m_accept_new(1) = 0
   like_object % accept_new(1) = 1

!!!!!!!!!!!!!!!!!!!!!!!!!

   !ui_ben and assets
   !low low
   like_object % mygrid_t_var(1) % rv( var_m_ui_ben ) = ui_low
   like_object % mygrid_wife_accept(1) % rv( var_m_ui_ben ) = ui_low

   like_object % mygrid_t_var(1) % rv( var_assets ) = asset_low
   like_object % mygrid_wife_accept(1) % rv( var_assets ) = asset_low

   choice_prob_matrix(1,1) = choice_prob_table_func( like_object, params, value_matrix, asset_matrix )

   !low high
   like_object % mygrid_t_var(1) % rv( var_m_ui_ben ) = ui_low
   like_object % mygrid_wife_accept(1) % rv( var_m_ui_ben ) = ui_low

   like_object % mygrid_t_var(1) % rv( var_assets ) = asset_high
   like_object % mygrid_wife_accept(1) % rv( var_assets ) = asset_high

   choice_prob_matrix(1,2) = choice_prob_table_func( like_object, params, value_matrix, asset_matrix )

   !high low
   like_object % mygrid_t_var(1) % rv( var_m_ui_ben ) = ui_high
   like_object % mygrid_wife_accept(1) % rv( var_m_ui_ben ) = ui_high

   like_object % mygrid_t_var(1) % rv( var_assets ) = asset_low
   like_object % mygrid_wife_accept(1) % rv( var_assets ) = asset_low

   choice_prob_matrix(2,1) = choice_prob_table_func( like_object, params, value_matrix, asset_matrix )

   !high high
   like_object % mygrid_t_var(1) % rv( var_m_ui_ben ) = ui_high
   like_object % mygrid_wife_accept(1) % rv( var_m_ui_ben ) = ui_high

   like_object % mygrid_t_var(1) % rv( var_assets ) = asset_high
   like_object % mygrid_wife_accept(1) % rv( var_assets ) = asset_high

   choice_prob_matrix(2,2) = choice_prob_table_func( like_object, params, value_matrix, asset_matrix )

   call gen_dd_table( unit_int, choice_prob_matrix, "wife accept dd table", "ui", "assets" )




!!!!!!!!!!!!!!!!!!!!!!!!!

   !new wage and wage mean
   !low low
   like_object % mygrid_wife_accept(1) % rv( var_wage ) = new_wage_low
   like_object % mygrid_t_invar % rv( var_m_wage_mean ) = wage_mean_low

   choice_prob_matrix(1,1) = choice_prob_table_func( like_object, params, value_matrix, asset_matrix )

   !low high
   like_object % mygrid_wife_accept(1) % rv( var_wage ) = new_wage_low
   like_object % mygrid_t_invar % rv( var_m_wage_mean ) = wage_mean_high

   choice_prob_matrix(1,2) = choice_prob_table_func( like_object, params, value_matrix, asset_matrix )

   !high low
   like_object % mygrid_wife_accept(1) % rv( var_wage ) = new_wage_high
   like_object % mygrid_t_invar % rv( var_m_wage_mean ) = wage_mean_low

   choice_prob_matrix(2,1) = choice_prob_table_func( like_object, params, value_matrix, asset_matrix )

   !high high
   like_object % mygrid_wife_accept(1) % rv( var_wage ) = new_wage_high
   like_object % mygrid_t_invar % rv( var_m_wage_mean ) = wage_mean_high

   choice_prob_matrix(2,2) = choice_prob_table_func( like_object, params, value_matrix, asset_matrix )

   call gen_dd_table( unit_int, choice_prob_matrix, "wife accept dd table", "new wage", "wage mean" )





contains

function choice_prob_table_func( like_object_inout, params_in, value_matrix_in, asset_matrix_in )
   !given that I've perturb the mygrid, this generates the perturb of the choice probabilities
   real*8 :: choice_prob_table_func
   type(like_type), intent(in out) :: like_object_inout
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension( max_n_grid_points, 0:1,0:1 ), intent(in) :: value_matrix_in
   real*8, dimension( max_n_grid_points, 0:1,0:1 ), intent(in) :: asset_matrix_in


   call utility_function_setup(  like_object_inout % mygrid_t_var(1) % rv(var_kids), &
                                 like_object_inout % mygrid_t_var(1) % rv(var_youngest), &
                                 params_in, &
                                 like_object_inout % utility_params(:,1) )

   call asset_choice_like_update_one_t(like_object_inout, asset_matrix_in, 1)

   call utility_like_update_one_t(like_object_inout, params_in,1)


   call choice_prob_like_update_one_t(like_object_inout,value_matrix_in,1)

   choice_prob_table_func = like_object_inout % choice_prob(1)


end function choice_prob_table_func

subroutine gen_dd_table( unit_in, choice_prob_matrix_in, table_label_in, row_label_in, column_label_in)
   !generates a difference in difference table given the choice probabilities  and some lables
   integer, intent(in) :: unit_in
   real*8, dimension(2,2), intent(in) :: choice_prob_matrix_in
   character(len=*), intent(in) :: table_label_in
   character(len=*), intent(in) :: row_label_in
   character(len=*), intent(in) :: column_label_in

   real*8, dimension(2) :: dif_row, dif_column
   real*8 :: dd

   dif_row     = choice_prob_matrix(:,2) - choice_prob_matrix(:,1)
   dif_column  = choice_prob_matrix(2,:) - choice_prob_matrix(1,:)
   dd = dif_row(2) - dif_row(1)

   call csv_write(unit_in, " ", .true. )
   call csv_write(unit_in, table_label_in, .true. )

   call csv_write(unit_in, " ", .false. )
   call csv_write(unit_in, column_label_in // " low", .false. )
   call csv_write(unit_in, column_label_in // " high", .false. )
   call csv_write(unit_in, "difference", .true. )

   call csv_write(unit_in, row_label_in // " low", .false. )
   call csv_write(unit_in, choice_prob_matrix_in(1,1), .false. )
   call csv_write(unit_in, choice_prob_matrix_in(1,2), .false. )
   call csv_write(unit_in, dif_row(1), .true. )

   call csv_write(unit_in, row_label_in // " high", .false. )
   call csv_write(unit_in, choice_prob_matrix_in(2,1), .false. )
   call csv_write(unit_in, choice_prob_matrix_in(2,2), .false. )
   call csv_write(unit_in, dif_row(2), .true. )

   call csv_write(unit_in, "difference", .false. )
   call csv_write(unit_in, dif_column(1), .false. )
   call csv_write(unit_in, dif_column(2), .false. )
   call csv_write(unit_in, dd, .true. )



end subroutine gen_dd_table



end program

