
!following modules are for parameters used in my mcmc routie


module mcmc_params_main_mod
   !main parameters that I need to adjust alot

   integer, parameter :: n_mcmc_iter             =    40000    !10000  !50000 !total number of iterations to do
   integer, parameter :: max_time                =  11*60*60         !2*60*60 is 2 hours in seconds
   integer, parameter :: n_mcmc_burn_in          = n_mcmc_iter/2 !how long the chain will take to burn in
   integer, parameter :: max_iter_saved          = 2500 !number of past iterations to store
   integer, parameter :: n_iter_saved_remove     = 50 !when reach max number of iterations, this is number of iterations to remove
   integer, parameter :: n_interp_keep           = 2 !number of value function to store, which are randomly selected from the posterior distribution
      !some output parametesr
   integer, parameter :: n_mcmc_means = 20 !number of means for the mcmc chain to store
   integer, parameter :: n_draws_converge = 10!number of draws used to construct whether chain has converged

   !following 2 controlls how many times to iterate the value function
 !  integer, parameter :: n_iterations_start = 1
 !  integer, parameter :: n_iterations_end = 1

   logical, parameter :: est_dynamic = .true. !If false, only estimate a static model, so don't do all the value function stuff
   logical, parameter :: est_savings = .true. !If false, estimate the models without incorporating savings
   logical, parameter :: est_hetero  = .true. !If false, estimate the model without unobserved heterogeneity
   logical, parameter :: ind_draws_free = .true. !says whether to do individual draws
   logical, parameter :: read_interp_matrices = .true. !for whether to init the ols matrices with other files
   logical, parameter :: save_params_switch = .true. !controls whether to save new kernel parameters and their respective value functions.  Useful for debugging purposes



   logical, parameter :: error_check = .false. !whether to check the likelihood and bayesian objects for errors during the program

   integer  :: print_iter_multiple =  100 !this parameters says when to print run time info
   logical :: print_extra = .false.  !controls whether to print extra output for the params step
   logical, parameter :: print_extra_ind = .false.  !controls whether to print extra output for the individual draw step
   logical, parameter :: print_stats = .true. !controls printing summary stats on wage, choice, and asset probs
   logical, parameter :: running_on_screen = .false. !controls whether to print the running output on screen or to the running_output_file
   character(len=30) :: running_output_file !I initialize this in estimate.f90

   !2/12/14, new parameters that affect how I iterate the value function
   integer, parameter :: val_update_multiple = 200 !says how often to update the value function
   integer, parameter :: val_update_n_draws = 15  !says how many times I need to iterate the value function for different parameter
   integer, parameter :: val_update_max_iter = 250 !maximum number of iterations for iterating the value function
   integer, parameter :: val_update_n_history = 100 !number of past parameters to use when determining the mean of the recent parameters
   integer, parameter :: val_update_sd_inc = 2 !factor by which to increate the random walk standard devations when updating the value function
   real*8, parameter  :: val_update_tol = 5.0D-2

   integer, parameter :: n_iter_bayes = 4

end module


module mcmc_init_mod
   !module containing parameters used for intializing the mcmc chain
   integer, parameter :: n_ind_draws_initial = 20 !200!20 !controlls how many times I draw the individual-specific parameters before I start the chain
      !useful to make sure these variables are starting in a good place

   integer, parameter :: n_draws_interp_init = 200   !75!# number of parameters to draw for initializing the interpolation matrix of the value function
   real*8, parameter :: rw_interp_init_adjust = 2.5 !increasing the variance of the random walks draws to increase the span of the parameter space the value functions cover

   character(len=40), parameter :: init_interp_file = "init_interp_bayes.csv"


end module mcmc_init_mod

module mcmc_block_kernel_mod
   !parameters related to the blocks and kernel weighting

   !number of parameters in each block
   integer, parameter :: n_params_joboffer_block1 = 7
   integer, parameter :: n_params_joboffer_block2 = 0
   integer, parameter :: n_params_wage_block1 = 1
   integer, parameter :: n_params_wage_block2 = 7
   integer, parameter :: n_params_wage_block3 = 4
   integer, parameter :: n_params_leisure_block1 = 4
   integer, parameter :: n_params_leisure_block2 = 2
   integer, parameter :: n_params_leisure_block3 = 1
   integer, parameter :: n_params_leisure_block4 = 1
   integer, parameter :: n_params_asset_error_block1 = 1
   integer, parameter :: n_params_asset_error_block2 = 1



   !number of blocks
   integer, parameter :: n_blocks_params = 11
   integer, parameter :: n_blocks_ind = 1
   integer, parameter :: n_blocks_hetero = 1
   integer, parameter :: n_blocks = n_blocks_params + n_blocks_ind + n_blocks_hetero

   integer, parameter :: n_kernel_params = 10 !number of parameters in which I do the kernel weighting stuff for.

   real*8, parameter :: kernel_lb = 1.0D-40 !use to determine if weights in the kernel summation are too low

   character(len=15), dimension(n_blocks_params) :: block_labels

end module

module mcmc_list_mod
   use mcmc_block_kernel_mod
   use model_params_mod

   !list of parameters used for either kernel weighting or drawing candidate parameters
  ! integer, dimension(n_kernel_params),target :: kernel_params_list
   real*8, dimension(n_kernel_params) :: kernel_params_weight !list of weight for putting more weight on certain variables
   real*8, dimension(n_main_params) :: kernel_var_values !list of variable values used in kernel weighting
   integer, dimension(n_params_joboffer_block1),target :: joboffer_block1_list
   integer, dimension(n_params_joboffer_block2),target :: joboffer_block2_list
   integer, dimension(n_params_wage_block1),target :: wage_block1_list
   integer, dimension(n_params_wage_block2),target :: wage_block2_list
   integer, dimension(n_params_wage_block3),target :: wage_block3_list
   integer, dimension(n_params_leisure_block1),target :: leisure_block1_list
   integer, dimension(n_params_leisure_block2),target :: leisure_block2_list
   integer, dimension(n_params_leisure_block3),target :: leisure_block3_list
   integer, dimension(n_params_leisure_block4),target :: leisure_block4_list
   integer, dimension(n_params_asset_error_block1),target :: asset_error_block1_list
   integer, dimension(n_params_asset_error_block2),target :: asset_error_block2_list
   logical :: block_list_created = .false.

end module mcmc_list_mod

module mcmc_rw_mod
   use mcmc_block_kernel_mod
   use model_params_mod

   !random_walk_cholesky matrices
   real*8, dimension(n_params_joboffer_block1, n_params_joboffer_block1),target :: rw_chol_joboffer_block1
   real*8, dimension(n_params_joboffer_block2, n_params_joboffer_block2),target :: rw_chol_joboffer_block2
   real*8, dimension(n_params_wage_block1, n_params_wage_block1),target :: rw_chol_wage_block1
   real*8, dimension(n_params_wage_block2, n_params_wage_block2),target :: rw_chol_wage_block2
   real*8, dimension(n_params_wage_block3, n_params_wage_block3),target :: rw_chol_wage_block3
   real*8, dimension(n_params_leisure_block1, n_params_leisure_block1),target :: rw_chol_leisure_block1
   real*8, dimension(n_params_leisure_block2, n_params_leisure_block2),target :: rw_chol_leisure_block2
   real*8, dimension(n_params_leisure_block3, n_params_leisure_block3),target :: rw_chol_leisure_block3
   real*8, dimension(n_params_leisure_block4, n_params_leisure_block4),target :: rw_chol_leisure_block4
   real*8, dimension(n_params_asset_error_block1, n_params_asset_error_block1),target :: &
      rw_chol_asset_error_block1
   real*8, dimension(n_params_asset_error_block2, n_params_asset_error_block2),target :: &
      rw_chol_asset_error_block2
   logical :: random_walk_created=.false. !use to determine if the random walk standard deviations have been created

   real*8  :: asset_error_sd_adjust = .2 !adusts the sd size if I'm not happy with the acceptance ratio for this step


 !  real*8, parameter :: rw_joboffer_import_factor = .5
 !  real*8, parameter :: rw_wage_import_factor = .3
 !  real*8, parameter :: rw_leisure_import_factor = .5


   real*8, parameter :: rw_joboffer_import_factor1 = .8
  ! real*8, parameter :: rw_joboffer_import_factor1 = .8
   real*8, parameter :: rw_joboffer_import_factor2 = .8
   real*8, parameter :: rw_wage_import_factor1 = 1
   real*8, parameter :: rw_wage_import_factor2 = 1
   real*8, parameter :: rw_wage_import_factor3 = .1
 !  real*8, parameter :: rw_leisure_import_factor = .2
   real*8, parameter :: rw_leisure_import_factor1 = 1
   real*8, parameter :: rw_leisure_import_factor2 = 1
   real*8, parameter :: rw_leisure_import_factor3 = .3
   real*8, parameter :: rw_leisure_import_factor4 = 1
 !  real*8, parameter :: rw_asset_error_import_factor = .0001
   real*8, parameter :: rw_asset_error_import_factor1 = 3
   real*8, parameter :: rw_asset_error_import_factor2 = 3




   logical, dimension(n_blocks_params) :: free_blocks
   logical, dimension(n_blocks_params) :: kernel_block = .true. !controls which blocks I need to do kernel weighting stuff for
   logical :: hetero_params_free
   logical, dimension(n_main_params) :: free_param_list

!   real*8, parameter :: rw_joboffer_import_factor = 0
!   real*8, parameter :: rw_wage_import_factor = 0
!   real*8, parameter :: rw_leisure_import_factor = 0
!   real*8, parameter :: rw_asset_error_import_factor = 0

end module mcmc_rw_mod

module mcmc_adjust_mod

use mcmc_params_main_mod
   use model_params_mod

   integer :: print_unit !controls where to print output

   !parameter for printing or adjustments in the main mcmc chain
   integer, parameter :: adjust_rw_multiple = 200 !25 !this parameter gives the iteration in which I should reconsider readjusting the random walk sd
   integer, parameter :: adjust_bandwidth_multiple = 1 !this parameter gives the iteration in which I should reconsider readjusting the random walk sd
   integer, parameter :: choice_update_min = huge(0) !for all iterations below this value, the choice probabilities need to be updated every iteration to reflect an updated version of the likelihood
   integer, parameter :: choice_update_multiple = 100 !after passing choice_update min, the choice probabilities are only updated for multiples of choice_update_multiple
   integer, parameter :: time_weighting_min = 0 !number of iterations to do the time weighting stuff

   logical, dimension(n_main_params) :: print_param_list !controls which params to print

   !following 4 parameters are use to adjust random walk sd and to determine when to adjust random walk sd
   real*8, parameter :: low_accept_cutoff =  .3-.15
   real*8, parameter :: high_accept_cutoff = .3+.15
   real*8, parameter :: low_adjust_val = sqrt(1.0/1.4)
   real*8, parameter :: high_adjust_val = sqrt(1.4)

end module mcmc_adjust_mod


module mcmc_output_mod

   !file names of some of the outpout
   character(len=40), parameter :: mcmc_output_file_main = "mcmc_output_file_main.csv"
   character(len=40), parameter :: mcmc_output_file_cov_matrix = "mcmc_output_cov_matrix.csv"
   character(len=40), parameter :: mcmc_output_file_ols_interp = "mcmc_output_ols_interp.csv"
   character(len=40), parameter :: file_cov_block_init = "cov_block_init"
   character(len=40), parameter :: file_cov_block_out = "cov_block_final"

end module mcmc_output_mod



module bayesian_mod

use enum_mod
use model_params_mod
use interp_support_mod

use sipp_data_mod

use utility_function_mod
use model_probs_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use likelihood_mod

use uniform_mod
use logit_mod
use normal_mod
use wishart_mod
use stat_estimates_mod

use csv_file
use, intrinsic :: iso_fortran_env !gives standard in, out and error
use mpi_mod






!list of module for all the parameters
use mcmc_params_main_mod
use mcmc_init_mod
use mcmc_block_kernel_mod
use mcmc_list_mod
use mcmc_rw_mod
use mcmc_adjust_mod
use mcmc_output_mod


implicit none

   integer :: i_global

   integer,parameter :: bayes_export_multiple = 2000 !how often to export the recent mean and sd of the bayesian parameters
   real*8, parameter :: bayes_export_frac = .5 !the fraction of parameters to toss
   character(len=30) :: bayes_export_file

!   integer, dimension(n_mcmc_iter) :: weight_util_direction
!   integer :: write_dir_multiple = 200
!   character(len=30) :: weight_util_file = "weight_util_dir.csv"
!   integer :: weight_util_unit

   type :: bayes_obj_type !collection of objects used in Bayesian McMC algorithm.  I create this type
      !so that I don't have to pass a ton of arguments to some functions

      real*8, dimension(n_params) :: params_current !the parameters that are currently active
      real*8, dimension(n_main_params) :: params_proposal !the proposal parameters I'm currently saving and will eventually store in the past_main
      real*8, dimension(n_grid_points) :: value_matrix_current
      real*8, dimension(n_grid_points) :: asset_matrix_current

      integer :: iter !McMC iter #
      integer :: iter_saved !number of proposal parametes and ols vecs saved from past iterations, which is less tha max_iter_saved
      integer ::  save_proc !keeps track of which node to save proposal matrices and parameters to

      real*8, dimension(n_kernel_params) :: inv_bandwidth  !inverse of the bandwidth for value function/ols kernel weighting stuff
      logical :: time_weighting
   !   integer :: n_iterations

      real*8, dimension(n_hetero,n_hetero) :: hetero_chol_current !choleksy decomp of the unobserved heterogeneity matrix

      real*8, dimension(n_main_params, n_mcmc_iter) :: params_history
    !  real*8, dimension(:,:), allocatable :: params_history
      real*8, dimension(n_kernel_params,max_iter_saved) :: past_params_kernel
      real*8, dimension(n_grid_points,max_iter_saved) :: past_value_matrix
      real*8, dimension(n_grid_points,max_iter_saved) :: past_asset_matrix

   !   logical, dimension(:,:), allocatable :: accept_history_params !acceptence history for the parameters
   !   integer, dimension(:), allocatable :: accept_history_ind !acceptance history for individual joboffers.  This vector stores the sum the joboffers that are accepted


      logical, dimension(n_blocks_params, n_mcmc_iter ) :: accept_history_params !acceptence history for the parameters
      integer, dimension(n_mcmc_iter) :: accept_history_ind !acceptance history for individual joboffers.  This vector stores the sum the joboffers that are accepted

      real*8 :: run_time

   contains
      procedure :: setup => bayesian_setup
      procedure :: update_bandwidth => update_bandwidth
      procedure :: save_bayesian_params_interp => save_bayesian_params_interp
      procedure :: save_param_history => save_param_history

      procedure :: read_interp_init => read_interp_init
      procedure :: interp_matrix_bayesian_init => interp_matrix_bayesian_init
      procedure :: val_update_bayes => val_update_bayes

      procedure :: mh_step_params => mh_step_params
      procedure :: mh_step_ind => mh_step_ind
      procedure :: choice_prob_bayesian_update => choice_prob_bayesian_update


      procedure :: mcmc_iter_once => mcmc_iter_once
      procedure :: mcmc_iter_main => mcmc_iter_main
    !  procedure :: random_walk_write => random_walk_write
   end type


   type :: mcmc_output_type !collect of output to store results of McMC routine

      integer :: final_iter
      real*8 :: final_time
      real*8,dimension(n_blocks_params) :: accept_param_average
      real*8 :: accept_ind_average

      real*8, dimension(n_main_params) :: posterior_mean
      real*8, dimension(n_main_params,n_mcmc_means) :: mcmc_means !means for different invervals of the mcmc chain

      real*8, dimension(n_main_params) :: posterior_sd
      real*8, dimension(n_main_params, n_main_params) :: posterior_cov_matrix

      real*8, dimension(n_grid_points) :: value_matrix_posterior_mean
      real*8, dimension(n_grid_points) :: asset_matrix_posterior_mean
      real*8, dimension(n_main_params, n_interp_keep) :: params_interp_matrix_save
      real*8, dimension(n_grid_points, n_interp_keep) :: value_matrix_save
      real*8, dimension(n_grid_points, n_interp_keep) :: asset_matrix_save

      logical :: kernel_error_mean !says whether there was an error in creating the kernels
      logical, dimension(n_interp_keep) :: kernel_error_vec


   contains
      procedure :: create => mcmc_output_create
      procedure :: write => mcmc_output_write
      procedure :: read => mcmc_output_read
   end type


   interface mh_accept_func !caclulates the metropolis-hastings acceptance ratio for when either the proposal densities cancel out or when have an influence on this ratio
      module procedure mh_accept_func_noprop
      module procedure mh_accept_func_prop
   end interface


contains






subroutine bayesian_lists_init() !initializes lists of parameters I use in my bayesian McMC routine

   block_list_created = .true.

!2/13/14, noticed something weired with the kernel weights, changing it




   kernel_params_weight(1) = .1   !stuff in joboffer probability

   kernel_params_weight(2) = .05   !stuff in wage distribution

   kernel_params_weight(3) = 0    !param_m_leisure_cons
   kernel_params_weight(4) = .7    !param_leisure_cons


   kernel_params_weight(5) = 0    !param_sd_wage
   kernel_params_weight(6) = .03      !param_prob_pt

   kernel_params_weight(7) = 0      !param_sd_non_pec
   kernel_params_weight(8) = 1      !param_weight_utility
   kernel_params_weight(9) = 1       !param_rra
   kernel_params_weight(10) = .4       !param_gender_reverse




   !joboffer block1
   joboffer_block1_list(1) = param_joboffer_cons
   joboffer_block1_list(2) = param_joboffer_ur
   joboffer_block1_list(3) = param_joboffer_work
   joboffer_block1_list(4) = param_joboffer_job_length
   joboffer_block1_list(5) = param_joboffer_unemp_length
   joboffer_block1_list(6) = param_joboffer_gender
   joboffer_block1_list(7) = param_joboffer_seam

   !wage block list1
   wage_block1_list(1) = param_wage_cons

   wage_block2_list(1) = param_wage_age
   wage_block2_list(2) = param_wage_age2
   wage_block2_list(3) = param_wage_race
   wage_block2_list(4) = param_wage_hs
   wage_block2_list(5) = param_wage_some_college
   wage_block2_list(6) = param_wage_college
   wage_block2_list(7) = param_wage_gender

   !wage block list2
   wage_block3_list(1) = param_wage_pt
   wage_block3_list(2) = param_wage_ur
   wage_block3_list(3) = param_sd_wage
   wage_block3_list(4) = param_prob_pt

   !leisure block list1
   leisure_block1_list(1) = param_m_leisure_cons
   leisure_block1_list(2) = param_leisure_cons
   leisure_block1_list(3) = param_leisure_kids
   leisure_block1_list(4) = param_leisure_youngest

   !leisure block list2
   leisure_block2_list(1) = param_sd_non_pec
   leisure_block2_list(2) = param_weight_utility

   !leisure block list3
   leisure_block3_list(1) = param_rra

   !leisure block list 4
   leisure_block4_list(1) = param_gender_reverse

   !asset error block list
   asset_error_block1_list(1) = param_asset_error_cons

   asset_error_block2_list(1) = param_asset_error_linear

   kernel_block = .true.
   kernel_block( block_wage_params1 ) = .false.  !all the parameters in this one are within the wage mean, so since that's a state
      !variable, I don't need to do any kernel weighting stuff
   kernel_block( block_wage_params2 ) = .false.  !all the parameters in this one are within the wage mean, so since that's a state
      !variable, I don't need to do any kernel weighting stuff


   kernel_var_values(param_joboffer_ur) =  6.635333
   kernel_var_values(param_joboffer_work) =  .8058212
   kernel_var_values(param_joboffer_job_length) = 32.13012
   kernel_var_values(param_joboffer_unemp_length) = 4.515402
   kernel_var_values(param_joboffer_gender) = .5
   kernel_var_values(param_joboffer_seam) = .25

   kernel_var_values(param_wage_pt) = 10.09 / .8058212
   kernel_var_values(param_wage_ur) =  6.635333

   kernel_var_values(param_leisure_kids) =  1.329324
   kernel_var_values(param_leisure_youngest) = 5.281818


   call free_params_sub

   call print_params_sub


   block_labels( block_joboffer_params1 )       = "job prob1"
   block_labels( block_joboffer_params2 )       = "job prob2"
   block_labels( block_wage_params1 )           = "wage dist1"
   block_labels( block_wage_params2 )           = "wage dist2"
   block_labels( block_wage_params3 )           = "wage dist3"
   block_labels( block_leisure_params1 )        = "leisure"
   block_labels( block_leisure_params2 )        = "util weight"
   block_labels( block_leisure_params3 )        = "rra"
   block_labels( block_leisure_params4 )        = "gender rev"
   block_labels( block_asset_error_params1 )    = "asset1"
   block_labels( block_asset_error_params2 )    = "asset2"


!   !first parameters used to the kernel weighting
!   kernel_params_list(1) = param_joboffer_ur
!   kernel_params_list(2) = param_joboffer_work

!   kernel_params_list(3) = param_wage_pt
!   kernel_params_list(4) = param_wage_ur

!   kernel_params_list(5) = param_m_leisure_kids
!   kernel_params_list(6) = param_m_leisure_youngest
!   kernel_params_list(7) = param_leisure_kids
!   kernel_params_list(8) = param_leisure_youngest

!   kernel_params_list(9) = param_sd_non_pec
!   kernel_params_list(10) = param_sd_wage
!   kernel_params_list(11) = param_prob_pt
!   kernel_params_list(12) = param_weight_utility

!   !joboffer block
!   joboffer_block_list(1) = param_joboffer_cons
!   joboffer_block_list(2) = param_joboffer_ur
!   joboffer_block_list(3) = param_joboffer_work
!   joboffer_block_list(4) = param_joboffer_age
!   joboffer_block_list(5) = param_joboffer_age2
!   joboffer_block_list(6) = param_joboffer_race
!   joboffer_block_list(7) = param_joboffer_hs
!   joboffer_block_list(8) = param_joboffer_some_college
!   joboffer_block_list(9) = param_joboffer_college
!   joboffer_block_list(10) = param_joboffer_gender
!   joboffer_block_list(11) = param_joboffer_seam

!   !wage block list
!   wage_block_list(1) = param_wage_cons
!   wage_block_list(2) = param_wage_pt
!   wage_block_list(3) = param_wage_ur
!   wage_block_list(4) = param_wage_age
!   wage_block_list(5) = param_wage_age2
!   wage_block_list(6) = param_wage_race
!   wage_block_list(7) = param_wage_hs
!   wage_block_list(8) = param_wage_some_college
!   wage_block_list(9) = param_wage_college
!   wage_block_list(10) = param_wage_gender

!   wage_block_list(11) = param_sd_wage
!   wage_block_list(12) = param_prob_pt

!   !list of leisure parameters
!   leisure_block_list(1) = param_m_leisure_cons
!   leisure_block_list(2) = param_m_leisure_age
!   leisure_block_list(3) = param_m_leisure_age2
!   leisure_block_list(4) = param_m_leisure_race
!   leisure_block_list(5) = param_m_leisure_hs
!   leisure_block_list(6) = param_m_leisure_some_college
!   leisure_block_list(7) = param_m_leisure_college
!   leisure_block_list(8) = param_m_leisure_kids
!   leisure_block_list(9) = param_m_leisure_youngest

!   leisure_block_list(10) = param_leisure_cons
!   leisure_block_list(11) = param_leisure_age
!   leisure_block_list(12) = param_leisure_age2
!   leisure_block_list(13) = param_leisure_race
!   leisure_block_list(14) = param_leisure_hs
!   leisure_block_list(15) = param_leisure_some_college
!   leisure_block_list(16) = param_leisure_college
!   leisure_block_list(17) = param_leisure_kids
!   leisure_block_list(18) = param_leisure_youngest

!   leisure_block_list(19) = param_weight_utility
!   leisure_block_list(20)    = param_sd_non_pec


contains

subroutine free_params_sub()
   !declares which variables are free



! 2/14, this is the main block of code for setting some variables to be free and others not to be free


   free_blocks = .false.
   free_param_list = .false.
   hetero_params_free = .false.


   free_blocks(block_joboffer_params1) = .true.
   free_param_list( param_joboffer_cons : param_joboffer_seam   ) = .true.
   free_param_list( param_joboffer_job_length  ) = .false.


   free_blocks(block_wage_params1) = .true.
   free_param_list( param_wage_cons   ) = .true.

   free_blocks(block_wage_params2) = .true.
   free_param_list( param_wage_age : param_wage_gender   ) = .true.

   free_blocks(block_wage_params3) = .true.
   free_param_list( param_wage_pt  ) = .true.
   free_param_list( param_wage_ur  ) = .true.
   free_param_list( param_sd_wage  ) = .true.
   free_param_list( param_prob_pt  ) = .true.



   free_blocks( block_leisure_params1 ) = .true.
 !  free_param_list( param_m_leisure_cons ) = .true.
   free_param_list( param_leisure_cons ) = .true.
!   free_param_list( param_leisure_youngest ) = .true.
!   free_param_list( param_leisure_kids ) = .true.


   free_blocks( block_leisure_params2 ) = .true.
   free_param_list( param_weight_utility ) = .true.

   free_blocks( block_leisure_params3 ) = .true.
   free_param_list( param_rra ) = .true.


   free_blocks( block_leisure_params4 ) = .true.
   free_param_list( param_gender_reverse ) = .true.






      hetero_params_free = .true.
      free_param_list( param_hetero_m_wage_var: param_hetero_wage_covar  ) = .true.



      free_blocks(block_asset_error_params1) = .true.
      free_param_list(param_asset_error_cons) = .true.

      free_blocks(block_asset_error_params2) = .true.
      free_param_list(param_asset_error_linear) = .true.


   if (est_hetero .eqv. .false. ) then
      hetero_params_free = .false.
      free_param_list( param_hetero_m_wage_var: param_hetero_wage_covar  ) = .false.
   end if



end subroutine free_params_sub

subroutine print_params_sub()

   print_param_list = free_param_list

   print_param_list( param_joboffer_ur : param_joboffer_seam ) = .false.
   print_param_list( param_wage_pt : param_wage_gender ) = .false.
   print_param_list( param_prob_pt ) = .false.


end subroutine print_params_sub


end subroutine bayesian_lists_init


function kernel_params_func( params_in )
   real*8, dimension(n_kernel_params) :: kernel_params_func
   real*8, dimension(n_main_params) :: params_in
   !transforms parameters for kernel founction interpolations

   kernel_params_func( 1 ) = &
      params_in(  param_joboffer_cons ) + &
      params_in(  param_joboffer_ur ) * kernel_var_values(   param_joboffer_ur)  + &
      params_in(  param_joboffer_work ) * kernel_var_values(   param_joboffer_work)  + &
      params_in(  param_joboffer_job_length ) * kernel_var_values(   param_joboffer_job_length)  + &
      params_in(  param_joboffer_unemp_length ) * kernel_var_values(   param_joboffer_unemp_length)  + &
      params_in(  param_joboffer_gender ) * kernel_var_values(   param_joboffer_gender)  + &
      params_in(  param_joboffer_seam ) * kernel_var_values(   param_joboffer_seam)

   kernel_params_func( 2 ) = &
      params_in(  param_wage_ur ) * kernel_var_values(   param_wage_ur)  + &
      params_in(  param_wage_pt ) * kernel_var_values(   param_wage_pt)

   kernel_params_func( 3 ) = params_in(  param_m_leisure_cons )

   kernel_params_func( 4 ) =   &
      params_in(  param_leisure_cons ) + &
      params_in(  param_leisure_kids ) * kernel_var_values(   param_leisure_kids)  + &
      params_in(  param_leisure_youngest ) * kernel_var_values(   param_leisure_youngest )

   kernel_params_func( 5 ) = params_in(  param_sd_wage )
   kernel_params_func( 6 ) = params_in(  param_prob_pt )
   kernel_params_func( 7 ) = params_in(  param_sd_non_pec )
   kernel_params_func( 8 ) = params_in(  param_weight_utility )
   kernel_params_func( 9 ) = params_in(  param_rra )

   kernel_params_func( 10 ) = params_in( param_gender_reverse )



end function



subroutine random_walk_init() !initialized the random walk standard deviations

   integer  :: read_unit
   real*8, dimension(:,:), allocatable :: rw_cov_temp
   real*8, dimension(:,:), allocatable :: A !matrix used for messing around with free variables
   real*8, dimension(:,:), pointer :: rw_chol_temp
   integer, dimension(:), pointer :: block_list_temp
   integer :: n_params_temp

   character(len=40) :: init_file, char_ignore
   character(len=1) :: int_char !for writting character to integer
   integer :: b,i,j,k1,k2

   random_walk_created = .true.

   if( myrank .eq. 0 ) then

      do b=1,n_blocks_params

         if(allocated(rw_cov_temp)) deallocate(rw_cov_temp)
         if(allocated(A)) deallocate(A)

         write(int_char,"(I1)") b
         init_file=trim(file_cov_block_init) // int_char // ".csv"

         select case(b)
            case (block_joboffer_params1)
               init_file = "cov_joboffer1.csv"
               n_params_temp = n_params_joboffer_block1
               rw_chol_temp => rw_chol_joboffer_block1 !I want this to actuall be the matrix of interest
               block_list_temp => joboffer_block1_list

            case (block_joboffer_params2)
               init_file = "cov_joboffer2.csv"
               n_params_temp = n_params_joboffer_block2
               rw_chol_temp => rw_chol_joboffer_block2 !I want this to actuall be the matrix of interest
               block_list_temp => joboffer_block2_list

            case  (block_wage_params1)
               init_file = "cov_wage1.csv"
               n_params_temp = n_params_wage_block1
               rw_chol_temp => rw_chol_wage_block1 !I want this to actuall be the matrix of interest
               block_list_temp => wage_block1_list

            case  (block_wage_params2)
               init_file = "cov_wage2.csv"
               n_params_temp = n_params_wage_block2
               rw_chol_temp => rw_chol_wage_block2 !I want this to actuall be the matrix of interest
               block_list_temp => wage_block2_list

            case  (block_wage_params3)
               init_file = "cov_wage3.csv"
               n_params_temp = n_params_wage_block3
               rw_chol_temp => rw_chol_wage_block3 !I want this to actuall be the matrix of interest
               block_list_temp => wage_block3_list

            case ( block_leisure_params1 )
               init_file = "cov_leisure1.csv"
               n_params_temp = n_params_leisure_block1
               rw_chol_temp => rw_chol_leisure_block1 !I want this to actuall be the matrix of interest
               block_list_temp => leisure_block1_list

            case ( block_leisure_params2 )
               init_file = "cov_leisure2.csv"
               n_params_temp = n_params_leisure_block2
               rw_chol_temp => rw_chol_leisure_block2 !I want this to actuall be the matrix of interest
               block_list_temp => leisure_block2_list

            case ( block_leisure_params3 )
               init_file = "cov_leisure3.csv"
               n_params_temp = n_params_leisure_block3
               rw_chol_temp => rw_chol_leisure_block3 !I want this to actuall be the matrix of interest
               block_list_temp => leisure_block3_list

            case ( block_leisure_params4 )
               init_file = "cov_leisure4.csv"
               n_params_temp = n_params_leisure_block4
               rw_chol_temp => rw_chol_leisure_block4 !I want this to actuall be the matrix of interest
               block_list_temp => leisure_block4_list


            case ( block_asset_error_params1 )
               init_file = "cov_asset1.csv"
               n_params_temp = n_params_asset_error_block1
               rw_chol_temp => rw_chol_asset_error_block1 !I want this to actuall be the matrix of interest
               block_list_temp => asset_error_block1_list

            case ( block_asset_error_params2 )
               init_file = "cov_asset2.csv"
               n_params_temp = n_params_asset_error_block2
               rw_chol_temp => rw_chol_asset_error_block2 !I want this to actuall be the matrix of interest
               block_list_temp => asset_error_block2_list

            case default
               print *, "error in select case for creating random walk vectors"
         end select


         if( n_params_temp > 0 ) then

            allocate(rw_cov_temp(n_params_temp,n_params_temp))
            allocate(A(n_params_temp,n_params_temp))

            open(newunit = read_unit,file=init_file, status = "old")

            read(read_unit,*) char_ignore
            do i=1,n_params_temp
               read(read_unit,*) char_ignore, (rw_cov_temp(i,j),j=1,i)
            end do
            close(read_unit)


            call chol_decomp_sub(n_params_temp, rw_cov_temp, rw_chol_temp)

            !setup of a transformation matrix to deal with free vs fixed variables.  If all variables are free, then this will just be the identity matrix, and if all fixed, then will be matrix of 0s.
            A=0
            do i=1,n_params_temp
               if (free_param_list(block_list_temp(i)) .eqv. .true. ) then
                  A(i,i)=1
               end if
            end do

            rw_chol_temp =matmul(A,rw_chol_temp)

         else
            rw_chol_temp = 0
            free_blocks(b) = .false.
         end if

      end do !over blocks

      !now adjust the overall size fo them
      rw_chol_joboffer_block1 = rw_chol_joboffer_block1 * rw_joboffer_import_factor1
      rw_chol_joboffer_block2 = rw_chol_joboffer_block2 * rw_joboffer_import_factor2
      rw_chol_wage_block1 = rw_chol_wage_block1 * rw_wage_import_factor1
      rw_chol_wage_block2 = rw_chol_wage_block2 * rw_wage_import_factor2
      rw_chol_wage_block3 = rw_chol_wage_block3 * rw_wage_import_factor3
      rw_chol_leisure_block1 = rw_chol_leisure_block1 * rw_leisure_import_factor1
      rw_chol_leisure_block2 = rw_chol_leisure_block2 * rw_leisure_import_factor2
      rw_chol_leisure_block3 = rw_chol_leisure_block3 * rw_leisure_import_factor3
      rw_chol_leisure_block4 = rw_chol_leisure_block4 * rw_leisure_import_factor4
      rw_chol_asset_error_block1 = rw_chol_asset_error_block1 * rw_asset_error_import_factor1
      rw_chol_asset_error_block2 = rw_chol_asset_error_block2 * rw_asset_error_import_factor2

   end if !myrank .eq. 0

   !now export the free_blocks vector (In case it was modified) because that is used by other processes
   call mpi_bcast( free_blocks, size(free_blocks), mpi_logical, 0, mpi_comm_world, err )


end subroutine random_walk_init





subroutine bayesian_setup(bayes_obj_out, params_filename_in, mcmc_output_in_opt)
   !intialized the bayesian object
   class(bayes_obj_type), intent(out) :: bayes_obj_out
   character(len=30), intent(in) :: params_filename_in
   type(mcmc_output_type), intent(in), optional :: mcmc_output_in_opt !for restarting the mcmc chain with previously saved output
   real*8 :: sd_temp
   real*8, dimension( n_hetero, n_hetero ) :: chol_temp, cov_temp
   integer :: i,j


   integer, parameter :: n_kernel_init = 20 !used for creating the intial kernel bandwidth
   real*8, parameter :: silverman_cons_init = ((4.0/(3.0*n_kernel_init))**(.2))*1000

   real*8, dimension(n_grid_points) :: value_matrix_temp
   real*8, dimension(n_grid_points) :: asset_matrix_temp

   !first for some setup
   call bayesian_lists_init()
   call gen_labels()
   call random_walk_init()

!   print *, "1st cov matrix"
!   rw_chol_joboffer_block = matmul(rw_chol_joboffer_block, transpose(rw_chol_joboffer_block))
!   do i=1,n_params_joboffer_block
!   print *, (rw_chol_joboffer_block(i,j),j=1,i)
!   end do
!   read(*,*)

!   print *, "2nd cov matrix"
!   rw_chol_wage_block = matmul(rw_chol_wage_block, transpose(rw_chol_wage_block))
!   do i=1,n_params_wage_block
!   print *, (rw_chol_wage_block(i,j),j=1,i)
!   end do
!   read(*,*)

!   print *, "3rd cov matrix"
!   rw_chol_leisure_block = matmul(rw_chol_leisure_block, transpose(rw_chol_leisure_block))
!   do i=1,n_params_leisure_block
!   print *, (rw_chol_leisure_block(i,j),j=1,i)
!   end do
!   read(*,*)



   call init_params( params_filename_in, bayes_obj_out % params_current )

   !init the unobserved hetero cholesky decomp
   if( est_hetero .eqv. .true. ) then

      !parameters in the output are stored in terms of the covariance matrix, but I want to store the cholesky decomposition
         !when estimating the model

      cov_temp(1,1) = bayes_obj_out % params_current( param_hetero_m_wage_var )
      cov_temp(2,1) = bayes_obj_out % params_current( param_hetero_wage_covar ) !only need the lower triangle
      cov_temp(2,2) = bayes_obj_out % params_current( param_hetero_wage_var )

      !get cholesky decomp
      call chol_decomp_sub(n_hetero, cov_temp, bayes_obj_out % hetero_chol_current )


      !now store the choleksy to the current parameters as well
      bayes_obj_out % params_current( param_hetero_m_wage_var ) = &
         bayes_obj_out % hetero_chol_current(1,1)

      bayes_obj_out % params_current( param_hetero_wage_covar ) = &
         bayes_obj_out % hetero_chol_current(2,1)

      bayes_obj_out % params_current( param_hetero_wage_var ) = &
         bayes_obj_out % hetero_chol_current(2,2)


   else
       bayes_obj_out % params_current( param_hetero_m_wage_var ) = 0
       bayes_obj_out % params_current( param_hetero_wage_covar ) = 0
       bayes_obj_out % params_current( param_hetero_wage_var ) = 0

   end if






   bayes_obj_out % iter = 0

   !only allocate the accept history and param history matrices for the root node
!   if( myrank .eq. 0 ) then
!      allocate( bayes_obj_out % accept_history_params( n_blocks_params, n_mcmc_iter ) )
!      allocate( bayes_obj_out % accept_history_ind( n_mcmc_iter ) )
!      allocate( bayes_obj_out % params_history( n_main_params, n_mcmc_iter ) )
!   end if

!   print *, "parameters I'm starting out with"
!   call write_params(bayes_obj_out % params_current ,6, .false.)

   !adjust the size down.
 !  bayes_obj_out % params_current(1:n_main_params) = &
    !  bayes_obj_out % params_current(1:n_main_params) / 100

   !this part of setup depends on whether I'm restarting mcmc chain
   if (present(mcmc_output_in_opt)) then

      print *, "stopping"
      stop

!      if ( myrank .eq. 0 ) then

!         bayes_obj_out % params_current(:n_main_params) = mcmc_output_in_opt % posterior_mean(:n_main_params)
!         bayes_obj_out % value_matrix_current = mcmc_output_in_opt % value_matrix_posterior_mean
!         bayes_obj_out % asset_matrix_current = mcmc_output_in_opt % asset_matrix_posterior_mean

!         !only import the ols vecs that didn't have any errors with their creation
!         do i=1,n_interp_keep
!            j=0
!            if ( mcmc_output_in_opt % kernel_error_vec(i) .eqv. .false. ) then
!               j = j+1
!               bayes_obj_out % past_params_kernel(:,j) = mcmc_output_in_opt % params_interp_matrix_save(:,i)
!               bayes_obj_out % past_value_matrix(:,:,:,j) = mcmc_output_in_opt % value_matrix_save(:,:,:,i)
!               bayes_obj_out % past_asset_matrix(:,:,:,j) = mcmc_output_in_opt % asset_matrix_save(:,:,:,i)
!            end if
!         end do

!         bayes_obj_out % iter_saved = j

!      end if

!      call mpi_bcast( bayes_obj_out % params_current, &
!         size(bayes_obj_out % params_current), mpi_real8, 0, mpi_comm_world, err )
!      call mpi_bcast( bayes_obj_out % value_matrix_current, &
!         size(bayes_obj_out % value_matrix_current), mpi_real8, 0, mpi_comm_world, err )
!      call mpi_bcast( bayes_obj_out % asset_matrix_current, &
!         size(bayes_obj_out % asset_matrix_current), mpi_real8, 0, mpi_comm_world, err )

!      !now initialize the bandwidth
!      call update_bandwidth(bayes_obj_out)

   else

      bayes_obj_out % save_proc = nprocs -1
      bayes_obj_out % iter_saved = 0


      value_matrix_temp = 0
      asset_matrix_temp = 0
      asset_matrix_temp(1) = 2.9957322735539909
      bayes_obj_out % params_proposal(:) = bayes_obj_out % params_current(1:n_main_params)

      !2/12/14, changed this bit of code since I've changed save_bayesian_params_interp
!      bayes_obj_out % iter = 1
      call save_bayesian_params_interp( bayes_obj_out, value_matrix_temp, &
            asset_matrix_temp)
!      bayes_obj_out % iter = 0



      bayes_obj_out % inv_bandwidth = 1.0 / silverman_cons_init


   end if

   bayes_obj_out % time_weighting = .false.
  ! bayes_obj_out % time_weighting = .true.
  ! bayes_obj_out % n_iterations = n_iterations_start
  ! print *, "through loop"




end subroutine bayesian_setup




subroutine update_bandwidth(bayes_obj_inout)
   !updates the bandwidth for the kernel weighting stuff
   !use silverman's rule of thumb
   class(bayes_obj_type), intent(in out) :: bayes_obj_inout
   real*8 :: mean_temp, sd_temp, bandwidth_temp, silverman_cons, mean_local, sd_local
   integer :: i,iter_saved, iter_saved_total

   real*8, dimension(max_iter_saved) :: vec_temp

   real*8, parameter :: min_sd_value = 2.0D-4

   iter_saved = bayes_obj_inout % iter_saved

   call mpi_allreduce( iter_saved, iter_saved_total, 1, mpi_integer, mpi_sum, mpi_comm_world, err)
   silverman_cons = (4.0/(3.0*iter_saved_total))**(.2)

   do i=1, n_kernel_params

      vec_temp(1:iter_saved) = bayes_obj_inout % past_params_kernel(i,1:iter_saved)

      !get mean locally first then across nodes
      if ( iter_saved > 0 ) then
       !  mean_local =  sum( bayes_obj_inout % past_params_kernel(j,1:iter_saved) )
         mean_local =  sum( vec_temp(1:iter_saved)  )
      else
         mean_local = 0
      end if

      !now across nodes
      call mpi_allreduce(mean_local, mean_temp, 1, mpi_real8, mpi_sum, mpi_comm_world, err)
      mean_temp = mean_temp / iter_saved_total

      !get sd locally first then across nodes
      if ( iter_saved > 0 ) then
       !  sd_local = sum_squares( iter_saved, &
      !               bayes_obj_inout % past_params_kernel(j,1:iter_saved), &
      !               mean_temp)
         sd_local = sum_squares( iter_saved, &
                     vec_temp(1:iter_saved), &
                     mean_temp)
      else
         sd_local = 0
      end if

      call mpi_allreduce(sd_local, sd_temp, 1, mpi_real8, mpi_sum, mpi_comm_world, err)
      sd_temp = sqrt( sd_temp / (iter_saved_total - 1 ) )

      sd_temp = max(sd_temp, min_sd_value)

      bandwidth_temp = sd_temp * silverman_cons
      bayes_obj_inout % inv_bandwidth(i) = 1.0/bandwidth_temp
    !  print *, i, sd_temp, bandwidth_temp, bayes_obj_inout % inv_bandwidth(i)
   end do

  ! print *, "done updating bandwidth"
  ! read(*,*)
contains

function sum_squares(n,x,mu)
   !gets sum of squares deviations
   integer, intent(in) :: n
   real*8, dimension(n), intent(in) :: x
   real*8, intent(in) :: mu
   real*8 :: sum_squares

   real*8, dimension(n) :: deviation_temp

   deviation_temp(:) = x(:) - mu

   sum_squares = dot_product(deviation_temp,deviation_temp)

end function sum_squares

end subroutine update_bandwidth


subroutine interp_kernel( bayes_obj_in, params_in, value_matrix_out, asset_matrix_out, kernel_error_out )
   !constructs the  interpolation coefficets bases on a weighted average of previous ols interpolation coefficients
   !this is similar to the procedure described in IJC, but instead of calculating weighted average past value functions, I construct
      !weighted average of past interpolation coefficeints.
   !I use the gaussian kernel for my kernel weighting
   !only do weighting with respect to some variables because some parameters (such as the education coefficient in the wage distribution) don't affect the value function since the effect of these parameters is captured by the function means.

   class(bayes_obj_type), intent(in) :: bayes_obj_in
   real*8, dimension(n_main_params), intent(in) :: params_in
   real*8, dimension(n_grid_points), intent(out) :: value_matrix_out
   real*8, dimension(n_grid_points), intent(out) :: asset_matrix_out
   logical, intent(out) :: kernel_error_out !true if there was a problem with getting the kernel estimates

   real*8, dimension(n_kernel_params) :: kernel_params_temp
   real*8 :: log_normal_kernel, dif, kernel_sum, kernel_sum_local, inv_iter_saved
   real*8, dimension( bayes_obj_in % iter_saved) :: kernel_weight
   integer :: i,  t

   real*8, dimension(n_grid_points) :: value_matrix_local
   real*8, dimension(n_grid_points) :: asset_matrix_local
   real*8, dimension(n_grid_points,nprocs) :: value_matrix_gather
   real*8, dimension(n_grid_points,nprocs) :: asset_matrix_gather



   kernel_params_temp = kernel_params_func( params_in )


   !first get the kernel weights
   do t=1, bayes_obj_in % iter_saved

      log_normal_kernel = 0

      do i=1,n_kernel_params

         dif = kernel_params_temp(i) - bayes_obj_in % past_params_kernel( i, t)

         log_normal_kernel=   log_normal_kernel -.5*dif*dif * kernel_params_weight(i) * &
                               ( bayes_obj_in % inv_bandwidth(i)**2 )

       !   print *, i, t, kernel_params_temp(i), bayes_obj_in % past_params_kernel( i, t), log_normal_kernel

      end do

      !now exponentiate the log of the kernel
      if ( log_normal_kernel < min_exp_arg ) then
         kernel_weight(t) = 0
      else
         kernel_weight(t) = exp( log_normal_kernel )
      end if

   end do !of getting kernel weights



   !if not pass burn in, do adjustments to weight down observations in the more distant past
   if ( bayes_obj_in % time_weighting .eqv. .true.) then
      inv_iter_saved = dble(1)/ bayes_obj_in % iter_saved
      do i=1, bayes_obj_in % iter_saved
         kernel_weight(i) = kernel_weight(i) * ( (i*inv_iter_saved) * (i*inv_iter_saved) )
      end do
   end if

   !get kernel sum
   if (bayes_obj_in % iter_saved > 0 ) then
      kernel_sum_local = sum(kernel_weight)
   else
      kernel_sum_local = 0
   end if
   call mpi_allreduce(kernel_sum_local, kernel_sum, 1, mpi_real8, mpi_sum, mpi_comm_world, err)

   kernel_error_out = nan_inf_check( kernel_sum )
   if ( kernel_sum < kernel_lb ) kernel_error_out = .true.


 !  print *, "kernel sum", kernel_sum

   if ( (kernel_error_out .eqv. .true.) .and. (myrank .eq. 0 ) ) print *, "error with the kernel sum being too low"
   kernel_weight = kernel_weight/ kernel_sum


!   print *, "printing kernel weight"
!   do i=1, bayes_obj_in % iter_saved
!      print *, i, kernel_weight(i)
!   end do
!   read(*,*)


!   if ( any(nan_inf_check(kernel_weight)) ) then
!      print *, "error with one of the kernels weights"
!      kernel_error_out = .true.
!   end if

   !now get the  interpolation coefficients
   !value function

   !first do local weighting
   value_matrix_local = 0

   do t=1,  bayes_obj_in % iter_saved
      value_matrix_local = value_matrix_local + kernel_weight(t) *  bayes_obj_in % past_value_matrix(:,t)
     ! print *, t, kernel_weight(t), bayes_obj_in % past_value_matrix(1,0,0,t)
   end do


   !now combine across processes
   call mpi_gather( value_matrix_local, size(value_matrix_local), MPI_REAL8, value_matrix_gather, &
      size(value_matrix_local), MPI_REAL8, 0, mpi_comm_world, err )

   if( myrank .eq. 0 ) then
      value_matrix_out = 0

      do t=1,  nprocs
         value_matrix_out = value_matrix_out + value_matrix_gather(:,t)
    !  print *, t, kernel_weight(t)
      end do

   end if

   call mpi_bcast( value_matrix_out, size(value_matrix_out), mpi_real8, 0, mpi_comm_world, err )




   !asset matrix

   !first do local weighting
   asset_matrix_local = 0

   do t=1,  bayes_obj_in % iter_saved
      asset_matrix_local = asset_matrix_local + kernel_weight(t) *  bayes_obj_in % past_asset_matrix(:,t)
    !  print *, t, kernel_weight(t)
   end do

   !now combine across processes
   call mpi_gather( asset_matrix_local, size(asset_matrix_local), MPI_REAL8, asset_matrix_gather, &
      size(asset_matrix_local), MPI_REAL8, 0, mpi_comm_world, err )

   if( myrank .eq. 0 ) then
      asset_matrix_out = 0

      do t=1,  nprocs
         asset_matrix_out = asset_matrix_out + asset_matrix_gather(:,t)
    !  print *, t, kernel_weight(t)
      end do

   end if

   call mpi_bcast( asset_matrix_out, size(asset_matrix_out), mpi_real8, 0, mpi_comm_world, err )




  ! print *, "done printing kernel weigthts"
  ! read(*,*)

end subroutine interp_kernel






subroutine draw_new_params( block_in, params_inout, params_proposal_inout )
   !draws candidate parameters, which are then either accepted or rejected by another subroutine
   !parameters are drawn with a random walk density
   integer, intent(in) :: block_in
   real*8, dimension(n_params), intent(in out) :: params_inout !these are the main parameters that are changes and used to determine whether to accept of reject these proposal parameters
   real*8, dimension(n_main_params), intent(in out) :: params_proposal_inout !this vector of parameters saves the proposal parameters that are drawn


   real*8, dimension(:,:), pointer :: rw_chol_pointer !pointers to make this subroutine easier to code
   integer, dimension(:), pointer :: list_pointer
   integer  :: n_params_temp
   real*8,dimension(:), allocatable :: theta_temp,theta_temp2 !temporary stores the theta of interest
   integer :: i

   !only draw if root node
   if(myrank .eq. 0 ) then

      !initialize the pointers
      select case(block_in)

         case ( block_joboffer_params1 )
            rw_chol_pointer  => rw_chol_joboffer_block1
            n_params_temp = n_params_joboffer_block1
            list_pointer     => joboffer_block1_list

         case ( block_joboffer_params2 )
            rw_chol_pointer  => rw_chol_joboffer_block2
            n_params_temp = n_params_joboffer_block2
            list_pointer     => joboffer_block2_list

         case ( block_wage_params1 )
            rw_chol_pointer  => rw_chol_wage_block1
            n_params_temp = n_params_wage_block1
            list_pointer     => wage_block1_list

         case ( block_wage_params2 )
            rw_chol_pointer  => rw_chol_wage_block2
            n_params_temp = n_params_wage_block2
            list_pointer     => wage_block2_list

         case ( block_wage_params3 )
            rw_chol_pointer  => rw_chol_wage_block3
            n_params_temp = n_params_wage_block3
            list_pointer     => wage_block3_list

         case ( block_leisure_params1 )
            rw_chol_pointer  => rw_chol_leisure_block1
            n_params_temp = n_params_leisure_block1
            list_pointer     => leisure_block1_list

         case ( block_leisure_params2 )
            rw_chol_pointer  => rw_chol_leisure_block2
            n_params_temp = n_params_leisure_block2
            list_pointer     => leisure_block2_list

         case ( block_leisure_params3 )
            rw_chol_pointer  => rw_chol_leisure_block3
            n_params_temp = n_params_leisure_block3
            list_pointer     => leisure_block3_list

         case ( block_leisure_params4 )
            rw_chol_pointer  => rw_chol_leisure_block4
            n_params_temp = n_params_leisure_block4
            list_pointer     => leisure_block4_list


         case ( block_asset_error_params1 )
            rw_chol_pointer  => rw_chol_asset_error_block1
            n_params_temp   = n_params_asset_error_block1
            list_pointer     => asset_error_block1_list

         case ( block_asset_error_params2 )
            rw_chol_pointer  => rw_chol_asset_error_block2
            n_params_temp   = n_params_asset_error_block2
            list_pointer     => asset_error_block2_list

         case default
            print *, "error in select case for draw new params"

      end select


      !converts parameters that have bounds
      call param_predraw_convert(params_inout)

      !have theta temp store the relevant parameters
      allocate(theta_temp(n_params_temp))
      allocate(theta_temp2(n_params_temp))



      do i=1,n_params_temp
         theta_temp(i) = params_inout( list_pointer(i) )
      end do


      !draw multivariate normal innovation
      call get_random_mvn(n_params_temp, theta_temp, rw_chol_pointer, theta_temp2)



      !now restore the new relevant parameters
      do i=1,n_params_temp
         params_inout( list_pointer(i) )        = theta_temp2(i)
      end do

      call param_postdraw_convert(params_inout)

      !now store the proposal parameters
      do i=1,n_params_temp
         params_proposal_inout( list_pointer(i) )        = params_inout( list_pointer(i) )
      end do

   end if !of whether this is the root node

   !now broadcast
   call mpi_bcast( params_inout, size(params_inout), mpi_real8, 0, mpi_comm_world, err )
   call mpi_bcast( params_proposal_inout, size(params_proposal_inout), mpi_real8, 0, mpi_comm_world, err )




end subroutine draw_new_params


subroutine save_bayesian_params_interp( bayes_obj_inout, value_matrix_in, &
   asset_matrix_in)
   !saves the interpolation coefficient and their respective parameters to the bayesian object
      !the parameters that are sames are the proposal parameters in the bayesian object

   class(bayes_obj_type), intent(in out), target :: bayes_obj_inout
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
   real*8, dimension(n_grid_points), intent(in) :: asset_matrix_in
   integer, pointer :: iter_saved
   integer :: i

   if ( ( est_dynamic .eqv. .true. ) .and. (save_params_switch .eqv. .true. ) ) then
      !increment save proc
      if (bayes_obj_inout % save_proc .ge. nprocs - 1 ) then
         bayes_obj_inout % save_proc = 0

      else
         bayes_obj_inout % save_proc = bayes_obj_inout % save_proc + 1

      end if


      !now only save proposal stuff if it's this node's turn
      if ( myrank .eq. bayes_obj_inout % save_proc ) then

         iter_saved  => bayes_obj_inout % iter_saved

         !remove some older once if reach max number of saved ols vec and proposal params
         if (iter_saved .eq. max_iter_saved) then

            print *, "before shift"
            print *, "min and max of proposal params (1st params)", &
               minval( bayes_obj_inout % past_params_kernel(1,:iter_saved) ), &
               maxval( bayes_obj_inout % past_params_kernel(1,:iter_saved) )
            print *, "min and max of ols vec (cons)", &
               minval( bayes_obj_inout % past_value_matrix(1,:iter_saved) ), &
               maxval( bayes_obj_inout % past_value_matrix(1,:iter_saved) )



            bayes_obj_inout % past_params_kernel = &
               eoshift( bayes_obj_inout % past_params_kernel, &
               shift = n_iter_saved_remove, dim = 2 )
            bayes_obj_inout % past_value_matrix = &
               eoshift( bayes_obj_inout % past_value_matrix, shift = n_iter_saved_remove, dim=2 )
            bayes_obj_inout % past_asset_matrix = &
               eoshift( bayes_obj_inout % past_asset_matrix, shift = n_iter_saved_remove, dim=2 )

            !IMPORTANT, need to check that eoshift is working now


            iter_saved = iter_saved - n_iter_saved_remove


            print *, "after shift"
            print *, "min and max of proposal params (1st params)", &
               minval( bayes_obj_inout % past_params_kernel(1,:iter_saved) ), &
               maxval( bayes_obj_inout % past_params_kernel(1,:iter_saved) )
            print *, "min and max of ols vec (cons)", &
               minval( bayes_obj_inout % past_value_matrix(1,:iter_saved) ), &
               maxval( bayes_obj_inout % past_value_matrix(1,:iter_saved) )

         !   print *, "iter_saved", iter_saved

         end if

         iter_saved = iter_saved + 1

         bayes_obj_inout % past_params_kernel(:,iter_saved) = &
            kernel_params_func( bayes_obj_inout % params_proposal )


!         print *, "kernel params"
!            do i=1, n_kernel_params
!               print *, bayes_obj_inout % past_params_kernel(i,iter_saved)
!            end do

!         read(*,*)


         bayes_obj_inout % past_value_matrix (:,iter_saved) = value_matrix_in
         bayes_obj_inout % past_asset_matrix (:,iter_saved) = asset_matrix_in



      end if !for myrank .eq. save_proc

   end if !est_dynamic .eqv. .true.

!2/12/14, decided to do this following bit of code in another subroutine
!   !store stuff for param history only for the root node

!   if (myrank .eq. 0 ) then
!      !now store the current parameters


!      bayes_obj_inout % params_history(:, bayes_obj_inout % iter) = &
!         bayes_obj_inout % params_current(:n_main_params)


!      !now store the unboserved heterogeneity terms (the choleksy decomp of them)
!      bayes_obj_inout % params_history( param_hetero_m_wage_var, bayes_obj_inout % iter) = &
!         bayes_obj_inout % hetero_chol_current(1,1)
!      bayes_obj_inout % params_history( param_hetero_wage_covar, bayes_obj_inout % iter) = &
!         bayes_obj_inout % hetero_chol_current(2,1)
!      bayes_obj_inout % params_history( param_hetero_wage_var, bayes_obj_inout % iter) = &
!         bayes_obj_inout % hetero_chol_current(2,2)


!   end if

end subroutine save_bayesian_params_interp

subroutine save_param_history( bayes_obj_inout )
   !save the current parameters to the history of past parameters
   class( bayes_obj_type) :: bayes_obj_inout
   real*8, dimension(n_hetero, n_hetero) :: cov_temp, chol_temp, chol_temp_transpose

   if (myrank .eq. 0 ) then
      !now store the current parameters


      bayes_obj_inout % params_history(:, bayes_obj_inout % iter) = &
         bayes_obj_inout % params_current(:n_main_params)


      !now store the unboserved heterogeneity (the covariance matrix for them)
      chol_temp = 0
      chol_temp(1,1) = bayes_obj_inout % hetero_chol_current(1,1)
      chol_temp(2,1) = bayes_obj_inout % hetero_chol_current(2,1)
      chol_temp(2,2) = bayes_obj_inout % hetero_chol_current(2,2)

      chol_temp_transpose = transpose( chol_temp )
      cov_temp = matmul( chol_temp, chol_temp_transpose )

      bayes_obj_inout % params_history( param_hetero_m_wage_var, bayes_obj_inout % iter) = &
         cov_temp(1,1)
      bayes_obj_inout % params_history( param_hetero_wage_covar, bayes_obj_inout % iter) = &
         cov_temp(2,1)
      bayes_obj_inout % params_history( param_hetero_wage_var, bayes_obj_inout % iter) = &
         cov_temp(2,2)


   end if

end subroutine

subroutine export_param_history( bayes_obj_in)
   !exports the mean and variance of a recent history of parameters to a file
   class( bayes_obj_type ), intent(in)  :: bayes_obj_in

   integer :: lb, ub, length
   real*8, dimension(n_main_params) :: mean, sd
   real*8, dimension(n_main_params, n_main_params) :: cov_matrix
   integer :: i, unit_int

   if ( myrank .eq. 0 ) then

      ub = bayes_obj_in % iter
      lb = nint (ub *bayes_export_frac)
      length = ub - lb + 1

      mean = mean_vec_est( length, n_main_params, bayes_obj_in % params_history (:, lb:ub) )
      cov_matrix = cov_matrix_est( length, n_main_params, bayes_obj_in % params_history(:, lb:ub), &
         mean )

      do i = 1, n_main_params
         sd(i) = sqrt( cov_matrix(i,i) )
      end do

      open(newunit = unit_int, file = bayes_export_file, status = "replace" )

         write(unit_int, * ) "upper bound  ", ub, " lower bound  ", lb
         write(unit_int, * ) "   "

         do i = 1, n_main_params
            call csv_write( unit_int, param_labels(i), .false. )
            call csv_write( unit_int, mean(i), .false. )
            call csv_write( unit_int, sd(i), .true. )
         end do

      close( unit_int)

   end if

   call mpi_barrier( mpi_comm_world, err)



end subroutine export_param_history


subroutine interp_matrix_bayesian_init( bayes_obj_inout, value_grid_inout )
   !this subroutine 1st iterates the value function at the initial parameters, then draws other parameters around this intial parameter
      !and iterates the value function at those parameters.  This is done to speed up converge of the value functions
   class( bayes_obj_type ), intent(in out), target :: bayes_obj_inout
   type(value_grid_type), intent(in out) :: value_grid_inout
   integer :: unit_int

   real*8, dimension(n_main_params) :: params_init
   real*8, dimension(n_grid_points) :: value_matrix_new, value_matrix_old
   real*8, dimension(n_grid_points) :: asset_matrix_new, asset_matrix_old
   real*8, dimension(n_kernel_params) :: kernel_params_temp

   integer :: i, b, m_job, job

   integer, parameter :: iter_init = 200
   integer, parameter :: max_iter = 200
   real*8, parameter :: tol = 1.0D-3
   integer, parameter :: n_howard = 0  !number of times to do the howard-improvement algorithm in which I only iterate and don't update the choice probability
   real*8, parameter :: tol_howard = 1.0D-3


   real*8 :: weight_utility_center !the utility constant for the central point
   real*8 :: init_factor !factor for adjusting the value_matrix_init


   if( myrank .eq. 0 ) print *, "within  interp_matrix_bayesian_init"

   bayes_obj_inout % iter_saved = 0

   if( bayes_obj_inout % iter .eq. 0 ) bayes_obj_inout % iter = 1

   value_matrix_old = 0
   asset_matrix_old = 0
   params_init = bayes_obj_inout % params_current(1:n_main_params)
   weight_utility_center =  bayes_obj_inout % params_current( param_weight_utility )

!   bayes_obj_inout % iter = 1

   !temporarily increase the variance of the random walk draws
   if( myrank .eq. 0 ) then
      rw_chol_joboffer_block1 = rw_chol_joboffer_block1 * rw_interp_init_adjust
      rw_chol_joboffer_block2 = rw_chol_joboffer_block2 * rw_interp_init_adjust
      rw_chol_wage_block1 = rw_chol_wage_block1 * rw_interp_init_adjust
      rw_chol_wage_block2 = rw_chol_wage_block2 * rw_interp_init_adjust
      rw_chol_wage_block3 = rw_chol_wage_block3 * rw_interp_init_adjust
      rw_chol_leisure_block1 = rw_chol_leisure_block1 * rw_interp_init_adjust
      rw_chol_leisure_block2 = rw_chol_leisure_block2 * rw_interp_init_adjust
      rw_chol_leisure_block3 = rw_chol_leisure_block3 * rw_interp_init_adjust
      rw_chol_leisure_block4 = rw_chol_leisure_block4 * rw_interp_init_adjust
      rw_chol_asset_error_block1 = rw_chol_asset_error_block1 * rw_interp_init_adjust
      rw_chol_asset_error_block2 = rw_chol_asset_error_block2 * rw_interp_init_adjust
   end if

   if (myrank .eq. 0 ) print *, "iterating for initial parameter"

   !first iterate for the initial parameter
   if( est_savings .eqv. .true. ) then
      call iterate_to_converge_all(    .false., &
                                       iter_init, &
                                       max_iter, &
                                       tol, &
                                       n_howard, &
                                       tol_howard, &
                                       bayes_obj_inout % params_current, &
                                       value_matrix_old, &
                                       asset_matrix_old, &
                                       value_grid_inout, &
                                       bayes_obj_inout % value_matrix_current, &
                                       bayes_obj_inout % asset_matrix_current )
   else

      call iterate_to_converge_value(  max_iter, &
                                       tol, &
                                       est_savings, &
                                       bayes_obj_inout % params_current, &
                                       value_matrix_old, &
                                       asset_matrix_old, &
                                       value_grid_inout, &
                                       bayes_obj_inout % value_matrix_current )
      bayes_obj_inout % asset_matrix_current = 0

   end if

   !2/14/16, having the value function form the center be the value function to start at
   value_matrix_old = bayes_obj_inout % value_matrix_current


   if( myrank .eq. 0 ) print *, "iterating for other paramters"
   !now iterate at other parameters, with the ols vec from the previous paramter as a starting value
   do i = 1, n_draws_interp_init
      if ( myrank .eq. 0 ) print *, "param draw #", i

      !draw parameters
      bayes_obj_inout % params_current(1:n_main_params) =  params_init

      do b=1, n_blocks_params
         call draw_new_params(b, bayes_obj_inout % params_current, &
                                 bayes_obj_inout % params_proposal)
      end do

      if (myrank .eq. 0 ) then
         call write_params( bayes_obj_inout % params_current, output_unit, .false. )
      end if

!2/14/16
!      !iterate, with using a modified version of the older value function as a starting value
!      init_factor = bayes_obj_inout % params_current( param_weight_utility ) / weight_utility_center
!      init_factor = init_factor * 0

!      value_matrix_old = init_factor *  bayes_obj_inout % value_matrix_current


      if ( est_savings .eqv. .true. ) then
         call iterate_to_converge_all( .false., &
                                       iter_init, &
                                       max_iter, &
                                       tol, &
                                       n_howard, &
                                       tol_howard, &
                                       bayes_obj_inout % params_current, &
                                       value_matrix_old, &
                                       bayes_obj_inout % asset_matrix_current, &
                                       value_grid_inout, &
                                       value_matrix_new, &
                                       asset_matrix_new )
      else

         call iterate_to_converge_value(  max_iter, &
                                          tol, &
                                          est_savings, &
                                          bayes_obj_inout % params_current, &
                                          value_matrix_old, &
                                          bayes_obj_inout % asset_matrix_current, &
                                          value_grid_inout, &
                                          value_matrix_new )
         asset_matrix_new = 0

      end if

      !save only to root node
      bayes_obj_inout % save_proc = nprocs -1
      call save_bayesian_params_interp( bayes_obj_inout, value_matrix_new, asset_matrix_new)

   end do

   !reset the parametes
   bayes_obj_inout % params_current(1:n_main_params) =  params_init



   !now write the file if root node
   if (myrank .eq. 0 ) then
      open(newunit = unit_int, file=init_interp_file)

      !write the initial parameter and its ols matrix to the file first
    !  kernel_params_temp = kernel_params_func( bayes_obj_inout % params_current(:) )
      call csv_write(unit_int, bayes_obj_inout % params_current(:), .true. )

      call csv_write(unit_int, bayes_obj_inout % value_matrix_current(:), .true. )
      call csv_write(unit_int, bayes_obj_inout % asset_matrix_current(:), .true. )


      !now the other matrices
      do i=1,bayes_obj_inout % iter_saved
         call csv_write(unit_int, bayes_obj_inout % past_params_kernel(:,i), .true. )

         call csv_write(unit_int, bayes_obj_inout % past_value_matrix(:,i), .true. )
         call csv_write(unit_int, bayes_obj_inout % past_asset_matrix(:,i), .true. )


      end do

      close(unit=unit_int)

   end if !root node


   !now decrease the variance back to original level
   if( myrank .eq. 0 ) then
      rw_chol_joboffer_block1 = rw_chol_joboffer_block1 / rw_interp_init_adjust
      rw_chol_joboffer_block2 = rw_chol_joboffer_block2 / rw_interp_init_adjust
      rw_chol_wage_block1 = rw_chol_wage_block1 / rw_interp_init_adjust
      rw_chol_wage_block2 = rw_chol_wage_block2 / rw_interp_init_adjust
      rw_chol_wage_block3 = rw_chol_wage_block3 / rw_interp_init_adjust
      rw_chol_leisure_block1 = rw_chol_leisure_block1 / rw_interp_init_adjust
      rw_chol_leisure_block2 = rw_chol_leisure_block2 / rw_interp_init_adjust
      rw_chol_leisure_block3 = rw_chol_leisure_block3 / rw_interp_init_adjust
      rw_chol_leisure_block4 = rw_chol_leisure_block4 / rw_interp_init_adjust
      rw_chol_asset_error_block1 = rw_chol_asset_error_block1 / rw_interp_init_adjust
      rw_chol_asset_error_block2 = rw_chol_asset_error_block2 / rw_interp_init_adjust
   end if

end subroutine interp_matrix_bayesian_init


subroutine read_interp_init(bayes_obj_inout)
   !reads in inital ols vectors I've created
   class(bayes_obj_type), intent(in out), target :: bayes_obj_inout

   integer :: i, j, m_job,job, real_ignore
   integer :: unit_int, proc_index
   real*8, dimension(n_grid_points ) :: value_matrix_temp
   real*8, dimension(n_grid_points ) :: asset_matrix_temp

   real*8, dimension( n_kernel_params, n_draws_interp_init ) :: params_read
   real*8, dimension( n_grid_points, n_draws_interp_init ) :: value_matrix_read
   real*8, dimension( n_grid_points, n_draws_interp_init ) :: asset_matrix_read

   logical :: kernel_error

   !do reading only for root
   if (myrank .eq. 0 ) then
      !read in and init the number of saved past iterations
      open(newunit = unit_int, file=init_interp_file, status = "old" )
      bayes_obj_inout % iter_saved = n_draws_interp_init

      !get the ols matrix for the stored main parameter and ols matrix
   !   read(unit_int,*) (bayes_obj_inout % params_current(j), j=1,n_main_params)

      !also have the proposal params be equal to params_current
   !   bayes_obj_inout % params_proposal(:n_main_params) = bayes_obj_inout % params_current(:n_main_params)

      !skip first line
      read(unit_int, *) real_ignore


      read(unit_int,*) ( bayes_obj_inout % value_matrix_current(j), j=1,n_grid_points)
      read(unit_int,*) ( bayes_obj_inout % asset_matrix_current(j),j=1,n_grid_points)

     ! read(*,*)


      !now read other ones centered around the posterior mean
      do i=1, n_draws_interp_init

         read(unit_int,*) ( params_read(j,i), j=1,n_kernel_params)

         read(unit_int,*) ( value_matrix_read(j,i), j=1,n_grid_points)
         read(unit_int,*) ( asset_matrix_read(j,i), j=1,n_grid_points)


      end do


!      print *, "value matrix"
!      do i=1,bayes_obj_inout % iter_saved

!         do job = 1,1
!         do m_job = 1,1
!            print *, bayes_obj_inout % past_value_matrix(1,m_job,job,i)
!         end do
!         end do

!      end do


   !   print *, "asset matrix"
   !   do i=1,bayes_obj_inout % iter_saved

   !      do job = 1,1
   !      do m_job = 1,1
   !         print *, bayes_obj_inout % past_asset_matrix(1,m_job,job,i)
   !      end do
   !      end do

   !   end do

      close(unit=unit_int)

     ! stop

   end if !for root node



   !now for passing the init parameters and value function to other processes and storing them there.
   call mpi_bcast( params_read, size(params_read), mpi_real8, 0, mpi_comm_world, err )
   call mpi_bcast( value_matrix_read, size(value_matrix_read), mpi_real8, 0, mpi_comm_world, err )
   call mpi_bcast( asset_matrix_read, size(asset_matrix_read), mpi_real8, 0, mpi_comm_world, err )

   proc_index = 0
   bayes_obj_inout % iter_saved = 0

   do i = 1, n_draws_interp_init

      bayes_obj_inout % save_proc = proc_index !check this

      if (proc_index .eq. myrank ) then

         bayes_obj_inout % iter_saved = bayes_obj_inout % iter_saved + 1
         bayes_obj_inout % past_params_kernel(:, bayes_obj_inout % iter_saved ) = &
            params_read(:,i)
         bayes_obj_inout % past_value_matrix(:, bayes_obj_inout % iter_saved ) = &
            value_matrix_read(:,i)
         bayes_obj_inout % past_asset_matrix(:, bayes_obj_inout % iter_saved ) = &
            asset_matrix_read(:,i)

      end if


      if ( proc_index .eq. nprocs - 1 ) then
         proc_index = 0
      else
         proc_index = proc_index + 1

      end if

   end do

   call mpi_barrier( mpi_comm_world, err )


   !now update bandwidth
   call update_bandwidth( bayes_obj_inout )

!   do i=1,n_kernel_params
!      print *, i, bayes_obj_inout % inv_bandwidth(i)
!   end do


 !  print *, "iter saved", bayes_obj_inout % iter_saved


   !for now, changing some things around to have value and asset matrix current to be based on kernel interpolation
   if( est_dynamic .eqv. .true. ) then

      kernel_error = .false.
      call interp_kernel(  bayes_obj_inout, &
                           bayes_obj_inout % params_current(:n_main_params), &
                           value_matrix_temp, &
                           asset_matrix_temp, &
                           kernel_error )

      bayes_obj_inout % value_matrix_current = value_matrix_temp
      bayes_obj_inout % asset_matrix_current = asset_matrix_temp



   !!   print *, "value matrix cons", bayes_obj_inout % value_matrix_current(1,1,1)
   !!   print *, "asset matrix cons", bayes_obj_inout % asset_matrix_current(1,1,1)

   !!   call interp_matrix_write(bayes_obj_inout % asset_matrix_current, output_unit)
   !!   stop

      if ( kernel_error .eqv. .true. ) then
         print *, "error with kernels for read_interp_init"
         stop
      end if

   end if !for estimate dynamic

   !now broadcast to other processes
   call mpi_bcast( bayes_obj_inout % params_current, &
      size(bayes_obj_inout % params_current), mpi_real8, 0, mpi_comm_world, err )
   call mpi_bcast( bayes_obj_inout % params_proposal, &
      size(bayes_obj_inout % params_proposal), mpi_real8, 0, mpi_comm_world, err )
   call mpi_bcast( bayes_obj_inout % value_matrix_current, &
      size(bayes_obj_inout % value_matrix_current), mpi_real8, 0, mpi_comm_world, err )
   call mpi_bcast( bayes_obj_inout % asset_matrix_current, &
      size(bayes_obj_inout % asset_matrix_current), mpi_real8, 0, mpi_comm_world, err )


end subroutine

subroutine val_update_bayes( bayes_obj_inout, value_grid_inout )
   !updates my set of value function to be used for the kernel-weighting mechanishm.  What is does
   ! 1) determines the recent mean of the parameters
   ! 2) get the asset and value function around this mean
   ! 3) draws parameters around that mean
   ! 4) gets the value function and asset policy function for those parameters
   ! 5) stores these value functions and parameters
   class( bayes_obj_type), intent(in out) :: bayes_obj_inout
   type(value_grid_type), intent(in out) :: value_grid_inout

   integer, parameter :: iter_init = -1 !prevents from doing the initial iteration
   integer, parameter :: n_howard = 0  !number of times to do the howard-improvement algorithm in which I only iterate and don't update the choice probability
   real*8, parameter :: tol_howard = 1.0D-3
   real*8, parameter :: dif_error_tol = 7000 !2/17/14, noticed that sometimes there would be a wierd paramete draw that yielded a value function
      !that diverged (numbers kept getting more and more giat), so I introduced this to excluded value functions where this happened

   integer :: b,i
   integer :: lb, ub, length
   real*8, dimension(n_main_params) :: mean_recent
   real*8, dimension(n_grid_points) :: value_matrix_start, asset_matrix_start
   real*8, dimension(n_grid_points) :: value_matrix_new, asset_matrix_new
   real*8, dimension(n_params) :: params_new
   logical :: kernel_error
   real*8 :: dif_new

!   if ( myrank .eq. 0 ) print *, "save_proc", bayes_obj_inout % save_proc
!   call mpi_barrier( mpi_comm_world, err )

!   do i = 0, nprocs - 1
!      if ( i .eq. myrank ) then
!         print *, "myrank", myrank, "iter_saved", bayes_obj_inout % iter_saved
!      end if
!      call mpi_barrier( mpi_comm_world, err )
!   end do


   !first get the recent mean of the parameters
   if (myrank .eq. 0 ) then
      print *, "****************************************"
      print *, "iter", bayes_obj_inout % iter
      print *, "  "
      print *, "   "

      lb = max( bayes_obj_inout % iter - val_update_n_history , 1 )
      ub = bayes_obj_inout % iter
      length = ub - lb + 1
      mean_recent = mean_vec_est( length, n_main_params, bayes_obj_inout % params_history(:,lb:ub) )

     ! print *, "recent estimated mean", mean_recent ( param_weight_utility )
     ! read(*,*)

      !setup new parameters
      params_new( : n_main_params  ) = mean_recent
      params_new( n_main_params +1 : ) = bayes_obj_inout % params_current( n_main_params+ 1: )

!      if ( 17200 .eq. bayes_obj_inout % iter ) then
!         call write_params(params_new, output_unit, .false.)
!      end if

   end if !myrank .eq. 0

  ! call mpi_barrier( mpi_comm_world, err )

   !now for setting up the initial value and asset matrices
   call mpi_bcast( params_new, size( params_new ), mpi_real8, 0, mpi_comm_world, err ) !need to broadcast before doing kernel weighting



  ! value_matrix_start = 0 !had issues in the past when this wasn't set to zero, I may change this in the future.



   !temporarily increase the variance of the random walk draws
   if( myrank .eq. 0 ) then
      rw_chol_joboffer_block1 = rw_chol_joboffer_block1 * val_update_sd_inc
      rw_chol_joboffer_block2 = rw_chol_joboffer_block2 * val_update_sd_inc
      rw_chol_wage_block1 = rw_chol_wage_block1 * val_update_sd_inc
      rw_chol_wage_block2 = rw_chol_wage_block2 * val_update_sd_inc
      rw_chol_wage_block3 = rw_chol_wage_block3 * val_update_sd_inc
      rw_chol_leisure_block1 = rw_chol_leisure_block1 * val_update_sd_inc
      rw_chol_leisure_block2 = rw_chol_leisure_block2 * val_update_sd_inc
      rw_chol_leisure_block3 = rw_chol_leisure_block3 * val_update_sd_inc
      rw_chol_leisure_block4 = rw_chol_leisure_block4 * val_update_sd_inc
      rw_chol_asset_error_block1 = rw_chol_asset_error_block1 * val_update_sd_inc
      rw_chol_asset_error_block2 = rw_chol_asset_error_block2 * val_update_sd_inc
   end if


   !now draw new parameters and get the value function for that parameter
   do i = 1, val_update_n_draws

      !set the new parameters equal to the mean before perturbing them
      if (myrank .eq. 0 ) params_new(: n_main_params) = mean_recent

      !drawing new parameters
      do b=1, n_blocks_params
         call draw_new_params(b, params_new, &
                                 bayes_obj_inout % params_proposal )
      end do

!   if ( ( 17200 .eq. bayes_obj_inout % iter ) .and. (myrank .eq. 0 ) ) then
!      print *, "value matrix"
!      call interp_matrix_write(value_matrix_start,output_unit)

!      print *, "asset matrix"
!      call interp_matrix_write(asset_matrix_start,output_unit)

!      print_test_global = .true.
!   end if

!      if ( ( 17200 .eq. bayes_obj_inout % iter ) .and. (myrank .eq. 0 ) ) then
!         print *, "draw no", i
!         call write_params(params_new, output_unit, .false.)
!      end if

     ! if ( myrank .eq. 0 ) then
     !    print *, params_new( param_weight_utility)
     !    read(*,*)
     ! end if

   !do kernel interpolation to get a starting value for the value function

      call interp_kernel(  bayes_obj_inout, &
                           params_new, &
                           value_matrix_start, &
                           asset_matrix_start, &
                           kernel_error )

      if ( ( kernel_error .eqv. .true. ) ) then
         if (myrank .eq. 0 )  print *, "error with kernel in val_update_bayes, setting matrices &
            to zero (with assset cons initialized to a value) "

         value_matrix_start = 0
         asset_matrix_start = 0  !really want to do kernel weighting for this one
         asset_matrix_start(1) = 2.9957322735539909

      end if

      call mpi_barrier( mpi_comm_world, err)

      !now iterate
      if ( est_savings .eqv. .true. ) then
         call iterate_to_converge_all( .false., &
                                       iter_init, &
                                       val_update_max_iter, &
                                       val_update_tol, &
                                       n_howard, &
                                       tol_howard, &
                                       params_new, &
                                       value_matrix_start, &
                                       asset_matrix_start, &
                                       value_grid_inout, &
                                       value_matrix_new, &
                                       asset_matrix_new, &
                                       dif_new )
      else

         call iterate_to_converge_value(  val_update_max_iter, &
                                          val_update_tol, &
                                          est_savings, &
                                          params_new, &
                                          value_matrix_start, &
                                          asset_matrix_start, &
                                          value_grid_inout, &
                                          value_matrix_new )
         asset_matrix_new = 0

      end if

!      if ( myrank .eq. 0 ) then
!         print *, "value and asset matrix cons", value_matrix_new(1), asset_matrix_new(1)
!       !  read(*,*)
!      end if

      !now save these interpolation coefficients along with their respective parameters
         !note, the respective parameters are stored in bayes_obj_inout % proposal_params, which was
         !stores from the draw_new_parameters routine
      if ( dif_new < dif_error_tol ) then
         call save_bayesian_params_interp( bayes_obj_inout, value_matrix_new, asset_matrix_new)
      else
         if (myrank .eq. 0 ) print *, "not saving these parameters because I got that they didn't converge to reasonable values"
         print *, "stopping"
         call mpi_finalize(err)
         stop
      end if

   end do

   !change the random walk matrices back to their original values
   if( myrank .eq. 0 ) then
      rw_chol_joboffer_block1 = rw_chol_joboffer_block1 / val_update_sd_inc
      rw_chol_joboffer_block2 = rw_chol_joboffer_block2 / val_update_sd_inc
      rw_chol_wage_block1 = rw_chol_wage_block1 / val_update_sd_inc
      rw_chol_wage_block2 = rw_chol_wage_block2 / val_update_sd_inc
      rw_chol_wage_block3 = rw_chol_wage_block3 / val_update_sd_inc
      rw_chol_leisure_block1 = rw_chol_leisure_block1 / val_update_sd_inc
      rw_chol_leisure_block2 = rw_chol_leisure_block2 / val_update_sd_inc
      rw_chol_leisure_block3 = rw_chol_leisure_block3 / val_update_sd_inc
      rw_chol_leisure_block4 = rw_chol_leisure_block4 / val_update_sd_inc
      rw_chol_asset_error_block1 = rw_chol_asset_error_block1 / val_update_sd_inc
      rw_chol_asset_error_block2 = rw_chol_asset_error_block2 / val_update_sd_inc
   end if

!   if ( myrank .eq. 0 ) print *, "save_proc", bayes_obj_inout % save_proc
!   call mpi_barrier( mpi_comm_world, err )

!   do i = 0, nprocs - 1
!      if ( i .eq. myrank ) then
!         print *, "myrank", myrank, "iter_saved", bayes_obj_inout % iter_saved
!      end if

!      call mpi_barrier( mpi_comm_world, err )
!   end do


!   if ( ( 17200 .eq. bayes_obj_inout % iter )  ) then
!      print *, "stopping from process", myrank
!      call mpi_finalize(err)
!   end if


end subroutine val_update_bayes



subroutine get_new_heteo_ind(like_object_inout, params_in,hetero_chol_in)
   !draws new unobserved heterogeneity for individual households

   type(like_type), intent(in out) :: like_object_inout
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(n_hetero,n_hetero), intent(in) :: hetero_chol_in

   real*8, dimension(n_hetero) :: standard_normals
   integer :: m_hours_new, hours_new
   real*8 :: m_wage_new, wage_new
   integer :: k,t




   !draw new unboserved heterogeneity
   do k=1,n_hetero
      standard_normals(k) = rnorm()
   end do

   like_object_inout % hetero = matmul( hetero_chol_in, standard_normals )

   !now update the wage means (time_invariant and time_varying)
   like_object_inout % mygrid_t_invar % rv(var_m_wage_mean) = &
      like_object_inout % m_wage_mean_nohetero + &
      like_object_inout % hetero(hetero_m_wage)

   like_object_inout % mygrid_t_invar % rv(var_wage_mean) = &
      like_object_inout % wage_mean_nohetero + &
      like_object_inout % hetero(hetero_wage)

   !incorporate bounds
   like_object_inout % mygrid_t_invar % rv(var_m_wage_mean)  = bounds_func( &
      like_object_inout % mygrid_t_invar % rv(var_m_wage_mean), &
      grid_min( var_m_wage_mean ), grid_max( var_m_wage_mean) )

   like_object_inout % mygrid_t_invar % rv(var_wage_mean)  = bounds_func( &
      like_object_inout % mygrid_t_invar % rv(var_wage_mean), &
      grid_min( var_wage_mean ), grid_max( var_wage_mean) )


!commenting this out because currently I calculate these things when I draw the job variables


!   do t=2, like_object_inout %  max_time

!      !first husband
!      if (like_object_inout % m_receive_new(t) .eq. 1) then
!         m_wage_new = like_object_inout % mygrid_husband_accept(t) % rv( var_m_wage )
!         m_hours_new = like_object_inout % mygrid_husband_accept(t) % iv( var_m_hours )

!         !wage probability
!         like_object_inout % m_wage_xbeta(t) = &
!            m_wage_xb_func(   like_object_inout % mygrid_t_invar, &
!                              like_object_inout % mygrid_t_var(t), &
!                              params_in, &
!                              m_hours_new )


!         !commenting out the wage probabilities because I'll be recomputing these for drawing
!            !new wages and joboffers

!!         like_object_inout % m_ln_wage_prob(t) = &
!!            ln_pdf_norm(   log( m_wage_new), &
!!                           like_object_inout % m_wage_xbeta(t), &
!!                           params_in(param_sd_wage) )

!      else
!!         like_object_inout % m_ln_wage_prob(t) = 0
!      end if


!      !now the wife
!      if (like_object_inout % receive_new(t) .eq. 1) then
!         !some pointing to make code easier to read
!         wage_new = like_object_inout % mygrid_wife_accept(t) % rv( var_wage )
!         hours_new = like_object_inout % mygrid_wife_accept(t) % iv( var_hours )

!         !wage probability
!         like_object_inout % wage_xbeta(t) = &
!            wage_xb_func(     like_object_inout % mygrid_t_invar, &
!                              like_object_inout % mygrid_t_var(t), &
!                              params_in, &
!                              hours_new )

!!         like_object_inout % ln_wage_prob(t) = &
!!            ln_pdf_norm(   log(wage_new), &
!!                           like_object_inout % wage_xbeta(t), &
!!                           params_in(param_sd_wage) )

!      else
!!         like_object_inout % ln_wage_prob(t) = 0
!      end if

!   end do !over t


end subroutine get_new_heteo_ind

!subroutine get_new_unemp_length_ind(like_object_inout, hh_data_in, params_in, max_time_out )
!   !draws new unemployment lenghts if the husband or wife starts the panel without a job

!   type(like_type), intent(in out) :: like_object_inout
!   type(data_i_type), intent(in) :: hh_data_in
!   real*8, dimension(n_params), intent(in) :: params_in
!   integer, intent(out) :: max_time_out  !says how many time periods I'm dealing with

!   real*8 :: m_ui_ben_init, ui_ben_init
!   integer :: m_unemp_length_new, unemp_length_new
!   integer :: t
!   integer :: time_span

!   max_time_out = 0

!   if ( like_object_inout % mygrid_t_var(1) % iv(var_m_job ) .eq. 0 ) then

!      m_unemp_length_new = -1 +  nint( r_uniform(  dble(hh_data_in % m_min_unemp_length), &
!                                                dble(hh_data_in % m_max_unemp_length) ) )
!      m_ui_ben_init = like_object_inout % mygrid_t_var(1) % rv(var_m_ui_ben)

!      t = 1
!      do while (  ( t .le. like_object_inout % m_time_unemp_end(1) ) .and. &
!                  ( hh_data_in % m_min_unemp_length + t .le. max_job_length) )
!                     !after hh_data_in % m_min_unemp_length + t, then I know for sure that no matter what the initial
!                        !draw for m_unemp_length_new was, the length unemployed is capped at 48

!         !increment length
!         like_object_inout % mygrid_t_var(t) % rv(var_m_unemp_length) = &
!            min( m_unemp_length_new + t, max_job_length )
!         like_object_inout % mygrid_wife_accept(t) % rv(var_m_unemp_length) = &
!            like_object_inout % mygrid_t_var(t) % rv(var_m_unemp_length)

!         !have ui if haven't exceded max length
!         if (  nint(like_object_inout % mygrid_t_var(t) % rv(var_m_unemp_length) ) .le. &
!               nint(like_object_inout % mygrid_t_var(t) % rv(var_uilength) ) ) then

!            like_object_inout % mygrid_t_var(t) % rv(var_m_ui_ben) = m_ui_ben_init
!            like_object_inout % mygrid_wife_accept(t) % rv(var_m_ui_ben) = m_ui_ben_init

!         else
!            like_object_inout % mygrid_t_var(t) % rv(var_m_ui_ben) = 0
!            like_object_inout % mygrid_wife_accept(t) % rv(var_m_ui_ben) = 0
!         end if

!         !change job offer prob
!         like_object_inout % m_joboffer_xbeta(t) = &
!            m_joboffer_xb_like_func(   like_object_inout % mygrid_t_invar, &
!                                       like_object_inout % mygrid_t_var(t), &
!                                       params_in, &
!                                       hh_data_in % data_t(t) % seam )

!         like_object_inout % m_joboffer_prob(t) = &
!            logit_prob( like_object_inout % m_joboffer_xbeta(t) )

!         t = t + 1
!      end do

!   max_time_out = t -1

!   end if !of whether the husband started out without a job


!   if ( like_object_inout % mygrid_t_var(1) % iv(var_job ) .eq. 0 ) then

!      unemp_length_new = -1 +  nint ( r_uniform(  dble(hh_data_in % min_unemp_length), &
!                                                dble(hh_data_in % max_unemp_length) ) )
!      ui_ben_init = like_object_inout % mygrid_t_var(1) % rv(var_ui_ben)

!      t = 1
!      do while (  ( t .le. like_object_inout % time_unemp_end(1) ) .and. &
!                  ( hh_data_in % min_unemp_length + t .le. max_job_length) )
!                     !after hh_data_in % m_min_unemp_length + t, then I know for sure that no matter what the initial
!                        !draw for m_unemp_length_new was, the length unemployed is capped at 48

!         !increment length
!         like_object_inout % mygrid_t_var(t) % rv(var_unemp_length) = &
!            min( unemp_length_new + t, max_job_length )
!         like_object_inout % mygrid_husband_accept(t) % rv(var_unemp_length) = &
!            like_object_inout % mygrid_t_var(t) % rv(var_unemp_length)

!         !have ui if haven't exceded max length
!         if (  nint(like_object_inout % mygrid_t_var(t) % rv(var_unemp_length) ) .le. &
!               nint(like_object_inout % mygrid_t_var(t) % rv(var_uilength) ) ) then

!            like_object_inout % mygrid_t_var(t) % rv(var_ui_ben) = ui_ben_init
!            like_object_inout % mygrid_husband_accept(t) % rv(var_ui_ben) = ui_ben_init

!         else
!            like_object_inout % mygrid_t_var(t) % rv(var_ui_ben) = 0
!            like_object_inout % mygrid_husband_accept(t) % rv(var_ui_ben) = 0
!         end if

!         !change job offer prob
!         like_object_inout % joboffer_xbeta(t) = &
!            joboffer_xb_like_func(  like_object_inout % mygrid_t_invar, &
!                                    like_object_inout % mygrid_t_var(t), &
!                                    params_in, &
!                                    hh_data_in % data_t(t) % seam )

!         like_object_inout % joboffer_prob(t) = &
!            logit_prob( like_object_inout % joboffer_xbeta(t) )

!         t = t + 1
!      end do

!   max_time_out = max( max_time_out, t- 1)

!   end if !of whether the husband started out without a job



!end subroutine get_new_unemp_length_ind

subroutine get_new_joboffers_ind(like_object_inout, hh_data_in, params_in,random_in)
   !draws new potential joboffers

   type(like_type), intent(in out) :: like_object_inout
   type(data_i_type), intent(in) :: hh_data_in
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(iter_m_job_offer: iter_hours,N_t), intent(in) :: random_in

   integer :: m_hours_new, hours_new
   real*8 :: m_wage_new, wage_new
   real*8 :: m_ui_ben_new, ui_ben_new
   real*8 :: m_non_pec_norm_new, non_pec_norm_new
   integer :: t, t_prime, lb, m_accept_new, accept_new
   logical :: non_pec_continue
   type(mygrid_type), pointer :: mygrid_temp

   real*8 :: m_wage_xb, wage_xb


   !draw new wage and non_pec for t=1 if a job is held, or draw new ui_benefit if job is not held
   !husband
   if ( like_object_inout % mygrid_t_var(1) % iv( var_m_job ) .eq. 1 ) then
      m_non_pec_norm_new = rnorm()
      m_wage_new =   hh_data_in % data_t(1) % m_wage * &
                     exp(  params_in( param_sd_m_wage_error) * rnorm() )

      !incorporate bounds
      m_non_pec_norm_new = bounds_func( m_non_pec_norm_new, grid_min( var_m_non_pec_norm ), &
                                                            grid_max( var_m_non_pec_norm ) )
      m_wage_new = bounds_func( m_wage_new, grid_min( var_m_wage ), grid_max( var_m_wage ) )


      do t_prime = 1, like_object_inout % m_time_job_ends(1)
         like_object_inout % mygrid_t_var(t_prime) % rv(var_m_non_pec_norm) = m_non_pec_norm_new
         like_object_inout % mygrid_t_var(t_prime) % rv(var_m_wage) = m_wage_new
      end do

   else
      m_ui_ben_new = min( hh_data_in % m_ui_ben_init * &
                     exp(  params_in( param_sd_m_wage_error) * rnorm() ), &
                     like_object_inout % mygrid_t_var(1) % rv(var_uimax) )

      m_ui_ben_new = bounds_func( m_ui_ben_new, grid_min( var_m_ui_ben ), grid_max( var_m_ui_ben ) )


      do t_prime = 1, like_object_inout % m_time_unemp_end(1)


         !have ui if haven't exceded max length
         if (  nint(like_object_inout % mygrid_t_var(t_prime) % rv(var_m_unemp_length) ) .le. &
               nint(like_object_inout % mygrid_t_var(t_prime) % rv(var_uilength) ) ) then

            like_object_inout % mygrid_t_var(t_prime) % rv(var_m_ui_ben) = m_ui_ben_new

         else
            like_object_inout % mygrid_t_var(t_prime) % rv(var_m_ui_ben) = 0
         end if

      end do

   end if

   !wife
   if ( like_object_inout % mygrid_t_var(1) % iv( var_job ) .eq. 1 ) then
      non_pec_norm_new = rnorm()
      wage_new =   hh_data_in % data_t(1) % wage * &
                     exp( params_in( param_sd_wage_error) * rnorm() )

      !incorporate bounds
      non_pec_norm_new = bounds_func( non_pec_norm_new,  grid_min( var_non_pec_norm ), &
                                                         grid_max( var_non_pec_norm ) )
      wage_new = bounds_func( wage_new, grid_min( var_wage ), grid_max( var_wage ) )

      do t_prime = 1, like_object_inout % time_job_ends(1)
         like_object_inout % mygrid_t_var(t_prime) % rv(var_non_pec_norm) = non_pec_norm_new
         like_object_inout % mygrid_t_var(t_prime) % rv(var_wage) = wage_new
      end do
   else
      ui_ben_new = min( hh_data_in % ui_ben_init * &
                     exp(  params_in( param_sd_wage_error) * rnorm() ) , &
                     like_object_inout % mygrid_t_var(1) % rv(var_uimax) )

      !incorporate bounds
      ui_ben_new = bounds_func( ui_ben_new, grid_min( var_ui_ben ), grid_max( var_ui_ben ) )

      do t_prime = 1, like_object_inout % time_unemp_end(1)


         !have ui if haven't exceded max length
         if (  nint(like_object_inout % mygrid_t_var(t_prime) % rv(var_unemp_length) ) .le. &
               nint(like_object_inout % mygrid_t_var(t_prime) % rv(var_uilength) ) ) then

            like_object_inout % mygrid_t_var(t_prime) % rv(var_ui_ben) = ui_ben_new

         else
            like_object_inout % mygrid_t_var(t_prime) % rv(var_ui_ben) = 0
         end if

      end do

   end if


   !now for other time periods
   do t=2, like_object_inout % max_time


      !if job was lost in the previous period, deal with assign this ui benefit over time
      if (  ( like_object_inout % m_jobloss(t-1) .eq. 1 ) .or. &
            ( like_object_inout % jobloss(t-1) .eq. 1 ) ) then

         !first figure out which mygrid needs to be dealt with
         m_accept_new = like_object_inout % m_accept_new(t-1)
         accept_new = like_object_inout % accept_new(t-1)

         if (( m_accept_new .eq. 0) .and. (accept_new .eq. 0)) then
            mygrid_temp => like_object_inout % mygrid_t_var(t-1)
         else if ( m_accept_new .eq. 0) then
            mygrid_temp => like_object_inout % mygrid_husband_accept(t-1)
         else if ( accept_new .eq. 0) then
            mygrid_temp => like_object_inout % mygrid_wife_accept(t-1)
         else
            mygrid_temp => like_object_inout % mygrid_both_accept(t-1)
         end if

         !husband
         if ( like_object_inout % m_jobloss(t-1) .eq. 1 ) then
            m_ui_ben_new = m_ui_ben_func(   mygrid_temp, like_object_inout % m_ui_ever(t-1) )

            !incorporate bounds
            m_ui_ben_new = bounds_func( m_ui_ben_new, grid_min( var_m_ui_ben ), grid_max( var_m_ui_ben ) )


            do t_prime = t, like_object_inout % m_time_unemp_end(t-1)

               if (  nint(like_object_inout % mygrid_t_var(t_prime) % rv(var_m_unemp_length) ) .le. &
                     nint(like_object_inout % mygrid_t_var(t_prime) % rv(var_uilength) ) ) then
                  like_object_inout % mygrid_t_var(t_prime) % rv(var_m_ui_ben) = m_ui_ben_new
               else
                  like_object_inout % mygrid_t_var(t_prime) % rv(var_m_ui_ben) = 0
               end if

            end do
         end if

         !wife
         if ( like_object_inout % jobloss(t-1) .eq. 1 ) then
            ui_ben_new = ui_ben_func(   mygrid_temp, like_object_inout % ui_ever(t-1) )

            !incorporate bounds
            ui_ben_new = bounds_func( ui_ben_new, grid_min( var_ui_ben ), grid_max( var_ui_ben ) )


            do t_prime = t, like_object_inout % time_unemp_end(t-1)

               if (  nint(like_object_inout % mygrid_t_var(t_prime) % rv(var_unemp_length) ) .le. &
                     nint(like_object_inout % mygrid_t_var(t_prime) % rv(var_uilength) ) ) then

                  like_object_inout % mygrid_t_var(t_prime) % rv(var_ui_ben) = ui_ben_new

               else
                  like_object_inout % mygrid_t_var(t_prime) % rv(var_ui_ben) = 0
               end if

            end do
         end if

      end if !of whether the husband or the wife lost a job



      !now draw job variable for accepted/received jobs as well as the received indicators
         !if a job wasn't accepted this month
      !first draw husband's variables
      if ( like_object_inout % m_accept_new(t) .eq. 1 ) then
         !if accepted new job, get new non pec and true wage

         !non pec
         m_non_pec_norm_new = rnorm()

         !hours (which doesn't change)
         m_hours_new = like_object_inout % mygrid_husband_accept(t) % iv( var_m_hours )

         !redo hours probability.  This doesn't change for spouses who accepted a job, but for spouse's that didnt' accept a job
            !the value of this probability could change depending on whether it was received.  so because I need to reswitch the pointers,
            !this need to be recomputed
         if ( like_object_inout % mygrid_husband_accept(t) % iv(var_m_hours) .eq. 20) then
            like_object_inout % m_hours_prob(t) = params_in(param_prob_pt)
         else
            like_object_inout % m_hours_prob(t) = 1 - params_in(param_prob_pt)
         end if




         !draw new wage based on observed wage
         m_wage_new =   hh_data_in % data_t(t) % m_wage * &
                        exp(  params_in( param_sd_m_wage_error) * rnorm() )

         !store the new_wag (which doesn't incorporate bounds yet)
         like_object_inout % m_wage_new(t)  = m_wage_new


         !wage probability (which doesn't incorporate bounds yet)
         m_wage_xb = &
            m_wage_xb_func(   like_object_inout % mygrid_t_invar, &
                              like_object_inout % mygrid_t_var(t), &
                              params_in, &
                              m_hours_new )

         like_object_inout % m_ln_wage_prob(t) = &
            m_ln_wage_prob_func( m_wage_new, &
                                 m_wage_xb, &
                                 params_in )


         !incorporate bounds
         m_non_pec_norm_new = bounds_func( m_non_pec_norm_new, grid_min( var_m_non_pec_norm ), &
                                                               grid_max( var_m_non_pec_norm ) )
         m_wage_new = bounds_func( m_wage_new, grid_min( var_m_wage ), grid_max( var_m_wage ) )

         !assign the new variables to the husband accept mygrids
         like_object_inout % mygrid_husband_accept(t) % rv(var_m_non_pec_norm) = m_non_pec_norm_new
         like_object_inout % mygrid_husband_accept(t) % rv(var_m_wage) = m_wage_new


         !assign these new variables as state variable over time
         if (t .ne. like_object_inout % max_time ) then
            do t_prime = t+1, like_object_inout % m_time_job_ends(t)
               like_object_inout % mygrid_t_var(t_prime) % rv(var_m_non_pec_norm) = m_non_pec_norm_new
               like_object_inout % mygrid_t_var(t_prime) % rv(var_m_wage) = m_wage_new
            end do
         end if


      else !job offer not accepted, so need to determine if a joboffer was received
         if ( random_in( iter_m_job_offer,t ) < like_object_inout % m_joboffer_prob(t) ) then
            like_object_inout % m_receive_new(t) = 1

            !non pec
            m_non_pec_norm_new = rnorm()
        !    like_object_inout % m_non_pec_prob(t) = ln_pdf_norm( m_non_pec_norm_new )

            !hours
            if ( random_in(iter_m_hours,t) < params_in(param_prob_pt) ) then
               m_hours_new = 20
               like_object_inout % m_hours_prob(t) = params_in(param_prob_pt)
            else
               m_hours_new = 40
               like_object_inout % m_hours_prob(t) = 1- params_in(param_prob_pt)
            end if

            !wage
           ! m_wage_new = rnorm()
         !   like_object_inout % m_ln_wage_prob(t) = ln_pdf_norm(m_wage_new, 0.0D+0, 1.0D+0) - &
         !      log( params_in(param_sd_wage) )

            m_wage_xb = &
               m_wage_xb_func(   like_object_inout % mygrid_t_invar, &
                                 like_object_inout % mygrid_t_var(t), &
                                 params_in, &
                                 m_hours_new)

            m_wage_new = exp( m_wage_xb + params_in(param_sd_wage) * rnorm() )

            !store new wage( which doesn't incorporate bounds)
            like_object_inout % m_wage_new(t) = m_wage_new


            !calculate wage probability
            like_object_inout % m_ln_wage_prob(t) = &
               m_ln_wage_prob_func( m_wage_new, &
                                    m_wage_xb, &
                                    params_in )

!            if ( (t .eq. 25 ) .and. (i_global .eq. 172) ) then
!               print *, "m_wage_new", m_wage_new
!               print *, "m_wage_xb", m_wage_xb
!               print *, "sd", params_in(param_sd_wage)
!               print *, "ln_pdf", like_object_inout % m_ln_wage_prob(t)

!               print *, "  "
!            end if

            !incorproate bounds
            m_non_pec_norm_new = bounds_func( m_non_pec_norm_new, grid_min( var_m_non_pec_norm ), &
                                                                  grid_max( var_m_non_pec_norm ) )
            m_wage_new = bounds_func( m_wage_new, grid_min( var_m_wage ), grid_max( var_m_wage ) )


            !now assign variable to the mygrid_husband_accept type
            !husband's variables with the newly draw variables
            like_object_inout % mygrid_husband_accept(t) % rv( var_m_wage )             = m_wage_new
            like_object_inout % mygrid_husband_accept(t) % rv( var_m_non_pec_norm )     = m_non_pec_norm_new
            like_object_inout % mygrid_husband_accept(t) % iv( var_m_hours )        = m_hours_new


         else
            like_object_inout % m_receive_new(t) = 0

            like_object_inout % m_ln_wage_prob(t) = 0
            like_object_inout % m_hours_prob(t) = 1

         end if !for whether a joboffer was received for the husband
      end if !for wheter a joboffer was accepted for the husband

      !now draw wife's variables
      if ( like_object_inout % accept_new(t) .eq. 1 ) then
         !if accepted new job, get new non pec and true wage

         !non pec
         non_pec_norm_new = rnorm()

         hours_new = like_object_inout % mygrid_wife_accept(t) % iv( var_hours )

         !redo hours probability
         if ( like_object_inout % mygrid_wife_accept(t) % iv(var_hours) .eq. 20) then
            like_object_inout % hours_prob(t) = params_in(param_prob_pt)
         else
            like_object_inout % hours_prob(t) = 1 - params_in(param_prob_pt)
         end if


         wage_new =   hh_data_in % data_t(t) % wage * &
                        exp(  params_in( param_sd_wage_error) * rnorm() )

         !store new wage (which doesn't incorporate bounds)
         like_object_inout % wage_new(t)  = wage_new


         !wage probability
         wage_xb = &
            wage_xb_func(  like_object_inout % mygrid_t_invar, &
                           like_object_inout % mygrid_t_var(t), &
                           params_in, &
                           hours_new )

         like_object_inout % ln_wage_prob(t) = &
            ln_wage_prob_func( wage_new, &
                                 wage_xb, &
                                 params_in )


         !incorporate bounds
         non_pec_norm_new = bounds_func( non_pec_norm_new,  grid_min( var_non_pec_norm ), &
                                                            grid_max( var_non_pec_norm ) )
         wage_new = bounds_func( wage_new, grid_min( var_wage ), grid_max( var_wage ) )

         ! assign the new variables to the wife accept mygrids
         like_object_inout % mygrid_wife_accept(t) % rv(var_non_pec_norm) = non_pec_norm_new
         like_object_inout % mygrid_wife_accept(t) % rv(var_wage) = wage_new


         !assign this new variables as a state variable over time
         if (t .ne. like_object_inout % max_time ) then
            do t_prime = t+1, like_object_inout % time_job_ends(t)
               like_object_inout % mygrid_t_var(t_prime) % rv(var_non_pec_norm) = non_pec_norm_new
               like_object_inout % mygrid_t_var(t_prime) % rv(var_wage) = wage_new
            end do
         end if

      else !job offer not received, so need to determine if a joboffer was received
         if ( random_in( iter_job_offer,t ) < like_object_inout % joboffer_prob(t) ) then
            like_object_inout % receive_new(t) = 1

            !non pec
            non_pec_norm_new = rnorm()
         !   like_object_inout % non_pec_prob(t) = ln_pdf_norm( non_pec_norm_new )

            !hours
            if ( random_in(iter_hours,t) < params_in(param_prob_pt) ) then
               hours_new = 20
               like_object_inout % hours_prob(t) = params_in(param_prob_pt)
            else
               hours_new = 40
               like_object_inout % hours_prob(t) = 1- params_in(param_prob_pt)
            end if


            !wage
          !  wage_new = rnorm()
          !  like_object_inout % ln_wage_prob(t) = ln_pdf_norm(wage_new, 0.0D+0, 1.0D+0) - &
          !     log( params_in(param_sd_wage) )

            wage_xb = &
               wage_xb_func(  like_object_inout % mygrid_t_invar, &
                              like_object_inout % mygrid_t_var(t), &
                              params_in, &
                              hours_new)

            wage_new = exp( wage_xb + params_in(param_sd_wage) * rnorm() )

            !store new wage (which doesn't incorporate bounds yet)
            like_object_inout % wage_new(t) = wage_new


            !now calcuate wage probability
            like_object_inout % ln_wage_prob(t) = &
               ln_wage_prob_func(   wage_new, &
                                    wage_xb, &
                                    params_in )


           !incorporate bounds
            non_pec_norm_new = bounds_func( non_pec_norm_new,  grid_min( var_non_pec_norm ), &
                                                               grid_max( var_non_pec_norm ) )
            wage_new = bounds_func( wage_new, grid_min( var_wage ), grid_max( var_wage ) )


            !now assign variable to the mygrid_wife_accept type
            like_object_inout % mygrid_wife_accept(t) % rv( var_wage )             = wage_new
            like_object_inout % mygrid_wife_accept(t) % rv( var_non_pec_norm )     = non_pec_norm_new
            like_object_inout % mygrid_wife_accept(t) % iv( var_hours )        = hours_new


         else
            like_object_inout % receive_new(t) = 0

            like_object_inout % ln_wage_prob(t) = 0
            like_object_inout % hours_prob(t) = 1

         end if !for whether a joboffer was received for the wife
      end if !for wheter a joboffer was accepted for the wife


      !If the husband got a job offer, update the wife's variables in mygrid_husband_accept that may have changed
      if ( like_object_inout % m_receive_new(t) .eq. 1 ) then
         like_object_inout % mygrid_husband_accept(t) % rv( var_non_pec_norm ) = &
            like_object_inout % mygrid_t_var(t) % rv( var_non_pec_norm )
         like_object_inout % mygrid_husband_accept(t) % rv( var_wage ) = &
            like_object_inout % mygrid_t_var(t) % rv( var_wage )
         like_object_inout % mygrid_husband_accept(t) % rv( var_ui_ben ) = &
            like_object_inout % mygrid_t_var(t) % rv( var_ui_ben )
      end if

      !If the wife got a job offer, update the husband's variables in mygrid_wife_accept that may have changed
      if ( like_object_inout % receive_new(t) .eq. 1 ) then
         like_object_inout % mygrid_wife_accept(t) % rv( var_m_non_pec_norm ) = &
            like_object_inout % mygrid_t_var(t) % rv( var_m_non_pec_norm )
         like_object_inout % mygrid_wife_accept(t) % rv( var_m_wage ) = &
            like_object_inout % mygrid_t_var(t) % rv( var_m_wage )
         like_object_inout % mygrid_wife_accept(t) % rv( var_m_ui_ben ) = &
            like_object_inout % mygrid_t_var(t) % rv( var_m_ui_ben )
      end if


      !now assign the variables if they both receive an offer
      if ((like_object_inout % m_receive_new(t) .eq. 1) .and. (like_object_inout % receive_new(t) .eq. 1)) then

         like_object_inout % mygrid_both_accept(t) % rv( var_m_non_pec_norm )  = m_non_pec_norm_new
         like_object_inout % mygrid_both_accept(t) % rv( var_non_pec_norm )    = non_pec_norm_new
         like_object_inout % mygrid_both_accept(t) % rv( var_m_wage )          = m_wage_new
         like_object_inout % mygrid_both_accept(t) % rv( var_wage )            = wage_new

         if ( like_object_inout % m_accept_new(t) .eq. 0 ) then
            like_object_inout % mygrid_both_accept(t) % iv( var_m_hours )     = m_hours_new
         end if

         if ( like_object_inout % accept_new(t) .eq. 0 ) then
            like_object_inout % mygrid_both_accept(t) % iv( var_hours )     = hours_new
         end if
      end if


   end do !over time


end subroutine get_new_joboffers_ind



subroutine get_new_asset_init( like_object_inout, like_target_in, params_in, ln_prop_old_out, ln_prop_cand_out)
   !draws new initial assets

   type(like_type), intent(in out) :: like_object_inout
   type(like_target_type), intent(in),target :: like_target_in
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, intent(out) :: ln_prop_old_out !proposal density for the old asset value
   real*8, intent(out) :: ln_prop_cand_out !proposal density for the candidate asset value

   real*8 :: asset_old, asset_cand, sd_for_old, sd_for_cand



   !get the old asset value
   if (associated(   like_object_inout % mygrid_t_var, &
                     like_target_in % mygrid_t_var1 )) then
      asset_old = like_target_in % mygrid_t_var2(1) % rv(var_assets)
   else
      asset_old = like_target_in % mygrid_t_var1(1) % rv(var_assets)
   end if


   !sd for candidate based on old asset value
   sd_for_cand = exp(   params_in( param_asset_error_cons ) + &
                  params_in( param_asset_error_linear )*abs(asset_old) )

   sd_for_cand = sd_for_cand * asset_error_sd_adjust

   !get candidate and incoroporate bounds
   asset_cand = asset_old + sd_for_cand*rnorm() !IMPORTANT: (1-13-14) This line might be unnecessary because of the following line, but I'm not sure if I shoudl comment it out because its been a while since I've looked at this code.

   asset_cand = rand_trunc_norm (   asset_old, &
                                    sd_for_cand, &
                                    grid_min( var_assets ) + 1, & !+1 so that I don't start at the lb of assets (IMPORTANT, not sure if I should do this)
                                    grid_max( var_assets ) )



   !store
   like_object_inout % mygrid_t_var(1) % rv(var_assets) = asset_cand


   !now get proposal density for candidate


      ln_prop_cand_out = &
         ln_pdf_trunc_norm(   asset_cand, &
                              asset_old, &
                              sd_for_cand, &
                              grid_min( var_assets ), &
                              grid_max( var_assets ) )



   !sd for older value
   sd_for_old = exp(   params_in( param_asset_error_cons ) + &
                  params_in( param_asset_error_linear )*abs(asset_cand) )

   sd_for_old = sd_for_old * asset_error_sd_adjust


   !now get proposal density for old
   ln_prop_old_out = &
      ln_pdf_trunc_norm(   asset_old, &
                           asset_cand, &
                           sd_for_old, &
                           grid_min( var_assets ), &
                           grid_max( var_assets ) )



end subroutine get_new_asset_init



!subroutine get_new_asset_init( like_object_inout, like_target_in, params_in, ln_prop_old_out, ln_prop_cand_out)
!   !draws new initial assets

!   type(like_type), intent(in out) :: like_object_inout
!   type(like_target_type), intent(in),target :: like_target_in
!   real*8, dimension(n_params), intent(in) :: params_in
!   real*8, intent(out) :: ln_prop_old_out !proposal density for the old asset value
!   real*8, intent(out) :: ln_prop_cand_out !proposal density for the candidate asset value

!   real*8 :: asset_old, asset_cand, sd_for_old, sd_for_cand



!   !get the old asset value
!   if (associated(   like_object_inout % mygrid_t_var, &
!                     like_target_in % mygrid_t_var1 )) then
!      asset_old = like_target_in % mygrid_t_var2(1) % rv(var_assets)
!   else
!      asset_old = like_target_in % mygrid_t_var1(1) % rv(var_assets)
!   end if


!   !sd for candidate based on old asset value
!   sd_for_cand = exp(   params_in( param_asset_error_cons ) + &
!                  params_in( param_asset_error_linear )*abs(asset_old) )

!   sd_for_cand = sd_for_cand * asset_error_sd_adjust

!   !get candidate and incoroporate bounds
!   asset_cand = asset_old + sd_for_cand*rnorm()
!   asset_cand = bounds_func( asset_cand, grid_min( var_assets ), grid_max( var_assets ) )

!   !store
!   like_object_inout % mygrid_t_var(1) % rv(var_assets) = asset_cand


!   !now get proposal density for candidate
!   if (  ( asset_cand > ( grid_min(var_assets) + zero_tol ) ) .and. &
!         ( asset_cand < ( grid_max(var_assets) - zero_tol ) )  ) then

!      ln_prop_cand_out = &
!         ln_pdf_norm(   asset_cand, &
!                        asset_old, &
!                        sd_for_cand )

!   else if ( asset_cand <= ( grid_min(var_assets) + zero_tol ) ) then
!      ln_prop_cand_out = &
!         log( cdf_norm( asset_cand, asset_old,  sd_for_cand ) )

!   else
!       ln_prop_cand_out = &
!         log( 1- cdf_norm( asset_cand, asset_old,  sd_for_cand ) )

!   end if



!   !sd for older value
!   sd_for_old = exp(   params_in( param_asset_error_cons ) + &
!                  params_in( param_asset_error_linear )*abs(asset_cand) )

!   sd_for_old = sd_for_old * asset_error_sd_adjust


!   !now get proposal density for old
!   if (  ( asset_old > ( grid_min(var_assets) + zero_tol ) ) .and. &
!         ( asset_old < ( grid_max(var_assets) - zero_tol ) )  ) then

!      ln_prop_old_out = &
!         ln_pdf_norm(   asset_old, &
!                        asset_cand, &
!                        sd_for_old )

!   else if ( asset_old <= ( grid_min(var_assets) + zero_tol ) ) then
!      ln_prop_old_out = &
!         log( cdf_norm( asset_old, asset_cand,  sd_for_old ) )

!   else
!       ln_prop_old_out = &
!         log( 1- cdf_norm( asset_old, asset_cand,  sd_for_old ) )

!   end if





!end subroutine get_new_asset_init





function mh_accept_func_noprop(log_like_old_in, log_like_cand_in,random_in) result(mh_accept_func)
   !determines whether to accept or reject the candidate parameter  using the
      !metropolis hastings acception ratio
   !this is the function for when the proposal density is symmetric

   logical :: mh_accept_func
   real*8, intent(in) :: log_like_old_in !this need to be a log likelihood
   real*8, intent(in) :: log_like_cand_in
   real*8, intent(in) :: random_in

   real*8 :: dif, ratio

   dif = log_like_cand_in - log_like_old_in

   if (dif < min_exp_arg) then
      ratio=0
      mh_accept_func = .false.
   else if (dif > max_exp_arg) then
      ratio=1
      mh_accept_func = .true.
   else
      ratio=min(exp(dif),dble(1))
      if (random_in < ratio ) then
         mh_accept_func = .true.
      else
         mh_accept_func = .false.
      end if
   end if

 !  if ( print_param_bayesian ) print *, "ratio", ratio


end function mh_accept_func_noprop



function mh_accept_func_prop(log_like_old_in, log_like_cand_in, ln_prop_old_in, ln_prop_cand_in, &
   random_in) result(mh_accept_func)
   !determines whether to accept or reject the candidate parameter  using the
      !metropolis hastings acception ratio
   !used when the proposal density is NOT symmetric

   logical :: mh_accept_func
   real*8, intent(in) :: log_like_old_in !this need to be a log likelihood
   real*8, intent(in) :: log_like_cand_in
   real*8, intent(in) :: ln_prop_old_in
   real*8, intent(in) :: ln_prop_cand_in
   real*8, intent(in) :: random_in

   real*8 :: dif, ratio

   dif = log_like_cand_in + ln_prop_old_in - log_like_old_in - ln_prop_cand_in

   if (dif < min_exp_arg) then
      ratio=0
      mh_accept_func = .false.
   else if (dif > max_exp_arg) then
      ratio=1
      mh_accept_func = .true.
   else
      ratio=min(exp(dif),dble(1))
      if (random_in < ratio ) then
         mh_accept_func = .true.
      else
         mh_accept_func = .false.
      end if
   end if



 !  if ( print_param_bayesian ) print *, "ratio", ratio


end function mh_accept_func_prop


subroutine mh_step_params( bayes_obj_inout, block_in, hh_data_in, like_object_inout, like_target_inout )
   !uses the metropolis hasting algorithm to draw a new set of population parameters in my model

   class(bayes_obj_type), intent(in out)  :: bayes_obj_inout
   integer, intent(in) :: block_in
   type(data_i_type), dimension(min_obs:max_obs), intent(in) ::  hh_data_in
   type(like_type), dimension(min_obs:max_obs), intent(in out) :: like_object_inout
   type(like_target_type), dimension(min_obs:max_obs), intent(in out) :: like_target_inout

   real*8, dimension(n_main_params) :: params_save
   real*8, dimension(n_grid_points ) :: value_matrix_save, value_matrix_temp
   real*8, dimension(n_grid_points ) :: asset_matrix_save, asset_matrix_temp
   real*8 :: log_like_val_old, log_like_val_new, random_temp
   logical :: accept_params, kernel_error
   integer :: i, t

   integer :: like_change, param_change

   kernel_error = .false.

   !first get old likelihood value
!   if( myrank .eq. 0 ) print *, "getting old like"
   log_like_val_old = log_like_func_all (like_object_inout, block_in)



   !switch pointers
!   if( myrank .eq. 0 ) print *, "switching pointers"
   call switch_pointers_params(  like_object_inout, &
                                 block_in, &
                                 like_target_inout )


   !draw new proposal params
!   if( myrank .eq. 0 ) print *, "drawing new parameters"
   params_save(1:n_main_params) = bayes_obj_inout % params_current(1:n_main_params)
   call draw_new_params(   block_in, &
                           bayes_obj_inout % params_current, &
                           bayes_obj_inout % params_proposal)


   if (error_check .eqv. .true.) then
      do i=1,n_main_params
         if ( nan_inf_check(bayes_obj_inout % params_current(i)) ) then
            print *, "error with parameter", param_labels(i), "at iter", bayes_obj_inout % iter, &
               "and block", block_in, bayes_obj_inout % params_current(i)
         end if
      end do
   end if



 !  print *, "now update parts of likelihood function"

   !update the respective part of the likelihood function
!   if( myrank .eq. 0 ) print *, "now updating parts"
   select case (block_in)
      case (block_joboffer_params1, block_joboffer_params2)

         do i=min_obs,max_obs
            call joboffer_like_update( like_object_inout(i), &
                                       hh_data_in(i), &
                                       bayes_obj_inout % params_current )
         end do

      case (block_wage_params1, block_wage_params2, block_wage_params3)

         do i=min_obs,max_obs
            call job_probs_like_update(   like_object_inout(i), &
                                          hh_data_in(i), &
                                          bayes_obj_inout % params_current )
         end do

         wage_prob_penalty_cutoff = wage_prob_penalty_cutoff_base - &
                                    log( bayes_obj_inout % params_current( param_sd_wage ) )


      case (block_leisure_params1, block_leisure_params2, block_leisure_params3, block_leisure_params4 )

         do i=min_obs,max_obs
            call utility_params_like_update(    like_object_inout(i), &
                                                hh_data_in(i), &
                                                bayes_obj_inout % params_current )

         end do

      case default
         print *, "error in selecting case for mh_step_params"
         stop
   end select

!   if (print_param_bayesian) then
!      print *, "proposal params"
!      params_for_printing(:n_main_params) = proposal_main_params_inout(:n_main_params)
!      call write_params(params_for_printing,6,.false.)
!   end if


 !  print *, "now do ols kernel weighting stuff"

   !get the interpolation coefficients using IJC's kernel weighted average idea
   if (est_dynamic .eqv. .true. ) then

      if( kernel_block(block_in) .eqv. .true. ) then

         value_matrix_save = bayes_obj_inout % value_matrix_current
         asset_matrix_save = bayes_obj_inout % asset_matrix_current


         call interp_kernel(  bayes_obj_inout, &
                              bayes_obj_inout % params_current(:n_main_params), &
                              value_matrix_temp, &
                              asset_matrix_temp, &
                              kernel_error )
         bayes_obj_inout % value_matrix_current = value_matrix_temp !do this because the bayesian object has intent(in) while the ols_matrix_temp has intent(out), so even though my program compiles with ols_matrix_current in that spot, I didn't know if I could have conflicts with the intent attributes
         bayes_obj_inout % asset_matrix_current = asset_matrix_temp

      end if

   end if

 !  print *, "ols cons for block = ", block_in, bayes_obj_inout % ols_vec_current(1)


   !get partial interpolation for a household, if needed, and update choice probabilities

!      call ols_partial_like_update( like_object_inout(i), &
!                                    bayes_obj_inout % ols_vec_current )

   if (est_savings .eqv. .true. ) then
      do i = min_obs,max_obs

         call asset_choice_like_update(   like_object_inout(i), &
                                          bayes_obj_inout % asset_matrix_current, &
                                          bayes_obj_inout % params_current )

         call asset_error_prob_like_update(  like_object_inout(i), &
                                             hh_data_in(i), &
                                             bayes_obj_inout % params_current )

      end do
   end if

 !  if( myrank .eq. 0 ) print *, "now utility and choice probs"

   do i = min_obs,max_obs

      call utility_like_update(  like_object_inout(i), &
                                 bayes_obj_inout % params_current )

      call choice_prob_like_update( like_object_inout(i), &
                                    bayes_obj_inout % value_matrix_current  )
   end do


!   !now check error in likelihood components
 !  print *, "checking error for iter", bayes_obj_inout % iter, "and block", block_in
   if (error_check .eqv. .true. ) then
      do i=min_obs,max_obs
         call like_object_inout(i) % like_error_hunt(i)
      end do
   end if


   !now get the new value for the likelihood
!   if( myrank .eq. 0 ) print *, "new old like"
   log_like_val_new = log_like_func_all( like_object_inout, block_in )




!   if (myrank .eq. 0 ) then
!      if (nan_inf_check(log_like_val_old) .or. nan_inf_check(log_like_val_new) ) then
!         print *, "error with the likelihood for"
!         print *, "iteration = ", bayes_obj_inout % iter, "block=", block_in
!         print *, "like_old", log_like_val_old, "like_new", log_like_val_new

!   !      do i=min_obs,max_obs
!   !         call like_object_inout(i) % like_error_hunt(i)
!   !      end do

!      end if
!   end if



   !determine whether the candidate parameters are accepted or rejected
   if (myrank .eq. 0 ) then
      call random_number(random_temp)
      accept_params = mh_accept_func( log_like_val_old, log_like_val_new, random_temp )
   end if

   call mpi_bcast( accept_params, 1, mpi_logical, 0, mpi_comm_world, err )


  ! print *, "accept for block = ", block_in, accept_params

   !if there was an error in getting the kernel weight, then reject these parameters
   if (est_dynamic .eqv. .true.) then
      if (kernel_error .eqv. .true.) then
         accept_params = .false.
         print *, "rejecting parameters because there was an error with getting the kernels"
         print *, "accept_params", accept_params
      end if
   end if


  ! if ( (bayes_obj_inout % iter >= 5762 ) .and. (myrank .eq. 0 )  ) then
  ! if ( (print_extra .eqv. .true. ) .and. (myrank .eq. 0 )  ) then

!      if (running_on_screen .eqv. .false.) open(newunit= print_unit,file=running_output_file,position="append")

!      write(print_unit,*), " "
!      write(print_unit,*), "iter", bayes_obj_inout % iter
!      write(print_unit,*), "block #", block_in
!      write(print_unit,*), "old likelihood", log_like_val_old
!      write(print_unit,*), "new likelihood", log_like_val_new
!      write(print_unit,*), "ratio", exp(log_like_val_new - log_like_val_old)
!      write(print_unit,*), "accept?", accept_params


!      if( block_in .eq. block_leisure_params2 ) then
!         write(print_unit,*), "util weight old", params_save(param_weight_utility), &
!            "util_weight_new",  bayes_obj_inout % params_current(param_weight_utility)
!         write(print_unit,*), "dif", bayes_obj_inout % params_current(param_weight_utility) - &
!            params_save(param_weight_utility)

!      end if

!      write(print_unit,*), "old value cons", value_matrix_save(1,0,0)
!      write(print_unit,*), "new value cons", bayes_obj_inout % value_matrix_current(1,0,0)


!      write(print_unit,*), "   "

!      if (running_on_screen .eqv. .false.) close(print_unit)

  ! end if


!   if ( ( block_in .eq. block_leisure_params2 ) .and. (myrank .eq. 0 ) ) then

!      if ( log_like_val_new - log_like_val_old > 0 ) then
!         like_change = 1
!      else
!         like_change = -1
!      end if

!      if( bayes_obj_inout % params_current(param_weight_utility) - &
!         params_save(param_weight_utility) > 0 ) then

!         param_change = 1
!      else
!         param_change = -1
!      end if

!      weight_util_direction( bayes_obj_inout % iter ) = like_change * param_change
!   end if



   !if accept parameters, then copy some of the time-invariant variables over to the inactive target
   if ( accept_params ) then
      !IMPORTANT: this one should be checked if I change the wage block lists
      if ( ( block_in .eq. block_wage_params1 ) .or. ( block_in .eq. block_wage_params2 ) ) then
         call copy_mygrid_t_invar( like_object_inout, block_in, like_target_inout )
      end if

   !if rejected candidate parameters, revert current_params, ols_vec, and the likelihood object to previous values
   else
      bayes_obj_inout % params_current(1:n_main_params) = params_save(1:n_main_params)

      wage_prob_penalty_cutoff = wage_prob_penalty_cutoff_base - &
                                 log( bayes_obj_inout % params_current( param_sd_wage ) )

      if ( (est_dynamic .eqv. .true.) .and. ( kernel_block(block_in) .eqv. .true. ) ) then
         bayes_obj_inout % value_matrix_current = value_matrix_save
         bayes_obj_inout % asset_matrix_current = asset_matrix_save
      end if

      call switch_pointers_params(  like_object_inout, &
                                    block_in, &
                                    like_target_inout )

      !put the reverted asset choice back onto they mygrid objects
      do i=min_obs,max_obs
         call asset_revert( like_object_inout(i), bayes_obj_inout % params_current)
      end do

   end if

   !now store acceptance to the root node
   if (myrank .eq. 0 ) then
      bayes_obj_inout % accept_history_params(block_in, bayes_obj_inout % iter ) = accept_params
   end if



end subroutine mh_step_params


subroutine mh_step_asset_error( bayes_obj_inout, block_in, hh_data_in, like_object_inout, like_target_inout  )
   !draws new parameters for the asset errors
   !this has its own subroutine because to draw these parameters, I only compared the current "real" assets to the assets in the data,
      !so I don't need the value function to calculate the likelihood function for this step

   class(bayes_obj_type), intent(in out)  :: bayes_obj_inout
   integer, intent(in) :: block_in
   type(data_i_type), dimension(min_obs:max_obs), intent(in) ::  hh_data_in
   type(like_type), dimension(min_obs:max_obs), intent(in out) :: like_object_inout
   type(like_target_type), dimension(min_obs:max_obs), intent(in out), target :: like_target_inout


   real*8, dimension(n_main_params) :: params_save
   real*8 :: log_like_val_old, log_like_val_new, random_temp, like_temp
   logical :: accept_params
   integer :: i


   !get old likelihood and switch pointers

   like_temp = 0

   do i=min_obs,max_obs
      like_temp = like_temp + like_object_inout(i) % ln_asset_prob

      if (associated(   like_object_inout(i) % ln_asset_prob, &
                        like_target_inout(i) % ln_asset_prob1 )) then
         like_object_inout(i) % ln_asset_prob  => like_target_inout(i) % ln_asset_prob2
      else
         like_object_inout(i) % ln_asset_prob  => like_target_inout(i) % ln_asset_prob1
      end if

   end do

   call mpi_reduce( like_temp, log_like_val_old, 1, mpi_real8, mpi_sum, 0, &
      mpi_comm_world, err )

   params_save(1:n_main_params) = bayes_obj_inout % params_current(1:n_main_params)
   call draw_new_params(   block_in, &
                           bayes_obj_inout % params_current, &
                           bayes_obj_inout % params_proposal)

   if (error_check .eqv. .true.) then
      do i=1,n_main_params
         if ( nan_inf_check(bayes_obj_inout % params_current(i)) ) then
            print *, "error with parameter", param_labels(i), "at iter", bayes_obj_inout % iter, &
               "and block", block_in, bayes_obj_inout % params_current(i)
         end if
      end do
   end if

   !update the asset error probability
   do i=min_obs,max_obs
      call asset_error_prob_like_update(  like_object_inout(i), &
                                          hh_data_in(i), &
                                          bayes_obj_inout % params_current )
   end do

!   !now check error in likelihood components
 !  print *, "checking error for iter", bayes_obj_inout % iter, "and block", block_in
   if (error_check .eqv. .true. ) then
      do i=min_obs,max_obs
         call like_object_inout(i) % like_error_hunt(i)
      end do
   end if


   !get new likelihood
   like_temp = 0
   do i=min_obs,max_obs
      like_temp = like_temp + like_object_inout(i) % ln_asset_prob
   end do

   call mpi_reduce( like_temp, log_like_val_new, 1, mpi_real8, mpi_sum, 0, &
      mpi_comm_world, err )

   !accept params

   if ( myrank .eq. 0 ) then
      call random_number(random_temp)
      accept_params = mh_accept_func( log_like_val_old, log_like_val_new, random_temp )
   end if

   call mpi_bcast( accept_params, 1, mpi_logical, 0, mpi_comm_world, err )


   if ( (print_extra .eqv. .true. ) .and. (myrank .eq. 0 )  ) then

      if (running_on_screen .eqv. .false.) open(newunit= print_unit,file=running_output_file,position="append")

      write(print_unit,*), " "
      write(print_unit,*), "iter", bayes_obj_inout % iter
      write(print_unit,*), "block #", block_in
      write(print_unit,*), "old likelihood", log_like_val_old
      write(print_unit,*), "new likelihood", log_like_val_new
      write(print_unit,*), "ratio", exp(log_like_val_new - log_like_val_old)
      write(print_unit,*), "accept?", accept_params


      write(print_unit,*), "   "

      if (running_on_screen .eqv. .false.) close(print_unit)

   end if

   !if rejected values, then revert to previous values

   if (.not. accept_params ) then
         bayes_obj_inout % params_current(1:n_main_params) = params_save(1:n_main_params)

      do i=min_obs,max_obs
         if (associated(   like_object_inout(i) % ln_asset_prob, &
                           like_target_inout(i) % ln_asset_prob1 )) then
            like_object_inout(i) % ln_asset_prob  => like_target_inout(i) % ln_asset_prob2
         else
            like_object_inout(i) % ln_asset_prob  => like_target_inout(i) % ln_asset_prob1
         end if

      end do

   end if


   !now store acceptance to the root
   if (myrank .eq. 0 ) then
      bayes_obj_inout % accept_history_params(block_in, bayes_obj_inout % iter ) = accept_params
   end if

end subroutine mh_step_asset_error




!!IMPORTANT: I haven't updated this routine yet
!subroutine mh_step_unemp_length(bayes_obj_inout, hh_data_in, like_object_inout, &
!   like_target_inout )
!   !uses the metropolis hasting algorithm to draw unemployment lenght variables in my model

!   class(bayes_obj_type), intent(in out), target  :: bayes_obj_inout
!   type(data_i_type), dimension(N_obs), intent(in) ::  hh_data_in
!   type(like_type), dimension(N_obs), intent(in out) :: like_object_inout
!   type(like_target_type), dimension(N_obs), intent(in out) :: like_target_inout

!   real*8 :: mh_random
!   real*8, dimension(N_t) :: m_ui_ben_old, ui_ben_old
!   integer, dimension(N_t) :: m_unemp_length_old, unemp_length_old
!   real*8, dimension(2:N_t) :: m_joboffer_prob_old, joboffer_prob_old, choice_prob_old

!   !stopped here
!   logical :: accept_temp
!   real*8 :: log_like_val_old, log_like_val_new
!   integer :: max_time_unemp_length, max_time
!   integer :: i,t


!   do i=min_obs,max_obs

!      if( like_object_inout(i) % unknown_unemp_length .eq. 1) then !only need to do this stuff for a new households

!         max_time = like_object_inout(i) % max_time

!         !first get old likelihood components
!         log_like_val_old = log_like_func ( like_object_inout(i), block_unemp_length )

!         !store old lenght and unemployment benefits
!         do t=1, like_object_inout(i) % max_time
!            m_ui_ben_old(t) = like_object_inout(i) % mygrid_t_var(t) % rv(var_m_ui_ben)
!            m_unemp_length_old(t) = like_object_inout(i) % mygrid_t_var(t) % rv(var_m_unemp_length)

!            ui_ben_old(t) = like_object_inout(i) % mygrid_t_var(t) % rv(var_ui_ben)
!            unemp_length_old(t) = like_object_inout(i) % mygrid_t_var(t) % rv(var_unemp_length)
!         end do

!         !store old joboffer and choice probs
!         m_joboffer_prob_old(2:max_time) = like_object_inout(i) % m_joboffer_prob(2:max_time)
!         joboffer_prob_old(2:max_time)   = like_object_inout(i) % joboffer_prob(2:max_time)
!         choice_prob_old(2:max_time)     = like_object_inout(i) % choice_prob(2:max_time)

!         !switch pointers
!         call switch_pointers_ind( like_object_inout(i), block_unemp_length, like_target_inout(i) )

!         !copy over the old choice probs, do this becuase I might not update all the choice probabilities
!         like_object_inout(i) % m_joboffer_prob(2:max_time) = m_joboffer_prob_old(2:max_time)
!         like_object_inout(i) % joboffer_prob(2:max_time)   = joboffer_prob_old(2:max_time)
!         like_object_inout(i) % choice_prob(2:max_time)     = choice_prob_old(2:max_time)


!         !draw new length
!         call get_new_unemp_length_ind(   like_object_inout(i), &
!                                          hh_data_in(i), &
!                                          bayes_obj_inout % params_current, &
!                                          max_time_unemp_length )

!         !update utility function and choice probabilities
!         do t=2, max_time_unemp_length
!            call utility_like_update_one_t(  like_object_inout(i), &
!                                             bayes_obj_inout % params_current , &
!                                             t)
!            call choice_prob_like_update_one_t( like_object_inout(i), &
!                                                bayes_obj_inout % ols_matrix_current, &
!                                                t)
!         end do



!         !now get new likelihood components
!         log_like_val_new = log_like_func ( like_object_inout(i), block_unemp_length )


!         if (error_check .eqv. .true. ) then
!            call like_object_inout(i) % like_error_hunt(i)
!         end if

!         if (nan_inf_check(log_like_val_old) .or. nan_inf_check(log_like_val_new) ) then
!            print *, "error with the likelihood for"
!            print *, "iteration = ", bayes_obj_inout % iter, "for hh", i
!            print *, "like_old", log_like_val_old, "like_new", log_like_val_new
!         end if


!         call random_number(mh_random)
!         accept_temp = mh_accept_func( log_like_val_old, log_like_val_new, mh_random )

!!         print *, i, log_like_val_old, log_like_val_new

!!         print *, "old likelihood", log_like_val_old
!!         print *, "new likelihood", log_like_val_new
!!         print *, "ratio", exp(log_like_val_new-log_like_val_old)
!!         print *, "accept?", accept_temp(i)


!         !if rejected candidate values, then switch pointer back to original value
!         if (.not. accept_temp ) then
!            call switch_pointers_ind( like_object_inout(i), block_unemp_length, like_target_inout(i) )

!            !switch the unemp benefits and unemp lenght back to their original values
!            do t=1, like_object_inout(i) % max_time
!               like_object_inout(i) % mygrid_t_var(t) % rv(var_m_ui_ben) = m_ui_ben_old(t)
!               like_object_inout(i) % mygrid_t_var(t) % rv(var_m_unemp_length) =  m_unemp_length_old(t)

!               like_object_inout(i) % mygrid_t_var(t) % rv(var_ui_ben) = ui_ben_old(t)
!               like_object_inout(i) % mygrid_t_var(t) % rv(var_unemp_length) = unemp_length_old(t)

!               like_object_inout(i) % mygrid_wife_accept(t) % rv(var_m_ui_ben) = &
!                  like_object_inout(i) % mygrid_t_var(t) % rv(var_m_ui_ben)
!               like_object_inout(i) % mygrid_wife_accept(t) % rv(var_m_unemp_length) = &
!                  like_object_inout(i) % mygrid_t_var(t) % rv(var_m_unemp_length)

!               like_object_inout(i) % mygrid_husband_accept(t) % rv(var_ui_ben) = &
!                  like_object_inout(i) % mygrid_t_var(t) % rv(var_ui_ben)
!               like_object_inout(i) % mygrid_husband_accept(t) % rv(var_unemp_length) = &
!                  like_object_inout(i) % mygrid_t_var(t) % rv(var_unemp_length)

!            end do
!         end if

!      end if !for whether I need to deal with unkown unemp length

!   end do !over i


!end subroutine mh_step_unemp_length



subroutine mh_step_ind(bayes_obj_inout, block_in, hh_data_in, like_object_inout, &
   like_target_inout, joboffer_random_in, mh_random_in)
   !uses the metropolis hasting algorithm to draw a new set of individual variables in my model

   class(bayes_obj_type), intent(in out), target  :: bayes_obj_inout
   integer, intent(in) :: block_in
   type(data_i_type), dimension(min_obs:max_obs), intent(in) ::  hh_data_in
   type(like_type), dimension(min_obs:max_obs), intent(in out) :: like_object_inout
   type(like_target_type), dimension(min_obs:max_obs), intent(in out) :: like_target_inout
   real*8, dimension(iter_m_job_offer: iter_hours,N_t,min_obs:max_obs), intent(in) :: joboffer_random_in
   real*8, dimension(min_obs:max_obs), intent(in) ::  mh_random_in


   logical, dimension(min_obs:max_obs) :: accept_temp
   integer :: accept_count_temp, accept_sum_temp
   real*8 :: log_like_val_old, log_like_val_new, asset_prop_old, asset_prop_new
   integer :: i,t

   do i=min_obs,max_obs

 !     i_global = i

!      print *, " "
!      print *, "****************************"
!      print *, "    "
!      print *, "doing individual draws for hh#, i"


      !first get old likelihood values
      log_like_val_old = log_like_func ( like_object_inout(i), block_in )

      !switch pointers
      call switch_pointers_ind( like_object_inout(i), block_in, like_target_inout(i) )


      !draw new unobserved heterogeneity terms
      call get_new_heteo_ind( like_object_inout(i), &
                              bayes_obj_inout % params_current, &
                              bayes_obj_inout % hetero_chol_current)

      !draw new assets
      if( est_savings .eqv. .true. ) then
         call get_new_asset_init(   like_object_inout(i), &
                                    like_target_inout(i), &
                                    bayes_obj_inout % params_current, &
                                    asset_prop_old, &
                                    asset_prop_new )
      end if

      !draw a new joboffer variables and update that part of the likelihood object
      call get_new_joboffers_ind(   like_object_inout(i), &
                                    hh_data_in(i), &
                                    bayes_obj_inout % params_current, &
                                    joboffer_random_in(:,:,i) )


      !now get utility parameters, if needed
      do t=2, like_object_inout(i) % max_time
         if (     (like_object_inout(i) % utility_params_created(t) .eq. 0) .and. &
               (  (like_object_inout(i) % m_receive_new(t) .eq. 1) .or. &
                  (like_object_inout(i) % receive_new(t) .eq. 1)    )) then

            like_object_inout(i) % utility_params_created(t) = 1
            call utility_function_setup(  &
               like_object_inout(i) % mygrid_t_var(t) % rv(var_kids), &
               like_object_inout(i) % mygrid_t_var(t) % rv(var_youngest), &
               bayes_obj_inout % params_current, &
               like_object_inout(i) % utility_params(:,t) )

         end if
      end do


      if( est_savings .eqv. .true. ) then
         call asset_choice_like_update(   like_object_inout(i), &
                                          bayes_obj_inout % asset_matrix_current, &
                                          bayes_obj_inout % params_current )

         call asset_error_prob_like_update(  like_object_inout(i), &
                                             hh_data_in(i), &
                                             bayes_obj_inout % params_current )
      end if

      !get the utility values
      call utility_like_update(  like_object_inout(i), bayes_obj_inout % params_current )

      !now update the partial interpolation function, if needed
!      if (  (  like_object_inout(i) % ols_partial_created .eq. 0   ) .and. &
!            (( maxval(like_object_inout(i) % m_receive_new) .eq. 1 ) .or. &
!            (  maxval(like_object_inout(i) % receive_new) .eq. 1   )  ))   then

!         call get_ols_partial(   like_object_inout(i) % mygrid_t_invar, &
!                                 bayes_obj_inout % ols_vec_current, &
!                                 like_object_inout(i) % ols_vec )
!         like_object_inout(i) % ols_partial_created = 1
!      end if

      !now update the choice probabilities
      call choice_prob_like_update( like_object_inout(i), &
                                    bayes_obj_inout % value_matrix_current )

      !now get new likelihood values
      log_like_val_new = log_like_func( like_object_inout(i), block_in )



      if (error_check .eqv. .true. ) then
         call like_object_inout(i) % like_error_hunt(i)
      end if

!      if (nan_inf_check(log_like_val_old) .or. nan_inf_check(log_like_val_new) ) then
!         print *, "error with the likelihood for"
!         print *, "iteration = ", bayes_obj_inout % iter, "for hh", i
!         print *, "like_old", log_like_val_old, "like_new", log_like_val_new
!      end if


      if ( est_savings .eqv. .true. ) then
         accept_temp(i) = mh_accept_func( log_like_val_old, log_like_val_new, asset_prop_old, &
            asset_prop_new, mh_random_in(i) )
      else
         accept_temp(i) = mh_accept_func( log_like_val_old, log_like_val_new, mh_random_in(i) )
      end if

!     print *, i, log_like_val_old, log_like_val_new

      if(print_extra_ind .eqv. .true.) then

      print *, "old likelihood", log_like_val_old
      print *, "new likelihood", log_like_val_new
      print *, "ratio", exp(log_like_val_new-log_like_val_old)
      print *, "accept?", accept_temp(i)

      end if

      !if rejected candidate values, then switch pointer back to original value
      if (.not. accept_temp(i) ) then
         call switch_pointers_ind( like_object_inout(i), block_in, like_target_inout(i) )

         !revert wage means to old values
         like_object_inout(i) % mygrid_t_invar % rv(var_m_wage_mean) = &
            like_object_inout(i) % m_wage_mean_nohetero + &
            like_object_inout(i) % hetero(hetero_m_wage)

         like_object_inout(i) % mygrid_t_invar % rv(var_wage_mean) = &
            like_object_inout(i) % wage_mean_nohetero + &
            like_object_inout(i) % hetero(hetero_wage)

         !incorporate bounds
         like_object_inout(i) % mygrid_t_invar % rv(var_m_wage_mean)  = bounds_func( &
            like_object_inout(i) % mygrid_t_invar % rv(var_m_wage_mean), &
            grid_min( var_m_wage_mean ), grid_max( var_m_wage_mean) )

         like_object_inout(i) % mygrid_t_invar % rv(var_wage_mean)  = bounds_func( &
            like_object_inout(i) % mygrid_t_invar % rv(var_wage_mean), &
            grid_min( var_wage_mean ), grid_max( var_wage_mean) )

         !don't need to revert asset choice here because I'm switching hte pointers for the time-varying mygrid objects

      end if

   end do !over i


   !final, export the sum of the variables that were accepted
   accept_count_temp = count(accept_temp)

   call mpi_reduce(accept_count_temp, accept_sum_temp, 1, mpi_integer, mpi_sum, 0, &
      mpi_comm_world, err)

   if( myrank .eq. 0 )  then
      bayes_obj_inout % accept_history_ind ( bayes_obj_inout % iter ) = accept_sum_temp
   end if

 !  print *,  bayes_obj_inout % accept_history_ind ( bayes_obj_inout % iter )

!read(*,*)



end subroutine mh_step_ind



subroutine wishart_step_hetero( bayes_obj_inout, like_object_inout)
   !draws new unobserved heterogeneity using the inverse_wishart distribution

   class(bayes_obj_type), intent(in out)  :: bayes_obj_inout
   type(like_type), dimension(min_obs:max_obs), intent(in out) :: like_object_inout

   real*8, dimension(n_hetero, n_hetero) :: sum_squares, sum_squares_temp, wishart_inv_draw
   integer :: i,k1,k2

   !get the sum of squares within each node
   sum_squares_temp = 0
   do i=min_obs,max_obs
      do k1 = 1, n_hetero
         do k2= 1, k1
            sum_squares_temp(k1,k2) = sum_squares_temp(k1,k2) + like_object_inout(i) % hetero(k1) * &
                                                      like_object_inout(i) % hetero(k2)
         end do
      end do
   end do

   !now sum across nodes
   call mpi_reduce(sum_squares_temp, sum_squares, size(sum_squares_temp), mpi_real8, mpi_sum, 0, &
      mpi_comm_world, err)



   if (myrank .eq. 0 ) then
      !draws a new covariance matrix
      wishart_inv_draw = random_wishart_inv(n_hetero,N_obs_total,sum_squares)

      !and now store the cholesky decomposition of this matrix
      call chol_decomp_sub(n_hetero, wishart_inv_draw, bayes_obj_inout %  hetero_chol_current )
   end if

   !now broadcast this choleksy decomp
   call mpi_bcast( bayes_obj_inout %  hetero_chol_current, &
      size(bayes_obj_inout %  hetero_chol_current), mpi_real8, 0, mpi_comm_world, err )

   !and store it on the current parameter vector
   bayes_obj_inout % params_current( param_hetero_m_wage_var ) = &
      bayes_obj_inout % hetero_chol_current(1,1)

   bayes_obj_inout % params_current( param_hetero_wage_covar ) = &
      bayes_obj_inout % hetero_chol_current(2,1)

   bayes_obj_inout % params_current( param_hetero_wage_var ) = &
      bayes_obj_inout % hetero_chol_current(2,2)



end subroutine wishart_step_hetero





subroutine choice_prob_bayesian_update(bayes_obj_inout, hh_data_in, like_object_inout, &
   value_matrix_iterate_post_in, asset_matrix_iterate_post_in)
   !updates the asset choice, utilities, and choice probabilities to account for the value function and asset policy function being updated
   class(bayes_obj_type), intent(in out) :: bayes_obj_inout
   type(data_i_type), dimension(min_obs:max_obs), intent(in) :: hh_data_in
   type(like_type), dimension(min_obs:max_obs), intent(in out) :: like_object_inout
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_iterate_post_in
   real*8, dimension(n_grid_points), intent(in) :: asset_matrix_iterate_post_in

   real*8, dimension(n_grid_points) :: value_matrix_temp
   real*8, dimension(n_grid_points) :: asset_matrix_temp
   logical :: kernel_error
   integer :: i,t



   call interp_kernel(  bayes_obj_inout, &
                        bayes_obj_inout % params_current(:n_main_params), &
                        value_matrix_temp, &
                        asset_matrix_temp, &
                        kernel_error )

   bayes_obj_inout % value_matrix_current = value_matrix_temp !do this because the bayesian object has intent(in) while the ols_matrix_temp has intent(out), so even though my program compiles with ols_matrix_current in that spot, I didn't know if I could have conflicts with the intent attributes
   bayes_obj_inout % asset_matrix_current = asset_matrix_temp

   if (kernel_error) then
      print *, "error with updating interpolation coefficients at end of mcmc_iter_once"
      print *, "setting interpolation coefficents equal to the proposal coefficents after iteration this loop"
      bayes_obj_inout % value_matrix_current = value_matrix_iterate_post_in
      bayes_obj_inout % asset_matrix_current = asset_matrix_iterate_post_in

      !this error is somewhat difficult to think what to do about

   end if



!  !get partial interpolation for a household, if needed, and update choice probabilities
   do i=min_obs,max_obs
  !    call ols_partial_like_update( like_object_inout(i), &
   !                                    bayes_obj_inout % ols_matrix_current )

      if (est_savings .eqv. .true. ) then
         call asset_choice_like_update(   like_object_inout(i), &
                                          bayes_obj_inout % asset_matrix_current, &
                                          bayes_obj_inout % params_current )

         call asset_error_prob_like_update(  like_object_inout(i), &
                                             hh_data_in(i), &
                                             bayes_obj_inout % params_current )

      end if

      call utility_like_update(  like_object_inout(i), &
                                 bayes_obj_inout % params_current )

      call choice_prob_like_update( like_object_inout(i), bayes_obj_inout % value_matrix_current )
   end do

!  !change the non-pecuniary benefits to correct for low choice probabilities post update
      !of the value function (this routine shouldn't have to be called after the burn-in stage is past
!     !because the updated value function should be basically the same as the old value function
!   do i=min_obs,max_obs
!      do t=2,like_object_inout(i) % max_time
!         if ( like_object_inout(i) % choice_prob(t) < choice_prob_min ) then

!            print *, "low choice prob for hh", i, "at time", t
!            call low_choice_prob_adjust(  like_object_inout(i), &
!                                          bayes_obj_inout % params_current, &
!                                          bayes_obj_inout % ols_matrix_current, &
!                                          t)
!         end if
!      end do
!   end do

!      do i=min_obs,max_obs
!         call like_object_inout(i) % like_error_hunt(i)
!      end do

end subroutine choice_prob_bayesian_update



!subroutine util_weight_bayes_der( bayes_obj_in, like_object_inout, hh_data_in, der_out )
!   !gets the derivative of the likelihood function wrt the utility weight
!   type(bayes_obj_type), intent(in) ::  bayes_obj_in
!   type(like_type), dimension(min_obs:max_obs), intent(in out) :: like_object_inout
!   type(data_i_type), dimension(min_obs:max_obs), intent(in out) :: hh_data_in
!   real*8, intent(out) :: der_out

!   real*8, dimension(n_grid_points ) :: value_matrix_temp
!   real*8, dimension(n_grid_points ) :: asset_matrix_temp
!   real*8, dimension(n_params) :: params_temp
!   real*8 :: perturb_val
!   logical :: kernel_error


!   if( myrank .eq. 0 ) perturb_val = rw_chol_leisure_block2(2,2)
!   call mpi_bcast( perturb_val, 1, mpi_real8, 0, mpi_comm_world, err )



!   params_temp =  bayes_obj_in % params_current
!   params_temp( param_weight_utility) = perturb_val + params_temp( param_weight_utility )


!   if (est_dynamic .eqv. .true. ) then

!      call interp_kernel(  bayes_obj_in, &
!                           params_temp, &
!                           value_matrix_temp, &
!                           asset_matrix_temp, &
!                           kernel_error )

!      if( kernel_error .eqv. .true. ) print *, "error with kernel for util_weight_bayes_der"

!   end if

!   call util_param_like_der(  like_object_inout, &
!                              hh_data_in, &
!                              params_temp, &
!                              value_matrix_temp, &
!                              perturb_val, &
!                              der_out )



!end subroutine util_weight_bayes_der





subroutine mcmc_iter_once( bayes_obj_inout, value_grid_inout, hh_data_in, &
   like_object_inout, like_target_inout )
   !does one Bayesian McMC iteration, iterating the value function as well at the end

   class(bayes_obj_type), intent(in out), target  :: bayes_obj_inout
   type(value_grid_type), intent(in out) :: value_grid_inout
   type(data_i_type), dimension(min_obs:max_obs) ::  hh_data_in
   type(like_type), dimension(min_obs:max_obs) :: like_object_inout
   type(like_target_type), dimension(min_obs:max_obs) :: like_target_inout


   real*8, dimension(iter_m_job_offer: iter_hours,N_t,min_obs:max_obs) :: joboffer_random
   real*8, dimension(min_obs:max_obs) ::  mh_random
   real*8, dimension(n_grid_points) :: value_matrix_iterate_pre, value_matrix_iterate_post
   real*8, dimension(n_grid_points) ::  asset_matrix_iterate_post, asset_matrix_scratch
   real*8, dimension(n_params) :: params_iterate
   integer :: block,i,t
   integer, pointer :: iter !to make code a bit easier to read
   logical :: kernel_error, accept_all

   iter        => bayes_obj_inout % iter



   !first draw parameters blocks
  ! if( myrank .eq. 0 ) print *, "iter #", iter


   do block = 1, block_leisure_params4
    !  if( myrank .eq. 0) print *, "block #", block
      if (free_blocks(block) .eqv. .true. ) then

         call mh_step_params( bayes_obj_inout, block, hh_data_in, like_object_inout, &
            like_target_inout)

      end if

   end do



   !asset error parameters
 !  if( myrank .eq. 0) print *, "asset error"
   do block = block_asset_error_params1, block_asset_error_params2

      if (free_blocks(block) .eqv. .true. ) then
         call mh_step_asset_error( bayes_obj_inout, block, hh_data_in, like_object_inout, like_target_inout  )
      end if

   end do

!  if(myrank .eq. 0) print *, "individual"

   !now individual joboffers
   if( ind_draws_free .eqv. .true. ) then
      call random_number(joboffer_random)
      call random_number(mh_random)
      call mh_step_ind( bayes_obj_inout, block_joboffers_ind, hh_data_in, like_object_inout, &
            like_target_inout, joboffer_random, mh_random)
   end if


   do i=min_obs,max_obs
      do t=2, like_object_inout(i) % max_time
         if ( (   like_object_inout(i) % m_ln_wage_prob(t) < -huge(0.0D+00) ) .or. &
                  ( like_object_inout(i) % m_ln_wage_prob(t) < -huge(0.0D+00) ) )  then

            print *, "problem after mh_step_ind at iter = ",  bayes_obj_inout % iter
            print *, "hh #", i, "at time ", t

            call like_object_inout(i) % print(t)

            stop

         end if
      end do
   end do


 ! if(myrank .eq. 0)  print *, "wishart"
   !now draw new unoboserved heterogeneity covariance matrix
   if( hetero_params_free .eqv. .true. ) then
      call wishart_step_hetero( bayes_obj_inout, like_object_inout)
   end if


!   if (iter .eq. 71 ) then
!    call like_target_inout(551) % print(like_object_inout(551), 4)
!      read(*,*)
!   end if

!   if (iter .eq. 74 ) then
!   call like_target_inout(551) % print(like_object_inout(551), 4)
!   read(*,*)
!   end if
!   print *, "iter", iter, "m receive", like_object_inout(551) % m_receive_new(4), "wage and non pec", &
!      like_object_inout(551) % mygrid_husband_accept(4) % rv(var_m_wage), &
!      like_object_inout(551) % mygrid_husband_accept(4) % rv(var_m_non_pec_norm), &
!      "choice prob", like_object_inout(551) % choice_prob(4)


   !store the accepted parameters
   call bayes_obj_inout % save_param_history()


!now iterate the value function and store these new value functions
   if (  ( mod(iter,val_update_multiple) .eq. 0 )  .and. &
         ( est_dynamic .eqv. .true.)               .and. &
         ( save_params_switch .eqv. .true. )             ) then

      call bayes_obj_inout % val_update_bayes( value_grid_inout )

!   !update choice probabilities to reflect updated value function  (if needed)
      if ( (iter < choice_update_min) .or. (mod(iter,choice_update_multiple) .eq. 0) ) then

         !2/12/14: don't think I need to update the value function stored in the bayesian object, but not completely sure
         call choice_prob_bayesian_update(   bayes_obj_inout, &
                                             hh_data_in, &
                                             like_object_inout, &
                                             value_matrix_iterate_post, &
                                             asset_matrix_iterate_post )

      end if

   end if

!2/12/14: This part of the code is old
!   if ( (est_dynamic .eqv. .true.) .and. ( save_params_switch .eqv. .true. ) ) then

!    !  print *, "getting new kernel"

!      if(myrank .eq. 0 )  then
!         accept_all = all(bayes_obj_inout % accept_history_params( :block_leisure_params3 ,iter) )
!      end if

!      call mpi_bcast( accept_all, 1, mpi_logical, 0, mpi_comm_world, err )


!      if ( accept_all .eqv. .true. ) then
!         value_matrix_iterate_pre = bayes_obj_inout % value_matrix_current
!      else
!         call interp_kernel(  bayes_obj_inout, &
!                              bayes_obj_inout % params_proposal(:n_main_params), &
!                              value_matrix_iterate_pre, &
!                              asset_matrix_scratch, &
!                              kernel_error )

!         if ( (kernel_error .eqv. .true.) .and. (myrank .eq. 0 ) )then
!            print *, "error with kernels before iterating the value function "
!            print *, "using a different ols_vector before interpolation"
!            value_matrix_iterate_pre = bayes_obj_inout % value_matrix_current

!            !need to also change the parameter vector these set of interpolation coefficients are based on
!            bayes_obj_inout % params_proposal(:n_main_params) = &
!               bayes_obj_inout % params_current(:n_main_params)

!         end if

!      end if

!   !   print *, "ols pre_iterate constant", ols_vec_iterate_pre(1)


!      !get the parameters for iteration
!      params_iterate(:n_main_params) = bayes_obj_inout % params_proposal(:n_main_params)
!      params_iterate(n_main_params+1:) = bayes_obj_inout % params_current(n_main_params+1:)




!      !now experimenting to see if iterating more will help
!      call iterate_bayes( &
!         max_switch_in     = .false., &
!         max_iter_in       = n_iter_bayes, &
!         tol_in            = 1.0D-2, &
!         params_in         = params_iterate, &
!         value_matrix_in   = value_matrix_iterate_pre, &
!         value_grid_inout  = value_grid_inout, &
!         value_matrix_out  = value_matrix_iterate_post, &
!         asset_matrix_out  = asset_matrix_iterate_post )




!!      !update the asset policy function (older code for only interating value function once
!!      if( est_savings .eqv. .true. ) then
!!         call update_asset_utility_params(   value_grid_inout, params_iterate )
!!         call update_asset_policy_function(  value_grid_inout, &
!!                                             value_matrix_iterate_pre, &
!!                                             params_iterate, &
!!                                             asset_matrix_iterate_post )
!!      else
!!         asset_matrix_iterate_post = bayes_obj_inout % asset_matrix_current
!!      end if

!!      !now iterate the value function
!!      call update_probs_all_node (  value_grid_inout, params_iterate )

!!      if (est_savings .eqv. .true. ) then
!!         call update_asset_util_all_node ( value_grid_inout, params_iterate, asset_matrix_iterate_post )
!!      else
!!         call update_asset_util_no_savings ( value_grid_inout, params_iterate )
!!      end if

!!      call iterate_value_all_node(  value_grid_inout, &
!!                                    value_matrix_iterate_pre, &
!!                                    params_iterate, &
!!                                    value_matrix_iterate_post )








!!  ! print *, "ols post_iterate constant", ols_vec_iterate_post(1)




!!


!!   !update choice probabilities to reflect updated value function (if needed)
!      if ( (iter < choice_update_min) .or. (mod(iter,choice_update_multiple) .eq. 0) ) then

!         call choice_prob_bayesian_update(   bayes_obj_inout, &
!                                             hh_data_in, &
!                                             like_object_inout, &
!                                             value_matrix_iterate_post, &
!                                             asset_matrix_iterate_post )

!      end if




!   end if !for estimate dynamic

!2/12/14, this part of the code is old
!   !now store parameters and interpolation matrices
!   call save_bayesian_params_interp(   bayes_obj_inout, &
!                                          value_matrix_iterate_post, &
!                                          asset_matrix_iterate_post )



end subroutine mcmc_iter_once



subroutine mcmc_iter_main( bayes_obj_inout, value_grid_inout, hh_data_in, &
   like_object_inout, like_target_inout)

   class(bayes_obj_type), intent(in out),target :: bayes_obj_inout
   type(value_grid_type), intent(in out) :: value_grid_inout
   type(data_i_type), dimension(min_obs:max_obs) ::  hh_data_in
   type(like_type), dimension(min_obs:max_obs) :: like_object_inout
   type(like_target_type), dimension(min_obs:max_obs) :: like_target_inout

   real*8 :: time_start, time_current
   integer :: lb, up, b, i, t,length,k
   integer, pointer :: iter
   real*8 :: accept_average
   real*8, dimension(:,:), pointer :: rw_chol_pointer

   real*8, dimension(iter_m_job_offer: iter_hours,N_t,min_obs:max_obs) :: joboffer_random
   real*8, dimension(min_obs:max_obs) ::  mh_random

   real*8 :: util_weight_der

   if( myrank .eq. 0 ) then
      if (running_on_screen .eqv. .true.) then
         print_unit = output_unit
      else
         open(newunit = print_unit, file=running_output_file, status = "replace" )
         write(print_unit,*) " "
         close(print_unit)
      end if

!      open(newunit = weight_util_unit, file=weight_util_file, status = "replace" )
!      call csv_write(weight_util_unit, "iter", .false. )
!      call csv_write(weight_util_unit, "weight_util_val", .false. )
!      call csv_write(weight_util_unit, "direction", .true. )
!      close(weight_util_unit)
   end if



   !pointing to make my life eaiser
   iter => bayes_obj_inout % iter

   !first draw some initial individual-specific variables to make sure that I'm starting in a good place with these variables

   if( error_check .eqv. .true. ) then
      do i=min_obs,max_obs
         call like_object_inout(i) % like_error_hunt(i)
      end do
   end if

   if( myrank .eq. 0) print *, "drawing series of individual-specific variables"

   iter = 1


   if(ind_draws_free .eqv. .true. ) then
      do k=1, n_ind_draws_initial

       !  print *, "initial ind iter #", k

         call random_number(joboffer_random)
         call random_number(mh_random)
         call mh_step_ind(bayes_obj_inout, block_joboffers_ind, hh_data_in, like_object_inout, &
            like_target_inout, joboffer_random, mh_random)

         if( error_check .eqv. .true. ) then
            do i=min_obs,max_obs
               call like_object_inout(i) % like_error_hunt(i)
            end do
         end if

      end do

   end if






!   !adjust the non-pecuniaries that are still too low
!   do i=min_obs,max_obs
!      do t=2,like_object_inout(i) % max_time
!         if ( like_object_inout(i) % choice_prob(t) < choice_prob_min ) then

!            print *, "low choice prob for hh", i, "at time", t
!            call low_choice_prob_adjust(  like_object_inout(i), &
!                                          bayes_obj_inout % params_current, &
!                                          bayes_obj_inout % ols_matrix_current, &
!                                          t)
!         end if
!      end do
!   end do





   if( myrank .eq. 0)  print *, "done with setting up initial variables"

   iter = 0


   call cpu_time(time_start)
   call cpu_time(time_current)

   call mpi_bcast( time_start, 1, mpi_real8, 0, mpi_comm_world, err )
   call mpi_bcast( time_current, 1, mpi_real8, 0, mpi_comm_world, err )





   do while ( (iter < n_mcmc_iter) .and. ((time_current - time_start) .le. max_time) )

    !  print *, "iter", iter, "n_mcmc_iter", n_mcmc_iter, "iter < n_mcmc_iter?", iter < n_mcmc_iter

      iter = iter +1


!      if ( (iter > 3000 ) .and. ( myrank .eq. 0 ) ) then

!         call csv_write(output_unit, iter, .false. )
!         call csv_write(output_unit, like_object_inout(6) % hetero(2), .false. )
!         call csv_write(output_unit, like_object_inout(6) % wage_mean_nohetero, .false. )
!         call csv_write(output_unit, like_object_inout(6) % receive_new(13), .false. )
!         call csv_write(output_unit, like_object_inout(6) % mygrid_wife_accept(13) % rv(var_wage), .false. )
!         call csv_write(output_unit, like_object_inout(6) % ln_wage_prob(13), .true. )


!      end if

!      if ( ( (iter .eq. 3398 ) .or. (iter .eq. 3399 ) ) .and. ( myrank .eq. 0 ) ) then

!         call csv_write(output_unit, iter, .false. )
!         call csv_write(output_unit, like_object_inout(6) % hetero(1), .false. )
!         call csv_write(output_unit, like_object_inout(6) % hetero(2), .false. )
!         call csv_write(output_unit, like_object_inout(6) % m_wage_mean_nohetero, .false. )
!         call csv_write(output_unit, like_object_inout(6) % wage_mean_nohetero, .true. )


!         do t=2, like_object_inout(6) % max_time

!            if ( like_object_inout(6) % m_receive_new(t) .eq. 1 ) then
!               call csv_write(output_unit, t, .false. )
!               call csv_write(output_unit, "m", .false. )
!               call csv_write(output_unit, like_object_inout(6) % &
!                  mygrid_husband_accept(t) % rv(var_m_wage), .false. )
!               call csv_write(output_unit, like_object_inout(6) % m_ln_wage_prob(t), .true. )

!            end if

!            if ( like_object_inout(6) % receive_new(t) .eq. 1 ) then
!               call csv_write(output_unit, t, .false. )
!               call csv_write(output_unit, "f", .false. )
!               call csv_write(output_unit, like_object_inout(6) % &
!                  mygrid_wife_accept(t) % rv(var_wage), .false. )
!               call csv_write(output_unit, like_object_inout(6) % ln_wage_prob(t), .true. )

!            end if

!         end do

!      end if



!      if( iter .eq. 1200 ) then
!         print_extra = .true.
!      end if

!      if( ( print_extra .eqv. .true. ) .and. (myrank .eq. 0 ) ) then
!         print *, "iter", iter
!      end if


     ! print *, "iter", iter
     ! read(*,*)

      if  ( mod(iter,print_iter_multiple) .eq. 0 ) then

         if (myrank .eq. 0 ) then

            if (running_on_screen .eqv. .false.) open(newunit= print_unit,file=running_output_file,position="append")

            write(print_unit,*), " "
            write(print_unit,*), "------------------------------------------------"
            write(print_unit,*), "------------------------------------------------"
            write(print_unit,*), " "

            write(print_unit,*), "iter=", iter, "run time", time_current - time_start
            write(print_unit,*), "recent accept average"

            b=1
            lb = max(iter - print_iter_multiple,1)
            up = iter-1
            length = up-lb+1

            do b=1,n_blocks_params
               if ( free_blocks(b) .eqv. .true. ) then
                  write(print_unit,*), block_labels(b), &
                  count( bayes_obj_inout % accept_history_params(b,lb:up))/ dble(length)
               end if
            end do

            write(print_unit,*), "recent ind average", sum(bayes_obj_inout % accept_history_ind(lb:up) )/&
               dble(length*N_obs_total)




            do i=1,n_main_params
               if( print_param_list(i) .eqv. .true. ) then
                  write(print_unit,*), param_labels(i), bayes_obj_inout % params_current(i)
               end if
            end do

         end if !myrank .eq. 0

         if ( print_stats .eqv. .true. ) then
            call like_stats_print( like_object_inout, hh_data_in, &
               bayes_obj_inout % params_current, print_unit, est_savings)
         end if

!         call util_weight_bayes_der(   bayes_obj_inout, &
!                                    like_object_inout, &
!                                    hh_data_in, &
!                                    util_weight_der)

         if ( myrank .eq. 0 ) then

            write(print_unit,*), "joboffer cons sd",     rw_chol_joboffer_block1(1,1)
            write(print_unit,*), "wage cons sd",         rw_chol_wage_block1(1,1)
            write(print_unit,*), "wage gender sd",         rw_chol_wage_block2( &
               n_params_wage_block2,n_params_wage_block2)
            write(print_unit,*), "wage sd sd",           rw_chol_wage_block3(3,3)
            write(print_unit,*), "leisure cons sd",      rw_chol_leisure_block1(2,2)
            write(print_unit,*), "gender reverse sd",      rw_chol_leisure_block4(1,1)
             write(print_unit,*), "weight utility sd",    rw_chol_leisure_block2(2,2)
            write(print_unit,*), "rra sd",               rw_chol_leisure_block3(1,1)
!            write(print_unit,*), "asset cons sd",        rw_chol_asset_error_block1(1,1)
!            write(print_unit,*), "asset linear sd",      rw_chol_asset_error_block2(1,1)

            write(print_unit,*), "value matrix cons",   bayes_obj_inout % value_matrix_current(1)
!            write(print_unit,*), "asset matrix cons",   bayes_obj_inout % asset_matrix_current(1)

           ! write(print_unit,*), "util weight der ", util_weight_der

            if (running_on_screen .eqv. .false.) close(print_unit)

         end if !myrank .eq. 0

      end if !iter multiple


!      if  ( ( mod(iter,write_dir_multiple) .eq. 0 ) .and. (myrank .eq. 0 ) ) then

!         if (myrank .eq. 0 ) then

!            lb = max(iter - write_dir_multiple,2)
!            up = iter-1
!            length = up-lb+1

!            open(newunit= weight_util_unit,file=weight_util_file,position="append")
!            do i=lb,up
!               call csv_write(weight_util_unit, i, .false. )
!               call csv_write(weight_util_unit, bayes_obj_inout %    &
!                  params_history(param_weight_utility,i), .false. )
!               call csv_write(weight_util_unit, weight_util_direction(i), .true. )
!            end do

!         end if
!      end if
   !   print *, bayes_obj_inout % params_proposal

      !this is the main part of this subroutine
      call mcmc_iter_once( bayes_obj_inout, value_grid_inout, hh_data_in, &
         like_object_inout, like_target_inout)

      if ( mod(iter,bayes_export_multiple) .eq. 0 ) then
         call export_param_history( bayes_obj_inout )
      end if



      !determine whether random walk covariance matrix needs to be adjusted
      if( ( mod(iter,adjust_rw_multiple) .eq. 0 ) .and. ( myrank .eq. 0 ) ) then


         !adjust the size of the random walk covariance matrices
         lb = max(iter-adjust_rw_multiple,1)
         up = iter
         length = up-lb+1

         do b=1,n_blocks_params

            if( free_blocks(b) .eqv. .true. ) then

               select case(b)
                  case ( block_joboffer_params1 )
                     rw_chol_pointer  => rw_chol_joboffer_block1
                  case ( block_joboffer_params2 )
                     rw_chol_pointer  => rw_chol_joboffer_block2
                  case ( block_wage_params1 )
                     rw_chol_pointer  => rw_chol_wage_block1
                  case ( block_wage_params2 )
                     rw_chol_pointer  => rw_chol_wage_block2
                  case ( block_wage_params3 )
                     rw_chol_pointer  => rw_chol_wage_block3
                  case ( block_leisure_params1 )
                     rw_chol_pointer  => rw_chol_leisure_block1
                  case ( block_leisure_params2 )
                     rw_chol_pointer  => rw_chol_leisure_block2
                  case ( block_leisure_params3 )
                     rw_chol_pointer  => rw_chol_leisure_block3
                  case ( block_leisure_params4 )
                     rw_chol_pointer  => rw_chol_leisure_block4
                  case ( block_asset_error_params1 )
                     rw_chol_pointer  => rw_chol_asset_error_block1
                  case ( block_asset_error_params2 )
                     rw_chol_pointer  => rw_chol_asset_error_block2
                  case default
                     print *, "error in select case for adjust rw"
               end select

               accept_average = &
                  dble(count((bayes_obj_inout % accept_history_params(b,lb:up)))) /length


               if (accept_average < low_accept_cutoff) then
                  rw_chol_pointer = rw_chol_pointer * low_adjust_val
               else if (accept_average > high_accept_cutoff) then
                  rw_chol_pointer = rw_chol_pointer * high_adjust_val
               end if

            end if !free blocks

         end do

      end if !adjust size of rw covariance matrices and myrank .eq. 0

      if ( ( est_dynamic .eqv. .true. ) .and. ( save_params_switch .eqv. .true. ) ) then
         if ( mod(iter,adjust_bandwidth_multiple) .eq. 0 ) then
            !now adjust the size of the bandwidth
            call bayes_obj_inout % update_bandwidth()
         end if
      end if

      !for changing time_weighting indicator and how many iterations to do
   !   if (iter < n_mcmc_burn_in) then
   !      bayes_obj_inout % time_weighting = .true.
   !      bayes_obj_inout % n_iterations = n_iterations_start
   !   else
    !     bayes_obj_inout % time_weighting = .false.
    !     bayes_obj_inout % n_iterations = n_iterations_end
    !  end if

      if( iter< time_weighting_min ) then
         bayes_obj_inout % time_weighting = .true.
      else
         bayes_obj_inout % time_weighting = .false.
      end if

      call cpu_time(time_current)
      call mpi_bcast( time_current, 1, mpi_real8, 0, mpi_comm_world, err )

      call mpi_barrier(mpi_comm_world, err) !2/18/14, putting this here because I had issues with not all of the processes exiting at the right time.  !might not be necessary or might slow things down, but will help me diagnose

      if  ( mod(iter,print_iter_multiple) .eq. 0 ) then
         print *, "end of iter", iter, "from proc", myrank
      end if

      call mpi_barrier(mpi_comm_world, err)

   end do !of the main do_while loop

   bayes_obj_inout % run_time = time_current - time_start

   print *, "exit from proc", myrank


end subroutine mcmc_iter_main


subroutine mcmc_output_create( mcmc_output_out, bayes_obj_in )
   !creates the output of the mcmc routine
   class(mcmc_output_type), intent(out), optional :: mcmc_output_out
   type(bayes_obj_type), intent(in) :: bayes_obj_in

   real*8, dimension(n_main_params, n_main_params) :: cov_temp, cholesky_temp, A
   real*8, dimension(n_main_params ) :: deviation_temp
   logical :: pos_def
   integer :: i, j1,j2, lb, up, b, length_temp

   real*8, dimension(n_main_params) :: params_temp
   real*8, dimension(n_grid_points) :: value_matrix_temp
   real*8, dimension(n_grid_points) :: asset_matrix_temp
   logical :: kernel_error_temp

   real*8, parameter :: fixed_param_sd = 1.0 !do this just to make covariance matrix pd


   if (myrank .eq. 0 ) then

      print *, "storing output"
      mcmc_output_out % final_iter = bayes_obj_in % iter
      mcmc_output_out % final_time = bayes_obj_in % run_time

      do b=1,n_blocks_params
         mcmc_output_out % accept_param_average(b) = &
            dble(count( bayes_obj_in % accept_history_params(b,:mcmc_output_out % final_iter)))/ &
            (mcmc_output_out % final_iter)
      end do

      mcmc_output_out % accept_ind_average = &
         dble( sum(bayes_obj_in % accept_history_ind(:mcmc_output_out % final_iter)))/ &
         ( mcmc_output_out % final_iter *N_obs_total )


      !means at various points in the mcmc chain
      length_temp = mcmc_output_out % final_iter /n_mcmc_means
      do i=1,n_mcmc_means
         lb = (i-1)*length_temp+1
         up = i*length_temp
         mcmc_output_out % mcmc_means(:,i) = mean_vec_est( length_temp, n_main_params, &
            bayes_obj_in % params_history(:,lb:up) )
      end do

   end if !myrankd .eq. 0

   !get mean and covariance if past the burn_in stage
   if ( bayes_obj_in % iter < n_mcmc_burn_in ) then

      if (myrank .eq. 0 ) then
         print *, "didn't do enough iterations to go pass burn in, will just store most recent parameter and ols instead"

         mcmc_output_out % posterior_mean = bayes_obj_in % params_current(1:n_main_params)
         mcmc_output_out % value_matrix_posterior_mean = bayes_obj_in % value_matrix_current
         mcmc_output_out % asset_matrix_posterior_mean = bayes_obj_in % asset_matrix_current
      end if


   else

      if (myrank .eq. 0 ) then

         length_temp = mcmc_output_out % final_iter - n_mcmc_burn_in
         lb=n_mcmc_burn_in+1
         up=mcmc_output_out % final_iter

         !mean
         mcmc_output_out % posterior_mean = mean_vec_est( length_temp, n_main_params, &
            bayes_obj_in % params_history(:,lb:up) )

         !covariance
         mcmc_output_out % posterior_cov_matrix = cov_matrix_est( length_temp, n_main_params, &
            bayes_obj_in % params_history(:,lb:up), mcmc_output_out % posterior_mean )


         !posterior standard deviations
         do i=1,n_main_params
            mcmc_output_out % posterior_sd(i) = sqrt( mcmc_output_out % posterior_cov_matrix(i,i) )
         end do

         do i=1,n_main_params
            if( free_param_list(i) .eqv. .false. ) then
               mcmc_output_out % posterior_cov_matrix(i,i) = fixed_param_sd
            end if
         end do

         params_temp = mcmc_output_out % posterior_mean

      end if !myrank .eq. 0

      call mpi_bcast( params_temp, size(params_temp), mpi_real8, 0, mpi_comm_world, err )


      !ols matrices at the posterior mean
      call interp_kernel(  bayes_obj_in, &
                           params_temp, &
                           value_matrix_temp, &
                           asset_matrix_temp, &
                           kernel_error_temp )


      if ( myrank .eq. 0 ) then

         !now store only for root node
         mcmc_output_out % value_matrix_posterior_mean = value_matrix_temp
         mcmc_output_out % asset_matrix_posterior_mean = asset_matrix_temp
         mcmc_output_out % kernel_error_mean = kernel_error_temp


         !now for interp coefficients centered around posterior mean

         print *, "ols coefficents center around posterior mean"

         !first cholesky
         call chol_decomp_sub(n_main_params, mcmc_output_out % posterior_cov_matrix, cholesky_temp, pos_def)

         A = 0
         do i=1,n_main_params
            if ( free_param_list(i) .eqv. .true. ) then
               A(i,i) = 1
            end if
         end do

         cholesky_temp = matmul(A,cholesky_temp)

         if (pos_def .eqv. .false.)  then
            print *, "posterior covariance matrix is not positive definite"
            print *, "will just draw random parameters by ignoring correlations"

            cholesky_temp=0
            do i= 1, n_main_params
               cholesky_temp(i,i) = mcmc_output_out % posterior_sd(i)
            end do
         end if

      end if !myrank .eq. 0

      do i=1, n_interp_keep

         if (myrank .eq. 0 ) then

            call get_random_mvn( n_main_params, &
                                 mcmc_output_out % posterior_mean, &
                                 cholesky_temp, &
                                 params_temp )

         end if

         call mpi_bcast( params_temp, size(params_temp), mpi_real8, 0, mpi_comm_world, err )


         call interp_kernel(  bayes_obj_in,  &
                              params_temp, &
                              value_matrix_temp, &
                              asset_matrix_temp, &
                              kernel_error_temp )


         if (myrank .eq. 0 ) then

            mcmc_output_out % params_interp_matrix_save(:,i) = params_temp
            mcmc_output_out % value_matrix_save(:,i) = value_matrix_temp
            mcmc_output_out % asset_matrix_save(:,i) = asset_matrix_temp
            mcmc_output_out % kernel_error_vec(i) = kernel_error_temp

         end if


      end do
   end if !for whether the burn_in period was passed or not



    if( myrank .eq. 0)  print *, "done with mcmc_output_create"

end subroutine mcmc_output_create



subroutine mcmc_output_write(mcmc_output_in,params_init_in)
   !writes the mcmc output to a text file
   class(mcmc_output_type),intent(in) :: mcmc_output_in
   real*8, dimension(n_main_params), intent(in) :: params_init_in
   integer, parameter :: write_unit = 1
   integer :: i,j,m_job,job

   if (myrank .eq. 0 ) then

      call gen_param_labels()

      !first the main output file that includes posterior means, sd, and other useful stats
      open(unit=write_unit, file = mcmc_output_file_main)

      call csv_write(write_unit, " ",.false.)
      call csv_write(write_unit, "param",.false.)
      call csv_write(write_unit, "sd",.true.)

      do i=1,n_main_params

         call csv_write(write_unit, param_labels(i), .false. )
         call csv_write(write_unit, mcmc_output_in % posterior_mean(i), .false. )
         call csv_write(write_unit, mcmc_output_in % posterior_sd(i), .true. )
      end do

      call csv_write(write_unit, "  ", .true. )

      call csv_write(write_unit, "final_iter", .false. )
      call csv_write(write_unit, mcmc_output_in % final_iter, .true. )

      call csv_write(write_unit, "final_time", .false. )
      call csv_write(write_unit, mcmc_output_in % final_time, .true. )

      call csv_write(write_unit, "accept_param_average", .false. )
      call csv_write(write_unit, mcmc_output_in % accept_param_average(:), .true. )

      call csv_write(write_unit, "accept_ind_averaege", .false. )
      call csv_write(write_unit, mcmc_output_in % accept_ind_average, .true. )

      call csv_write(write_unit, "  ", .true. )
      call csv_write(write_unit, "mcmc_means", .true. )

      do i=1,n_main_params
         call csv_write(write_unit, param_labels(i), .false. )
         call csv_write(write_unit, params_init_in(i), .false.)
         call csv_write(write_unit, mcmc_output_in % mcmc_means(i,1:n_mcmc_means), .true. )
      end do


      close(unit=write_unit)

      !now create the covariance matrix
      open(unit=write_unit, file = mcmc_output_file_cov_matrix)

      call csv_write(write_unit, "  ", .false. )
      do i=1,n_main_params
         call csv_write(write_unit, param_labels(i), .false. )
      end do
      call csv_write(write_unit, "  ", .true. )

      do i=1,n_main_params
         call csv_write(write_unit, param_labels(i), .false. )
         call csv_write(write_unit, mcmc_output_in % posterior_cov_matrix(i,1:i), .true. )
      end do

      close(unit=write_unit)

      !now for the interpolation coefficients
      open(unit=write_unit, file = mcmc_output_file_ols_interp)



      call csv_write(write_unit, mcmc_output_in % value_matrix_posterior_mean(:), .true. )
      call csv_write(write_unit, mcmc_output_in % asset_matrix_posterior_mean(:), .true. )




      do i=1, n_interp_keep
         call csv_write(write_unit, mcmc_output_in % params_interp_matrix_save(:,i), .true. )

         call csv_write(write_unit, mcmc_output_in % value_matrix_save(:,i), .true. )
         call csv_write(write_unit, mcmc_output_in % asset_matrix_save(:,i), .true. )

      end do

      !now write the kernel error logicals
      write(write_unit, *) mcmc_output_in % kernel_error_mean, &
               (mcmc_output_in % kernel_error_vec(i), i=1,n_interp_keep )

      close(unit=write_unit)

   end if !myrank .eq. 0


end subroutine mcmc_output_write

subroutine mcmc_output_read(mcmc_output_out)
   !reads the mcmc output from test files
   class(mcmc_output_type),intent(out) :: mcmc_output_out

   integer :: i,j, m_job, job, read_unit
   character(len=50) :: char_ignore

   if (myrank .eq. 0 ) then

      !first the main output file that includes posterior means, sd, and other useful stats
      open(newunit = read_unit, file = mcmc_output_file_main)

      read(read_unit,*)

      do i=1,n_main_params
         read(read_unit,*) char_ignore, mcmc_output_out % posterior_mean(i), mcmc_output_out % posterior_sd(i)
      end do


     ! print *, mcmc_output_out % posterior_mean(n_main_params), mcmc_output_out % posterior_sd(n_main_params)
    !  read(*,*)

      read(read_unit,*)

      read(read_unit,*) char_ignore, mcmc_output_out % final_iter
      read(read_unit,*) char_ignore, mcmc_output_out % final_time
      read(read_unit,*) char_ignore, (mcmc_output_out % accept_param_average(j), j=1,n_blocks_params)
      read(read_unit,*) char_ignore, mcmc_output_out % accept_ind_average

      read(read_unit,*)
      read(read_unit,*)

      do i=1,n_main_params
         read(read_unit,*) char_ignore, ( mcmc_output_out % mcmc_means(i,j), j=1,n_mcmc_means )
      end do

      close(unit=read_unit)

      !now the covariance matrix
      open(unit=read_unit, file = mcmc_output_file_cov_matrix)

      read(read_unit,*)

      do i=1,n_main_params
         read(read_unit,*) char_ignore, (mcmc_output_out % posterior_cov_matrix(i,j), j=1,i)
      end do

      close(unit=read_unit)

      !now for the interpolation coefficients
      open(unit=read_unit, file = mcmc_output_file_ols_interp)


      read(read_unit,*) (mcmc_output_out % value_matrix_posterior_mean(j), j=1, n_grid_points )
      read(read_unit,*) (mcmc_output_out % asset_matrix_posterior_mean(j), j=1, n_grid_points )


      do i=1, n_interp_keep
         read(read_unit,*) (mcmc_output_out % params_interp_matrix_save(j,i), j=1,n_main_params )


         read(read_unit,*) (mcmc_output_out % value_matrix_save(j,i), j=1,n_grid_points )
         read(read_unit,*) (mcmc_output_out % asset_matrix_save(j,i), j=1,n_grid_points )


      end do


      !now read the kernel error logicals
      read(read_unit, *) mcmc_output_out % kernel_error_mean, &
               (mcmc_output_out % kernel_error_vec(i), i=1,n_interp_keep )


      close(unit=read_unit)

   end if


end subroutine mcmc_output_read



!subroutine random_walk_write(bayes_obj_in, cov_in_opt)
!   !this writes the covariance matrices for the blocks
!   class(bayes_obj_type), intent(in) :: bayes_obj_in
!   real*8, dimension(n_main_params, n_main_params), intent(in), optional :: cov_in_opt !optional input of the larger covariance matrix for the converted
!      !parameters

!   integer :: write_unit
!   real*8, dimension(:,:), allocatable :: rw_cov_temp
!   integer, dimension(:), pointer :: list_pointer
!   character(len=30), dimension(:),allocatable :: param_labels_temp
!   integer :: n_params_temp


!   real*8, dimension(n_main_params, n_mcmc_iter ) :: params_history_convert
!   real*8, dimension(n_main_params) :: mean_convert
!   real*8, dimension(n_main_params,n_main_params) :: cov_convert
!   integer :: iter


!   character(len=40) :: out_file, char_ignore
!   character(len=1) :: int_char !for writting character to integer
!   integer :: b,i,j,i1,i2,j1,j2
!   integer :: length_temp, lb, up

!   if (myrank .eq. 0 ) then

!      call gen_param_labels()

!      !get covariance matrix
!      if (present(cov_in_opt)) then
!         cov_convert = cov_in_opt

!      else
!         iter = bayes_obj_in % iter

!         !determine if past burn_in stage
!         if ( iter < n_mcmc_burn_in ) then
!            print *, "didn't do enough iterations to go pass burn in, not doing this routine"
!            return
!         end if


!         params_history_convert = bayes_obj_in % params_history

!         length_temp = iter - n_mcmc_burn_in
!         lb=n_mcmc_burn_in+1
!         up=iter


!         !covert the stored variables of interest that have bounds
!         do i = lb, up
!            call param_predraw_convert(params_history_convert(:,i) )
!         end do


!         !mean
!         mean_convert = mean_vec_est( length_temp, n_main_params, &
!            params_history_convert(:,lb:up) )

!         !covariance
!         cov_convert = cov_matrix_est( length_temp, n_main_params, &
!            params_history_convert(:,lb:up), mean_convert )

!         print *, "got the converted means"

!      end if


!     ! now generate the covariance matrices based on these converted parameters

!      do b=1,n_blocks_params
!         print *, "now writing rw cov matrix for block", b

!         !get file name
!         write(int_char,"(I1)") b
!         out_file=trim(file_cov_block_out) // trim(int_char) // ".csv"

!         !set up integers and pointers
!         select case(b)
!            case (block_joboffer_params)
!               n_params_temp = n_params_joboffer_block
!               list_pointer     => joboffer_block_list

!            case  (block_wage_params)
!               n_params_temp = n_params_wage_block
!               list_pointer     => wage_block_list

!            case ( block_leisure_params1 )
!               n_params_temp = n_params_leisure_block1
!               list_pointer     => leisure_block1_list

!            case ( block_leisure_params2 )
!               n_params_temp = n_params_leisure_block2
!               list_pointer     => leisure_block2_list

!            case ( block_asset_error_params )
!               n_params_temp = n_params_asset_error_block
!               list_pointer     => asset_error_block_list

!            case default
!               print *, "error in select case for writting the random walk covariances"
!         end select

!         !get this smaller covariance matrix from the larger covariance matrix
!         if(allocated(rw_cov_temp)) deallocate(rw_cov_temp)
!         allocate(rw_cov_temp(n_params_temp,n_params_temp))

!         rw_cov_temp = 0

!         do i1=1,n_params_temp
!            do i2=1,i1
!               j1=list_pointer(i1)
!               j2=list_pointer(i2)

!               rw_cov_temp(i1,i2) = cov_convert(j1,j2)
!            end do
!         end do

!         !get smaller param labels
!         if(allocated(param_labels_temp)) deallocate(param_labels_temp)
!         allocate(param_labels_temp(n_params_temp))

!         do i=1,n_params_temp
!            j = list_pointer(i)
!            param_labels_temp(i) = param_labels(j)
!         end do

!         !now write this file
!         open(newunit = write_unit, file=out_file)

!         call csv_write(write_unit, " ", .false. )
!         call csv_write(write_unit, param_labels_temp, .true.)

!         do i=1,n_params_temp
!            call csv_write(write_unit, param_labels_temp(i), .false. )
!            call csv_write(write_unit, rw_cov_temp(i,1:i), .true. )
!         end do

!         close(write_unit)

!      end do !over blocks

!   end if

!end subroutine random_walk_write

end module bayesian_mod


