!IMPORTANT: need to think about how to do the P or 1-P think for getting an offer/not receiving an offer or for a job_loss



module likelihood_mod

use enum_mod
use model_params_mod
use interp_support_mod

use sipp_data_mod

use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use normal_mod
use stat_estimates_mod

use csv_file
use mpi_mod




implicit none

   logical, parameter :: print_param_like = .false.
   logical, parameter :: choice_prob_lb_enforce = .true. !whether to enfore a lower bound on the choice probabilities
   logical, parameter :: choice_prob_penalty_enforce = .true. !whether to incoporate a penalty for low choice probs
   logical, parameter :: asset_error_lb_enforce = .true. !whether to enfore a lower bound for the asset error
   logical, parameter :: wage_prob_penalty_enforce = .true. !whether to incoporate a penalty for low choice probs

   real*8, parameter  :: choice_prob_penalty_cutoff = 1.0D-8   !1.0D-4
   real*8, parameter  :: choice_prob_penalty_val = 100
   real*8, parameter :: choice_prob_lb = tiny(1.0D-200) !if choice_prob_lb_enforce is true, then the choice probability will be the max of the actual choice prob and this one. !I've had this raised to 1.0D-8 before
   real*8, parameter :: asset_error_lb = -huge(0.0D+0)

   real*8, parameter    :: wage_prob_penalty_cutoff_base = log(1.0D-50)
   real*8               :: wage_prob_penalty_cutoff =  wage_prob_penalty_cutoff_base !this changes throughout my program to be a function of wage sd
   real*8, parameter    :: wage_prob_penalty_val = 100

   integer, parameter :: der_vars_convert = 10 !conversion integer used for constructing the derivatives of the choice probabilities

 !  real*8, parameter :: asset_median = 31745.1 !used for initialization

   real*8, parameter :: asset_median = 10000 !used for initialization

   !this type contains pointers for various components of the likelihood function.  For parts that change as parameters change, I make them pointers to reduce inefficient copying for my Bayesian McMC routine
   type :: like_type

      logical :: pointer_initialized = .false.

      integer :: max_time

      !these are integers that don't change as the parameters change
      integer, dimension(:), allocatable :: m_accept_new !whether accepted a new job
      integer, dimension(:), allocatable :: accept_new

      integer, dimension(:), allocatable :: m_time_job_ends !for an accepted job or job held at t=1, this integer for index t gives the time index for the last period a job was held in the data
      integer, dimension(:), allocatable :: time_job_ends !for an accepted job or job held at t=1, this integer for index t gives the time index for the last period a job was held in the data

      integer, dimension(:), allocatable :: birth
      integer, dimension(:), allocatable :: m_jobloss !whether loss a job
      integer, dimension(:), allocatable :: jobloss

      integer, dimension(:), allocatable :: m_ui_ever !whether ui was receives in the event of a jobloss
      integer, dimension(:), allocatable :: ui_ever
      integer, dimension(:), allocatable :: m_time_unemp_end !time an unemployment spell ends
      integer, dimension(:), allocatable :: time_unemp_end !time an unemployment spell ends

      integer :: unknown_unemp_length !says whether I need to deal with unknown unemployment length, which I need to do
         !if a spouse is unemployed at the beginnign of the panel

      !these are variables that change as the parameters change or unknown variables change, so I make them pointers

      real*8, dimension(:), pointer  :: hetero

      real*8, dimension(:), pointer :: m_joboffer_prob
      real*8, dimension(:), pointer :: joboffer_prob


      real*8, dimension(:), pointer :: m_hours_prob
      real*8, dimension(:), pointer :: hours_prob

      real*8, pointer :: m_wage_mean_nohetero
      real*8, pointer :: wage_mean_nohetero

      real*8, dimension(:), pointer :: m_ln_wage_prob
      real*8, dimension(:), pointer :: ln_wage_prob
      real*8, dimension(:), pointer :: m_wage_new !stores the new wage, which isn't forced to be between the grid mins and maxs
      real*8, dimension(:), pointer :: wage_new


      integer, dimension(:), pointer :: m_receive_new
      integer, dimension(:), pointer :: receive_new

      type(mygrid_type), pointer :: mygrid_t_invar  !this mygrid contains the time invariant variables
      type(mygrid_type), dimension(:), pointer :: mygrid_t_var !contains time varing variables
      type(mygrid_type), dimension(:), pointer :: mygrid_husband_accept
      type(mygrid_type), dimension(:), pointer :: mygrid_wife_accept
      type(mygrid_type), dimension(:), pointer :: mygrid_both_accept

      integer, dimension(:), pointer:: utility_params_created !says whether the utilty parameters have been created for this current draw of the leisure parameters
         !used because I only need these parameters when a job_offer is received
      real*8, dimension(:,:), pointer :: utility_params
      real*8, dimension(:), pointer :: utility_both_reject
      real*8, dimension(:), pointer :: utility_husband_accept
      real*8, dimension(:), pointer :: utility_wife_accept
      real*8, dimension(:), pointer :: utility_both_accept

      real*8, dimension(:), pointer :: asset_choice_both_reject
      real*8, dimension(:), pointer :: asset_choice_husband_accept
      real*8, dimension(:), pointer :: asset_choice_wife_accept
      real*8, dimension(:), pointer :: asset_choice_both_accept

      real*8, pointer :: ln_asset_prob

      real*8, dimension(:), pointer :: choice_prob !this is the probability of a joboffer choice conditional on a set of receive offers

    !  real*8, dimension(:), pointer :: m_jobloss_prob
    !  real*8, dimension(:), pointer :: jobloss_prob



   contains
      procedure :: print => print_like_object
      procedure :: like_init_pointer => like_init_pointer
      procedure :: like_main_setup => like_main_setup
     ! procedure :: get_birth_ui_means => get_birth_ui_means

      procedure :: joboffer_like_update         => joboffer_like_update
      procedure :: job_probs_like_update        => job_probs_like_update
      procedure :: asset_choice_like_update     => asset_choice_like_update
      procedure :: utility_params_like_update   => utility_params_like_update
      procedure :: utility_like_update          => utility_like_update
   !   procedure :: jobloss_like_update          => jobloss_like_update

  !    procedure :: ols_partial_like_update      => ols_partial_like_update
      procedure :: choice_prob_like_update      => choice_prob_like_update
      procedure :: asset_error_prob_like_update => asset_error_prob_like_update
  !    procedure :: low_choice_prob_adjust             => low_choice_prob_adjust

      procedure :: like_max_value => like_max_value


      procedure :: print_which_like_target      => print_which_like_target
      procedure :: like_error_hunt              => like_error_hunt
      procedure :: log_like_func                => log_like_func


   end type

   !this contains targets for the pointers in the object written above
   type :: like_target_type

      logical :: target_allocated = .false.

      real*8, dimension(n_hetero)       :: hetero1, hetero2

      real*8, dimension(:),allocatable  :: m_joboffer_prob1, m_joboffer_prob2
      real*8, dimension(:), allocatable :: joboffer_prob1, joboffer_prob2

      real*8, dimension(:), allocatable :: m_hours_prob1, m_hours_prob2
      real*8, dimension(:), allocatable :: hours_prob1, hours_prob2


      real*8 :: m_wage_mean_nohetero1, m_wage_mean_nohetero2
      real*8 :: wage_mean_nohetero1, wage_mean_nohetero2

      real*8, dimension(:), allocatable :: m_ln_wage_prob1, m_ln_wage_prob2
      real*8, dimension(:), allocatable :: ln_wage_prob1, ln_wage_prob2
      real*8, dimension(:), allocatable :: m_wage_new1, m_wage_new2
      real*8, dimension(:), allocatable :: wage_new1, wage_new2


      integer, dimension(:), allocatable :: m_receive_new1, m_receive_new2
      integer, dimension(:), allocatable :: receive_new1, receive_new2
      type(mygrid_type)  :: mygrid_t_invar1, mygrid_t_invar2
      type(mygrid_type), dimension(:), allocatable :: mygrid_t_var1, mygrid_t_var2
      type(mygrid_type), dimension(:), allocatable :: mygrid_husband_accept1, mygrid_husband_accept2
      type(mygrid_type), dimension(:), allocatable :: mygrid_wife_accept1, mygrid_wife_accept2
      type(mygrid_type), dimension(:), allocatable :: mygrid_both_accept1, mygrid_both_accept2


      integer, dimension(:), allocatable:: utility_params_created1, utility_params_created2 !says whether the utilty parameters have been created for this current draw of the leisure parameters
      real*8, dimension(:,:), allocatable :: utility_params1, utility_params2
      real*8, dimension(:), allocatable :: utility_both_reject1, utility_both_reject2
      real*8, dimension(:), allocatable :: utility_husband_accept1, utility_husband_accept2
      real*8, dimension(:), allocatable :: utility_wife_accept1, utility_wife_accept2
      real*8, dimension(:), allocatable :: utility_both_accept1, utility_both_accept2

      real*8, dimension(:), allocatable :: asset_choice_both_reject1, asset_choice_both_reject2
      real*8, dimension(:), allocatable :: asset_choice_husband_accept1, asset_choice_husband_accept2
      real*8, dimension(:), allocatable :: asset_choice_wife_accept1, asset_choice_wife_accept2
      real*8, dimension(:), allocatable :: asset_choice_both_accept1, asset_choice_both_accept2

      real*8 :: ln_asset_prob1, ln_asset_prob2

      real*8, dimension(:), allocatable :: choice_prob1, choice_prob2 !this is the probability of a joboffer choice conditional on a set of receive offers

   !   real*8, dimension(:), allocatable :: m_jobloss_prob1, m_jobloss_prob2
   !   real*8, dimension(:), allocatable :: jobloss_prob1, jobloss_prob2

   contains
      procedure :: print => print_like_target
      procedure :: allocate => like_target_allocate
      procedure :: copy_targets_init => copy_targets_init
   end type


contains

subroutine print_like_object(like_in,t,value_matrix_in_opt)
   !prints the components of the likelihood object at time t
   class(like_type) :: like_in
   integer,intent(in) :: t
   real*8, dimension(n_grid_points), intent(in), optional :: value_matrix_in_opt
   integer :: i

   print *, " "
   print *,"!!!!!!!!!"
   print *, "variables for the joboffer stage"

   if (t .ne. 1 ) then
      print *, "m_receive_new", like_in % m_receive_new(t)
      print *, "receive_new", like_in % receive_new(t)
   end if

   print *, "m_accept_new", like_in % m_accept_new(t)
   print *, "accept_new", like_in % accept_new(t)

   if (t .ne. 1 ) then
      if (like_in % m_accept_new(t) .eq. 1) print *, "end date for husband's job", like_in % m_time_job_ends(t)
      if (like_in % accept_new(t) .eq. 1  ) print *, "end date for wife's    job", like_in % time_job_ends(t)
   end if

   print *, "unobserved hetero", like_in % hetero

  ! stop

   if (t .ne. 1 ) then
      print *, " "
      print *, "m_joboffer_prob", like_in % m_joboffer_prob(t)
      print *, "joboffer_prob", like_in % joboffer_prob(t)

      print *, " "
      if (like_in % m_receive_new(t) .eq. 1) print *, "m_hours_prob", like_in % m_hours_prob(t)
      if (like_in % receive_new(t) .eq. 1)   print *, "hours_prob", like_in % hours_prob(t)

      if (like_in % m_receive_new(t) .eq. 1) print *, "m_ln_wage_prob", like_in % m_ln_wage_prob(t)
      if (like_in % receive_new(t) .eq. 1)   print *, "ln_wage_prob", like_in % ln_wage_prob(t)

      if (like_in % m_receive_new(t) .eq. 1) print *, "m_new_wage", like_in % m_wage_new(t)
      if (like_in % receive_new(t) .eq. 1)   print *, "new_wage", like_in % wage_new(t)

   end if

   print *, " "
   print *, "m_wage_mean_nohetero", like_in % m_wage_mean_nohetero
   print *, "wage_mean_nohetero", like_in % wage_mean_nohetero


   print *, " "
   print *, "mygrid_t_invar"
   print *, " "
   call like_in % mygrid_t_invar % print(.true. , .false. )

   print *, " "
   print *, "mygrid_t_var"
   print *, " "
   call like_in % mygrid_t_var(t) % print( .false. , .true. )


   if (t .ne. 1 ) then
      if (like_in % m_receive_new(t) .eq.1) then
         print *, " "
         print *, "mygird_husband_accept"
         print *, " "
         call like_in % mygrid_husband_accept(t) % print( .false. , .true. )
      end if

      if (like_in % receive_new(t) .eq. 1)  then
         print *, " "
         print *, "mygrid_wife_accept"
         print *, " "
         call like_in % mygrid_wife_accept(t) % print( .false. , .true. )
      end if

      if ( (like_in % m_receive_new(t) .eq.1) .and. (like_in % receive_new(t) .eq. 1) ) then
         print *, " "
         print *, "mygrid_both_accept"
         print *, " "
         call like_in % mygrid_both_accept(t) % print( .false. , .true. )
      end if

      print *, " "

      if ( (like_in % m_receive_new(t) .eq.1) .or. (like_in % receive_new(t) .eq. 1) )then
         print *, "utility params"
         call print_utility_params( like_in % utility_params(:,t) )

         print *, " "

         print *, "utility_both_reject", like_in % utility_both_reject(t)
      end if

      if (like_in % m_receive_new(t) .eq.1) then
         print *, "utility_husband_accept", like_in % utility_husband_accept(t)
      end if
      if (like_in % receive_new(t) .eq. 1) then
         print *, "utility_wife_accept", like_in % utility_wife_accept(t)
      end if
      if ( (like_in % m_receive_new(t) .eq.1) .and. (like_in % receive_new(t) .eq. 1) )then
         print *, "utility_both_accept", like_in % utility_both_accept(t)
      end if


      !assets
      print *, " "

      print *, "asset_choice_both_reject" , like_in % asset_choice_both_reject(t)

      if (like_in % m_receive_new(t) .eq.1) then
         print *, "asset_choice_husband_accept", like_in % asset_choice_husband_accept(t)
      end if
      if (like_in % receive_new(t) .eq. 1) then
         print *, "asset_choice_wife_accept", like_in % asset_choice_wife_accept(t)
      end if
      if ( (like_in % m_receive_new(t) .eq.1) .and. (like_in % receive_new(t) .eq. 1) )then
         print *, "asset_choice_both_accept", like_in % asset_choice_both_accept(t)
      end if

      print *, "ln_asset_prob", like_in % ln_asset_prob

      print *, " "

      !value functions
      if ( present(value_matrix_in_opt) ) then
         if ( (like_in % m_receive_new(t) .eq.1) .or. (like_in % receive_new(t) .eq. 1) )then
            print *, "value_both_reject", &
               value_function_split_in(   like_in % mygrid_t_invar, &
                                          like_in % mygrid_t_var(t), &
                                          like_in % asset_choice_both_reject(t), &
                                          value_matrix_in_opt )
         end if
         if (like_in % m_receive_new(t) .eq.1) then
            print *, "value_husband_accept", &
               value_function_split_in(   like_in % mygrid_t_invar, &
                                          like_in % mygrid_husband_accept(t), &
                                          like_in % asset_choice_husband_accept(t), &
                                          value_matrix_in_opt )
         end if
         if (like_in % receive_new(t) .eq. 1) then
            print *, "value_wife_accept", &
               value_function_split_in(   like_in % mygrid_t_invar, &
                                          like_in % mygrid_wife_accept(t), &
                                          like_in % asset_choice_wife_accept(t), &
                                          value_matrix_in_opt )
         end if
         if ( (like_in % m_receive_new(t) .eq.1) .and. (like_in % receive_new(t) .eq. 1) )then
            print *, "value_both_accept", &
               value_function_split_in(   like_in % mygrid_t_invar, &
                                          like_in % mygrid_both_accept(t), &
                                          like_in % asset_choice_both_accept(t), &
                                          value_matrix_in_opt )
         end if
      end if




      print *,  " "
      print *, "choice_prob", like_in % choice_prob(t)
   end if !t .ne. 1

   if ( t .ne. 1)  print *, "utility_params_created?", like_in % utility_params_created(t)
!   print *, "ols_partial_created?", like_in % ols_partial_created

   if ( t .ne. like_in % max_time) then
      print *, " "
      print *, "!!!!!!!!!!!!!!!!!"
      !group of coefficients used for evaluating the jobloss event
      print *, "birth", like_in % birth(t)
      print *, "m_jobloss", like_in % m_jobloss(t) !whether loss a job
      print *, "jobloss", like_in % jobloss(t)

   !   print *, " "
   !   print *, "m_jobloss_prob", like_in % m_jobloss_prob(t)
   !   print *, "jobloss_prob", like_in % jobloss_prob(t)


      print *, " "
      if (  (like_in % m_jobloss(t) .eq. 1 ) .or. &
            (t .eq. 1) .and. (like_in % mygrid_t_var(t) % iv(var_m_job) .eq. 0) ) then
         print *, "m_ui_ever", like_in % m_ui_ever(t)
         print *, "m_unemp end date", like_in % m_time_unemp_end(t)
      end if

      if (  (like_in % jobloss(t) .eq. 1 ) .or. &
            (t .eq. 1) .and. (like_in % mygrid_t_var(t) % iv(var_job) .eq. 0) ) then
         print *, "ui_ever", like_in % ui_ever(t)
         print *, "unemp end date", like_in % time_unemp_end(t)
      end if
   end if

end subroutine print_like_object



subroutine print_like_target(like_target_in,like_in,t)
   !prints the components of the likelihood object at time t
   class(like_target_type) :: like_target_in
   type(like_type) :: like_in
   integer,intent(in) :: t
   integer :: i

   print *, " "
   print *,"!!!!!!!!!"
   print *, "variables for the joboffer stage"

   if (t .ne. 1 ) then
      print *, "m_receive_new", like_target_in % m_receive_new1(t), like_target_in % m_receive_new2(t)
      print *, "receive_new", like_target_in % receive_new1(t), like_target_in % receive_new2(t)
   end if

   print *, "m_accept_new", like_in % m_accept_new(t)
   print *, "accept_new", like_in % accept_new(t)

   if (like_in % m_accept_new(t) .eq. 1) print *, "end date for husband's job", like_in % m_time_job_ends(t)
   if (like_in % accept_new(t) .eq. 1  ) print *, "end date for wife's    job", like_in % time_job_ends(t)

   print *, "hetero"
   do i=1,n_hetero
      print *, i, like_target_in % hetero1(i), like_target_in % hetero2(i)
   end do


   if (t .ne. 1 ) then
      print *, " "
      print *, "m_joboffer_prob", like_target_in % m_joboffer_prob1(t), like_target_in % m_joboffer_prob2(t)
      print *, "joboffer_prob", like_target_in % joboffer_prob1(t), like_target_in % joboffer_prob2(t)


      print *, " "
      print *, "m_hours_prob", like_target_in % m_hours_prob1(t), like_target_in % m_hours_prob2(t)
      print *, "hours_prob", like_target_in % hours_prob1(t), like_target_in % hours_prob2(t)

      print *, "m_ln_wage_prob", like_target_in % m_ln_wage_prob1(t), like_target_in % m_ln_wage_prob2(t)
      print *, "ln_wage_prob", like_target_in % ln_wage_prob1(t), like_target_in % ln_wage_prob2(t)

      print *, "m_wage_new", like_target_in % m_wage_new1(t), like_target_in % m_wage_new2(t)
      print *, "wage_new", like_target_in % wage_new1(t), like_target_in % wage_new2(t)

   end if

   print *, " "
   print *, "m_wage_mean_nohetero", like_target_in % m_wage_mean_nohetero1, &
      like_target_in % m_wage_mean_nohetero2
   print *, "wage_mean_nohetero", like_target_in % wage_mean_nohetero1, &
      like_target_in % wage_mean_nohetero2


   print *, " "
   print *, "mygrid_t_invar"
   print *, " "
   call print_mygrid2( like_target_in % mygrid_t_invar1, like_target_in % mygrid_t_invar2, .true., .false. )

   print *, " "
   print *, "mygrid_t_var"
   print *, " "
   call print_mygrid2( like_target_in % mygrid_t_var1(t), like_target_in % mygrid_t_var2(t), .false., .true. )

   if (t .ne. 1 ) then
      print *, " "
      print *, "mygird_husband_accept"
      print *, " "
      call print_mygrid2( like_target_in % mygrid_husband_accept1(t), like_target_in % mygrid_husband_accept2(t), .false., .true. )

      print *, " "
      print *, "mygrid_wife_accept"
      print *, " "
      call print_mygrid2( like_target_in % mygrid_wife_accept1(t), like_target_in % mygrid_wife_accept2(t), .false., .true. )


      print *, " "
      print *, "mygrid_both_accept"
      print *, " "
      call print_mygrid2( like_target_in % mygrid_both_accept1(t), like_target_in % mygrid_both_accept2(t), .false., .true. )

      print *, " "


      print *, "utility params"
      print *, "cons_weight",    like_target_in % utility_params1( util_cons_weight,t ), &
         like_target_in % utility_params2( util_cons_weight,t )
      print *, "m_leisure_weight",   like_target_in % utility_params1( util_m_leisure_weight,t ), &
         like_target_in % utility_params2( util_m_leisure_weight,t )
      print *, "leisure_weight",     like_target_in % utility_params1( util_leisure_weight,t ), &
         like_target_in % utility_params2( util_leisure_weight,t )



      print *, " "
      print *, "utility_both_reject", like_target_in % utility_both_reject1(t), like_target_in % utility_both_reject2(t)
      print *, "utility_husband_accept", like_target_in % utility_husband_accept1(t), like_target_in % utility_husband_accept2(t)
      print *, "utility_wife_accept", like_target_in % utility_wife_accept1(t), like_target_in % utility_wife_accept2(t)
      print *, "utility_both_accept", like_target_in % utility_both_accept1(t), like_target_in % utility_both_accept2(t)

      print *, " "
      print *, "asset_choice_both_reject", like_target_in % asset_choice_both_reject1(t), &
         like_target_in % asset_choice_both_reject2(t)
      print *, "asset_choice_husband_accept", like_target_in % asset_choice_husband_accept1(t), &
         like_target_in % asset_choice_husband_accept2(t)
      print *, "asset_choice_wife_accept", like_target_in % asset_choice_wife_accept1(t), &
         like_target_in % asset_choice_wife_accept2(t)
      print *, "asset_choice_both_accept", like_target_in % asset_choice_both_accept1(t), &
         like_target_in % asset_choice_both_accept2(t)

      print *, "ln_asset_prob", like_target_in % ln_asset_prob1, like_target_in % ln_asset_prob2


      print *,  " "
      print *, "choice_prob", like_target_in % choice_prob1(t), like_target_in % choice_prob2(t)
   end if !t .ne. 1

   if (t .ne. like_in % max_time ) then
      print *, " "
      print *, "!!!!!!!!!!!!!!!!!"
      !group of coefficients used for evaluating the jobloss event
      print *, "birth", like_in % birth(t)
      print *, "m_jobloss", like_in % m_jobloss(t) !whether loss a job
      print *, "jobloss", like_in % jobloss(t)

      print *, " "
  !    print *, "m_jobloss_prob", like_target_in % m_jobloss_prob1(t), like_target_in % m_jobloss_prob2(t)
  !    print *, "jobloss_prob", like_target_in % jobloss_prob1(t), like_target_in % jobloss_prob2(t)
   end if


end subroutine print_like_target




subroutine like_target_allocate (like_target_out, hh_data_in)
   !allocate the components of the like_target_type
   class(like_target_type), intent(out) :: like_target_out
   type(data_i_type), intent(in)  :: hh_data_in
   integer :: t,max_time

   like_target_out % target_allocated = .true.


   max_time = hh_data_in % max_time

   !allocate the targets
   allocate( like_target_out % m_joboffer_prob1(2:max_time) )
   allocate( like_target_out % m_joboffer_prob2(2:max_time) )
   allocate( like_target_out % joboffer_prob1(2:max_time) )
   allocate( like_target_out % joboffer_prob2(2:max_time) )


   allocate( like_target_out % m_hours_prob1(2:max_time) )
   allocate( like_target_out % m_hours_prob2(2:max_time) )
   allocate( like_target_out % hours_prob1(2:max_time) )
   allocate( like_target_out % hours_prob2(2:max_time) )

   allocate( like_target_out % m_ln_wage_prob1(2:max_time) )
   allocate( like_target_out % m_ln_wage_prob2(2:max_time) )
   allocate( like_target_out % ln_wage_prob1(2:max_time) )
   allocate( like_target_out % ln_wage_prob2(2:max_time) )


   allocate( like_target_out % m_wage_new1(2:max_time) )
   allocate( like_target_out % m_wage_new2(2:max_time) )
   allocate( like_target_out % wage_new1(2:max_time) )
   allocate( like_target_out % wage_new2(2:max_time) )


   like_target_out % m_ln_wage_prob1(2:max_time) = 0
   like_target_out % m_ln_wage_prob2(2:max_time) = 0
   like_target_out % ln_wage_prob1(2:max_time) = 0
   like_target_out % ln_wage_prob2(2:max_time) = 0


   allocate( like_target_out % m_receive_new1(2:max_time) )
   allocate( like_target_out % m_receive_new2(2:max_time) )
   allocate( like_target_out % receive_new1(2:max_time) )
   allocate( like_target_out % receive_new2(2:max_time) )

   allocate( like_target_out % mygrid_t_var1(max_time) )
   allocate( like_target_out % mygrid_t_var2(max_time) )
   allocate( like_target_out % mygrid_husband_accept1(2:max_time) )
   allocate( like_target_out % mygrid_husband_accept2(2:max_time) )
   allocate( like_target_out % mygrid_wife_accept1(2:max_time) )
   allocate( like_target_out % mygrid_wife_accept2(2:max_time) )
   allocate( like_target_out % mygrid_both_accept1(2:max_time) )
   allocate( like_target_out % mygrid_both_accept2(2:max_time) )

   allocate( like_target_out % utility_params_created1(2:max_time) )
   allocate( like_target_out % utility_params_created2(2:max_time) )
   allocate( like_target_out % utility_params1(n_utility_params,2:max_time) )
   allocate( like_target_out % utility_params2(n_utility_params,2:max_time) )
   allocate( like_target_out % utility_both_reject1(2:max_time) )
   allocate( like_target_out % utility_both_reject2(2:max_time) )
   allocate( like_target_out % utility_husband_accept1(2:max_time) )
   allocate( like_target_out % utility_husband_accept2(2:max_time) )
   allocate( like_target_out % utility_wife_accept1(2:max_time) )
   allocate( like_target_out % utility_wife_accept2(2:max_time) )
   allocate( like_target_out % utility_both_accept1(2:max_time) )
   allocate( like_target_out % utility_both_accept2(2:max_time) )


   allocate( like_target_out % asset_choice_both_reject1(2:max_time) )
   allocate( like_target_out % asset_choice_both_reject2(2:max_time) )
   allocate( like_target_out % asset_choice_husband_accept1(2:max_time) )
   allocate( like_target_out % asset_choice_husband_accept2(2:max_time) )
   allocate( like_target_out % asset_choice_wife_accept1(2:max_time) )
   allocate( like_target_out % asset_choice_wife_accept2(2:max_time) )
   allocate( like_target_out % asset_choice_both_accept1(2:max_time) )
   allocate( like_target_out % asset_choice_both_accept2(2:max_time) )



   allocate( like_target_out % choice_prob1(2:max_time) )
   allocate( like_target_out % choice_prob2(2:max_time) )

 !  allocate( like_target_out % m_jobloss_prob1(max_time-1) )
 !  allocate( like_target_out % m_jobloss_prob2(max_time-1) )
 !  allocate( like_target_out % jobloss_prob1(max_time-1) )
 !  allocate( like_target_out % jobloss_prob2(max_time-1) )


   !now allocate the mygrid objects
   call like_target_out % mygrid_t_invar1 % allocate()
   call like_target_out % mygrid_t_invar2 % allocate()

   call like_target_out % mygrid_t_var1(1) % t_var_allocate()
   call like_target_out % mygrid_t_var2(1) % t_var_allocate()


   do t=2,max_time
      call like_target_out % mygrid_t_var1(t) % t_var_allocate()
      call like_target_out % mygrid_t_var2(t) % t_var_allocate()

      call like_target_out % mygrid_husband_accept1(t) % t_var_allocate()
      call like_target_out % mygrid_husband_accept2(t) % t_var_allocate()

      call like_target_out % mygrid_wife_accept1(t) % t_var_allocate()
      call like_target_out % mygrid_wife_accept2(t) % t_var_allocate()

      call like_target_out % mygrid_both_accept1(t) % t_var_allocate()
      call like_target_out % mygrid_both_accept2(t) % t_var_allocate()
   end do


end subroutine like_target_allocate

subroutine like_init_pointer(like_object_out,like_target_in,hh_data_in)
   !initializes the pointers in the likelihood_object as well as allocates some of the components that aren't pointers

   class(like_type), intent(out) :: like_object_out
   type(like_target_type), intent(in), target :: like_target_in
   type(data_i_type), intent(in) :: hh_data_in
   integer :: max_time

   !making sure the targets have been allocated already
   if (.not. like_target_in % target_allocated) then
      print *, "like_target_in not allocated"
      stop
   end if

   like_object_out % pointer_initialized = .true.

   max_time = hh_data_in % max_time

   !allocate the integers vectors in like_object_out that aren't pointers
   allocate( like_object_out % m_accept_new(max_time) )
   allocate( like_object_out % accept_new(max_time) )

   allocate( like_object_out % m_time_job_ends(max_time) )
   allocate( like_object_out % time_job_ends(max_time) )

   allocate( like_object_out % birth(max_time-1) )
   allocate( like_object_out % m_jobloss(max_time) )
   allocate( like_object_out % jobloss(max_time) )

   allocate( like_object_out % m_ui_ever(max_time-1) )
   allocate( like_object_out % ui_ever(max_time-1) )
   allocate( like_object_out % m_time_unemp_end(max_time-1) )
   allocate( like_object_out % time_unemp_end(max_time-1) )

 !  allocate( like_object_out % utility_params_created(max_time) )


   !now initalize the pointer object
   like_object_out % hetero                  => like_target_in % hetero1

   like_object_out % m_joboffer_prob        => like_target_in % m_joboffer_prob1
   like_object_out % joboffer_prob          => like_target_in % joboffer_prob1

   like_object_out % m_hours_prob           => like_target_in % m_hours_prob1
   like_object_out % hours_prob             => like_target_in % hours_prob1


   like_object_out % m_wage_mean_nohetero    => like_target_in % m_wage_mean_nohetero1
   like_object_out % wage_mean_nohetero    => like_target_in % wage_mean_nohetero1

   like_object_out % m_ln_wage_prob            => like_target_in % m_ln_wage_prob1
   like_object_out % ln_wage_prob              => like_target_in % ln_wage_prob1
   like_object_out % m_wage_new                => like_target_in % m_wage_new1
   like_object_out % wage_new                  => like_target_in % wage_new1

   like_object_out % m_receive_new          => like_target_in % m_receive_new1
   like_object_out % receive_new            => like_target_in % receive_new1
   like_object_out % mygrid_t_invar         => like_target_in % mygrid_t_invar1
   like_object_out % mygrid_t_var           => like_target_in % mygrid_t_var1
   like_object_out % mygrid_husband_accept  => like_target_in % mygrid_husband_accept1
   like_object_out % mygrid_wife_accept     => like_target_in % mygrid_wife_accept1
   like_object_out % mygrid_both_accept     => like_target_in % mygrid_both_accept1

   like_object_out % utility_params_created => like_target_in % utility_params_created1
   like_object_out % utility_params         => like_target_in % utility_params1
   like_object_out % utility_both_reject    => like_target_in % utility_both_reject1
   like_object_out % utility_husband_accept => like_target_in % utility_husband_accept1
   like_object_out % utility_wife_accept    => like_target_in % utility_wife_accept1
   like_object_out % utility_both_accept    => like_target_in % utility_both_accept1

   like_object_out % asset_choice_both_reject    => like_target_in % asset_choice_both_reject1
   like_object_out % asset_choice_husband_accept => like_target_in % asset_choice_husband_accept1
   like_object_out % asset_choice_wife_accept    => like_target_in % asset_choice_wife_accept1
   like_object_out % asset_choice_both_accept    => like_target_in % asset_choice_both_accept1

   like_object_out % ln_asset_prob    => like_target_in % ln_asset_prob1

   like_object_out % choice_prob            => like_target_in % choice_prob1

 !  like_object_out % m_jobloss_prob         => like_target_in % m_jobloss_prob1
 !  like_object_out % jobloss_prob           => like_target_in % jobloss_prob1

end subroutine like_init_pointer




subroutine like_main_setup(like_object_inout, hh_data_in )
   !updates the joboffer probablities for when I change the parameters for this function
   !this subroutines needs to be called after the target has been setup and the pointers have been intialized

   class(like_type), intent(in out) :: like_object_inout
   type(data_i_type), intent(in)  :: hh_data_in

   real*8, parameter :: init_non_pec_val=0
 !  real*8, parameter :: init_non_pec_prob=inv_sqrt_2pi

   type( mygrid_type), pointer ::  mygrid_temp
   type( mygrid_type), pointer ::  mygrid_temp_lag

   integer :: m_job_preloss, job_preloss!whether a job was had before the jobloss event
   integer :: m_job_start, job_start
   integer :: i,t,max_time,t_prime
   logical :: non_pec_continue


  !making sure the targets have been allocated already
   if (.not. (like_object_inout % pointer_initialized) ) then
      print *, "pointer in like_object_inout haven't been made yet"
      stop
   end if

   !get max time
   max_time = hh_data_in % max_time
   like_object_inout % max_time = max_time

   !first assign the non job related variables and some others that are easy to assign
   do t = 1,max_time

      !initalize the pointer
      mygrid_temp => like_object_inout % mygrid_t_var(t)

      !copy non-job related data here data to mygrid here
      mygrid_temp % rv( var_ur_state ) = hh_data_in % data_t(t) % ur_state
      mygrid_temp % rv( var_uimax )    = hh_data_in % data_t(t) % uimax
 !    mygrid_temp % rv( var_uirate )   = hh_data_in % data_t(t) % uirate
      mygrid_temp % rv( var_kids ) = hh_data_in % data_t(t) % kids
      mygrid_temp % rv( var_youngest ) = hh_data_in % data_t(t) % youngest


      !get ui length here
      mygrid_temp % rv( var_uilength )   =  &
         ui_length_func(   hh_data_in % data_t(t) % ur_state, &
                           hh_data_in % data_t(t) % eb_opt_trig, &
                           hh_data_in % data_t(t) % euc_bill_1, &
                           hh_data_in % data_t(t) % euc_bill_2, &
                           hh_data_in % data_t(t) % euc_bill_3)

      !initialize assets
      if ( hh_data_in % liquid_time1 .le. 0 ) then
         mygrid_temp % rv( var_assets ) = asset_median
      else
         mygrid_temp % rv( var_assets ) = hh_data_in % liquid_net1
      end if

      !incorporate bounds
      do i=var_ur_state, var_assets
         mygrid_temp % rv(i) = bounds_func( mygrid_temp % rv(i), grid_min(i), grid_max(i) )
      end do

      !now duplicate these values to the wife,husband,and both accept mygrids
      !first husband accepting
      !note: here, the ui variables are set to 0 already, so I don't need to worry about copy ui benefits that I don't need
      if( t .ne. 1) then
         like_object_inout % mygrid_husband_accept(t)   = mygrid_temp
         like_object_inout % mygrid_wife_accept(t)      = mygrid_temp
         like_object_inout % mygrid_both_accept(t)      = mygrid_temp

         !here set the appropriate job indicatores equal to 1
         like_object_inout % mygrid_husband_accept(t) % iv(var_m_job)  = 1
         like_object_inout % mygrid_wife_accept(t) % iv(var_job)       = 1
         like_object_inout % mygrid_both_accept(t) % iv(var_m_job)     = 1
         like_object_inout % mygrid_both_accept(t) % iv(var_job)       = 1
      end if

   end do !t

   !now get the job-related variables for the initial time period
   t=1

   !set up pointers
   mygrid_temp => like_object_inout % mygrid_t_var(t)


   !set accept and receive new to zero
   like_object_inout % m_accept_new(t) = 0
 !  like_object_inout % m_receive_new(t) = 0
   like_object_inout % accept_new(t) = 0
 !  like_object_inout % receive_new(t) = 0


   if (hh_data_in % data_t(t) % m_job_no .ne. 0 ) then !husband has job
      mygrid_temp % iv( var_m_job )      = 1

      mygrid_temp % iv( var_m_hours )    = hh_data_in % data_t(t) % m_hours
      mygrid_temp % rv( var_m_wage )         = hh_data_in % data_t(t) % m_wage
      mygrid_temp % rv( var_m_non_pec_norm ) = init_non_pec_val

      mygrid_temp % rv( var_m_job_length)    = hh_data_in % m_job_length_start

   else !husband doesn't have a job
   !   mygrid_temp % iv( var_m_job )      = 0
      mygrid_temp % rv( var_m_ui_ben )       = min(   dble(hh_data_in % m_ui_ben_init), &
                                                      mygrid_temp % rv( var_uimax ) )
      like_object_inout % m_ui_ever(t) =  hh_data_in % data_t(t) % m_ui_ever
      mygrid_temp % rv( var_m_unemp_length)    = &
         min ( ( hh_data_in % m_min_unemp_length + hh_data_in % m_max_unemp_length   )/2 , &
               max_job_length)



   end if

   if (hh_data_in % data_t(t) % job_no .ne. 0 ) then !wife has job
      mygrid_temp % iv( var_job )      = 1

      mygrid_temp % iv( var_hours )    = hh_data_in % data_t(t) % hours
      mygrid_temp % rv( var_wage )         = hh_data_in % data_t(t) % wage
      mygrid_temp % rv( var_non_pec_norm ) = init_non_pec_val

      mygrid_temp % rv( var_job_length)    = hh_data_in % job_length_start

   else !wife doesn't have a job
      mygrid_temp % rv( var_ui_ben )       = min(   dble(hh_data_in % ui_ben_init), &
                                                      mygrid_temp % rv( var_uimax ) )
      like_object_inout % ui_ever(t) =  hh_data_in % data_t(t) % ui_ever
      mygrid_temp % rv( var_unemp_length)    = &
         min( ( hh_data_in % min_unemp_length + hh_data_in % max_unemp_length   )/2, &
               max_job_length)
   end if





   !now assign the job variables for the other time periods
   do t = 2,max_time


      !do some pointer stuff here
      mygrid_temp       => like_object_inout % mygrid_t_var(t)
      mygrid_temp_lag   => like_object_inout % mygrid_t_var(t-1)



      !first figure out whether had a job at the end of the previous joboffer period (i.e. had a job right before the jobloss event)
      if ( ( mygrid_temp_lag % iv( var_m_job ) .eq. 1 ) .or. (like_object_inout % m_accept_new(t-1) .eq. 1) ) then
         m_job_preloss = 1
      else
         m_job_preloss = 0
      end if

      if ( ( mygrid_temp_lag % iv( var_job ) .eq. 1) .or. (like_object_inout % accept_new(t-1) .eq. 1) ) then
         job_preloss = 1
      else
         job_preloss = 0
      end if

      !assign the jobloss integers and ui integers
      like_object_inout % m_jobloss(t-1)  = hh_data_in % data_t(t-1) % m_jobloss
      like_object_inout % jobloss(t-1)    = hh_data_in % data_t(t-1) % jobloss

      !assign the ui ever integers to the likelihood object
      if (like_object_inout % m_jobloss(t-1) .eq. 1 ) then
         like_object_inout % m_ui_ever(t-1) = hh_data_in % data_t(t) % m_ui_ever
      end if

      if (like_object_inout % jobloss(t-1) .eq. 1 ) then
         like_object_inout % ui_ever(t-1) = hh_data_in % data_t(t) % ui_ever
      end if


      !now whether a birth happened
      if ( (mygrid_temp % rv( var_kids ) > mygrid_temp_lag % rv( var_kids )) .and. &
         (mygrid_temp % rv( var_youngest ) < 2) ) then
         like_object_inout % birth(t-1) = 1
      else
         like_object_inout % birth(t-1) = 0
      end if

      !now get the state variables for the following period
      !first the husband
      if (m_job_preloss .eq. 0) then !didn't have job last period
         mygrid_temp % rv( var_m_ui_ben )    = mygrid_temp_lag % rv( var_m_ui_ben )

      else if ( like_object_inout % m_jobloss(t-1) .eq. 1) then !just lost a job
          mygrid_temp % rv( var_m_ui_ben ) = m_ui_ben_func( mygrid_temp_lag, &
                                                like_object_inout % m_ui_ever(t-1) )

      else !kept job, so the job state variables for period t are the job variables from the data for time period t-1
         !the data represents the variables held at the end of period t
         mygrid_temp % iv( var_m_hours )    = hh_data_in % data_t(t-1) % m_hours
         mygrid_temp % rv( var_m_wage )         = hh_data_in % data_t(t-1) % m_wage
         mygrid_temp % rv( var_m_non_pec_norm ) = init_non_pec_val
         mygrid_temp % iv( var_m_job )      = 1
      end if

      !now the wife
      if (job_preloss .eq. 0) then !didn't have job last period
         mygrid_temp % rv( var_ui_ben )      = mygrid_temp_lag % rv( var_ui_ben )

      else if ( like_object_inout % jobloss(t-1) .eq. 1) then !just lost a job
          mygrid_temp % rv( var_ui_ben ) = ui_ben_func( mygrid_temp_lag, &
                                                like_object_inout % ui_ever(t-1) )

      else !kept job, so the job state variables for period t are the job variables from the data for time period t-1
         !the data represents the variables held at the end of period t
         mygrid_temp % iv( var_hours )     = hh_data_in % data_t(t-1) % hours
         mygrid_temp % rv( var_wage )          = hh_data_in % data_t(t-1) % wage
         mygrid_temp % rv( var_non_pec_norm )  = init_non_pec_val
         mygrid_temp % iv( var_job )       = 1
      end if


      !now increment the lenght of employment or unemployment
      if( mygrid_temp % iv( var_m_job ) .eq. 1 ) then
         if (like_object_inout % m_accept_new(t-1) .eq. 1 ) then
            mygrid_temp % rv( var_m_job_length ) = 1
         else
            mygrid_temp % rv( var_m_job_length ) = &
               min(mygrid_temp_lag % rv( var_m_job_length ) + 1, dble(max_job_length) )
         end if
      else
         mygrid_temp % rv( var_m_unemp_length ) = &
            min( mygrid_temp_lag % rv( var_m_unemp_length ) + 1, dble(max_job_length) )
      end if

      if( mygrid_temp % iv( var_job ) .eq. 1 ) then
         if (like_object_inout % accept_new(t-1) .eq. 1 ) then
            mygrid_temp % rv( var_job_length ) = 1
         else
            mygrid_temp % rv( var_job_length ) = &
               min( mygrid_temp_lag % rv( var_job_length ) + 1, dble(max_job_length) )
         end if
      else
         mygrid_temp % rv( var_unemp_length ) = &
            min( mygrid_temp_lag % rv( var_unemp_length ) + 1, dble(max_job_length) )
      end if



      !now determine if a new job was started
      if (  ( hh_data_in % data_t(t) % m_job_no .ne. hh_data_in % data_t(t-1) % m_job_no) .and. &
            ( hh_data_in % data_t(t) % m_job_no .ne. 0 ) )  then! last condition to ensure employed and not a job loss
         like_object_inout % m_accept_new(t) = 1
         like_object_inout % m_receive_new(t) = 1
      else
         like_object_inout % m_accept_new(t) = 0
         like_object_inout % m_receive_new(t) = 0
      end if

      if (  ( hh_data_in % data_t(t) % job_no .ne. hh_data_in % data_t(t-1) % job_no) .and. &
            ( hh_data_in % data_t(t) % job_no .ne. 0 ) )  then!last condition to ensure employed and not a job loss
         like_object_inout % accept_new(t) = 1
         like_object_inout % receive_new(t) = 1
      else
         like_object_inout % accept_new(t) = 0
         like_object_inout % receive_new(t) = 0
      end if

      !initalize the jobvars in the husband_accept, wife_accept, and both accept grid
      !ealier, I've copied the non-job time-varying data (e.g. ur_rate) to these grids

      !copy the wife's current job variable to the mygrid_husband_accept grid
      like_object_inout % mygrid_husband_accept(t) % iv( var_job )       = mygrid_temp % iv( var_job )
      like_object_inout % mygrid_husband_accept(t) % rv( var_wage )          = mygrid_temp % rv( var_wage )
      like_object_inout % mygrid_husband_accept(t) % iv( var_hours )     = mygrid_temp % iv( var_hours )
      like_object_inout % mygrid_husband_accept(t) % rv( var_non_pec_norm )  = mygrid_temp % rv( var_non_pec_norm )
      like_object_inout % mygrid_husband_accept(t) % rv( var_ui_ben )        = mygrid_temp % rv( var_ui_ben )

      like_object_inout % mygrid_husband_accept(t) % rv( var_job_length )    = mygrid_temp % rv( var_job_length )
      like_object_inout % mygrid_husband_accept(t) % rv( var_unemp_length )  = mygrid_temp % rv( var_unemp_length )

      !copy the husband's current job variable to the mygrid_wife_accept grid
      like_object_inout % mygrid_wife_accept(t) % iv( var_m_job )       = mygrid_temp % iv( var_m_job )
      like_object_inout % mygrid_wife_accept(t) % rv( var_m_wage )          = mygrid_temp % rv( var_m_wage )
      like_object_inout % mygrid_wife_accept(t) % iv( var_m_hours )     = mygrid_temp % iv( var_m_hours )
      like_object_inout % mygrid_wife_accept(t) % rv( var_m_non_pec_norm )  = mygrid_temp % rv( var_m_non_pec_norm )
      like_object_inout % mygrid_wife_accept(t) % rv( var_m_ui_ben )        = mygrid_temp % rv( var_m_ui_ben )

      like_object_inout % mygrid_wife_accept(t) % rv( var_m_job_length )      = mygrid_temp % rv( var_m_job_length )
      like_object_inout % mygrid_wife_accept(t) % rv( var_m_unemp_length )    = mygrid_temp % rv( var_m_unemp_length )

      !for if the husband gets a new job, initialize with this new job data
      if (like_object_inout % m_accept_new(t) .eq. 1) then
         like_object_inout % mygrid_husband_accept(t) % rv( var_m_wage )           = hh_data_in % data_t(t) % m_wage
         like_object_inout % mygrid_husband_accept(t) % iv( var_m_hours )      = hh_data_in % data_t(t) % m_hours
         like_object_inout % mygrid_husband_accept(t) % rv( var_m_non_pec_norm )   = init_non_pec_val
     !    like_object_inout % m_non_pec_prob(t) = init_non_pec_prob

         like_object_inout % mygrid_both_accept(t) % rv( var_m_wage )           = hh_data_in % data_t(t) % m_wage
         like_object_inout % mygrid_both_accept(t) % iv( var_m_hours )      = hh_data_in % data_t(t) % m_hours
         like_object_inout % mygrid_both_accept(t) % rv( var_m_non_pec_norm )   = init_non_pec_val

      end if


      !for if the wife starts a new job, initialize with this new job data
      if (like_object_inout % accept_new(t) .eq. 1)  then
         like_object_inout % mygrid_wife_accept(t) % rv( var_wage )          = hh_data_in % data_t(t) % wage
         like_object_inout % mygrid_wife_accept(t) % iv( var_hours )     = hh_data_in % data_t(t) % hours
         like_object_inout % mygrid_wife_accept(t) % rv( var_non_pec_norm )  = init_non_pec_val
      !   like_object_inout % non_pec_prob(t) = init_non_pec_prob

         like_object_inout % mygrid_both_accept(t) % rv( var_wage )          = hh_data_in % data_t(t) % wage
         like_object_inout % mygrid_both_accept(t) % iv( var_hours )     = hh_data_in % data_t(t) % hours
         like_object_inout % mygrid_both_accept(t) % rv( var_non_pec_norm )  = init_non_pec_val

      end if



   end do !over t



   !now for figuring out the last time period an newly accepted job was held
   m_job_start = like_object_inout % mygrid_t_var(1) % iv( var_m_job )
   job_start = like_object_inout % mygrid_t_var(1) % iv( var_job )

   do t=1,max_time
      if (  (like_object_inout % m_accept_new(t) .eq. 1) .or. &
            ( (t .eq. 1) .and. (m_job_start .eq. 1 ) )      )then

         !this if determines if the job is held in subsequent time periods, which is the case if (1) this isn't the last period or (2)the husband doesn't instanly loose his job
         if ( ( t .eq. max_time ) .or. ( like_object_inout % m_jobloss(t) .eq. 1 ) ) then
            non_pec_continue = .false.
            like_object_inout % m_time_job_ends(t) = t
         else
            non_pec_continue = .true.
            t_prime = t +1
         end if

         do while ( non_pec_continue )
            !have reached the last time period for this job if (1) a new job was accepted, (2) the job was lost this period, of (3) this is the last time period
            if (  ( like_object_inout % m_accept_new(t_prime)  .eq. 1 ) .or. &
                  ( like_object_inout % m_jobloss(t_prime)     .eq. 1 ) .or. &
                  ( t_prime                                    .eq. max_time ) ) then

               like_object_inout % m_time_job_ends(t) = t_prime
               non_pec_continue = .false.
            else
               t_prime = t_prime + 1
            end if
         end do

      end if !for the end date fo the husband's job

      !now do the same thing for the wife
      if (  (like_object_inout % accept_new(t) .eq. 1 ) .or. &
            ( (t .eq. 1) .and. (job_start .eq. 1 ) )  ) then

         !this if determines if the job is held in subsequent time periods, which is the case if (1) this isn't the last period or (2)the husband doesn't instanly loose his job
         if ( ( t .eq. max_time ) .or. ( like_object_inout % jobloss(t) .eq. 1 ) ) then
            non_pec_continue = .false.
            like_object_inout % time_job_ends(t) = t
         else
            non_pec_continue = .true.
            t_prime = t +1
         end if

         do while ( non_pec_continue )
            !have reached the last time period for this job if (1) a new job was accepted, (2) the job was lost this period, of (3) this is the last time period
            if (  ( like_object_inout % accept_new(t_prime)  .eq. 1 ) .or. &
                  ( like_object_inout % jobloss(t_prime)     .eq. 1 ) .or. &
                  ( t_prime                                  .eq. max_time ) ) then

               like_object_inout % time_job_ends(t) = t_prime
               non_pec_continue = .false.
            else
               t_prime = t_prime + 1
            end if
         end do

      end if !for the end date of the husband's job
   end do !over t


   !now for figuring out when an unemployment spell ends
   do t=1,max_time-1

      if (  ( like_object_inout % m_jobloss(t) .eq. 1 ) .or. &
            ( (t .eq. 1) .and. (m_job_start .eq. 0 ) ) ) then
         t_prime = t+1

         do while ( ( t_prime .ne. max_time) .and. (like_object_inout % m_accept_new(t_prime) .ne. 1) )
            t_prime = t_prime + 1
         end do

         like_object_inout % m_time_unemp_end(t) = t_prime

      end if


      if (  ( like_object_inout % jobloss(t) .eq. 1 ) .or. &
            ( (t .eq. 1) .and. (job_start .eq. 0 ) ) ) then
         t_prime = t+1

         do while ( ( t_prime .ne. max_time) .and. (like_object_inout % accept_new(t_prime) .ne. 1) )
            t_prime = t_prime + 1
         end do

         like_object_inout % time_unemp_end(t) = t_prime

      end if

   end do



   !now for setting ui equal to 0 if household has exceded the maximum duration of benefits
   do t=1, max_time
         if (  nint(like_object_inout % mygrid_t_var(t) % rv(var_m_unemp_length) ) > &
               nint(like_object_inout % mygrid_t_var(t) % rv(var_uilength) ) ) then

            like_object_inout % mygrid_t_var(t) % rv(var_m_ui_ben) = 0
            if (t .ne. 1) like_object_inout % mygrid_wife_accept(t) % rv(var_m_ui_ben) = 0

         end if


         if (  nint(like_object_inout % mygrid_t_var(t) % rv(var_unemp_length) ) > &
               nint(like_object_inout % mygrid_t_var(t) % rv(var_uilength) ) ) then

            like_object_inout % mygrid_t_var(t) % rv(var_ui_ben) = 0
            if( t .ne. 1) like_object_inout % mygrid_husband_accept(t) % rv(var_ui_ben) = 0

         end if

   end do

   !deal with some of the unknown unemployment lenght variables here
   if(  &
         ( ( like_object_inout % mygrid_t_var(1) % iv(var_m_job) .eq. 0 ) .and. &
         (hh_data_in % m_min_unemp_length < max_job_length) ) &
         .or. &
         ( ( like_object_inout % mygrid_t_var(1) % iv(var_job) .eq. 0 ) .and. &
         (hh_data_in % min_unemp_length < max_job_length) ) &
         ) then


      like_object_inout % unknown_unemp_length = 1

   else
      like_object_inout % unknown_unemp_length = 0
   end if

   !incorporating bounds of jobs-specific variables
  ! mygrid_t_var
   do t=1, like_object_inout % max_time
      do i= var_m_wage, var_unemp_length
         if( var_present(i,   like_object_inout % mygrid_t_var(t) % iv(var_m_job), &
                              like_object_inout % mygrid_t_var(t) % iv(var_job) ) .eq. 1) then

            like_object_inout % mygrid_t_var(t) % rv(i) = bounds_func( &
               like_object_inout % mygrid_t_var(t) % rv(i), grid_min(i), grid_max(i) )
         end if

      end do
   end do

  ! mygrid_husband_accept
   do t=2, like_object_inout % max_time
      if ( like_object_inout % m_receive_new(t) .eq. 1 ) then
         do i= var_m_wage, var_unemp_length
            if( var_present(i,   like_object_inout % mygrid_husband_accept(t) % iv(var_m_job), &
                                 like_object_inout % mygrid_husband_accept(t) % iv(var_job) ) .eq. 1 ) then

               like_object_inout % mygrid_husband_accept(t) % rv(i) = bounds_func( &
                  like_object_inout % mygrid_husband_accept(t) % rv(i), grid_min(i), grid_max(i) )
            end if
         end do
      end if
   end do

  ! mygrid_wife_accept
   do t=2, like_object_inout % max_time
      if ( like_object_inout % receive_new(t) .eq. 1 ) then
         do i= var_m_wage, var_unemp_length
            if( var_present(i,   like_object_inout % mygrid_wife_accept(t) % iv(var_m_job), &
                                 like_object_inout % mygrid_wife_accept(t) % iv(var_job) ) .eq. 1 ) then

               like_object_inout % mygrid_wife_accept(t) % rv(i) = bounds_func( &
                  like_object_inout % mygrid_wife_accept(t) % rv(i), grid_min(i), grid_max(i) )
            end if
         end do
      end if
   end do

  ! mygrid_both_accept
   do t=2, like_object_inout % max_time
      if (   ( like_object_inout % m_receive_new(t) .eq. 1 ) .and. &
            ( like_object_inout % receive_new(t) .eq. 1 )  ) then
         do i= var_m_wage, var_unemp_length
            if( var_present(i,   like_object_inout % mygrid_both_accept(t) % iv(var_m_job), &
                                 like_object_inout % mygrid_both_accept(t) % iv(var_job) ) .eq. 1 ) then

               like_object_inout % mygrid_both_accept(t) % rv(i) = bounds_func( &
                  like_object_inout % mygrid_both_accept(t) % rv(i), grid_min(i), grid_max(i) )
            end if
         end do
      end if
   end do

   !givin incorporated bounds, now have the wage new variables equal the mygrid counterparts
   do t=2, like_object_inout % max_time

      if ( like_object_inout % m_receive_new(t) .eq. 1 ) then
         like_object_inout % m_wage_new(t) = like_object_inout % mygrid_husband_accept(t) % rv( var_m_wage )
      end if

      if  ( like_object_inout % receive_new(t) .eq. 1 )  then
         like_object_inout % wage_new(t)   = like_object_inout % mygrid_wife_accept(t) % rv( var_wage )
      end if

   end do


   !initialize asset choice to be equal to the state value ( useful for turning off the saving part of my model )
     ! print *, "both reject"
   do t = 2, like_object_inout % max_time

      like_object_inout % asset_choice_both_reject(t) = &
         like_object_inout % mygrid_t_var(t) % rv(var_assets )

      !  print *, "husband accept"
      like_object_inout % asset_choice_husband_accept(t) = &
         like_object_inout % mygrid_husband_accept(t) % rv(var_assets)

      ! print *, "wife accept"
      like_object_inout % asset_choice_wife_accept(t) = &
         like_object_inout % mygrid_wife_accept(t) % rv(var_assets)


      !  print *, "both accept"
      like_object_inout % asset_choice_both_accept(t) = &
         like_object_inout % mygrid_both_accept(t) % rv(var_assets )

   end do

   !also, initialzie the asset error prob to 0 in case I'm not dealing with assets
   like_object_inout % ln_asset_prob = 0



end subroutine like_main_setup


!subroutine get_birth_ui_means(like_object_inout, hh_data_in, params_in )
!   !gets the birth and ui menas, and then store them on the vector of state variables
!      !I only have to call this once, and I don't need the birth and ui probabilities because I don't need them to
!      !construct the likelihood function

!   class(like_type), intent(in out) :: like_object_inout
!   type(data_i_type), intent(in) :: hh_data_in
!   real*8, dimension(n_params), intent(in) :: params_in


!   like_object_inout % mygrid_t_invar % rv(var_m_ui_mean) = &
!      params_in(param_ui_cons)  + &
!      params_in(param_ui_age)          * hh_data_in % m_age + &
!      params_in(param_ui_age2)         * hh_data_in % m_age * hh_data_in % m_age + &
!      params_in(param_ui_race)         * hh_data_in % m_white + &
!      params_in(param_ui_hs)           * hh_data_in % m_hs + &
!      params_in(param_ui_some_college) * hh_data_in % m_some_college + &
!      params_in(param_ui_college)      * hh_data_in % m_college

!   like_object_inout % mygrid_t_invar % rv(var_ui_mean) = &
!      params_in(param_ui_cons)  + &
!      params_in(param_ui_age)          * hh_data_in % age + &
!      params_in(param_ui_age2)         * hh_data_in % age * hh_data_in % age + &
!      params_in(param_ui_race)         * hh_data_in % white + &
!      params_in(param_ui_hs)           * hh_data_in % hs + &
!      params_in(param_ui_some_college) * hh_data_in % some_college + &
!      params_in(param_ui_college)      * hh_data_in % college + &
!      params_in(param_ui_gender)

!   like_object_inout % mygrid_t_invar % rv(var_birth_mean) = &
!      params_in(param_birth_cons)  + &
!      params_in(param_birth_age)          * hh_data_in % age + &
!      params_in(param_birth_age2)         * hh_data_in % age * hh_data_in % age


!end subroutine get_birth_ui_means


subroutine copy_targets_init(like_target_inout)
   !copy's some of the targets after other initializations
   class(like_target_type), intent(in out) :: like_target_inout

   !copy the mygrid targets, do this such that I don't have to copy the unemployment or kids data again
   like_target_inout % mygrid_t_invar2         = like_target_inout % mygrid_t_invar1
   like_target_inout % mygrid_t_var2           = like_target_inout % mygrid_t_var1
   like_target_inout % mygrid_husband_accept2  = like_target_inout % mygrid_husband_accept1
   like_target_inout % mygrid_wife_accept2     = like_target_inout % mygrid_wife_accept1
   like_target_inout % mygrid_both_accept2     = like_target_inout % mygrid_both_accept1

   !also copy the receive job indicators for spouses who start a new job
   like_target_inout % m_receive_new2           = like_target_inout % m_receive_new1
   like_target_inout % receive_new2             = like_target_inout % receive_new1

   !and also the asset error prob and asset_choice
   like_target_inout % ln_asset_prob2                = like_target_inout % ln_asset_prob1
   like_target_inout % asset_choice_both_reject2     = like_target_inout % asset_choice_both_reject1
   like_target_inout % asset_choice_husband_accept2  = like_target_inout % asset_choice_husband_accept1
   like_target_inout % asset_choice_wife_accept2     = like_target_inout % asset_choice_wife_accept1
   like_target_inout % asset_choice_both_accept2     = like_target_inout % asset_choice_both_accept1


end subroutine copy_targets_init



subroutine joboffer_like_update(like_object_inout, hh_data_in, params_in )
   !updates the joboffer probablities for when I change the parameters for this function

   class(like_type), intent(in out) :: like_object_inout
   type(data_i_type), intent(in) :: hh_data_in
   real*8, dimension(n_params), intent(in) :: params_in

   real*8 :: m_joboffer_xbeta_temp, joboffer_xbeta_temp

   !various pointers

   integer :: t


!   !update the joboffer means
!   like_object_inout % mygrid_t_invar % rv(var_m_joboffer_mean) = &
!      params_in(param_joboffer_cons)  + &
!      params_in(param_joboffer_seam)*.25
!  !    params_in(param_joboffer_age)          * hh_data_in % m_age + &
!  !    params_in(param_joboffer_age2)         * hh_data_in % m_age * hh_data_in % m_age + &
!  !    params_in(param_joboffer_race)         * hh_data_in % m_white + &
!  !    params_in(param_joboffer_hs)           * hh_data_in % m_hs + &
!  !    params_in(param_joboffer_some_college) * hh_data_in % m_some_college + &
!  !    params_in(param_joboffer_college)      * hh_data_in % m_college

!   like_object_inout % mygrid_t_invar % rv(var_joboffer_mean) = &
!      params_in(param_joboffer_cons)  + &
!      params_in(param_joboffer_seam)*.25  + &
!      params_in(param_joboffer_gender)
! !     params_in(param_joboffer_age)          * hh_data_in % age + &
! !     params_in(param_joboffer_age2)         * hh_data_in % age * hh_data_in % age + &
! !     params_in(param_joboffer_race)         * hh_data_in % white + &
! !     params_in(param_joboffer_hs)           * hh_data_in % hs + &
! !     params_in(param_joboffer_some_college) * hh_data_in % some_college + &
! !     params_in(param_joboffer_college)      * hh_data_in % college + &



   !now go through all the time periods
   do t = 2, like_object_inout %  max_time
   !   print *, "t", t


      !now create the time-specific inputs for the joboffer probablities

      !first do husband
      m_joboffer_xbeta_temp = &
         m_joboffer_xb_like_func(   like_object_inout % mygrid_t_invar, &
                                    like_object_inout % mygrid_t_var(t), &
                                    params_in, &
                                    hh_data_in % data_t(t) % seam )


      like_object_inout % m_joboffer_prob(t) = &
         logit_prob( m_joboffer_xbeta_temp )



   !  print *, "m_joboffer_prob_temp", m_joboffer_prob_temp

      !now do wife
      joboffer_xbeta_temp = &
         joboffer_xb_like_func(  like_object_inout % mygrid_t_invar, &
                                 like_object_inout % mygrid_t_var(t), &
                                 params_in, &
                                 hh_data_in % data_t(t) % seam )

      !now get the probabilities
      like_object_inout % joboffer_prob(t) = &
         logit_prob( joboffer_xbeta_temp )

   ! print *, "joboffer_prob_temp", joboffer_prob_temp

   end do !t

end subroutine joboffer_like_update



subroutine job_probs_like_update( like_object_inout, hh_data_in, params_in )
   !updates the potential joboffer variables (i.e. the collect of hours, wage, and non-pec for the potential offers)
      !and get the probability for the new wages, if needed

   class(like_type), intent(in out) :: like_object_inout
   type(data_i_type), intent(in) :: hh_data_in
   real*8, dimension(n_params), intent(in) :: params_in

   real*8 :: m_wage_new, wage_new
   real*8 :: m_wage_xb, wage_xb
   integer :: m_hours_new, hours_new

   integer :: t



   !get wage means (not including hetero yet)
   like_object_inout % m_wage_mean_nohetero = &
      params_in(param_wage_cons)  + &
      params_in(param_wage_age)          * hh_data_in  % m_age + &
      params_in(param_wage_age2)         * hh_data_in  % m_age * hh_data_in  % m_age + &
      params_in(param_wage_race)         * hh_data_in  % m_white + &
      params_in(param_wage_hs)           * hh_data_in  % m_hs + &
      params_in(param_wage_some_college) * hh_data_in  % m_some_college + &
      params_in(param_wage_college)      * hh_data_in  % m_college




   like_object_inout % wage_mean_nohetero = &
      params_in(param_wage_cons)  + &
      params_in(param_wage_age)          * hh_data_in  % age + &
      params_in(param_wage_age2)         * hh_data_in  % age * hh_data_in  % age + &
      params_in(param_wage_race)         * hh_data_in  % white + &
      params_in(param_wage_hs)           * hh_data_in  % hs + &
      params_in(param_wage_some_college) * hh_data_in  % some_college + &
      params_in(param_wage_college)      * hh_data_in  % college + &
      params_in(param_wage_gender)

   !now add the heterogeneity
   like_object_inout % mygrid_t_invar % rv(var_m_wage_mean) = &
      like_object_inout % m_wage_mean_nohetero + &
      like_object_inout % hetero(hetero_m_wage)

   like_object_inout % mygrid_t_invar % rv(var_wage_mean) = &
      like_object_inout % wage_mean_nohetero + &
      like_object_inout % hetero(hetero_wage)

   !incorporate bounds
   like_object_inout % mygrid_t_invar % rv(var_m_wage_mean)  = bounds_func( &
      like_object_inout % mygrid_t_invar % rv(var_m_wage_mean), &
      grid_min( var_m_wage_mean ), grid_max( var_m_wage_mean) )

   like_object_inout % mygrid_t_invar % rv(var_wage_mean)  = bounds_func( &
      like_object_inout % mygrid_t_invar % rv(var_wage_mean), &
      grid_min( var_wage_mean ), grid_max( var_wage_mean) )

   do t=2, like_object_inout %  max_time


      !now get the "probabilities" for the receive wage and non_pec_benefit, but only have to do this
         !if a job offer was received, which is determined if they accept a new offer or from the potentail
         !joboffers generated from the McMC chain

      !first husband
      if (like_object_inout % m_receive_new(t) .eq. 1) then
         m_wage_new = like_object_inout % m_wage_new(t)
         m_hours_new = like_object_inout % mygrid_husband_accept(t) % iv( var_m_hours )

         !wage probability
         m_wage_xb = &
            m_wage_xb_func(   like_object_inout % mygrid_t_invar, &
                              like_object_inout % mygrid_t_var(t), &
                              params_in, &
                              m_hours_new )

         like_object_inout % m_ln_wage_prob(t) = &
            m_ln_wage_prob_func( m_wage_new, &
                                 m_wage_xb, &
                                 params_in )

         !hours probability
         if ( m_hours_new .eq. 20) then
            like_object_inout % m_hours_prob(t) = params_in(param_prob_pt)
         else
            like_object_inout % m_hours_prob(t) = 1 - params_in(param_prob_pt)
         end if


      else
         like_object_inout % m_ln_wage_prob(t) = 0
         like_object_inout % m_hours_prob(t) = 1
      end if


      !now the wife
      if (like_object_inout % receive_new(t) .eq. 1) then
         !some pointing to make code easier to read
         wage_new = like_object_inout % wage_new(t)
         hours_new = like_object_inout % mygrid_wife_accept(t) % iv( var_hours )

         !wage probability
         wage_xb = &
            wage_xb_func(  like_object_inout % mygrid_t_invar, &
                           like_object_inout % mygrid_t_var(t), &
                           params_in, &
                           hours_new )

         like_object_inout % ln_wage_prob(t) = &
            ln_wage_prob_func(   wage_new, &
                                 wage_xb, &
                                 params_in )


         !hours probability
         if ( hours_new .eq. 20) then
            like_object_inout % hours_prob(t) = params_in(param_prob_pt)
         else
            like_object_inout % hours_prob(t) = 1 - params_in(param_prob_pt)
         end if
      else
         like_object_inout % ln_wage_prob(t) = 0
         like_object_inout % hours_prob(t) = 1
      end if

   end do !over t


end subroutine job_probs_like_update


subroutine asset_choice_like_update_one_t( like_object_inout, asset_matrix_in, t)
   !updates the asset choice
   class(like_type), intent(in out) :: like_object_inout
   real*8, dimension( n_grid_points ), intent(in) :: asset_matrix_in
   integer, intent(in) :: t


   ! print *, "both reject"
   like_object_inout % asset_choice_both_reject(t) = &
      asset_policy_func_split_in(   like_object_inout % mygrid_t_invar, &
                                    like_object_inout % mygrid_t_var(t), &
                                    asset_matrix_in )



   !  print *, "husband accept"
   if(like_object_inout % m_receive_new(t) .eq. 1)  then
      like_object_inout % asset_choice_husband_accept(t) = &
         asset_policy_func_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_husband_accept(t), &
                                       asset_matrix_in )
   end if

   ! print *, "wife accept"
   if(like_object_inout % receive_new(t) .eq. 1)  then
      like_object_inout % asset_choice_wife_accept(t) = &
         asset_policy_func_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_wife_accept(t), &
                                       asset_matrix_in )
   end if

   !  print *, "both accept"
   if (   (like_object_inout % m_receive_new(t) .eq. 1) .and. &
         (like_object_inout % receive_new(t) .eq. 1) ) then

      like_object_inout % asset_choice_both_accept(t) = &
         asset_policy_func_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_both_accept(t), &
                                       asset_matrix_in )
   end if



end subroutine asset_choice_like_update_one_t


subroutine asset_state_like_update_one_t( like_object_inout, params_in, t)
   !given an newly updated asset choice from t-1, this updates the state variable for period t
   class(like_type), intent(in out) :: like_object_inout
   real*8, dimension(n_params), intent(in) :: params_in
   integer, intent(in) :: t

   real*8 :: new_asset

   !first, figure out the new asset choice based on the accept indicators, expect for
   !t-1=1, the mygrid_t_var object has the drawn initial asset (before interest rates are incorpored)

   if( t .eq. 2 ) then
      new_asset = like_object_inout % mygrid_t_var(1) % rv( var_assets )


   else

      if (  ( like_object_inout % m_accept_new(t-1) .eq. 1 ) .and. &
            ( like_object_inout % accept_new(t-1) .eq. 1 ) ) then
         new_asset = like_object_inout % asset_choice_both_accept(t-1)

      else if ( like_object_inout % m_accept_new(t-1) .eq. 1 ) then
         new_asset = like_object_inout % asset_choice_husband_accept(t-1)

      else if ( like_object_inout % accept_new(t-1) .eq. 1 ) then
         new_asset = like_object_inout % asset_choice_wife_accept(t-1)

      else
         new_asset = like_object_inout % asset_choice_both_reject(t-1)

      end if

   end if

   !increase by the interest rate
   new_asset = new_asset * (1+ params_in( param_interest_rate ) )

   new_asset = bounds_func( new_asset, grid_min(var_assets), grid_max(var_assets) )

   !now assign to the mygrid object
   like_object_inout % mygrid_t_var(t) % rv( var_assets )            = new_asset
   like_object_inout % mygrid_husband_accept(t) % rv( var_assets )   = new_asset
   like_object_inout % mygrid_wife_accept(t) % rv( var_assets )      = new_asset
   like_object_inout % mygrid_both_accept(t) % rv( var_assets )      = new_asset


end subroutine asset_state_like_update_one_t


subroutine asset_choice_like_update( like_object_inout, asset_matrix_in, params_in )
   !updates the asset choices

   class(like_type), intent(in out) :: like_object_inout
   real*8, dimension(n_grid_points), intent(in) :: asset_matrix_in
   real*8, dimension(n_params), intent(in) :: params_in
   integer :: t
  ! print *, "within utility like update"

   do t=2, like_object_inout % max_time
    !  print *, "asset choice updating, time = ", t
      !update asset state variable 1st.  for t=2, the value is based on the initial draw asset variable
      call asset_state_like_update_one_t( like_object_inout, params_in, t)
      call asset_choice_like_update_one_t( like_object_inout, asset_matrix_in, t)

   end do !over t

end subroutine asset_choice_like_update




subroutine utility_params_like_update(like_object_inout, hh_data_in, params_in)
   !updates the utility parameters

   class(like_type), intent(in out) :: like_object_inout
   type(data_i_type), intent(in) :: hh_data_in
   real*8, dimension(n_params), intent(in) :: params_in

   !various pointers
   type( mygrid_type), pointer ::  mygrid_temp
!   real*8, pointer :: m_leisure_mean, leisure_mean

!   real*8 :: m_leisure_mean_temp, leisure_mean_temp
   integer :: t

   !some initial pointing
!   m_leisure_mean    => like_object_inout % mygrid_t_invar % rv(var_m_leisure_mean)
!   leisure_mean      => like_object_inout % mygrid_t_invar % rv(var_leisure_mean)

!   !now set the leisure means
!   m_leisure_mean = &
!      params_in(param_m_leisure_cons)
!!      params_in(param_m_leisure_age)          * hh_data_in % m_age + &
!!      params_in(param_m_leisure_age2)         * hh_data_in % m_age * hh_data_in % m_age + &
!!      params_in(param_m_leisure_race)         * hh_data_in % m_white + &
!!      params_in(param_m_leisure_hs)           * hh_data_in % m_hs + &
!!      params_in(param_m_leisure_some_college) * hh_data_in % m_some_college + &
!!      params_in(param_m_leisure_college)      * hh_data_in % m_college

!   leisure_mean = &
!      params_in(param_leisure_cons)
!    !  params_in(param_leisure_age)          * hh_data_in % age + &
!    !  params_in(param_leisure_age2)         * hh_data_in % age * hh_data_in % age
!!      params_in(param_leisure_race)         * hh_data_in % white + &
!!      params_in(param_leisure_hs)           * hh_data_in % hs + &
!!      params_in(param_leisure_some_college) * hh_data_in % some_college + &
!!      params_in(param_leisure_college)      * hh_data_in % college

   !now loop over time
   do t = 2,like_object_inout % max_time

      !now get the utility function parameters, which I only need to calculate if the household receives any job offers
      if ((like_object_inout % m_receive_new(t) .eq. 1) .or. (like_object_inout % receive_new(t) .eq. 1)) then
         like_object_inout % utility_params_created(t) = 1
         call utility_function_setup(  like_object_inout % mygrid_t_var(t) % rv(var_kids), &
                                       like_object_inout % mygrid_t_var(t) % rv(var_youngest), &
                                       params_in, &
                                       like_object_inout % utility_params(:,t) )
      else
         like_object_inout % utility_params_created(t) = 0
      end if
   end do !over t




end subroutine utility_params_like_update



subroutine utility_like_update_one_t(like_object_inout, params_in, t)
   !updates the potential utilities from the joboffer stage
   class(like_type), intent(in out) :: like_object_inout
   real*8, dimension(n_params), intent(in) :: params_in
   integer, intent(in) :: t

  !     print *, "both accept"
      if(   (like_object_inout % m_receive_new(t) .eq. 1) .and. &
            (like_object_inout % receive_new(t) .eq. 1) ) then

         like_object_inout % utility_both_accept(t) = &
            utility_function(    like_object_inout % mygrid_both_accept(t), &
                                 like_object_inout % asset_choice_both_accept(t), &
                                 like_object_inout % utility_params(:,t), &
                                 params_in )
      end if

  !  print *, "husband accept"
      if(like_object_inout % m_receive_new(t) .eq. 1)  then
         like_object_inout % utility_husband_accept(t) = &
            utility_function(    like_object_inout % mygrid_husband_accept(t), &
                                 like_object_inout % asset_choice_husband_accept(t), &
                                 like_object_inout % utility_params(:,t), &
                                 params_in )
      end if

   !    print *, "wife accept"
      if(like_object_inout % receive_new(t) .eq. 1)  then
         like_object_inout % utility_wife_accept(t) = &
            utility_function(    like_object_inout % mygrid_wife_accept(t), &
                                 like_object_inout % asset_choice_wife_accept(t), &
                                 like_object_inout % utility_params(:,t), &
                                 params_in )
      end if

   ! print *, "both reject"
      if(   (like_object_inout % m_receive_new(t) .eq. 1) .or. &
            (like_object_inout % receive_new(t) .eq. 1) ) then
         like_object_inout % utility_both_reject(t) = &
            utility_function(    like_object_inout % mygrid_t_var(t), &
                                 like_object_inout % asset_choice_both_reject(t), &
                                 like_object_inout % utility_params(:,t), &
                                 params_in )
      end if

end subroutine utility_like_update_one_t





subroutine utility_like_update(like_object_inout, params_in )
   !updates the potential utilities from the joboffer stage

   class(like_type), intent(in out) :: like_object_inout
   real*8, dimension(n_params), intent(in) :: params_in
   integer :: t

  ! print *, "within utility like update"

   do t=2, like_object_inout % max_time
   !    print *, "t=", t
      call utility_like_update_one_t(like_object_inout, params_in, t)

   end do !over t

end subroutine utility_like_update


!don't need this because I'm estimating the jobloss parameters outside my structural model, for now

!subroutine jobloss_like_update( like_object_inout, hh_data_in, params_in )
!   !updates the jobloss probablities for when I change the parameters for this function

!   class(like_type), intent(in out)  :: like_object_inout
!   type(data_i_type), intent(in),target  :: hh_data_in
!   real*8, dimension(n_params), intent(in) :: params_in

!   !various pointers
!   type( mygrid_type), pointer ::  mygrid_temp
!   real*8 :: m_jobloss_xb, m_jobloss_xb
! !  real*8, pointer :: m_jobloss_mean, jobloss_mean

!   integer :: i,t


!   !some initial pointing
!!   m_jobloss_mean    => like_object_inout % mygrid_t_invar % rv(var_m_jobloss_mean)
!!   jobloss_mean      => like_object_inout % mygrid_t_invar % rv(var_jobloss_mean)

!!   !update the jobloss means
!!   m_jobloss_mean = &
!!      params_in(param_jobloss_cons)  + &
!!      params_in(param_jobloss_seam_future)*.25
!!    !  params_in(param_jobloss_age)          * hh_data_in % m_age + &
!!    !  params_in(param_jobloss_age2)         * hh_data_in % m_age * hh_data_in % m_age + &
!!    !  params_in(param_jobloss_race)         * hh_data_in % m_white + &
!!    !  params_in(param_jobloss_hs)           * hh_data_in % m_hs + &
!!    !  params_in(param_jobloss_some_college) * hh_data_in % m_some_college + &
!!    !  params_in(param_jobloss_college)      * hh_data_in % m_college

!!   jobloss_mean = &
!!      params_in(param_jobloss_cons)  + &
!!      params_in(param_jobloss_seam_future)*.25  + &
!!      params_in(param_jobloss_gender)
!!    !  params_in(param_jobloss_age)          * hh_data_in % age + &
!!    !  params_in(param_jobloss_age2)         * hh_data_in % age * hh_data_in % age + &
!!    !  params_in(param_jobloss_race)         * hh_data_in % white + &
!!    !  params_in(param_jobloss_hs)           * hh_data_in % hs + &
!!    !  params_in(param_jobloss_some_college) * hh_data_in % some_college + &
!!    !  params_in(param_jobloss_college)      * hh_data_in % college + &
!!    !  params_in(param_jobloss_gender)


!   !now go through all the time periods
!   !IMPORTANT, I only go up to max_time_in-1, not max_time_in, because I don't know the jobloss outcome
!      !for the last period since I don't know the employment status for the following non-existent period
!      !and I also start at t=1
!   do t = 1, like_object_inout % max_time-1
!      mygrid_temp          => like_object_inout % mygrid_t_var(t)

!      !now create the time-specific inputs for the jobloss probablities, which I only need to do
!      !if they have a job

!      !first do husband

!      if (  ( mygrid_temp % iv( var_m_job ) .eq. 1 ) .or. &
!            ( like_object_inout % m_accept_new(t) .eq. 1) ) then

!         m_jobloss_xb = &
!            m_jobloss_xb_like_func( like_object_inout % mygrid_t_invar, &
!                                    like_object_inout % mygrid_t_var(t), &
!                                    params_in, &
!                                    hh_data_in % data_t(t+1) % seam )

!         !now get the jobloss_probabilities
!         like_object_inout % m_jobloss_prob(t) = &
!            logit_prob( m_jobloss_xb )

!         !do either P or 1-P depending on whether a job was lost
!         if ( like_object_inout % m_jobloss(t) .eq. 0 ) then
!            like_object_inout % m_jobloss_prob(t) = 1- like_object_inout % m_jobloss_prob(t)
!         end if

!      else
!         like_object_inout % m_jobloss_prob(t) = 1
!      end if

!      !now do wife
!      if (  ( mygrid_temp % iv( var_job ) .eq. 1 ) .or. &
!            ( like_object_inout % accept_new(t) .eq. 1) ) then

!         jobloss_xb = &
!            jobloss_xb_like_func(   like_object_inout % mygrid_t_invar, &
!                                    like_object_inout % mygrid_t_var(t), &
!                                    params_in, &
!                                    hh_data_in % data_t(t+1) % seam )


!         !now get the jobloss_probabilities
!         like_object_inout % jobloss_prob(t) = &
!            logit_prob ( jobloss_xb )

!         !do either P or 1-P depending on whether a job was lost
!         if ( like_object_inout % jobloss(t) .eq. 0 ) then
!            like_object_inout % jobloss_prob(t) = 1- like_object_inout % jobloss_prob(t)
!         end if

!      else
!         like_object_inout % jobloss_prob(t) = 1
!      end if

!   end do !over t

!end subroutine jobloss_like_update


!subroutine ols_partial_like_update( like_object_inout, ols_vec_in)
!    !updates the ols partial interpolation function for a household
!   class(like_type), intent(in out) :: like_object_inout
!   real*8, dimension(length_ols_vec_high), intent(in) :: ols_vec_in

!   !determine if need to ever get the value function for this household, and if so, get
!      !the partial interpolation function
!   if ( ( maxval(like_object_inout % m_receive_new) .eq. 1 ) .or. &
!        ( maxval(like_object_inout % receive_new) .eq. 1 )  ) then

!      like_object_inout % ols_partial_created = 1
!      call get_ols_partial( like_object_inout % mygrid_t_invar, &
!                              ols_vec_in, &
!                              like_object_inout % ols_vec )
!   else
!      like_object_inout % ols_partial_created = 0
!   end if


!end subroutine ols_partial_like_update


subroutine choice_prob_like_update_one_t(like_object_inout, value_matrix_in, t)
   !gets the accept/reject choice probability, given that a particular joboffer has been received
   class(like_type), intent(in out) :: like_object_inout
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
   integer, intent(in) :: t

   real*8,dimension(4) :: vals_prob_input
   integer :: choice_index

      if (  (like_object_inout % m_receive_new(t) .eq. 0) .and.  &
            (like_object_inout % receive_new(t) .eq. 0) ) then
     !    print *, "both no offer"
         like_object_inout % choice_prob(t) = 1

      else if (like_object_inout % m_receive_new(t) .eq. 0) then !only wife receives an offer
      !   print *, "only wife receives offer"
         vals_prob_input(1) = like_object_inout % utility_both_reject(t) + &
            value_function_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_t_var(t), &
                                       like_object_inout % asset_choice_both_reject(t), &
                                       value_matrix_in )


         vals_prob_input(2) = like_object_inout % utility_wife_accept(t) + &
            value_function_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_wife_accept(t), &
                                       like_object_inout % asset_choice_wife_accept(t), &
                                       value_matrix_in )

         choice_index = like_object_inout % accept_new(t) + 1
         like_object_inout % choice_prob(t) = mlogit_prob(2,choice_index,vals_prob_input(1:2) )

      else if (like_object_inout % receive_new(t) .eq. 0) then !only husband
       !  print *, "only husband receives offer"
         vals_prob_input(1) = like_object_inout % utility_both_reject(t) + &
            value_function_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_t_var(t), &
                                       like_object_inout % asset_choice_both_reject(t), &
                                       value_matrix_in )


         vals_prob_input(2) = like_object_inout % utility_husband_accept(t) + &
            value_function_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_husband_accept(t), &
                                       like_object_inout % asset_choice_husband_accept(t), &
                                       value_matrix_in )

         choice_index = like_object_inout % m_accept_new(t) + 1
         like_object_inout % choice_prob(t) = mlogit_prob(2,choice_index,vals_prob_input(1:2) )

      else !both receive an offer
       !  print *, "both receive offer"

         vals_prob_input(1) = like_object_inout % utility_both_reject(t) + &
            value_function_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_t_var(t), &
                                       like_object_inout % asset_choice_both_reject(t), &
                                       value_matrix_in )

         vals_prob_input(2) = like_object_inout % utility_husband_accept(t) + &
            value_function_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_husband_accept(t), &
                                       like_object_inout % asset_choice_husband_accept(t), &
                                       value_matrix_in )

         vals_prob_input(3) = like_object_inout % utility_wife_accept(t) + &
            value_function_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_wife_accept(t), &
                                       like_object_inout % asset_choice_wife_accept(t), &
                                       value_matrix_in )

         vals_prob_input(4) = like_object_inout % utility_both_accept(t) + &
            value_function_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_both_accept(t), &
                                       like_object_inout % asset_choice_both_accept(t), &
                                       value_matrix_in )

         choice_index = 1 + like_object_inout % m_accept_new(t) &
            + 2*like_object_inout % accept_new(t)
         like_object_inout % choice_prob(t) = mlogit_prob(4,choice_index,vals_prob_input )

      end if

      if (choice_prob_lb_enforce .eqv. .true. ) like_object_inout % choice_prob(t) = &
         max( like_object_inout % choice_prob(t), choice_prob_lb )


end subroutine choice_prob_like_update_one_t


subroutine like_max_value_one_t( like_object_inout, params_in, value_matrix_in, t)
   !note: this subroutine hasn't been debugged yet

   !gets the choice probabilities based on maximizing the value function wrt assets rather than
      !doing interpolation to get the assets
   !bypasses changing the utilties in the likelihood object


   class(like_type), intent(in out) :: like_object_inout
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
   integer, intent(in) :: t

   real*8,dimension(4) :: value_vec
   integer :: choice_index

   !first get the values and the asset for each potential outcome (which depends on whether
      ! job offers were received or not)

   !both reject
   call max_value_sub_split(  like_object_inout % mygrid_t_invar, &
                              like_object_inout % mygrid_t_var(t), &
                              like_object_inout % utility_params(:,t), &
                              params_in, &
                              value_matrix_in, &
                              like_object_inout % asset_choice_both_reject(t), &
                              value_vec(1) )


   !husband accept
   if (like_object_inout % m_receive_new(t) .eq. 1) then

      call max_value_sub_split(  like_object_inout % mygrid_t_invar, &
                                 like_object_inout % mygrid_husband_accept(t), &
                                 like_object_inout % utility_params(:,t), &
                                 params_in, &
                                 value_matrix_in, &
                                 like_object_inout % asset_choice_husband_accept(t), &
                                 value_vec(2) )


   end if

   !wife accept
   if (like_object_inout % receive_new(t) .eq. 1) then

      call max_value_sub_split(  like_object_inout % mygrid_t_invar, &
                                 like_object_inout % mygrid_wife_accept(t), &
                                 like_object_inout % utility_params(:,t), &
                                 params_in, &
                                 value_matrix_in, &
                                 like_object_inout % asset_choice_wife_accept(t), &
                                 value_vec(3) )
   end if

   !both accept
   if (  (like_object_inout % m_receive_new(t) .eq. 1) .and. &
         (like_object_inout % receive_new(t) .eq. 1) ) then

      call max_value_sub_split(  like_object_inout % mygrid_t_invar, &
                                 like_object_inout % mygrid_both_accept(t), &
                                 like_object_inout % utility_params(:,t), &
                                 params_in, &
                                 value_matrix_in, &
                                 like_object_inout % asset_choice_both_accept(t), &
                                 value_vec(4) )
   end if


   !now get the choice probabilities
   if (  (like_object_inout % m_receive_new(t) .eq. 0) .and.  &
         (like_object_inout % receive_new(t) .eq. 0) ) then
  !    print *, "both no offer"
      like_object_inout % choice_prob(t) = 1

   else if (like_object_inout % m_receive_new(t) .eq. 0) then !only wife receives an offer
   !   print *, "only wife receives offer"

      !need to move element 3 to element 2 in value_vec because I want to input a 2x1 vector
      value_vec(2) = value_vec(3)

      choice_index = like_object_inout % accept_new(t) + 1
      like_object_inout % choice_prob(t) = mlogit_prob(2,choice_index,value_vec(1:2) )

   else if (like_object_inout % receive_new(t) .eq. 0) then !only husband
    !  print *, "only husband receives offer"



      choice_index = like_object_inout % m_accept_new(t) + 1
      like_object_inout % choice_prob(t) = mlogit_prob(2,choice_index,value_vec(1:2) )

   else !both receive an offer
    !  print *, "both receive offer"

      choice_index = 1 + like_object_inout % m_accept_new(t) &
         + 2*like_object_inout % accept_new(t)
      like_object_inout % choice_prob(t) = mlogit_prob(4,choice_index,value_vec )

   end if

   if (choice_prob_lb_enforce .eqv. .true. ) like_object_inout % choice_prob(t) = &
      max( like_object_inout % choice_prob(t), choice_prob_lb )


end subroutine like_max_value_one_t

subroutine like_max_value( like_object_inout, params_in, value_matrix_in )
   class(like_type), intent(in out) :: like_object_inout
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
   integer :: t

   !note: this subroutine hasn't been debugged yet

   do t=2, like_object_inout % max_time
    !  print *, "like max value for, time = ", t
      !update asset state variable 1st.  for t=2, the value is based on the initial draw asset variable
      call asset_state_like_update_one_t( like_object_inout, params_in, t)
      call like_max_value_one_t( like_object_inout, params_in, value_matrix_in, t)

   end do !over t



end subroutine like_max_value



subroutine value_args_one_t(like_object_inout, value_matrix_in, t, value_vec_out)
   !gets the value functions that would go into the logit probabilities
   class(like_type), intent(in out) :: like_object_inout
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
   integer, intent(in) :: t
   real*8, dimension(:), allocatable :: value_vec_out


      if (  (like_object_inout % m_receive_new(t) .eq. 0) .and.  &
            (like_object_inout % receive_new(t) .eq. 0) ) then
     !    print *, "both no offer"
         allocate(value_vec_out(1))
         value_vec_out = 0

      else if (like_object_inout % m_receive_new(t) .eq. 0) then !only wife receives an offer
      !   print *, "only wife receives offer"
         allocate( value_vec_out(2) )

         value_vec_out(1) = &
            value_function_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_t_var(t), &
                                       like_object_inout % asset_choice_both_reject(t), &
                                       value_matrix_in )


         value_vec_out(2) = &
            value_function_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_wife_accept(t), &
                                       like_object_inout % asset_choice_wife_accept(t), &
                                       value_matrix_in )

      else if (like_object_inout % receive_new(t) .eq. 0) then !only husband
       !  print *, "only husband receives offer"
         allocate( value_vec_out(2) )

         value_vec_out(1) =  &
            value_function_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_t_var(t), &
                                       like_object_inout % asset_choice_both_reject(t), &
                                       value_matrix_in )


         value_vec_out(2) =  &
            value_function_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_husband_accept(t), &
                                       like_object_inout % asset_choice_husband_accept(t), &
                                       value_matrix_in )

      else !both receive an offer
       !  print *, "both receive offer"
         allocate( value_vec_out(4) )

         value_vec_out(1) =  &
            value_function_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_t_var(t), &
                                       like_object_inout % asset_choice_both_reject(t), &
                                       value_matrix_in )

         value_vec_out(2) =  &
            value_function_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_husband_accept(t), &
                                       like_object_inout % asset_choice_husband_accept(t), &
                                       value_matrix_in )

         value_vec_out(3) =  &
            value_function_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_wife_accept(t), &
                                       like_object_inout % asset_choice_wife_accept(t), &
                                       value_matrix_in )

         value_vec_out(4) =  &
            value_function_split_in(   like_object_inout % mygrid_t_invar, &
                                       like_object_inout % mygrid_both_accept(t), &
                                       like_object_inout % asset_choice_both_accept(t), &
                                       value_matrix_in )

      end if



end subroutine value_args_one_t


subroutine choice_prob_like_update(like_object_inout, value_matrix_in)
   !gets the accept/reject choice probability, given that a particular joboffer has been received
   class(like_type), intent(in out) :: like_object_inout
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in

   real*8,dimension(4) :: vals_prob_input
   integer :: t, choice_index



   !now go through time to get the choice probabilities
   do t=2, like_object_inout % max_time
    !  print *, t
      call choice_prob_like_update_one_t(like_object_inout,value_matrix_in,t)

   end do !over t

end subroutine choice_prob_like_update


subroutine asset_error_prob_like_update( like_object_inout, hh_data_in, params_in )
   !updates the log of the measurement error probabilities
   class(like_type), intent(in out) :: like_object_inout
   type(data_i_type), intent(in) :: hh_data_in
   real*8, dimension(n_params), intent(in) :: params_in

   like_object_inout % ln_asset_prob = 0

   if (( hh_data_in % liquid_time1 > 0 ) .and. ( hh_data_in % liquid_time1 <= like_object_inout % max_time )) then

      like_object_inout % ln_asset_prob = like_object_inout % ln_asset_prob + &
         asset_error_func( hh_data_in % liquid_time1, hh_data_in % liquid_net1 )

   end if

   if (( hh_data_in % liquid_time2 > 0 ) .and. ( hh_data_in % liquid_time2 <= like_object_inout % max_time ))  then

      like_object_inout % ln_asset_prob = like_object_inout % ln_asset_prob + &
         asset_error_func( hh_data_in % liquid_time2, hh_data_in % liquid_net2 )

   end if

   if (( hh_data_in % liquid_time3 > 0 ) .and. ( hh_data_in % liquid_time3 <= like_object_inout % max_time ))  then

      like_object_inout % ln_asset_prob = like_object_inout % ln_asset_prob + &
         asset_error_func( hh_data_in % liquid_time3, hh_data_in % liquid_net3 )

   end if

   if ( asset_error_lb_enforce .eqv. .true. ) then

       like_object_inout % ln_asset_prob =  max( like_object_inout % ln_asset_prob, asset_error_lb )

   end if

contains

function asset_error_func( t_in, asset_data_in )
   real*8 :: asset_error_func
   integer, intent(in) :: t_in
   real*8, intent(in) :: asset_data_in

   real*8 :: asset_real, sd

   asset_real = like_object_inout %  mygrid_t_var( t_in ) % rv( var_assets )
   sd = exp(   params_in( param_asset_error_cons ) + &
               params_in( param_asset_error_linear )* abs(asset_real) )

   asset_error_func = ln_pdf_norm(asset_real, asset_data_in , sd )

end function



end subroutine asset_error_prob_like_update




!commenting this subroutine out because I think it'll be complicated to deal with with the new incorporation of assets


!subroutine low_choice_prob_adjust(like_object_inout, params_in, value_matrix_in, &
!   asset_matrix_in, t)
!   !changes the non_pecuniary benefits and measurement error value to make choice probabilities
!   class(like_type), intent(in out) :: like_object_inout
!   real*8, dimension(n_params), intent(in) :: params_in
!   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
!   real*8, dimension(max_n_ols_coef_asset, 0:1, 0:1), intent(in) :: asset_matrix_in
!   integer, intent(in) :: t

!   real*8, parameter :: non_pec_inc_val =.33D+00 !controls how much to change the non_pecuniary benefits
!   real*8, parameter :: wage_inc_val =.4D+00 !controls how much to change the wage_error

!   real*8, parameter :: max_wage_val = 120 !max amount I'm willing to increase wages to
!   real*8, parameter :: max_non_pec_val = 7
!   real*8, parameter :: min_wage_val = 1
!   real*8, parameter :: min_non_pec_val = -7

!   integer, parameter :: max_iter = 100 !max number of times the non_pecuniary benefit can be changed


!   real*8 :: m_non_pec_norm, non_pec_norm, m_wage, wage
!   integer :: i,t_prime,ub

!   i=0
!   m_non_pec_norm = like_object_inout % mygrid_husband_accept(t) % rv(var_m_non_pec_norm)
!   non_pec_norm = like_object_inout % mygrid_wife_accept(t) % rv(var_non_pec_norm)
!   m_wage = like_object_inout % mygrid_husband_accept(t) % rv(var_m_wage)
!   wage = like_object_inout % mygrid_wife_accept(t) % rv(var_wage)

!      ! print *, "problematic hh before I changed the non pecuniary benefit"
!      !   call like_object_inout % print(t)
!     ! read(*,*)

!   do while( (i .le. max_iter) .and. (like_object_inout % choice_prob(t) < choice_prob_min))
!      i=i+1


!      if (like_object_inout % m_receive_new(t) .eq. 1 ) then
!         if (like_object_inout % m_accept_new(t) .eq. 1 ) then
!            m_non_pec_norm = m_non_pec_norm + non_pec_inc_val
!            m_wage = m_wage*exp( wage_inc_val*params_in( param_sd_wage) )

!            !incoprorate bounds
!            m_non_pec_norm = min(m_non_pec_norm,max_non_pec_val)
!            m_wage = min(m_wage,max_wage_val)


!         else
!            !make it such that this person didn't receive a job offer
!            like_object_inout % m_receive_new(t) = 0
!            m_wage = 0
!            m_non_pec_norm = 0

!            like_object_inout % mygrid_husband_accept(t)  % iv(var_m_hours) = 0
!            like_object_inout % mygrid_both_accept(t)     % iv(var_m_hours) = 0

!         end if

!         like_object_inout % mygrid_husband_accept(t) % rv(var_m_wage) = m_wage
!         like_object_inout % mygrid_both_accept(t) % rv(var_m_wage)    = m_wage

!         like_object_inout % mygrid_husband_accept(t) % rv(var_m_non_pec_norm) = m_non_pec_norm
!         like_object_inout % mygrid_both_accept(t) % rv(var_m_non_pec_norm)    = m_non_pec_norm
!      end if

!      if (like_object_inout % receive_new(t) .eq. 1 ) then
!         if (like_object_inout % accept_new(t) .eq. 1 ) then
!            non_pec_norm = non_pec_norm + non_pec_inc_val
!            wage = wage*exp( wage_inc_val*params_in( param_sd_wage) )

!            !incoprorate bounds
!            non_pec_norm = min(non_pec_norm,max_non_pec_val)
!            wage = min(wage,max_wage_val)

!         else
!            !make it such that this person didn't receive a job offer
!            like_object_inout % receive_new(t) = 0
!            wage = 0
!            non_pec_norm = 0

!            like_object_inout % mygrid_wife_accept(t)  % iv(var_hours) = 0
!            like_object_inout % mygrid_both_accept(t)  % iv(var_hours) = 0
!         end if

!         like_object_inout % mygrid_wife_accept(t) % rv(var_wage)    = wage
!         like_object_inout % mygrid_both_accept(t) % rv(var_wage)    = wage

!         like_object_inout % mygrid_wife_accept(t) % rv(var_non_pec_norm)    = non_pec_norm
!         like_object_inout % mygrid_both_accept(t) % rv(var_non_pec_norm)    = non_pec_norm
!      end if

!      call asset_choice_like_update_one_t( like_object_inout, asset_matrix_in, t)
!      call utility_like_update_one_t( like_object_inout, params_in, t)
!      call choice_prob_like_update_one_t( like_object_inout, ols_matrix_in, t)

!      if (i .eq. max_iter) then
!!         call like_object_inout % print(t)

!!         print *, "value both reject", &
!!            value_function_split_in(   like_object_inout % mygrid_t_invar, &
!!                                       like_object_inout % mygrid_t_var(t), &
!!                                       ols_matrix_in )

!!         if (like_object_inout % m_receive_new(t) .eq. 1 ) then
!!            print *, "value husband accept", &
!!                  value_function_split_in(   like_object_inout % mygrid_t_invar, &
!!                              like_object_inout % mygrid_husband_accept(t), &
!!                              ols_matrix_in )
!!         end if

!!         if (like_object_inout % receive_new(t) .eq. 1 ) then
!!            print *, "value wife accept", &
!!                  value_function_split_in(   like_object_inout % mygrid_t_invar, &
!!                              like_object_inout % mygrid_wife_accept(t), &
!!                              ols_matrix_in )
!!         end if


!    !     print *, value_function_partial( like_object_inout % mygrid_t_var(t), &
!     !                               like_object_inout % ols_vec )

!     !    like_object_inout % mygrid_husband_accept(t) % rv(var_m_wage) = 100
!      !   print *, value_function_partial( like_object_inout % mygrid_husband_accept(t), &
!       !                             like_object_inout % ols_vec )

!         print *, "kept of changing non_pecuniary benefits without reaching the needed choice probability minimum"
!       !  read(*,*)
!      end if
!   end do

!   !now change the wage probabilities (if a job offer was received
!   if (like_object_inout % m_receive_new(t) .eq. 1 ) then
!      like_object_inout % m_ln_wage_prob(t) = &
!         ln_pdf_norm(   log(m_wage), &
!                        like_object_inout % m_wage_xbeta(t), &
!                        params_in(param_sd_wage) )
!   end if

!   if (like_object_inout % receive_new(t) .eq. 1 ) then
!      like_object_inout % ln_wage_prob(t) = &
!         ln_pdf_norm(   log(wage), &
!                        like_object_inout % wage_xbeta(t), &
!                        params_in(param_sd_wage) )
!   end if


!   !assign this new wage and non_pec as a state variable over time, if needed
!   if ((like_object_inout % m_accept_new(t) .eq. 1) .and. (t .ne. like_object_inout % max_time )) then

!      do t_prime = t+1, like_object_inout % m_time_job_ends(t)
!         like_object_inout % mygrid_t_var(t_prime) % rv(var_m_wage) = m_wage
!         like_object_inout % mygrid_t_var(t_prime) % rv(var_m_non_pec_norm) = m_non_pec_norm

!         !if wife received a job offer in t_prime, need to copy over these values to mygrid_wife_accept
!         if (like_object_inout % receive_new(t_prime) .eq. 1 ) then
!            like_object_inout % mygrid_wife_accept(t_prime) % rv(var_m_wage) = m_wage
!            like_object_inout % mygrid_wife_accept(t_prime) % rv(var_m_non_pec_norm) = m_non_pec_norm
!         end if

!      end do

!   end if

!   if ((like_object_inout % accept_new(t) .eq. 1) .and. (t .ne. like_object_inout % max_time )) then

!      do t_prime = t+1, like_object_inout % time_job_ends(t)
!         like_object_inout % mygrid_t_var(t_prime) % rv(var_wage) = wage
!         like_object_inout % mygrid_t_var(t_prime) % rv(var_non_pec_norm) = non_pec_norm

!         !if husband received a job offer in t_prime, need to copy over these values to mygrid_husband_accept
!         if (like_object_inout % m_receive_new(t_prime) .eq. 1 ) then
!            like_object_inout % mygrid_husband_accept(t_prime) % rv(var_wage) = wage
!            like_object_inout % mygrid_husband_accept(t_prime) % rv(var_non_pec_norm) = non_pec_norm
!         end if

!      end do

!   end if

!   !now update the subsequent asset choice, utilities, and choice_probabilities, if needed
!   if ( ((like_object_inout % m_accept_new(t) .eq. 1) .or. &
!         (like_object_inout % accept_new(t)   .eq. 1)) .and. &
!         (t .ne. like_object_inout % max_time )) then

!      ub = max(like_object_inout % m_time_job_ends(t), like_object_inout % time_job_ends(t))
!      do t_prime = t+1, ub
!         call utility_like_update_one_t(like_object_inout,params_in,t_prime)
!         call asset_choice_like_update_one_t( like_object_inout, asset_matrix_in, t_prime)
!         call choice_prob_like_update_one_t(like_object_inout,ols_matrix_in,t_prime)
!      end do

!   end if


!end subroutine low_choice_prob_adjust



subroutine switch_pointers_params(like_object_inout, block_in, like_target_in )
   !switches the likelihood pointers within a bayesian McMC routine for the parameter blocks
   type(like_type), dimension(min_obs:max_obs), intent(in out) :: like_object_inout
   integer,intent(in) :: block_in
   type(like_target_type), dimension(min_obs:max_obs), intent(in), target :: like_target_in
   integer :: i

   !first change variables that depend of the block I'm dealing with
   select case(block_in)

      !joboffer probabilities
      case (block_joboffer_params1, block_joboffer_params2)

         do i=min_obs,max_obs
            if (associated(   like_object_inout(i) % m_joboffer_prob, &
                              like_target_in(i) % m_joboffer_prob1 )) then
               like_object_inout(i) % m_joboffer_prob => like_target_in(i) % m_joboffer_prob2
               like_object_inout(i) % joboffer_prob   => like_target_in(i) % joboffer_prob2

            else
               like_object_inout(i) % m_joboffer_prob => like_target_in(i) % m_joboffer_prob1
               like_object_inout(i) % joboffer_prob   => like_target_in(i) % joboffer_prob1

            end if
         end do


      !wage, hours, and non_pec probs
      case  (block_wage_params1, block_wage_params2, block_wage_params3)

         do i=min_obs,max_obs

            if (associated(   like_object_inout(i) % m_ln_wage_prob, &
                              like_target_in(i) % m_ln_wage_prob1 )) then

               like_object_inout(i) % m_ln_wage_prob => like_target_in(i) % m_ln_wage_prob2
               like_object_inout(i) % ln_wage_prob   => like_target_in(i) % ln_wage_prob2

               like_object_inout(i) % m_hours_prob => like_target_in(i) % m_hours_prob2
               like_object_inout(i) % hours_prob   => like_target_in(i) % hours_prob2

            else

               like_object_inout(i) % m_ln_wage_prob => like_target_in(i) % m_ln_wage_prob1
               like_object_inout(i) % ln_wage_prob   => like_target_in(i) % ln_wage_prob1

               like_object_inout(i) % m_hours_prob => like_target_in(i) % m_hours_prob1
               like_object_inout(i) % hours_prob   => like_target_in(i) % hours_prob1

            end if

            if (associated(   like_object_inout(i) % m_wage_mean_nohetero, &
                              like_target_in(i) % m_wage_mean_nohetero1 )) then

               like_object_inout(i) % m_wage_mean_nohetero => like_target_in(i) % m_wage_mean_nohetero2
               like_object_inout(i) % wage_mean_nohetero   => like_target_in(i) % wage_mean_nohetero2

            else
               like_object_inout(i) % m_wage_mean_nohetero => like_target_in(i) % m_wage_mean_nohetero1
               like_object_inout(i) % wage_mean_nohetero   => like_target_in(i) % wage_mean_nohetero1

            end if


         end do


         !time-invariant state variables, for now, only have to worry about wage unobserved heterogeneity
         do i=min_obs,max_obs
            if (associated(   like_object_inout(i) % mygrid_t_invar, &
                           like_target_in(i) % mygrid_t_invar1 )) then
               like_object_inout(i) % mygrid_t_invar => like_target_in(i) % mygrid_t_invar2
            else
               like_object_inout(i) % mygrid_t_invar => like_target_in(i) % mygrid_t_invar1
            end if
         end do


      !utility parameters created indicator and utility parameters
         !only switch this when I draw a new utility parameters
      case ( block_leisure_params1, block_leisure_params2, block_leisure_params3, block_leisure_params4 )

         do i=min_obs,max_obs
            if (associated(   like_object_inout(i) % utility_params, &
                              like_target_in(i) % utility_params1 )) then
               like_object_inout(i) % utility_params_created   => like_target_in(i) % utility_params_created2
               like_object_inout(i) % utility_params           => like_target_in(i) % utility_params2
            else
               like_object_inout(i) % utility_params_created   => like_target_in(i) % utility_params_created1
               like_object_inout(i) % utility_params           => like_target_in(i) % utility_params1
            end if
         end do

      case default
         print *, "error in select case for switch_pointers_params"

   end select


   !now for variables that I always change regardless of which parameter block I'm dealing with



!   do i=min_obs,max_obs
!      if (associated(   like_object_inout(i) % ols_partial_created, &
!                        like_target_in(i)  % ols_partial_created1 )) then
!         like_object_inout(i) % ols_partial_created => like_target_in(i) % ols_partial_created2
!         like_object_inout(i) % ols_vec             => like_target_in(i) % ols_vec2
!      else
!         like_object_inout(i) % ols_partial_created => like_target_in(i) % ols_partial_created1
!         like_object_inout(i) % ols_vec             => like_target_in(i) % ols_vec1
!      end if
!   end do

   !asset choice
   do i=min_obs,max_obs
      if (associated(   like_object_inout(i) % asset_choice_both_reject, &
                        like_target_in(i) % asset_choice_both_reject1 )) then
         like_object_inout(i) % asset_choice_both_reject      => like_target_in(i) % asset_choice_both_reject2
         like_object_inout(i) % asset_choice_husband_accept   => like_target_in(i) % asset_choice_husband_accept2
         like_object_inout(i) % asset_choice_wife_accept      => like_target_in(i) % asset_choice_wife_accept2
         like_object_inout(i) % asset_choice_both_accept      => like_target_in(i) % asset_choice_both_accept2
      else
         like_object_inout(i) % asset_choice_both_reject      => like_target_in(i) % asset_choice_both_reject1
         like_object_inout(i) % asset_choice_husband_accept   => like_target_in(i) % asset_choice_husband_accept1
         like_object_inout(i) % asset_choice_wife_accept      => like_target_in(i) % asset_choice_wife_accept1
         like_object_inout(i) % asset_choice_both_accept      => like_target_in(i) % asset_choice_both_accept1
      end if
   end do

   !asset error
   do i=min_obs,max_obs
      if (associated(   like_object_inout(i) % ln_asset_prob, &
                        like_target_in(i) % ln_asset_prob1 )) then

         like_object_inout(i) % ln_asset_prob  => like_target_in(i) % ln_asset_prob2
      else
         like_object_inout(i) % ln_asset_prob  => like_target_in(i) % ln_asset_prob1
      end if
   end do


   !utility values
   do i=min_obs,max_obs
      if (associated(   like_object_inout(i) % utility_both_reject, &
                        like_target_in(i) % utility_both_reject1 )) then
         like_object_inout(i) % utility_both_reject      => like_target_in(i) % utility_both_reject2
         like_object_inout(i) % utility_husband_accept   => like_target_in(i) % utility_husband_accept2
         like_object_inout(i) % utility_wife_accept      => like_target_in(i) % utility_wife_accept2
         like_object_inout(i) % utility_both_accept      => like_target_in(i) % utility_both_accept2
      else
         like_object_inout(i) % utility_both_reject      => like_target_in(i) % utility_both_reject1
         like_object_inout(i) % utility_husband_accept   => like_target_in(i) % utility_husband_accept1
         like_object_inout(i) % utility_wife_accept      => like_target_in(i) % utility_wife_accept1
         like_object_inout(i) % utility_both_accept      => like_target_in(i) % utility_both_accept1
      end if
   end do



   !choice-probability, which I change for all blocks
   do i=min_obs,max_obs
      if (associated(   like_object_inout(i) % choice_prob, &
                        like_target_in(i) % choice_prob1 )) then
         like_object_inout(i) % choice_prob => like_target_in(i) % choice_prob2
      else
         like_object_inout(i) % choice_prob => like_target_in(i) % choice_prob1
      end if
   end do

end subroutine switch_pointers_params





subroutine switch_pointers_ind(like_object_inout, block_in, like_target_in )
   !switches the likelihood pointers within a bayesian McMC routine for individual variables
   class(like_type),intent(in out) :: like_object_inout
   integer,intent(in) :: block_in
   type(like_target_type), intent(in), target :: like_target_in


!   select case( block_in)

!      case(block_joboffers_ind)

         !the following only need to be changed for the hetero block
         if (associated(   like_object_inout % hetero, &
                           like_target_in % hetero1 )) then
            like_object_inout % hetero => like_target_in % hetero2

         else
            like_object_inout % hetero => like_target_in % hetero1

         end if



      !wage prob would need to be changed for both hetero and joboffers block

      !wage, hours, and non_pec probs
         if (associated(   like_object_inout % m_ln_wage_prob, &
                           like_target_in % m_ln_wage_prob1 )) then
            like_object_inout % m_ln_wage_prob => like_target_in % m_ln_wage_prob2
            like_object_inout % ln_wage_prob   => like_target_in % ln_wage_prob2

            like_object_inout % m_hours_prob => like_target_in % m_hours_prob2
            like_object_inout % hours_prob   => like_target_in % hours_prob2

         else
            like_object_inout % m_ln_wage_prob => like_target_in % m_ln_wage_prob1
            like_object_inout % ln_wage_prob   => like_target_in % ln_wage_prob1

            like_object_inout % m_hours_prob => like_target_in % m_hours_prob1
            like_object_inout % hours_prob   => like_target_in % hours_prob1

         end if


      !all these parameters only need to be changed for the joboffer block

      !receiving job indicators
         if (associated(   like_object_inout % m_receive_new, &
                           like_target_in % m_receive_new1 )) then
            like_object_inout % m_receive_new => like_target_in % m_receive_new2
            like_object_inout % receive_new   => like_target_in % receive_new2
         else
            like_object_inout % m_receive_new => like_target_in % m_receive_new1
            like_object_inout % receive_new   => like_target_in % receive_new1
         end if



      !time-invariant state variables and ols


      !time-varing variables and potential joboffer state variables

         if (associated(   like_object_inout % mygrid_t_var, &
                           like_target_in % mygrid_t_var1 )) then
            like_object_inout % mygrid_t_var          => like_target_in % mygrid_t_var2
            like_object_inout % mygrid_husband_accept => like_target_in % mygrid_husband_accept2
            like_object_inout % mygrid_wife_accept    => like_target_in % mygrid_wife_accept2
            like_object_inout % mygrid_both_accept    => like_target_in % mygrid_both_accept2

            like_object_inout % m_wage_new            => like_target_in % m_wage_new2
            like_object_inout % wage_new              => like_target_in % wage_new2

         else
            like_object_inout % mygrid_t_var          => like_target_in % mygrid_t_var1
            like_object_inout % mygrid_husband_accept => like_target_in % mygrid_husband_accept1
            like_object_inout % mygrid_wife_accept    => like_target_in % mygrid_wife_accept1
            like_object_inout % mygrid_both_accept    => like_target_in % mygrid_both_accept1

            like_object_inout % m_wage_new            => like_target_in % m_wage_new1
            like_object_inout % wage_new              => like_target_in % wage_new1
         end if

!      case (block_unemp_length)

!         if (associated(   like_object_inout % m_joboffer_prob, &
!                              like_target_in % m_joboffer_prob1 )) then
!            like_object_inout % m_joboffer_prob => like_target_in % m_joboffer_prob2
!            like_object_inout % joboffer_prob   => like_target_in % joboffer_prob2

!         else
!            like_object_inout % m_joboffer_prob => like_target_in % m_joboffer_prob1
!            like_object_inout % joboffer_prob   => like_target_in % joboffer_prob1

!         end if


!      case default
!         print *, "error select case for switch_pointers_ind"
!         stop

!   end select

   !asset choice

   if (associated(   like_object_inout % asset_choice_both_reject, &
                     like_target_in % asset_choice_both_reject1 )) then
      like_object_inout % asset_choice_both_reject      => like_target_in % asset_choice_both_reject2
      like_object_inout % asset_choice_husband_accept   => like_target_in % asset_choice_husband_accept2
      like_object_inout % asset_choice_wife_accept      => like_target_in % asset_choice_wife_accept2
      like_object_inout % asset_choice_both_accept      => like_target_in % asset_choice_both_accept2
   else
      like_object_inout % asset_choice_both_reject      => like_target_in % asset_choice_both_reject1
      like_object_inout % asset_choice_husband_accept   => like_target_in % asset_choice_husband_accept1
      like_object_inout % asset_choice_wife_accept      => like_target_in % asset_choice_wife_accept1
      like_object_inout % asset_choice_both_accept      => like_target_in % asset_choice_both_accept1
   end if

   !asset prob

   if (associated(   like_object_inout % ln_asset_prob, &
                     like_target_in % ln_asset_prob1 )) then

      like_object_inout % ln_asset_prob  => like_target_in % ln_asset_prob2
   else
      like_object_inout % ln_asset_prob  => like_target_in % ln_asset_prob1
   end if


   !potential utilities from all the choices

   if (associated(   like_object_inout % utility_both_reject, &
                     like_target_in % utility_both_reject1 )) then
      like_object_inout % utility_both_reject      => like_target_in % utility_both_reject2
      like_object_inout % utility_husband_accept   => like_target_in % utility_husband_accept2
      like_object_inout % utility_wife_accept      => like_target_in % utility_wife_accept2
      like_object_inout % utility_both_accept      => like_target_in % utility_both_accept2
   else
      like_object_inout % utility_both_reject      => like_target_in % utility_both_reject1
      like_object_inout % utility_husband_accept   => like_target_in % utility_husband_accept1
      like_object_inout % utility_wife_accept      => like_target_in % utility_wife_accept1
      like_object_inout % utility_both_accept      => like_target_in % utility_both_accept1
   end if



   !choice-probability, which I change for all blocks
   if (associated(   like_object_inout % choice_prob, &
                     like_target_in % choice_prob1 )) then
      like_object_inout % choice_prob => like_target_in % choice_prob2
   else
      like_object_inout % choice_prob => like_target_in % choice_prob1
   end if


end subroutine switch_pointers_ind

subroutine print_which_like_target(like_object_in, like_target_in)
   class(like_type) :: like_object_in
   type(like_target_type), target :: like_target_in

   !prints the association between the like target and like pointers
   print *, "printing association between like targets and pointers"


   print *, "hetero", &
      target_int( associated( like_object_in % hetero, &
                              like_target_in % hetero1 ) )

   print *, "m_joboffer_prob", &
      target_int( associated( like_object_in % m_joboffer_prob, &
                              like_target_in % m_joboffer_prob1 ) )

   print *, "joboffer_prob", &
      target_int( associated( like_object_in % joboffer_prob, &
                              like_target_in % joboffer_prob1 ) )

   print *, " "

   print *, "m_ln_wage_prob", &
      target_int( associated( like_object_in % m_ln_wage_prob, &
                              like_target_in % m_ln_wage_prob1 ) )
   print *, "ln_wage_prob", &
      target_int( associated( like_object_in % m_ln_wage_prob, &
                              like_target_in % m_ln_wage_prob1 ) )

   print *, "m_wage_new", &
      target_int( associated( like_object_in % m_wage_new, &
                              like_target_in % m_wage_new1 ) )
   print *, "wage_new", &
      target_int( associated( like_object_in % wage_new, &
                              like_target_in % wage_new1 ) )

   print *, "m_wage_mean_nohetero", &
      target_int( associated( like_object_in % m_wage_mean_nohetero, &
                              like_target_in % m_wage_mean_nohetero1 ) )
   print *, "wage_mean_nohetero", &
      target_int( associated( like_object_in % wage_mean_nohetero, &
                              like_target_in % wage_mean_nohetero1 ) )

   print *, "m_hours_prob", &
      target_int( associated( like_object_in % m_hours_prob, &
                              like_target_in % m_hours_prob1 ) )

   print *, "hours_prob", &
      target_int( associated( like_object_in % hours_prob, &
                              like_target_in % hours_prob1 ) )


   print *, " "

   print *, "m_receive_new", &
      target_int( associated( like_object_in % m_receive_new, &
                              like_target_in % m_receive_new1 ) )

   print *, "receive_new", &
      target_int( associated( like_object_in % receive_new, &
                              like_target_in % receive_new1 ) )

   print *, "mygrid_t_invar", &
      target_int( associated( like_object_in % mygrid_t_invar, &
                              like_target_in % mygrid_t_invar1 ) )

   print *, "mygrid_t_var", &
      target_int( associated( like_object_in % mygrid_t_var, &
                              like_target_in % mygrid_t_var1 ) )

   print *, "mygrid_husband_accept", &
      target_int( associated( like_object_in % mygrid_husband_accept, &
                              like_target_in % mygrid_husband_accept1 ) )

   print *, "mygrid_wife_accept", &
      target_int( associated( like_object_in % mygrid_wife_accept, &
                              like_target_in % mygrid_wife_accept1 ) )

   print *, "mygrid_both_accept", &
      target_int( associated( like_object_in % mygrid_both_accept, &
                              like_target_in % mygrid_both_accept1 ) )

   print *, " "

   print *, "utility_params_created", &
      target_int( associated( like_object_in % utility_params_created, &
                              like_target_in % utility_params_created1 ) )

   print *, "utility_params", &
      target_int( associated( like_object_in % utility_params, &
                              like_target_in % utility_params1 ) )

   print *, "utility_both_reject", &
      target_int( associated( like_object_in % utility_both_reject, &
                              like_target_in % utility_both_reject1 ) )

   print *, "utility_husband_accept", &
      target_int( associated( like_object_in % utility_husband_accept, &
                              like_target_in % utility_husband_accept1 ) )

   print *, "utility_wife_accept", &
      target_int( associated( like_object_in % utility_wife_accept, &
                              like_target_in % utility_wife_accept1 ) )

   print *, "utility_both_accept", &
      target_int( associated( like_object_in % utility_both_accept, &
                              like_target_in % utility_both_accept1 ) )

   print *, " "


   print *, "asset_choice_both_reject", &
      target_int( associated( like_object_in % asset_choice_both_reject, &
                              like_target_in % asset_choice_both_reject1 ) )

   print *, "asset_choice_husband_accept", &
      target_int( associated( like_object_in % asset_choice_husband_accept, &
                              like_target_in % asset_choice_husband_accept1 ) )

   print *, "asset_choice_wife_accept", &
      target_int( associated( like_object_in % asset_choice_wife_accept, &
                              like_target_in % asset_choice_wife_accept1 ) )

   print *, "asset_choice_both_accept", &
      target_int( associated( like_object_in % asset_choice_both_accept, &
                              like_target_in % asset_choice_both_accept1 ) )


   print *, " "

   print *, "ln_asset_prob", &
      target_int( associated( like_object_in % ln_asset_prob, &
                              like_target_in % ln_asset_prob1 ) )


!   print *, "ols_partial_created", &
!      target_int( associated( like_object_in % ols_partial_created, &
!                              like_target_in % ols_partial_created1 ) )

!   print *, "ols_vec", &
!      target_int( associated( like_object_in % ols_vec, &
!                              like_target_in % ols_vec1 ) )

   print *, "choice_prob", &
      target_int( associated( like_object_in % choice_prob, &
                              like_target_in % choice_prob1 ) )

!   print *, " "

!   print *, "m_jobloss_prob", &
!      target_int( associated( like_object_in % m_jobloss_prob, &
!                              like_target_in % m_jobloss_prob1 ) )

!   print *, "jobloss_prob", &
!      target_int( associated( like_object_in % jobloss_prob, &
!                              like_target_in % jobloss_prob1 ) )


contains

function target_int(logical_in)
   !converts the true or false indicator to the integer of interest
   logical, intent(in) :: logical_in
   integer :: target_int

   if (logical_in .eqv. .true. ) then
      target_int = 1
   else
      target_int = 2
   end if

end function


end subroutine print_which_like_target




subroutine copy_mygrid_t_invar( like_object_in, block_in, like_target_inout )
   !copies some of the time invariante variant variables to the inactive target, needed because this object is only updated
      !partially for every parameter block
   type(like_type), dimension(min_obs:max_obs),intent(in out) :: like_object_in
   integer,intent(in) :: block_in
   type(like_target_type), dimension(min_obs:max_obs), intent(in out), target :: like_target_inout
   integer :: i


   select case( block_in )

      !joboffer probabilities
      case (block_joboffer_params1, block_joboffer_params2 )

!         do i=min_obs,max_obs
!            if (associated(   like_object_in(i) % mygrid_t_invar, &
!                              like_target_inout(i) % mygrid_t_invar1 )) then
!               like_target_inout(i) % mygrid_t_invar2 % rv(var_m_joboffer_mean) =  &
!                  like_target_inout(i) % mygrid_t_invar1 % rv(var_m_joboffer_mean)
!               like_target_inout(i) % mygrid_t_invar2 % rv(var_joboffer_mean) =  &
!                  like_target_inout(i) % mygrid_t_invar1 % rv(var_joboffer_mean)
!            else
!               like_target_inout(i) % mygrid_t_invar1 % rv(var_m_joboffer_mean) =  &
!                  like_target_inout(i) % mygrid_t_invar2 % rv(var_m_joboffer_mean)
!               like_target_inout(i) % mygrid_t_invar1 % rv(var_joboffer_mean) =  &
!                  like_target_inout(i) % mygrid_t_invar2 % rv(var_joboffer_mean)
!            end if
!         end do

      !wage, hours, and non_pec probs
      case  (block_wage_params1, block_wage_params2, block_wage_params3)

         do i=min_obs,max_obs
            if (associated(   like_object_in(i) % mygrid_t_invar, &
                              like_target_inout(i) % mygrid_t_invar1 )) then
               like_target_inout(i) % mygrid_t_invar2 % rv(var_m_wage_mean) =  &
                  like_target_inout(i) % mygrid_t_invar1 % rv(var_m_wage_mean)
               like_target_inout(i) % mygrid_t_invar2 % rv(var_wage_mean) =  &
                  like_target_inout(i) % mygrid_t_invar1 % rv(var_wage_mean)
            else
               like_target_inout(i) % mygrid_t_invar1 % rv(var_m_wage_mean) =  &
                  like_target_inout(i) % mygrid_t_invar2 % rv(var_m_wage_mean)
               like_target_inout(i) % mygrid_t_invar1 % rv(var_wage_mean) =  &
                  like_target_inout(i) % mygrid_t_invar2 % rv(var_wage_mean)
            end if
         end do

      !utility parameters created indicator and utility parameters
         !only switch this when I draw a new utility parameters
      case ( block_leisure_params1, block_leisure_params2, block_leisure_params3, block_leisure_params4 )

!         do i= min_obs,max_obs
!            if (associated(   like_object_in(i) % mygrid_t_invar, &
!                              like_target_inout(i) % mygrid_t_invar1 )) then
!               like_target_inout(i) % mygrid_t_invar2 % rv(var_m_leisure_mean) =  &
!                  like_target_inout(i) % mygrid_t_invar1 % rv(var_m_leisure_mean)
!               like_target_inout(i) % mygrid_t_invar2 % rv(var_leisure_mean) =  &
!                  like_target_inout(i) % mygrid_t_invar1 % rv(var_leisure_mean)
!            else
!               like_target_inout(i) % mygrid_t_invar1 % rv(var_m_leisure_mean) =  &
!                  like_target_inout(i) % mygrid_t_invar2 % rv(var_m_leisure_mean)
!               like_target_inout(i) % mygrid_t_invar1 % rv(var_leisure_mean) =  &
!                  like_target_inout(i) % mygrid_t_invar2 % rv(var_leisure_mean)
!            end if
!         end do

      case default
         print *, "error in select case for switch_pointers_params"

   end select


end subroutine copy_mygrid_t_invar



subroutine copy_mygrid_t_invar_ind( like_object_in, block_in, like_target_inout )
   !same as the subroutine from above, but for when I only want to do it for one household
   type(like_type), intent(in out) :: like_object_in
   integer,intent(in) :: block_in
   type(like_target_type), intent(in out), target :: like_target_inout

   !only have one block for now

 !  select case( block_in )

      !wage, hours, and non_pec probs
  !    case  (block_joboffers_ind)

         if (associated(   like_object_in % mygrid_t_invar, &
                           like_target_inout % mygrid_t_invar1 )) then
            like_target_inout % mygrid_t_invar2 % rv(var_m_wage_mean) =  &
               like_target_inout % mygrid_t_invar1 % rv(var_m_wage_mean)
            like_target_inout % mygrid_t_invar2 % rv(var_wage_mean) =  &
               like_target_inout % mygrid_t_invar1 % rv(var_wage_mean)
         else
            like_target_inout % mygrid_t_invar1 % rv(var_m_wage_mean) =  &
               like_target_inout % mygrid_t_invar2 % rv(var_m_wage_mean)
            like_target_inout % mygrid_t_invar1 % rv(var_wage_mean) =  &
               like_target_inout % mygrid_t_invar2 % rv(var_wage_mean)
         end if


   !   case default
   !      print *, "error in select case for switch_pointers_params"

  ! end select


end subroutine copy_mygrid_t_invar_ind

subroutine asset_revert(like_object_inout, params_in)
   !if rejected some parameters, this reverts the asset variables in the mygrid object to previous values
   class(like_type), intent(in out) :: like_object_inout
   real*8, dimension(n_params), intent(in) :: params_in

   integer :: t

   do t=2, like_object_inout % max_time
      call asset_state_like_update_one_t( like_object_inout, params_in, t)
   end do

end subroutine asset_revert




function log_like_func( like_object_in, block_in )
   !gets the log likelihood for an individual household

   class(like_type), intent(in) :: like_object_in
   integer, intent(in) :: block_in
   real*8  :: log_like_func

   integer :: t

   !first incorporate the parts that depend on the block I'm dealing with
   select case(block_in)
      case ( block_joboffer_params1, block_joboffer_params2, block_unemp_length )
         !do P or 1-P thing here depending of whether a joboffer was received
         log_like_func =  sum( log(like_object_in % m_joboffer_prob), &
                                       mask=like_object_in % m_receive_new .eq. 1) + &
                           sum( log(1-like_object_in % m_joboffer_prob), &
                                       mask=like_object_in % m_receive_new .eq. 0) + &
                           sum( log(like_object_in % joboffer_prob), &
                                       mask=like_object_in % receive_new .eq. 1) + &
                           sum( log(1-like_object_in % joboffer_prob), &
                                       mask=like_object_in % receive_new .eq. 0)

      case ( block_wage_params1, block_wage_params2, block_wage_params3)
         !don't include the non-pecuniary "probabilities" because they're already normalized, so the probability doesn't change
            !as the standard deviation changes.

         log_like_func= sum( (like_object_in % m_ln_wage_prob)    ) + &
                        sum( (like_object_in % ln_wage_prob)      ) + &
                        sum( log(like_object_in % m_hours_prob)   ) + &
                        sum( log(like_object_in % hours_prob)     )


         !now incorporate wage probs penalties
         if( wage_prob_penalty_enforce .eqv. .true. ) then

            do t=2, like_object_in % max_time
               if ( like_object_in % m_ln_wage_prob(t) < wage_prob_penalty_cutoff ) then
                  log_like_func = log_like_func +  like_object_in % m_ln_wage_prob(t) * wage_prob_penalty_val
               end if

               if ( like_object_in % ln_wage_prob(t) < wage_prob_penalty_cutoff ) then
                  log_like_func = log_like_func +  like_object_in % ln_wage_prob(t) * wage_prob_penalty_val
               end if
            end do

         end if

      case ( block_leisure_params1, block_leisure_params2, block_leisure_params3, block_leisure_params4  )
         !don't need to do anything
          log_like_func = 0

      case ( block_joboffers_ind )
         !get the choice probabiliities of the accepted jobs
          log_like_func = sum( ( like_object_in % m_ln_wage_prob), &
                                 mask = like_object_in % m_accept_new(2:) .eq. 1) + &
                          sum( ( like_object_in % ln_wage_prob), &
                                 mask = like_object_in % accept_new(2:) .eq. 1)

      case default
         print *, "error in select case for get_log_like_params"

   end select

   !now incorporate choice probabilities
   if( choice_prob_penalty_enforce .eqv. .true. ) then

      do t=2, like_object_in % max_time
         if ( like_object_in % choice_prob(t) < choice_prob_penalty_cutoff ) then
            log_like_func = log_like_func + log( like_object_in % choice_prob(t) )*choice_prob_penalty_val
         else
            log_like_func = log_like_func + log( like_object_in % choice_prob(t) )
         end if
      end do

   else

      log_like_func = log_like_func +  sum( log(like_object_in % choice_prob) )

   end if

   !finally, asset_error prob
   log_like_func = log_like_func + like_object_in % ln_asset_prob

end function log_like_func

function log_like_func_all( like_object_in, block_in )
   !Note: the final likelihood only goes to the root node

   type(like_type), dimension(min_obs:max_obs), intent(in) :: like_object_in
   integer, intent(in) :: block_in
   real*8  :: log_like_func_all

   integer :: i
   real*8  :: log_like_func_temp

   log_like_func_temp = 0

   !sum within a nod
   do i=min_obs, max_obs
      log_like_func_temp = log_like_func_temp + log_like_func(like_object_in(i), block_in)
   end do

   call mpi_allreduce( log_like_func_temp, log_like_func_all, 1, &
      mpi_real8, mpi_sum,  mpi_comm_world, err )


end function log_like_func_all




subroutine choice_prob_der_setup( mygrid_in, m_wage_new_in, wage_new_in, m_non_pec_new_in, &
   non_pec_new_in, m_hours_new_in, hours_new_in, like_object_out )

   !set's up a likelihood object to be used as an argument in the choice_prob_derivatives routine
      !if m_job_opt and job_opt are present, then object is base on the origin value for this (m_job, job) combo
      !if t_opt and like_object_opt are present, then copies over these parts.

   type(mygrid_type), intent(in) :: mygrid_in
   real*8, intent(in) :: m_wage_new_in, wage_new_in, m_non_pec_new_in, non_pec_new_in
   integer, intent(in) :: m_hours_new_in, hours_new_in
   type(like_type), intent(out) :: like_object_out


   !setup the like_object_out of interest
   allocate( like_object_out % m_receive_new(1) )
   allocate( like_object_out % receive_new(1) )
   allocate( like_object_out % m_accept_new(1) )
   allocate( like_object_out % accept_new(1) )

   allocate( like_object_out % mygrid_t_invar )
   allocate( like_object_out % mygrid_t_var(1) )
   allocate( like_object_out % mygrid_husband_accept(1) )
   allocate( like_object_out % mygrid_wife_accept(1) )
   allocate( like_object_out % mygrid_both_accept(1) )

   allocate( like_object_out % utility_params_created(1) )
   allocate( like_object_out % utility_params(n_utility_params, 1) )
   allocate( like_object_out % utility_both_reject(1) )
   allocate( like_object_out % utility_husband_accept(1) )
   allocate( like_object_out % utility_wife_accept(1) )
   allocate( like_object_out % utility_both_accept(1) )

   allocate( like_object_out % asset_choice_both_reject(1) )
   allocate( like_object_out % asset_choice_husband_accept(1) )
   allocate( like_object_out % asset_choice_wife_accept(1) )
   allocate( like_object_out % asset_choice_both_accept(1) )

   allocate( like_object_out % choice_prob(1) )

   like_object_out % utility_params_created = 1

   !now setup the mygrids
   !first mygrid_t_invar and mygrid_t_var

   call like_object_out % mygrid_t_invar % allocate()
   call like_object_out % mygrid_t_var(1) % t_var_allocate()

   like_object_out % mygrid_t_invar % rv( : n_t_invar ) = mygrid_in % rv( : n_t_invar )

   like_object_out % mygrid_t_var(1) % rv(  n_t_invar + 1 :  ) = mygrid_in % rv( n_t_invar + 1 : )

   like_object_out % mygrid_t_var(1) % iv =  mygrid_in % iv


   !mygrid husband accept

   like_object_out % mygrid_husband_accept(1) = like_object_out % mygrid_t_var(1)

   like_object_out % mygrid_husband_accept(1) % rv( var_m_wage )          = m_wage_new_in
   like_object_out % mygrid_husband_accept(1) % rv( var_m_non_pec_norm )  = m_non_pec_new_in
   like_object_out % mygrid_husband_accept(1) % rv( var_m_job_length )    = 0

   like_object_out % mygrid_husband_accept(1) % rv( var_m_ui_ben )         = 0
   like_object_out % mygrid_husband_accept(1) % rv( var_m_unemp_length )   = 0

   like_object_out % mygrid_husband_accept(1) % iv( var_m_hours )         = m_hours_new_in
   like_object_out % mygrid_husband_accept(1) % iv( var_m_job )           = 1


   !mygrid wife accept
   like_object_out % mygrid_wife_accept(1) = like_object_out % mygrid_t_var(1)

   like_object_out % mygrid_wife_accept(1) % rv( var_wage )          = wage_new_in
   like_object_out % mygrid_wife_accept(1) % rv( var_non_pec_norm )  = non_pec_new_in
   like_object_out % mygrid_wife_accept(1) % rv( var_job_length )    = 0

   like_object_out % mygrid_wife_accept(1) % rv( var_ui_ben )         = 0
   like_object_out % mygrid_wife_accept(1) % rv( var_unemp_length )   = 0

   like_object_out % mygrid_wife_accept(1) % iv( var_hours )         = hours_new_in
   like_object_out % mygrid_wife_accept(1) % iv( var_job )           = 1

   !mygrid both accept
   like_object_out % mygrid_both_accept(1) = like_object_out % mygrid_t_var(1)

   like_object_out % mygrid_both_accept(1) % rv( var_m_wage )          = m_wage_new_in
   like_object_out % mygrid_both_accept(1) % rv( var_m_non_pec_norm )  = m_non_pec_new_in
   like_object_out % mygrid_both_accept(1) % rv( var_m_job_length )    = 0

   like_object_out % mygrid_both_accept(1) % rv( var_m_ui_ben )         = 0
   like_object_out % mygrid_both_accept(1) % rv( var_m_unemp_length )   = 0

   like_object_out % mygrid_both_accept(1) % iv( var_m_hours )         = m_hours_new_in
   like_object_out % mygrid_both_accept(1) % iv( var_m_job )           = 1

   like_object_out % mygrid_both_accept(1) % rv( var_wage )          = wage_new_in
   like_object_out % mygrid_both_accept(1) % rv( var_non_pec_norm )  = non_pec_new_in
   like_object_out % mygrid_both_accept(1)   % rv( var_job_length )    = 0

   like_object_out % mygrid_both_accept(1) % rv( var_ui_ben )         = 0
   like_object_out % mygrid_both_accept(1) % rv( var_unemp_length )   = 0

   like_object_out % mygrid_both_accept(1) % iv( var_hours )         = hours_new_in
   like_object_out % mygrid_both_accept(1) % iv( var_job )           = 1



end subroutine choice_prob_der_setup

subroutine choice_prob_derivatives( m_receive_new_in, &
                                    receive_new_in, &
                                    like_object_in, &
                                    params_in, &
                                    value_matrix_in, &
                                    asset_matrix_in, &
                                    first_derivative_out, &
                                    second_derivative_out )
   !gets the 1st and 2nd derivative of the choice probabilities
   !these derivatives are a matrix for the 4 possible accept/reject choices

   integer, intent(in) :: m_receive_new_in, receive_new_in
   type(like_type), intent(in) :: like_object_in
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
   real*8, dimension(n_grid_points), intent(in) :: asset_matrix_in
   real*8, dimension(0:1,0:1,n_like_vars), intent(out) :: first_derivative_out
   real*8, dimension(0:1,0:1,n_like_vars, n_like_vars), intent(out) :: second_derivative_out

   real*8, dimension(0:1,0:1) :: probs_origin, probs_perturb1, probs_perturb2, probs_perturb3, probs_perturb4
   type(like_type) :: like_temp
   type(mygrid_type) :: mygrid_t_invar_save, mygrid_t_var_save, mygrid_husband_accept_save, mygrid_wife_accept_save, &
      mygrid_both_accept_save
   integer ::  i, j, m_job, job

   like_temp = like_object_in

   first_derivative_out = 0
   second_derivative_out = 0

   m_job = like_temp % mygrid_t_var(1) % iv( var_m_job )
   job = like_temp % mygrid_t_var(1) % iv( var_job )


 !  like_temp % ols_partial_created = 1
   like_temp % m_receive_new(1) = m_receive_new_in
   like_temp % receive_new(1) = receive_new_in


   !save the mygrid types so I can revert back to them after I'm finished perturbing one parameter
   mygrid_t_invar_save          = like_temp % mygrid_t_invar
   mygrid_t_var_save            = like_temp % mygrid_t_var(1)
   mygrid_husband_accept_save   = like_temp % mygrid_husband_accept(1)
   mygrid_wife_accept_save      = like_temp % mygrid_wife_accept(1)
   mygrid_both_accept_save      = like_temp % mygrid_both_accept(1)
   call gen_perturb_choice_probs( probs_origin )

   !now get first and second derivative
   do i = 1, n_like_vars
      if ( like_var_present(i, m_job, job, m_receive_new_in, receive_new_in) .eq. 1) then


         call mygrid_revert()
         call perturb_mygrid( i,1)
         call gen_perturb_choice_probs( probs_perturb1 )

         call mygrid_revert()
         call perturb_mygrid( i,-1)
         call gen_perturb_choice_probs( probs_perturb2 )

         first_derivative_out(:,:,i)    = ( probs_perturb1 - probs_perturb2 )/(2*der_delta(i))
         second_derivative_out(:,:,i,i) = (probs_perturb1 -2*probs_origin + probs_perturb2 ) / &
            ( der_delta(i)**2 )

      end if
   end do


   !now get second derivative
   do i = 1, n_like_vars
      do j=1,i-1
         if ( ( like_var_present(i, m_job, job, m_receive_new_in, receive_new_in) .eq. 1) .and. &
              ( like_var_present(j, m_job, job, m_receive_new_in, receive_new_in) .eq. 1) ) then

            !first revert back to original values
            call mygrid_revert()
            call perturb_mygrid( i,1)
            call perturb_mygrid( j,1)
            call gen_perturb_choice_probs( probs_perturb1 )

            call mygrid_revert()
            call perturb_mygrid( i,-1)
            call perturb_mygrid( j,1)
            call gen_perturb_choice_probs( probs_perturb2 )

            call mygrid_revert()
            call perturb_mygrid( i,1)
            call perturb_mygrid( j,-1)
            call gen_perturb_choice_probs( probs_perturb3 )

            call mygrid_revert()
            call perturb_mygrid( i,-1)
            call perturb_mygrid( j,-1)
            call gen_perturb_choice_probs( probs_perturb4 )


            second_derivative_out(:,:,i,j) = ( probs_perturb1 - probs_perturb2  - &
                probs_perturb3 + probs_perturb4 ) /( 4*der_delta(i) * der_delta(j) )

!            if ( ( i .eq. var_m_job_length ) .and. ( j .eq. var_m_wage ) ) then
!               print *, "second der:", second_derivative_out(:,:,i,j)
!               print *, "probs_perturb1:", probs_perturb1
!               print *, "probs_perturb2:", probs_perturb2
!               print *, "probs_perturb3:", probs_perturb3
!               print *, "probs_perturb4:", probs_perturb4

!               stop
!            end if


         end if

      end do
   end do


contains

subroutine perturb_mygrid( k, sign_in)
   !for peterbing the points

   integer, intent(in) :: k !which variable to perturb
   integer, intent(in) :: sign_in !whether to increase or decrease

   integer :: k2


   if ( .not. ( (sign_in .eq. 1) .or. (sign_in .eq. -1 ) ) ) then
      print *, "error with sign", sign_in
      stop
   end if


   select case(k)

      case ( : var_wage_mean )

         like_temp % mygrid_t_invar % rv(k) =  &
            like_temp % mygrid_t_invar % rv(k) + der_delta(k)*sign_in

      case (var_ur_state : var_assets)

         like_temp % mygrid_t_var(1) % rv(k) =  &
            like_temp % mygrid_t_var(1) % rv(k) + der_delta(k)*sign_in

         if (m_receive_new_in .eq. 1) like_temp % mygrid_husband_accept(1) % rv(k) =  &
            like_temp % mygrid_husband_accept(1) % rv(k) + der_delta(k)*sign_in

         if (receive_new_in .eq. 1) like_temp % mygrid_wife_accept(1) % rv(k) =  &
            like_temp % mygrid_wife_accept(1) % rv(k) + der_delta(k)*sign_in

         if ( (m_receive_new_in .eq. 1) .and. (receive_new_in .eq. 1) ) &
            like_temp % mygrid_both_accept(1) % rv(k) =  &
            like_temp % mygrid_both_accept(1) % rv(k) + der_delta(k)*sign_in

      case ( var_m_wage,  var_m_non_pec_norm, var_m_job_length )

         if( m_job .eq. 1) then

            like_temp % mygrid_t_var(1) % rv(k) =  &
               like_temp % mygrid_t_var(1) % rv(k) + der_delta(k)*sign_in

            if (receive_new_in .eq. 1) like_temp % mygrid_wife_accept(1) % rv(k) =  &
               like_temp % mygrid_wife_accept(1) % rv(k) + der_delta(k)*sign_in

         end if

      case ( var_m_ui_ben, var_m_unemp_length  )

         if( m_job .eq. 0) then

            like_temp % mygrid_t_var(1) % rv(k) =  &
               like_temp % mygrid_t_var(1) % rv(k) + der_delta(k)*sign_in

            if (receive_new_in .eq. 1) like_temp % mygrid_wife_accept(1) % rv(k) =  &
               like_temp % mygrid_wife_accept(1) % rv(k) + der_delta(k)*sign_in

         end if

      case ( var_wage,  var_non_pec_norm, var_job_length )

         if( job .eq. 1) then

            like_temp % mygrid_t_var(1) % rv(k) =  &
               like_temp % mygrid_t_var(1) % rv(k) + der_delta(k)*sign_in

            if (m_receive_new_in .eq. 1) like_temp % mygrid_husband_accept(1) % rv(k) =  &
               like_temp % mygrid_husband_accept(1) % rv(k) + der_delta(k)*sign_in

         end if

      case ( var_ui_ben, var_unemp_length )

         if( job .eq. 0) then

            like_temp % mygrid_t_var(1) % rv(k) =  &
               like_temp % mygrid_t_var(1) % rv(k) + der_delta(k)*sign_in

            if (m_receive_new_in .eq. 1) like_temp % mygrid_husband_accept(1) % rv(k) =  &
               like_temp % mygrid_husband_accept(1) % rv(k) + der_delta(k)*sign_in

         end if

      case ( var_m_wage_new,  var_m_non_pec_norm_new )

         if( m_receive_new_in .eq. 1) then
            k2= k - der_vars_convert !gets the appropriate variable index from k2

            like_temp % mygrid_husband_accept(1) % rv(k2) =  &
               like_temp % mygrid_husband_accept(1) % rv(k2) + der_delta(k2)*sign_in

            if (receive_new_in .eq. 1) like_temp % mygrid_both_accept(1) % rv(k2) =  &
               like_temp % mygrid_both_accept(1) % rv(k2) + der_delta(k2)*sign_in

         end if

      case ( var_wage_new,  var_non_pec_norm_new )

         if( receive_new_in .eq. 1) then
            k2= k - der_vars_convert !gets the appropriate variable index from k2

            like_temp % mygrid_wife_accept(1) % rv(k2) =  &
               like_temp % mygrid_wife_accept(1) % rv(k2) + der_delta(k2)*sign_in

            if (m_receive_new_in .eq. 1) like_temp % mygrid_both_accept(1) % rv(k2) =  &
               like_temp % mygrid_both_accept(1) % rv(k2) + der_delta(k2)*sign_in

         end if

      case default
         print *, "error in select case for getting the choice probabilitiy derivatives"
         stop

   end select

end subroutine perturb_mygrid



subroutine gen_perturb_choice_probs( perturb_choice_probs_out )
   !given that I've perturb the mygrid, this generates the perturb of the choice probabilities
   real*8, dimension(0:1,0:1) :: perturb_choice_probs_out

   integer :: m_accept_new, accept_new

   perturb_choice_probs_out =  0

   call utility_function_setup(  like_temp % mygrid_t_var(1) % rv(var_kids), &
                                 like_temp % mygrid_t_var(1) % rv(var_youngest), &
                                 params_in, &
                                 like_temp % utility_params(:,1) )

   call asset_choice_like_update_one_t(like_temp, asset_matrix_in, 1)

   call utility_like_update_one_t(like_temp, params_in,1)

   do m_accept_new = 0, m_receive_new_in
   do accept_new = 0, receive_new_in

      if ( ( m_accept_new .eq. 1 ) .or. ( accept_new .eq. 1 ) ) then

         like_temp % m_accept_new(1) = m_accept_new
         like_temp % accept_new(1) = accept_new

         call choice_prob_like_update_one_t(like_temp,value_matrix_in,1)

         perturb_choice_probs_out(m_accept_new, accept_new ) = like_temp % choice_prob(1)

      end if

   end do
   end do

   !now get the prob of both reject, which is just 1 - sum of other probs calculated
   perturb_choice_probs_out(0, 0 ) = 1 -sum( perturb_choice_probs_out )


end subroutine gen_perturb_choice_probs


function like_var_present(index_in,m_job_in, job_in, m_receive_in, receive_in)
   !says whether a variable is present
   integer :: like_var_present
   integer, intent(in) :: index_in, m_job_in, job_in, m_receive_in, receive_in

   select case(index_in)
      case( :n_cont_vars)
         like_var_present = var_present(index_in,m_job_in,job_in )

      case(var_m_wage_new, var_m_non_pec_norm_new)
         like_var_present = m_receive_in

      case(var_wage_new, var_non_pec_norm_new)
         like_var_present = receive_in

      case default
         print *, "error in select case"
         stop

   end select


end function like_var_present


subroutine mygrid_revert()
   !reverts the mygrids back to their original values

   like_temp % mygrid_t_invar             = mygrid_t_invar_save
   like_temp % mygrid_t_var(1)            = mygrid_t_var_save
   like_temp % mygrid_husband_accept(1)   = mygrid_husband_accept_save
   like_temp % mygrid_wife_accept(1)      = mygrid_wife_accept_save
   like_temp % mygrid_both_accept(1)      = mygrid_both_accept_save


end subroutine mygrid_revert







end subroutine choice_prob_derivatives


subroutine choice_prob_der_write( m_receive_new_in, receive_new_in, first_der_in, second_der_in, unit_in )
   !write the derivatives of the choice probabilities to a file
   integer, intent(in) :: m_receive_new_in
   integer, intent(in) :: receive_new_in
   real*8, dimension(0:1,0:1,n_like_vars)  :: first_der_in
   real*8, dimension(0:1,0:1,n_like_vars, n_like_vars) :: second_der_in
   integer, intent(in) :: unit_in

   integer :: m_accept_new, accept_new, i, j

   do m_accept_new = 0, m_receive_new_in
   do accept_new = 0, receive_new_in

      call csv_write(unit_in, "   ", .true. )
      call csv_write(unit_in, "m_accept = ", .false. )
      call csv_write(unit_in, m_accept_new, .false. )
      call csv_write(unit_in, "accept = ", .false. )
      call csv_write(unit_in, accept_new, .true. )

      call csv_write(unit_in, "first derivative", .true. )

      do i = 1, n_like_vars

         call csv_write( unit_in, like_var_labels(i), .false. )
         call csv_write( unit_in, first_der_in(m_accept_new, accept_new, i), .true. )

      end do

      call csv_write(unit_in, "second derivative", .true. )

      call csv_write(unit_in, "  ", .false.)
      do i=1,  n_like_vars
         call csv_write(unit_in, like_var_labels(i),.false.)
      end do
      call csv_next_record(unit_in)

      do i = 1, n_like_vars

         call csv_write(unit_in, like_var_labels(i), .false.)

         do j=1, i
            call csv_write(unit_in, second_der_in(m_accept_new, accept_new,i,j),.false.)

         end do
         call csv_next_record(unit_in)

      end do


   end do
   end do



end subroutine choice_prob_der_write








subroutine like_setup_all(hh_data_in, params_in, savings_switch_in, value_matrix_in, asset_matrix_in, &
   like_object_out, like_target_out)
   !sets up all the components of the likelihood

   type(data_i_type), dimension(min_obs:max_obs), intent(in) :: hh_data_in
   real*8, dimension(n_params), intent(in) :: params_in
   logical, intent(in) :: savings_switch_in !if false, then don't update savings choice, keep it at the intial level of the current assets
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
   real*8, dimension(n_grid_points), intent(in) :: asset_matrix_in
   type(like_type), dimension(:), allocatable, intent(out) :: like_object_out
   type(like_target_type), dimension(:), allocatable, intent(out) :: like_target_out

   integer :: i,b

   allocate( like_object_out(min_obs:max_obs) )
   allocate( like_target_out(min_obs:max_obs) )

   do i=min_obs,max_obs

    !  print *, "hh #", i

      !first the main setup routines
      call like_target_out(i) % allocate( hh_data_in(i) )
      call like_object_out(i) % like_init_pointer( like_target_out(i), hh_data_in(i) )
      call like_object_out(i) % like_main_setup( hh_data_in(i) )
    !  call like_object_out(i) % get_birth_ui_means( hh_data_in(i), params_in)
      call like_target_out(i) % copy_targets_init()

      !now initialize the other parts
      call like_object_out(i) % joboffer_like_update( hh_data_in(i), params_in )
      call like_object_out(i) % job_probs_like_update( hh_data_in(i), params_in )
      call like_object_out(i) % utility_params_like_update( hh_data_in(i), params_in )

      if( savings_switch_in .eqv. .true. ) then
         call like_object_out(i) % asset_choice_like_update( asset_matrix_in, params_in  )
      end if

      call like_object_out(i) % utility_like_update( params_in )
   !   call like_object_out(i) % jobloss_like_update( hh_data_in(i), params_in )
  !    call like_object_out(i) % ols_partial_like_update( ols_vec_in )
      call like_object_out(i) % choice_prob_like_update(value_matrix_in  )

      if( savings_switch_in .eqv. .true. ) then
         call like_object_out(i) % asset_error_prob_like_update( hh_data_in(i), params_in )
      end if

   end do

   !now copy some of the time_invariant variables to the inactive parts
   do b= 1, block_leisure_params4
      call copy_mygrid_t_invar( like_object_out, b, like_target_out )
   end do





   !now do for jobloss, which I haven't incorporated into the copy_mygrid_t_invar function yet
!   do i=min_obs:max_obs
!      like_target_out(i) % mygrid_t_invar2 % rv(var_m_jobloss_mean) = &
!         like_target_out(i) % mygrid_t_invar1 % rv(var_m_jobloss_mean)
!      like_target_out(i) % mygrid_t_invar2 % rv(var_jobloss_mean) = &
!         like_target_out(i) % mygrid_t_invar1 % rv(var_jobloss_mean)
!   end do





end subroutine like_setup_all


subroutine like_error_hunt(like_object_in,i)
   !hunts for a nan or inf in the likelihood function, or an error with the indicators
   class(like_type), intent(in) :: like_object_in
   integer, intent(in) :: i !index for this household
   integer :: t,k

   if ( like_object_in % pointer_initialized .eqv. .false.) then
      print *, "pointer non intialized for hh=", i
   end if


   if ((like_object_in % max_time <1) .or. (like_object_in % max_time >N_t)) then
      print *, "error with max time for hh=", i, like_object_in % max_time
   end if


   !some time_invariant objects
   do k=1,n_hetero
      if ( nan_inf_check(like_object_in % hetero(k) ) ) then
         print *, "error for hetero", k, "for hh", i, like_object_in % hetero(k)
      end if
   end do

   if ( nan_inf_check(like_object_in % m_wage_mean_nohetero ) ) then
      print *, "error with  m_wage_mean_nohetero for hh", i, like_object_in % m_wage_mean_nohetero
   end if

   if ( nan_inf_check(like_object_in % wage_mean_nohetero ) ) then
      print *, "error with  wage_mean_nohetero for hh", i, like_object_in % wage_mean_nohetero
   end if


   !first for some indicators
   do t=2, like_object_in % max_time
      if (.not.((like_object_in % m_accept_new(t) .eq. 0) .or. (like_object_in % m_accept_new(t) .eq. 1))) then
         print *, "error with accep new for hh", i, "at time", t, like_object_in % m_accept_new(t)
      end if

      if (.not.((like_object_in % accept_new(t) .eq. 0) .or. (like_object_in % accept_new(t) .eq. 1))) then
         print *, "error with accep new for hh", i, "at time", t, like_object_in % accept_new(t)
      end if

      if ( ((like_object_in % m_time_job_ends(t) < 0) .or. (like_object_in % m_time_job_ends(t) > N_t)) .and. &
           (like_object_in % m_accept_new(t) .eq. 1)) then
         print *, "error with new job start time for hh", i, "at time", t, like_object_in % m_time_job_ends(t)
      end if

      if ( ((like_object_in % time_job_ends(t) < 0) .or. (like_object_in % time_job_ends(t) > N_t)) .and. &
           (like_object_in % accept_new(t) .eq. 1)) then
         print *, "error with new job start time for hh", i, "at time", t, like_object_in % time_job_ends(t)
      end if


      if (.not.((like_object_in % m_receive_new(t) .eq. 0) .or. (like_object_in % m_receive_new(t) .eq. 1))) then
         print *, "error with m_receive new for hh", i, "at time", t, like_object_in % m_receive_new(t)
      end if

      if (.not.((like_object_in % receive_new(t) .eq. 0) .or. (like_object_in % receive_new(t) .eq. 1))) then
         print *, "error with recieve new for hh", i, "at time", t, like_object_in % receive_new(t)
      end if

      if (t .ne. like_object_in % max_time) then

         if (.not.((like_object_in % birth(t) .eq. 0) .or. (like_object_in % birth(t) .eq. 1))) then
            print *, "error with birth for hh", i, "at time", t, like_object_in % birth(t)
         end if

         if (.not.((like_object_in % m_jobloss(t) .eq. 0) .or. (like_object_in % m_jobloss(t) .eq. 1))) then
            print *, "error with m_jobloss for hh", i, "at time", t, like_object_in % m_jobloss(t)
         end if

         if (.not.((like_object_in % jobloss(t) .eq. 0) .or. (like_object_in % jobloss(t) .eq. 1))) then
            print *, "error with jobloss for hh", i, "at time", t, like_object_in % jobloss(t)
         end if

      end if

      !now for indicators combos that shouldn't happen
      if ((like_object_in % m_accept_new(t) .eq. 1) .and. (like_object_in % m_receive_new(t) .eq. 0)) then
         print *, "error with accep and receive new for hh", i, "at time", t, &
            like_object_in % m_accept_new(t), like_object_in % m_receive_new(t)
      end if

      if ((like_object_in % accept_new(t) .eq. 1) .and. (like_object_in % receive_new(t) .eq. 0)) then
         print *, "error with accep and receive new for hh", i, "at time", t, &
            like_object_in % accept_new(t), like_object_in % receive_new(t)
      end if

      !now for probabilities
      if( .not. ((like_object_in % m_joboffer_prob(t)>0) .and. (like_object_in % m_joboffer_prob(t)<1))) then
         print *, "error with m_joboffer_prob for hh", i, "at time", t, like_object_in % m_joboffer_prob(t)
      end if

      if( .not. ((like_object_in % joboffer_prob(t)>0) .and. (like_object_in % joboffer_prob(t)<1))) then
         print *, "error with joboffer_prob for hh", i, "at time", t, like_object_in % joboffer_prob(t)
      end if





      if (like_object_in % m_receive_new(t) .eq. 1) then
         if( .not. ((like_object_in % m_hours_prob(t)>0) .and. &
                  (like_object_in % m_hours_prob(t)<1))) then
            print *, "error with m_hours_prob for hh", i, "at time", t, like_object_in % m_hours_prob(t)
         end if

         if( .not. ((like_object_in % m_ln_wage_prob(t)>-huge(0.0D+00)) .and. &
                     (like_object_in % m_ln_wage_prob(t)< 2 ))) then
            print *, "error with m_ln_wage_prob for hh", i, "at time", t, like_object_in % m_ln_wage_prob(t)
         end if



         if( .not.   ( (   (like_object_in % m_wage_new(t) >= grid_max( var_m_wage) ) .and. &
                              ( abs( like_object_in % mygrid_husband_accept(t) % rv(var_m_wage) - &
                              grid_max( var_m_wage) ) < zero_tol ) ) .or. &
                        (   (like_object_in % m_wage_new(t) <= grid_min( var_m_wage) ) .and. &
                              ( abs( like_object_in % mygrid_husband_accept(t) % rv(var_m_wage) - &
                              grid_min( var_m_wage) ) < zero_tol ) ) .or. &
                        (   (like_object_in % m_wage_new(t) > grid_min( var_m_wage) ) .and. &
                              (like_object_in % m_wage_new(t) < grid_max( var_m_wage) ) .and. &
                              ( abs( like_object_in % mygrid_husband_accept(t) % rv(var_m_wage) - &
                              like_object_in % m_wage_new(t) ) < zero_tol ) )    ) ) then

            print *, "error with m_wages_new for hh", i, "at time", t, like_object_in % m_wage_new(t), &
               like_object_in % mygrid_husband_accept(t) % rv(var_m_wage)
         end if



      else
         if( abs(like_object_in % m_hours_prob(t)-1) > zero_tol ) then
            print *, "error with m_hours_prob for hh", i, "at time", t, like_object_in % m_hours_prob(t)
         end if

         if( like_object_in % m_ln_wage_prob(t) < -zero_tol ) then
            print *, "error with m_ln_wage_prob for hh", i, "at time", t, like_object_in % m_ln_wage_prob(t), &
             "it should be 0"
         end if

      end if



      if (like_object_in % receive_new(t) .eq. 1) then
         if( .not. ((like_object_in % hours_prob(t)>0) .and. (like_object_in % hours_prob(t)<1))) then
            print *, "error with hours_prob for hh", i, "at time", t, like_object_in % hours_prob(t)
         end if

         if( .not. ((like_object_in % ln_wage_prob(t)>-huge(0.0D+00)) .and. &
            (like_object_in % ln_wage_prob(t)<2))) then
            print *, "error with ln_wage_prob for hh", i, "at time", t, like_object_in % ln_wage_prob(t)
         end if

         if( .not.   ( (   (like_object_in % wage_new(t) >= grid_max( var_wage) ) .and. &
                              ( abs( like_object_in % mygrid_wife_accept(t) % rv(var_wage) - &
                              grid_max( var_wage) ) < zero_tol ) ) .or. &
                        (   (like_object_in % wage_new(t) <= grid_min( var_wage) ) .and. &
                              ( abs( like_object_in % mygrid_wife_accept(t) % rv(var_wage) - &
                              grid_min( var_wage) ) < zero_tol ) ) .or. &
                        (   (like_object_in % wage_new(t) > grid_min( var_wage) ) .and. &
                              (like_object_in % wage_new(t) < grid_max( var_wage) ) .and. &
                              ( abs( like_object_in % mygrid_wife_accept(t) % rv(var_wage) - &
                              like_object_in % wage_new(t) ) < zero_tol ) )    ) ) then

            print *, "error with wages_new for hh", i, "at time", t, like_object_in % wage_new(t), &
               like_object_in % mygrid_wife_accept(t) % rv(var_wage)
         end if


      else
         if( abs(like_object_in % hours_prob(t)-1) > zero_tol ) then
            print *, "error with hours_prob for hh", i, "at time", t, like_object_in % hours_prob(t)
         end if

         if( like_object_in % ln_wage_prob(t) < -zero_tol ) then
            print *, "error with ln_wage_prob for hh", i, "at time", t, like_object_in % ln_wage_prob(t), &
            "it should be 0"
         end if

      end if

   end do

   !now check error in mygrid
   call like_object_in % mygrid_t_invar % mygrid_error_hunt(.true.,.false.,i,0,0)

   do t=2, like_object_in % max_time
      call like_object_in % mygrid_t_var(t) % mygrid_error_hunt(.false.,.true.,i,t,1)

      if (like_object_in % m_receive_new(t) .eq. 1) then
         call like_object_in % mygrid_husband_accept(t) % mygrid_error_hunt(.false.,.true.,i,t,2)
      end if

      if (like_object_in % receive_new(t) .eq. 1) then
         call like_object_in % mygrid_wife_accept(t) % mygrid_error_hunt(.false.,.true.,i,t,3)
      end if

      if ((like_object_in % m_receive_new(t) .eq. 1) .and. (like_object_in % receive_new(t) .eq. 1))then
         call like_object_in % mygrid_both_accept(t) % mygrid_error_hunt(.false.,.true.,i,t,4)
      end if


      !checks mygrids that should be the same in regarsd to certain variables
      if (like_object_in % m_receive_new(t) .eq. 1) then
         call mygrid_same_check( "f",  like_object_in % mygrid_t_var(t), &
                                       like_object_in % mygrid_husband_accept(t), i,t )

      end if

      if (like_object_in % receive_new(t) .eq. 1) then
         call mygrid_same_check( "m",  like_object_in % mygrid_t_var(t), &
                                       like_object_in % mygrid_wife_accept(t), i,t )
      end if

      if ( (like_object_in % m_receive_new(t) .eq. 1) .and. &
           (like_object_in % receive_new(t) .eq. 1) ) then

         call mygrid_same_check( "m",  like_object_in % mygrid_both_accept(t), &
                                       like_object_in % mygrid_husband_accept(t), i,t )

         call mygrid_same_check( "f",  like_object_in % mygrid_both_accept(t), &
                                       like_object_in % mygrid_wife_accept(t), i,t )
      end if

   end do

   !now utility parameters
   do t=2,like_object_in % max_time
      if ((like_object_in % m_receive_new(t) .eq. 1) .or. &
         (like_object_in % receive_new(t) .eq. 1)) then

         if (like_object_in % utility_params_created(t) .ne. 1) then
            print *, "utility parameters not created for hh", i, "at time", t
         end if


         do k=1,n_utility_params
            if (nan_inf_check(like_object_in % utility_params(k,t))) then
               print *, "error in utility parameter for hh", i, "at time", t, &
                  like_object_in % utility_params(k,t)
            end if
         end do

         if (nan_inf_check(like_object_in % utility_both_reject(t))) then
            print *, "error in utility both reject for hh", i, "at time", t, &
               like_object_in % utility_both_reject(t)
         end if

      end if

      if (like_object_in % m_receive_new(t) .eq. 1) then
         if (nan_inf_check(like_object_in % utility_husband_accept(t))) then
            print *, "error in utility husband accept for hh", i, "at time", t, &
               like_object_in % utility_husband_accept(t)
         end if
      end if

      if (like_object_in % receive_new(t) .eq. 1) then
         if (nan_inf_check(like_object_in % utility_wife_accept(t))) then
            print *, "error in utility wife accept for hh", i, "at time", t, &
               like_object_in % utility_wife_accept(t)
         end if
      end if

      if ((like_object_in % m_receive_new(t) .eq. 1) .and. &
         (like_object_in % receive_new(t) .eq. 1)) then
         if (nan_inf_check(like_object_in % utility_both_accept(t))) then
            print *, "error in utility both accept for hh", i, "at time", t, &
               like_object_in % utility_both_accept(t)
         end if
      end if

   end do

   !ols vec
!   if (any(like_object_in % m_receive_new .eq. 1) .or. any(like_object_in % receive_new .eq. 1)) then
!      if (like_object_in % ols_partial_created .ne. 1) then
!         print *, "ols partial not created for hh", i
!      end if
!   end if

!   do k=1,length_ols_vec_t_var
!      if (nan_inf_check(like_object_in % ols_vec(k))) then
!         print *, "error in ols vec for hh ", i, "at time", t, like_object_in % ols_vec(k)
!      end if
!   end do


   !choice prob
   do t=2,like_object_in % max_time
      if( .not. ((like_object_in % choice_prob(t)>0) .and. (like_object_in % choice_prob(t)<1+zero_tol))) then
         print *, "error with choice prob for hh", i, "at time", t, like_object_in % choice_prob(t)
      end if
   end do

   !jobloss prob
!   do t=1,like_object_in % max_time - 1
!      if ( like_object_in % mygrid_t_var(t) % iv(var_m_job) .eq. 1) then
!         if( .not. ((like_object_in % m_jobloss_prob(t)>0) .and. &
!            (like_object_in % m_jobloss_prob(t)<1))) then
!            print *, "error with m_jobloss_prob for hh", i, "at time", t, &
!               like_object_in % m_jobloss_prob(t)
!         end if
!      end if

!      if ( like_object_in % mygrid_t_var(t) % iv(var_job) .eq. 1) then
!         if( .not. ((like_object_in % jobloss_prob(t)>0) .and. &
!            (like_object_in % jobloss_prob(t)<1))) then
!            print *, "error with jobloss_prob for hh", i, "at time", t, &
!               like_object_in % jobloss_prob(t)
!         end if
!      end if

!   end do

   !checker bounds of variables
   !mygrid_t_invar
   do k= 1, var_wage_mean

      if( var_present(k,   like_object_in % mygrid_t_invar % iv(var_m_job), &
                           like_object_in % mygrid_t_invar % iv(var_job) ) .eq. 1) then

         if (  (like_object_in % mygrid_t_invar % rv(k) < grid_min(k) - zero_tol ) .or. &
               (like_object_in % mygrid_t_invar % rv(k) > grid_max(k) + zero_tol )) then

            print *, "problem with variable ", var_labels(k), "with mygrid_t_invar at time", t, &
            like_object_in % mygrid_t_invar % rv(k)
         end if

      end if

   end do



  ! mygrid_t_var
   do t=1, like_object_in % max_time
      do k= var_ur_state, var_unemp_length
         if( var_present(k,   like_object_in % mygrid_t_var(t) % iv(var_m_job), &
                              like_object_in % mygrid_t_var(t) % iv(var_job) ) .eq. 1) then

            if (  (like_object_in % mygrid_t_var(t) % rv(k) < grid_min(k) - zero_tol ) .or. &
                  (like_object_in % mygrid_t_var(t) % rv(k) > grid_max(k) + zero_tol )) then

               print *, "problem with variable ", var_labels(k), "with mygrid_t_var at time", t, &
               like_object_in % mygrid_t_var(t) % rv(k)
            end if

         end if

      end do
   end do

  ! mygrid_husband_accept
   do t=2, like_object_in % max_time
      if ( like_object_in % m_receive_new(t) .eq. 1 ) then
         do k= var_ur_state, var_unemp_length
            if( var_present(k,   like_object_in % mygrid_husband_accept(t) % iv(var_m_job), &
                                 like_object_in % mygrid_husband_accept(t) % iv(var_job) ) .eq. 1 ) then

               if (  (like_object_in % mygrid_husband_accept(t) % rv(k) < grid_min(k) - zero_tol ) .or. &
                     (like_object_in % mygrid_husband_accept(t) % rv(k) > grid_max(k) + zero_tol )) then

                  print *, "problem with variable ", var_labels(k), "with mygrid_husband_accept at time", t, &
                  like_object_in % mygrid_husband_accept(t) % rv(k)
               end if

            end if
         end do
      end if
   end do

  ! mygrid_wife_accept
   do t=2, like_object_in % max_time
      if ( like_object_in % receive_new(t) .eq. 1 ) then
         do k= var_ur_state, var_unemp_length
            if( var_present(k,   like_object_in % mygrid_wife_accept(t) % iv(var_m_job), &
                                 like_object_in % mygrid_wife_accept(t) % iv(var_job) ) .eq. 1 ) then


               if (  (like_object_in % mygrid_wife_accept(t) % rv(k) < grid_min(k) - zero_tol ) .or. &
                     (like_object_in % mygrid_wife_accept(t) % rv(k) > grid_max(k) + zero_tol )) then

                  print *, "problem with variable ", var_labels(k), "with mygrid_wife_accept at time", t, &
                  like_object_in % mygrid_wife_accept(t) % rv(k)
               end if

            end if
         end do
      end if
   end do

  ! mygrid_both_accept
   do t=2, like_object_in % max_time
      if (   ( like_object_in % m_receive_new(t) .eq. 1 ) .and. &
            ( like_object_in % receive_new(t) .eq. 1 )  ) then
         do k= var_ur_state, var_unemp_length
            if( var_present(k,   like_object_in % mygrid_both_accept(t) % iv(var_m_job), &
                                 like_object_in % mygrid_both_accept(t) % iv(var_job) ) .eq. 1 ) then


               if (  (like_object_in % mygrid_both_accept(t) % rv(k) < grid_min(k) - zero_tol ) .or. &
                     (like_object_in % mygrid_both_accept(t) % rv(k) > grid_max(k) + zero_tol )) then

                  print *, "problem with variable ", var_labels(k), "with mygrid_both_accept at time", t, &
                  like_object_in % mygrid_both_accept(t) % rv(k)
               end if

            end if
         end do
      end if
   end do




end subroutine like_error_hunt


subroutine like_same_check(like_object_in, like_target_in, block_in)
   !test whether two target components are the same for a given block, given that for this block, I've drawn two parameters that are the same
   type(like_type), dimension(min_obs:max_obs), intent(in) :: like_object_in
   type(like_target_type), dimension(min_obs:max_obs), intent(in) :: like_target_in
   integer, intent(in) :: block_in
   integer :: i, t

   select case(block_in)
      case ( block_joboffer_params1, block_joboffer_params2, block_unemp_length )

         do i = min_obs,max_obs
            do t = 2, like_object_in(i) % max_time

               if( abs (like_target_in(i) % m_joboffer_prob1(t) - &
                        like_target_in(i) % m_joboffer_prob2(t) )  > zero_tol ) then

                  print *, "m_joboffer_prob targets different for hh", i, "at time", t, &
                     like_target_in(i) % m_joboffer_prob1(t), like_target_in(i) % m_joboffer_prob2(t)

               end if

               if( abs (like_target_in(i) % joboffer_prob1(t) - &
                        like_target_in(i) % joboffer_prob2(t) )  > zero_tol ) then

                  print *, "joboffer_prob targets different for hh", i, "at time", t, &
                     like_target_in(i) % joboffer_prob1(t), like_target_in(i) % joboffer_prob2(t)

               end if

            end do
         end do

      case ( block_wage_params1, block_wage_params2, block_wage_params3 )

         do i = min_obs,max_obs
            do t = 2, like_object_in(i) % max_time

               if( abs (like_target_in(i) % m_ln_wage_prob1(t) - &
                        like_target_in(i) % m_ln_wage_prob2(t) )  > zero_tol ) then

                  print *, "m_ln_wage_prob targets different for hh", i, "at time", t, &
                     like_target_in(i) % m_ln_wage_prob1(t), like_target_in(i) % m_ln_wage_prob2(t)

               end if

               if( abs (like_target_in(i) % ln_wage_prob1(t) - &
                        like_target_in(i) % ln_wage_prob2(t) )  > zero_tol ) then

                  print *, "ln_wage_prob targets different for hh", i, "at time", t, &
                     like_target_in(i) % ln_wage_prob1(t), like_target_in(i) % ln_wage_prob2(t)

               end if

            end do
         end do


         do i = min_obs,max_obs
            do t = 2, like_object_in(i) % max_time

               if( abs (like_target_in(i) % m_hours_prob1(t) - &
                        like_target_in(i) % m_hours_prob2(t) )  > zero_tol ) then

                  print *, "m_hours_prob targets different for hh", i, "at time", t, &
                     like_target_in(i) % m_hours_prob1(t), like_target_in(i) % m_hours_prob2(t)

               end if

               if( abs (like_target_in(i) % hours_prob1(t) - &
                        like_target_in(i) % hours_prob2(t) )  > zero_tol ) then

                  print *, "hours_prob targets different for hh", i, "at time", t, &
                     like_target_in(i) % hours_prob1(t), like_target_in(i) % hours_prob2(t)

               end if

            end do
         end do


      case ( block_leisure_params1, block_leisure_params2, block_leisure_params3, block_leisure_params4 )

      case default
         print *, "error in select case for get_log_like_params"

   end select

   !now asset error and joboffer prob
   do i = min_obs,max_obs
      do t = 2, like_object_in(i) % max_time

         if( abs (like_target_in(i) % choice_prob1(t) - &
                  like_target_in(i) % choice_prob2(t) )  > zero_tol ) then

            print *, "choice_prob targets different for hh", i, "at time", t, &
               like_target_in(i) % choice_prob1(t), like_target_in(i) % choice_prob2(t)

         end if

      end do

      if( abs (like_target_in(i) % ln_asset_prob1 - &
               like_target_in(i) % ln_asset_prob2 )  > zero_tol ) then

         print *, "ln_asset_prob targets different for hh", i,  &
            like_target_in(i) % ln_asset_prob1, like_target_in(i) % ln_asset_prob2

      end if

   end do





end subroutine like_same_check



subroutine like_stats_print( like_object_in, hh_data_in, params_in, write_unit_in, savings_switch_in)
   !prints percentiles of various components of the likelihood
   type(like_type), dimension(min_obs:max_obs), intent(in out) :: like_object_in
   type(data_i_type), dimension(min_obs:max_obs), intent(in) :: hh_data_in
   real*8, dimension(n_params), intent(in) :: params_in
   integer, intent(in) :: write_unit_in
   logical, intent(in) :: savings_switch_in

   integer :: n_receive_local, n_choice_local, n_asset_local, n_wage_mean_local
   integer :: receive_counter, choice_counter, asset_counter, wage_mean_counter
   real*8, dimension(:), allocatable :: joboffer_prob_local, wage_prob_local, choice_prob_local, &
      asset_prob_local, wage_mean_local
   integer :: i, t
   real*8 :: m_wage_new, wage_new

   !first get sizes
   n_receive_local = 0
   n_choice_local = 0
   n_asset_local = 0
   n_wage_mean_local = (max_obs - min_obs + 1) *2

   do i=min_obs,max_obs
      do t=2 , like_object_in(i) % max_time

         if ( like_object_in(i) % m_receive_new(t) .eq. 1 ) then
            n_receive_local = n_receive_local + 1

         end if

         if ( like_object_in(i) % receive_new(t) .eq. 1 ) then
            n_receive_local = n_receive_local + 1

         end if


         if ( ( like_object_in(i) % m_receive_new(t) .eq. 1 ) .or. &
               ( like_object_in(i) % receive_new(t) .eq. 1 ) ) then
            n_choice_local = n_choice_local + 1

         end if

      end do !t
   end do !i


   if( savings_switch_in .eqv. .true. ) then
      do i=min_obs,max_obs
         !now n_asset_ind
         if (( hh_data_in(i) % liquid_time1 > 0 ) .and. &
            ( hh_data_in(i) % liquid_time1 <= like_object_in(i) % max_time )) then

            n_asset_local = n_asset_local + 1

         end if

         if (( hh_data_in(i) % liquid_time2 > 0 ) .and. &
            ( hh_data_in(i) % liquid_time2 <= like_object_in(i) % max_time ))  then

            n_asset_local = n_asset_local + 1

         end if

         if (( hh_data_in(i) % liquid_time3 > 0 ) .and. &
            ( hh_data_in(i) % liquid_time3 <= like_object_in(i) % max_time ))  then

            n_asset_local = n_asset_local + 1

         end if


      end do !over i
   end if !savings switch



   !now allocate
   allocate(joboffer_prob_local(n_receive_local))
   allocate(wage_prob_local(n_receive_local))
   allocate(wage_mean_local(n_wage_mean_local))
   allocate(choice_prob_local(n_choice_local))
   if(savings_switch_in .eqv. .true.) allocate(asset_prob_local(n_asset_local))

   receive_counter = 0
   wage_mean_counter = 0
   choice_counter = 0
   asset_counter = 0


   !now put these probabilities into a vector
   do i=min_obs,max_obs
      do t=2,like_object_in(i) % max_time

         if ( like_object_in(i) % m_receive_new(t) .eq. 1 ) then
            receive_counter = receive_counter + 1
            joboffer_prob_local(receive_counter) = like_object_in(i) % m_joboffer_prob(t)


            m_wage_new = like_object_in(i) % mygrid_husband_accept(t) % rv( var_m_wage )

!            if (  ( m_wage_new > ( grid_min(var_m_wage) + zero_tol ) ) .and. &
!                  ( m_wage_new < ( grid_max(var_m_wage) - zero_tol ) )  ) then

               wage_prob_local(receive_counter) = exp( like_object_in(i) % m_ln_wage_prob(t) ) * &
                  params_in( param_sd_wage ) !convert to the standard normal pdf to make stats easier to interpret

       !     else
         !       wage_prob_local(receive_counter) = exp( like_object_in(i) % m_ln_wage_prob(t) )
       !     end if

!            if ( wage_prob_local(receive_counter) < tiny(0.0D+00) ) then
!               print *, "hh ", i, "t", t
!               call like_object_in(i) % print(t)
!               call mpi_abort(mpi_comm_world, 911, err)
!               stop

!            end if

!           if ( wage_prob_local(receive_counter) > .4D+00 ) then
!               print *, "problem with wage probs too high"
!               print *, "wage prob", wage_prob_local(receive_counter)
!               print *, "param sd", params_in( param_sd_wage )
!               print *, "hh ", i, "t", t
!               print *, "  "
!               call like_object_in(i) % print(t)
!               call mpi_abort(mpi_comm_world, 911, err)
!               stop

!            end if


         end if

         if ( like_object_in(i) % receive_new(t) .eq. 1 ) then
            receive_counter = receive_counter + 1
            joboffer_prob_local(receive_counter) = like_object_in(i) % joboffer_prob(t)

            wage_new = like_object_in(i) % mygrid_wife_accept(t) % rv( var_wage )

!            if (  ( wage_new > ( grid_min(var_wage) + zero_tol ) ) .and. &
!                  ( wage_new < ( grid_max(var_wage) - zero_tol ) )  ) then

               wage_prob_local(receive_counter) = exp( like_object_in(i) % ln_wage_prob(t) ) * &
                  params_in( param_sd_wage ) !convert to the standard normal pdf to make stats easier to interpret

!            else
!                wage_prob_local(receive_counter) = exp( like_object_in(i) % ln_wage_prob(t) )
!            end if

!            if ( wage_prob_local(receive_counter) < tiny(0.0D+00) ) then
!               print *, "hh ", i, "t", t
!               call like_object_in(i) % print(t)
!               call mpi_abort(mpi_comm_world, 911, err)
!               stop

!            end if

!           if ( wage_prob_local(receive_counter) > .4D+00 ) then
!               print *, "problem with wage probs too high"
!               print *, "hh ", i, "t", t
!               call like_object_in(i) % print(t)
!               call mpi_abort(mpi_comm_world, 911, err)
!               stop

!            end if

         end if


         if ( ( like_object_in(i) % m_receive_new(t) .eq. 1 ) .or. &
               ( like_object_in(i) % receive_new(t) .eq. 1 ) ) then
            choice_counter = choice_counter + 1
            choice_prob_local(choice_counter) = like_object_in(i) % choice_prob(t)

         end if

      end do !t
   end do !i

   if( savings_switch_in .eqv. .true. ) then
      do i=min_obs,max_obs
         !now n_asset_ind
         if (( hh_data_in(i) % liquid_time1 > 0 ) .and. &
            ( hh_data_in(i) % liquid_time1 <= like_object_in(i) % max_time )) then

            asset_counter = asset_counter + 1
            asset_prob_local(asset_counter) = asset_pdf_func( hh_data_in(i) % liquid_time1, &
               hh_data_in(i) % liquid_net1 )

         end if

         if (( hh_data_in(i) % liquid_time2 > 0 ) .and. &
            ( hh_data_in(i) % liquid_time2 <= like_object_in(i) % max_time ))  then

            asset_counter = asset_counter + 1
            asset_prob_local(asset_counter) = asset_pdf_func( hh_data_in(i) % liquid_time2, &
               hh_data_in(i) % liquid_net2 )

         end if

         if (( hh_data_in(i) % liquid_time3 > 0 ) .and. &
            ( hh_data_in(i) % liquid_time3 <= like_object_in(i) % max_time ))  then

            asset_counter = asset_counter + 1
            asset_prob_local(asset_counter) = asset_pdf_func( hh_data_in(i) % liquid_time3, &
               hh_data_in(i) % liquid_net3 )

         end if
      end do !over i
   end if !savings switch

   !now wage mean
   do i = min_obs, max_obs
      wage_mean_counter = wage_mean_counter + 1
      wage_mean_local( wage_mean_counter ) = like_object_in(i) % mygrid_t_invar % rv( var_m_wage_mean )

      wage_mean_counter = wage_mean_counter + 1
      wage_mean_local( wage_mean_counter ) = like_object_in(i) % mygrid_t_invar % rv( var_wage_mean )
   end do


 !  call stats_support_sub( n_receive_local,     joboffer_prob_local, "joboffer prob" )
 !  call stats_support_sub( n_receive_local,     wage_prob_local,      "wage prob" )
 !  call stats_support_sub( n_wage_mean_local,   wage_mean_local,      "wage mean" )
   call stats_support_sub( n_choice_local ,     choice_prob_local,    "choice prob" )
   if( savings_switch_in .eqv. .true. ) then
  !    call stats_support_sub(n_asset_local,     asset_prob_local, "asset prob" )
   end if


contains

function asset_pdf_func( t_in, asset_data_in )
   !gets the standard normal pdf for the asset error
   real*8 :: asset_pdf_func
   integer, intent(in) :: t_in
   real*8, intent(in) :: asset_data_in

   real*8 :: asset_real, asset_sd_arg, sd

   asset_real = like_object_in(i) %  mygrid_t_var( t_in ) % rv( var_assets )
   asset_sd_arg = ( abs(asset_data_in) + abs(asset_real) ) * .5 !flexbible way of allow error variance to be greater the larger either the "real" assets are or the assets in the data are
   sd = exp(   params_in( param_asset_error_cons ) + &
               params_in( param_asset_error_linear )* asset_sd_arg )

   asset_pdf_func = pdf_norm(asset_real, asset_data_in , sd ) * sd

end function


subroutine stats_support_sub(n_local, vec_local, stats_label_in ) !make this a subroutine because I do similar things for different stats
   integer, intent(in) :: n_local
   real*8, dimension(n_local), intent(in) :: vec_local
   character(len=*), intent(in) :: stats_label_in !prints which statistic this is for

   integer, dimension(0:nprocs-1) :: recvcounts, displs
   integer :: n_global
   real*8, dimension(:), allocatable :: vec_global
   real*8, dimension(n_percent) :: percent_vec
   integer :: i, t

   call mpi_gather( n_local, 1, mpi_int, recvcounts, 1, mpi_int, 0, mpi_comm_world, err )
   n_global = sum( recvcounts )


   !allocate global vector
   if( myrank .eq. 0 ) then
      allocate( vec_global( n_global ) )
   else
      allocate( vec_global( n_global ) )
   end if

   !set up displacement vectors
   displs(0) = 0
   do i=1,nprocs-1
      displs(i) = recvcounts(i-1) + displs(i-1)
   end do

   !now get the global vector
   call mpi_gatherv( vec_local, n_local, mpi_real8, vec_global, recvcounts, &
        displs, mpi_real8, 0, mpi_comm_world, err )

   if( myrank .eq. 0 ) then


      percent_vec = percent_func(n_global, vec_global)


      write(write_unit_in,*) " "
      write(write_unit_in,*) stats_label_in // "  percentiles"
      do i=1,n_percent
         write(write_unit_in,*)  percent_nodes(i), percent_vec(i)
      end do

      write(write_unit_in,*) "min", minval(vec_global)
      write(write_unit_in,*) "max", maxval(vec_global)

   end if


end subroutine stats_support_sub


end subroutine like_stats_print

subroutine util_param_like_der(like_object_inout, hh_data_in, params_in, value_matrix_in, h_in, der_out )
   !this gets the derivative of the likelihood function wrt a parameters in the utility funciton
   !currently, this ignores assets
   !the parameters need to have been already perturbed, and the value matrix needs to be these perturbed values

   type(like_type), dimension(min_obs:max_obs), intent(in out) :: like_object_inout
   type(data_i_type), dimension(min_obs:max_obs), intent(in) :: hh_data_in
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension(n_grid_points), intent(in) :: value_matrix_in
   real*8, intent(in) :: h_in
   real*8, intent(out) :: der_out

   integer, dimension(:), allocatable:: utility_params_created
   real*8, dimension(:,:), allocatable :: utility_params
   real*8, dimension(:), allocatable :: utility_both_reject
   real*8, dimension(:), allocatable :: utility_husband_accept
   real*8, dimension(:), allocatable :: utility_wife_accept
   real*8, dimension(:), allocatable :: utility_both_accept
   real*8, dimension(:), allocatable :: choice_prob

   real*8, dimension(min_obs:max_obs) :: der_ind
   real*8 :: der_local

   integer :: i, max_time, t



   !first, back up the likelihood components of interest
   do i = min_obs, max_obs

      max_time = like_object_inout(i) % max_time

      allocate( utility_params_created (2:max_time) )
      allocate( utility_params (n_utility_params,2:max_time) )
      allocate( utility_both_reject (2:max_time) )
      allocate( utility_husband_accept (2:max_time) )
      allocate( utility_wife_accept (2:max_time) )
      allocate( utility_both_accept (2:max_time) )
      allocate( choice_prob (2:max_time) )

      utility_params_created     = like_object_inout(i) % utility_params_created
      utility_params             = like_object_inout(i) % utility_params
      utility_both_reject        = like_object_inout(i) % utility_both_reject
      utility_husband_accept     = like_object_inout(i) % utility_husband_accept
      utility_wife_accept        = like_object_inout(i) % utility_wife_accept
      utility_both_accept        = like_object_inout(i) % utility_both_accept
      choice_prob                = like_object_inout(i) % choice_prob

      !now get the components of interest
      call like_object_inout(i) % utility_params_like_update( hh_data_in(i), params_in )
      call like_object_inout(i) % utility_like_update( params_in )
      call like_object_inout(i) % choice_prob_like_update(value_matrix_in  )


      der_ind(i) = sum( log(like_object_inout(i) % choice_prob) - log(choice_prob) ) / h_in


      like_object_inout(i) % utility_params_created   = utility_params_created
      like_object_inout(i) % utility_params           = utility_params
      like_object_inout(i) % utility_both_reject      = utility_both_reject
      like_object_inout(i) % utility_husband_accept   = utility_husband_accept
      like_object_inout(i) % utility_wife_accept      = utility_wife_accept
      like_object_inout(i) % utility_both_accept      = utility_both_accept
      like_object_inout(i) % choice_prob              = choice_prob

      deallocate(utility_params_created)
      deallocate(utility_params)
      deallocate(utility_both_reject)
      deallocate(utility_husband_accept)
      deallocate(utility_wife_accept)
      deallocate(utility_both_accept)
      deallocate(choice_prob)

   end do !i

   der_local = sum(der_ind)

   call mpi_allreduce( der_local, der_out, 1, &
      mpi_real8, mpi_sum,  mpi_comm_world, err )


end subroutine util_param_like_der







!subroutine choice_prob_like_update_one_t(like_object_inout,t) (old)
!   !gets the accept/reject choice probability, given that a particular joboffer has been received
!   class(like_type), intent(in out) :: like_object_inout
!   integer, intent(in) :: t

!   real*8,dimension(4) :: vals_prob_input
!   integer :: choice_index

!      if (  (like_object_inout % m_receive_new(t) .eq. 0) .and.  &
!            (like_object_inout % receive_new(t) .eq. 0) ) then
!     !    print *, "both no offer"
!         like_object_inout % choice_prob(t) = 1

!      else if (like_object_inout % m_receive_new(t) .eq. 0) then !only wife receives an offer
!      !   print *, "only wife receives offer"
!         vals_prob_input(1) = like_object_inout % utility_both_reject(t) + &
!            value_function_partial( like_object_inout % mygrid_t_var(t), &
!                                    like_object_inout % ols_vec )

!         vals_prob_input(2) = like_object_inout % utility_wife_accept(t) + &
!            value_function_partial( like_object_inout % mygrid_wife_accept(t), &
!                                    like_object_inout % ols_vec )

!         choice_index = like_object_inout % accept_new(t) + 1
!         like_object_inout % choice_prob(t) = mlogit_prob(2,choice_index,vals_prob_input(1:2) )

!      else if (like_object_inout % receive_new(t) .eq. 0) then !only husband
!       !  print *, "only husband receives offer"
!         vals_prob_input(1) = like_object_inout % utility_both_reject(t) + &
!            value_function_partial( like_object_inout % mygrid_t_var(t), &
!                                    like_object_inout % ols_vec )

!         vals_prob_input(2) = like_object_inout % utility_husband_accept(t) + &
!            value_function_partial( like_object_inout % mygrid_husband_accept(t), &
!                                    like_object_inout % ols_vec )

!         choice_index = like_object_inout % m_accept_new(t) + 1
!         like_object_inout % choice_prob(t) = mlogit_prob(2,choice_index,vals_prob_input(1:2) )

!      else !both receive an offer
!       !  print *, "both receive offer"

!         vals_prob_input(1) = like_object_inout % utility_both_reject(t) + &
!            value_function_partial( like_object_inout % mygrid_t_var(t), &
!                                    like_object_inout % ols_vec )

!         vals_prob_input(2) = like_object_inout % utility_husband_accept(t) + &
!            value_function_partial( like_object_inout % mygrid_husband_accept(t), &
!                                    like_object_inout % ols_vec )

!         vals_prob_input(3) = like_object_inout % utility_wife_accept(t) + &
!            value_function_partial( like_object_inout % mygrid_wife_accept(t), &
!                                    like_object_inout % ols_vec )

!         vals_prob_input(4) = like_object_inout % utility_both_accept(t) + &
!            value_function_partial( like_object_inout % mygrid_both_accept(t), &
!                                    like_object_inout % ols_vec )

!         choice_index = 1 + like_object_inout % m_accept_new(t) &
!            + 2*like_object_inout % accept_new(t)
!         like_object_inout % choice_prob(t) = mlogit_prob(4,choice_index,vals_prob_input )

!      end if


!end subroutine choice_prob_like_update_one_t












end module likelihood_mod
